/*
 * Common (OS-independent) portion of
 * Broadcom 802.11abg Networking Device Driver
 *
 * Copyright 2005-2006, Broadcom Corporation
 * All Rights Reserved.                
 * 
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id$
 */

#include <wlc_cfg.h>
#include <typedefs.h>
#include <bcmdefs.h>
#include <osl.h>
#include <sbutils.h>
#include <bcmendian.h>
#include <bcmutils.h>
#include <proto/802.1d.h>
#include <proto/802.11.h>
#include <proto/802.11e.h>
#include <proto/wpa.h>
#include <proto/vlan.h>
#include <sbconfig.h>
#include <pcicfg.h>
#include <bcmsrom.h>
#include <wlioctl.h>
#include <epivers.h>
#include <proto/eapol.h>
#include <bcmwpa.h>

#include <sbhndpio.h>
#include <sbhnddma.h>
#include <hnddma.h>
#include <d11.h>
#include <wlc_rate.h>
#include <wlc_pub.h>
#include <wlc_rate_sel.h>
#include <wlc_key.h>
#include <wlc_bsscfg.h>
#include <wlc_channel.h>
#include <wlc_bsscfg.h>
#include <wlc_pio.h>
#include <wlc.h>
#include <wlc_scb.h>
#include <wlc_phy.h>
#include <wlc_led.h>
#include <wlc_frmutil.h>
#include <wlc_security.h>


#include <wl_export.h>
#include "d11ucode.h"


/*
 * buffer length needed for wlc_format_ssid
 * 32 SSID chars, max of 4 chars for each SSID char "\xFF", plus NULL.
 */
#define SSID_FMT_BUF_LEN	((4 * DOT11_MAX_SSID_LEN) + 1)

/* antenna swap threshold */
#define	ANTCNT			10		/* vanilla M_MAX_ANTCNT value */

#define SCAN_IN_PROGRESS(wlc)	((wlc)->scan.channel_idx != -1)

#define	WLC_ACTION_ASSOC	1 /* Association request for MAC */
#define	WLC_ACTION_ROAM		2 /* Roaming request for MAC */
#define	WLC_ACTION_SCAN		3 /* Scan request for MAC */
#define	WLC_ACTION_QUIET	4 /* Quiet request for MAC */
#define	WLC_ACTION_RM		5 /* Radio Measure request for MAC */

#define	TIMER_INTERVAL_WATCHDOG	1000	/* watchdog timer, in unit of ms */
#define	TIMER_INTERVAL_RADIOCHK	800	/* radio monitor timer, in unit of ms */
#define	BEACON_INTERVAL_DEFAULT	100	/* beacon interval, in unit of ms */
#define	DTIM_INTERVAL_DEFAULT	3	/* DTIM interval, in unit of beacon interval */
#define	SYNTHPU_DLY_APHY_US	3700	/* a phy synthpu_dly time in us */
#define	PRETBTT_APHY_US		120	/* a phy pretbtt time in us */
#define	SYNTHPU_DLY_BPHY_US	1050	/* b/g phy synthpu_dly time in us, default */
#define	PRETBTT_BPHY_US		250	/* b/g phy pretbtt time in us */

/*
 * driver maintains internal 'tick'(wlc->pub.now) which increments in 1s OS timer(soft
 * watchdog) it is not a wall clock and won't increment when driver is in 'down" state
 * this low resolution driver tick can be used for maintenance tasks such as phy
 * calibration and scb update
 */
#define	SW_TIMER_IBSS_GMODE_RATEPROBE	60	/* periodic IBSS gmode rate probing */
#define	SW_TIMER_MAC_STAT_UPD		30	/* periodic MAC stats update */

#define	LED_TIME			200	/* 200ms wlc_led_timer() period */


/* Find basic rate for a given rate : pren simply returns the same rspec */
#define WLC_BASIC_RATE(wlc, rspec) ((wlc)->band->basic_rate[rspec & RSPC_RATE_MASK])

/* Do we support this rate? */
#define VALID_RATE(wlc, rspec) wlc_valid_rate(wlc, rspec, WLC_BAND_AUTO, FALSE)
#define VALID_RATE_DBG(wlc, rspec) wlc_valid_rate(wlc, rspec, WLC_BAND_AUTO, TRUE)

#define PLCP_PHYRATE(plcp, is_ofdm) (is_ofdm ? (((ofdm_phy_hdr_t *)plcp)->rlpt[0] & 0xf) : \
	(((cck_phy_hdr_t *)plcp)->signal))

/* To inform the ucode of the last mcast frame posted so that it can clear moredata bit */
#define BCMCFID(wlc, fid) wlc_write_shm((wlc), M_BCMC_FID, (fid))

#define WLC_IS_CURRENT_BSSID(wlc, bssid) \
	(!bcmp((char*)(bssid), (char*)&((wlc)->BSSID), ETHER_ADDR_LEN))
#define WLC_IS_TARGET_BSSID(wlc, bssid) \
	(!bcmp((char*)(bssid), (char*)&((wlc)->target_bss.BSSID), ETHER_ADDR_LEN))
#define WLC_IS_CURRENT_SSID(wlc, ssid, len) \
	(((len) == (wlc)->pub.current_bss.SSID_len) && \
	 !bcmp((char*)(ssid), (char*)&((wlc)->pub.current_bss.SSID), (len)))
#define WLC_IS_TARGET_SSID(wlc, ssid, len) \
	(((len) == (wlc)->target_bss.SSID_len) && \
	 !bcmp((char*)(ssid), (char*)&((wlc)->target_bss.SSID), (len)))
#define WLC_BSS_CONNECTED(wlc) \
	((wlc)->pub.associated && (!(wlc)->pub.BSS || !ETHER_ISNULLADDR(&(wlc)->BSSID)))

#define IS_SINGLEBAND_5G(device) \
	((device == BCM4306_D11A_ID) || \
	 (device == BCM4311_D11A_ID) || \
	 (device == BCM4318_D11A_ID))

#define START_CORE_INDEX(device, ncores) \
	((unsigned)((IS_SINGLEBAND_5G(device)) && ((ncores) == 2) ? 1 : 0))

#define END_CORE_INDEX(device, ncores, nbands) \
	((unsigned)((((ncores) == 2) && (((nbands) == 2) || (IS_SINGLEBAND_5G(device)))) ? 2 : 1))

#define FOR_ACTIVE_D11_CORES(loopindex, device, ncores, nbands) \
	for (loopindex = START_CORE_INDEX(device, ncores); \
		loopindex < END_CORE_INDEX(device, ncores, nbands); \
			loopindex++)

#define DMAREG(wlc, direction, fifonum)	(D11REV_LT(wlc->pub.corerev, 11) ? \
	((direction == DMA_TX) ? (void*)&(wlc->regs->fifo.f32regs.dmaregs[fifonum].xmt) : \
				 (void*)&(wlc->regs->fifo.f32regs.dmaregs[fifonum].rcv)) : \
	((direction == DMA_TX) ? (void*)&(wlc->regs->fifo.f64regs[fifonum].dmaxmt) : \
				 (void*)&(wlc->regs->fifo.f64regs[fifonum].dmarcv)))

#define DATA_BLOCK_SCAN		(1 << 0)
#define DATA_BLOCK_QUIET	(1 << 1)
#define DATA_BLOCK_JOIN		(1 << 2)
#define DATA_BLOCK_PS		(1 << 3)

#define WL_DUMP_BUF_SIZE	(32 * 1024)

#define FRAMETYPE(r, frame)	((IS_CCK(r) ? FT_CCK : FT_OFDM))

/* join pref width in bits */
#define WLC_JOIN_PREF_BITS_RSSI		8 /* # of bits for TLV in Join Pref for RSSI */

/* join pref formats */
#define WLC_JOIN_PREF_OFF_COUNT		1 /* Offset for Count for Join Pref TLV */
#define WLC_JOIN_PREF_OFF_BAND		1 /* Offset for Band for Join Pref TLV */

#define WLC_JOIN_PREF_LEN_FIXED		2 /* Fixed length for Join PREF TLV */

/* handy macros */
#define WLCWLUNIT(wlc)		((wlc)->pub.unit)
#define WLCTYPEBMP(type)	(1 << (type))
#define WLCMAXCNT(type)		(1 << (type))
#define WLCMAXVAL(bits)		((1 << (bits)) - 1)
#define WLCBITMASK(bits)	((1 << (bits)) - 1)

#define WLC_BCMC_PSMODE(wlc) (TRUE)

#define WLC_WAR16165(wlc) (wlc->war16165)

#define RFDISABLE_DEFAULT	10000000 /* rfdisable delay timer 500 ms, runs of ALP clock  */

/* debug/trace */
uint wl_msg_level =
	0;

/* IOVar table */

/* Parameter IDs, for use only internally to wlc -- in the wlc_iovars
 * table and by the wlc_doiovar() function.  No ordering is imposed:
 * the table is keyed by name, and the function uses a switch.
 */
enum {
	IOV_RTSTHRESH = 1,
	IOV_5G_RATE,
	IOV_2G_RATE,
	IOV_FRAGTHRESH,
	IOV_WSEC,
	IOV_PWSEC,
	IOV_GWSEC,
	IOV_WSEC_KEY,
	IOV_WPA_AUTH,
	IOV_D11_AUTH,
	IOV_WPAIE,		/* set/get the wpaie to/from the driver */
	IOV_WSEC_RESTRICT,
	IOV_TKIP_CMS,		/* TKIP counter measures */
	IOV_URXEAPOL_RESTRICT,
	IOV_LAST 		/* In case of a need to check max ID number */
};

static uint8 WPA_info_element[] = {
	DOT11_MNG_WPA_ID, 0x18,
	0x00, 0x50, 0xf2, 0x01, 0x01, 0x00,
	0x00, 0x50, 0xf2, 0xff,
	0x01, 0x00, 0x00, 0x50, 0xf2, 0xff,
	0x01, 0x00, 0x00, 0x50, 0xf2, 0xff,
	0x00, 0x00
};

#ifdef BCMWPA2
static uint8 WPA2_info_element[] = {
	DOT11_MNG_RSN_ID, 0x14, 0x01, 0x00,
	0x00, 0x0F, 0xAC, 0xff,
	0x01, 0x00, 0x00, 0x0F, 0xAC, 0xff,
	0x01, 0x00, 0x00, 0x0F, 0xAC, 0xff,
	0x00, 0x00
};
#endif /* BCMWPA2 */


const wlc_iovar_t wlc_iovars[] = {
	{"rtsthresh", IOV_RTSTHRESH,
	(IOVF_WHL), IOVT_UINT16, 0
	},
	{"a_rate", IOV_5G_RATE,
	(0), IOVT_UINT32, 0
	},
	{"bg_rate", IOV_2G_RATE,
	(0), IOVT_UINT32, 0
	},
	{"fragthresh", IOV_FRAGTHRESH,
	(0), IOVT_UINT16, 0
	},
	{"wsec", IOV_WSEC,
	(0), IOVT_UINT32, 0
	},
	{"pwsec", IOV_PWSEC,
	(0), IOVT_UINT32, 0
	},
	{"gwsec", IOV_GWSEC,
	(0), IOVT_UINT32, 0
	},
	{"wsec_key", IOV_WSEC_KEY,
	(0), IOVT_BUFFER, sizeof(wl_wsec_key_t)
	},
	{"wpa_auth", IOV_WPA_AUTH,
	(0), IOVT_INT32, 0
	}, 
	{"auth", IOV_D11_AUTH,
	0, IOVT_INT32, 0
	},
	{"wpaie", IOV_WPAIE,
	0, IOVT_BUFFER, 16,
	},
	{"wsec_restrict", IOV_WSEC_RESTRICT,
	(0), IOVT_BOOL, 0
	},
	{"tkip_countermeasures", IOV_TKIP_CMS,
	(0), IOVT_BOOL, 0
	},
	{"rx_unecrypted_restrict", IOV_URXEAPOL_RESTRICT,
	(0), IOVT_BOOL, 0
	},
	{NULL, 0, 0, 0, 0 }
};


uint8 wme_prio2fifo[NUMPRIO] = {
	TX_AC_BE_FIFO,	/* 0	BE	AC_BE	Best Effort */
	TX_AC_BK_FIFO,	/* 1	BK	AC_BK	Background */
	TX_AC_BK_FIFO,	/* 2	--	AC_BK	Background */
	TX_AC_BE_FIFO,	/* 3	EE	AC_BE	Best Effort */
	TX_AC_VI_FIFO,	/* 4	CL	AC_VI	Video */
	TX_AC_VI_FIFO,	/* 5	VI	AC_VI	Video */
	TX_AC_VO_FIFO,	/* 6	VO	AC_VO	Voice */
	TX_AC_VO_FIFO	/* 7	NC	AC_VO	Voice */
};

/* precedences numbers for wlc queues. These are twice as may levels as
 * 802.1D priorities.
 * Odd numbers are used for HI priority traffic at same precedence levels
 * These constants are used ONLY by wlc_prio2prec_map.  Do not use them elsewhere.
 */
#define	_WLC_PREC_NONE		0	/* None = - */
#define	_WLC_PREC_BK		2	/* BK - Background */
#define	_WLC_PREC_BE		4	/* BE - Best-effort */
#define	_WLC_PREC_EE		6	/* EE - Excellent-effort */
#define	_WLC_PREC_CL		8	/* CL - Controlled Load */
#define	_WLC_PREC_VI		10	/* Vi - Video */
#define	_WLC_PREC_VO		12	/* Vo - Voice */
#define	_WLC_PREC_NC		14	/* NC - Network Control */

/* 802.1D Priority to precedence queue mapping */
const uint8 wlc_prio2prec_map[] = {
	_WLC_PREC_BE,		/* 0 BE - Best-effort */
	_WLC_PREC_BK,		/* 1 BK - Background */
	_WLC_PREC_NONE,		/* 2 None = - */
	_WLC_PREC_EE,		/* 3 EE - Excellent-effort */
	_WLC_PREC_CL,		/* 4 CL - Controlled Load */
	_WLC_PREC_VI,		/* 5 Vi - Video */
	_WLC_PREC_VO,		/* 6 Vo - Voice */
	_WLC_PREC_NC,		/* 7 NC - Network Control */
};

/* Define a bitmap of precedences comprised by each AC */
#define WLC_PREC_BMP_AC_BE	(NBITVAL(WLC_PRIO_TO_PREC(PRIO_8021D_BE)) |	\
				 NBITVAL(WLC_PRIO_TO_HI_PREC(PRIO_8021D_BE)) |	\
				 NBITVAL(WLC_PRIO_TO_PREC(PRIO_8021D_EE)) |	\
				 NBITVAL(WLC_PRIO_TO_HI_PREC(PRIO_8021D_EE)))
#define WLC_PREC_BMP_AC_BK	(NBITVAL(WLC_PRIO_TO_PREC(PRIO_8021D_BK)) |	\
				 NBITVAL(WLC_PRIO_TO_HI_PREC(PRIO_8021D_BK)) |	\
				 NBITVAL(WLC_PRIO_TO_PREC(PRIO_8021D_NONE)) |	\
				 NBITVAL(WLC_PRIO_TO_HI_PREC(PRIO_8021D_NONE)))
#define WLC_PREC_BMP_AC_VI	(NBITVAL(WLC_PRIO_TO_PREC(PRIO_8021D_CL)) |	\
				 NBITVAL(WLC_PRIO_TO_HI_PREC(PRIO_8021D_CL)) |	\
				 NBITVAL(WLC_PRIO_TO_PREC(PRIO_8021D_VI)) |	\
				 NBITVAL(WLC_PRIO_TO_HI_PREC(PRIO_8021D_VI)))
#define WLC_PREC_BMP_AC_VO	(NBITVAL(WLC_PRIO_TO_PREC(PRIO_8021D_VO)) |	\
				 NBITVAL(WLC_PRIO_TO_HI_PREC(PRIO_8021D_VO)) |	\
				 NBITVAL(WLC_PRIO_TO_PREC(PRIO_8021D_NC)) |	\
				 NBITVAL(WLC_PRIO_TO_HI_PREC(PRIO_8021D_NC)))

/* AC bitmap to precedence bitmap mapping (constructed in wlc_attach) */
uint wlc_acbitmap2precbitmap[16];

/* Sanity check for tx_prec_map and fifo synchup
 * Either there are some packets pending for the fifo, else if fifo is empty then
 * all the corresponding precmap bits should be set
 */
#define WLC_TX_FIFO_CHECK(wlc, fifo) ((wlc)->core->txpktpend[(fifo)] || \
	((wlc)->core->txpktpend[(fifo)] == 0 && \
	((wlc)->tx_prec_map & (wlc)->fifo2prec_map[(fifo)]) == \
	(wlc)->fifo2prec_map[(fifo)]) || \
	(PIO_ENAB((wlc)) && (wlc->tx_suspended || \
		 !wlc_pio_txavailable((wlc)->core->pio[fifo], 1, 1))))

/* TX FIFO number to WME/802.1E Access Category */
const uint8 wme_fifo2ac[] = { AC_BK, AC_BE, AC_VI, AC_VO, AC_BE, AC_BE };

/* Shared memory location index for various AC params */
static uint8 wme_shmemacindex[] = {
	1,	/* AC_BE index */
	0, 	/* AC_BK index */
	2, 	/* AC_VI index */
	3	/* AC_VO index */
};


/* conditions under which we want to keep the core awake */
#define STAY_AWAKE(wlc) (wlc->wake || SCAN_IN_PROGRESS(wlc) || \
	wlc->PMpending || wlc->assoc_state || wlc->check_for_unaligned_tbtt || \
	wlc->PSpoll || wlc->apsd_sta_usp || wlc->forcefastclk || \
	wlc->wakeforclkctl || wlc->pub.wakeforphyreg || wlc->wakefortxfifo || \
	wlc->wakeformacsuspend || wlc->txpend16165war || wlc->PMawakebcn)

#define WLC_CHAN_ID(x)		(((x) & RXS_CHAN_ID_MASK) >> RXS_CHAN_ID_SHIFT)
#define WLC_CHAN_PHYTYPE(x)	(((x) & RXS_CHAN_PHYTYPE_MASK) >> RXS_CHAN_PHYTYPE_SHIFT)

/* currently the best mechanism for determining SIFS is the band in use */
#define SIFS(band) ((band)->bandtype == WLC_BAND_5G ? APHY_SIFS_TIME : BPHY_SIFS_TIME);

/* value for # replay counters currently supported */
#define WLC_REPLAY_CNTRS_VALUE	WPA_CAP_16_REPLAY_CNTRS

/* priority to replay counter (Rx IV) entry index mapping. */
/*
* It is one-to-one mapping when there are 16 replay counters.
* Otherwise it is many-to-one mapping when there are only 4
* counters which are one-to-one mapped to 4 ACs.
*/
#if WLC_REPLAY_CNTRS_VALUE == WPA_CAP_16_REPLAY_CNTRS
#define PRIO2IVIDX(prio)	(prio)
#elif WLC_REPLAY_CNTRS_VALUE == WPA_CAP_4_REPLAY_CNTRS
#define PRIO2IVIDX(prio)	WME_PRIO2AC(prio)
#else
#error "Neither WPA_CAP_4_REPLAY_CNTRS nor WPA_CAP_16_REPLAY_CNTRS is used"
#endif /* WLC_REPLAY_CNTRS_VALUE == WPA_CAP_16_REPLAY_CNTRS */

/* local prototypes */
static void wlc_monitor(wlc_info_t *wlc, d11rxhdr_t *rxh, void *p);
static void wlc_write_inits(wlc_info_t *wlc, const d11init_t *inits);
static void wlc_wme_setparams(wlc_info_t *wlc, bool suspend);
static bool wlc_is_wme_ie(wlc_info_t *wlc, uint8 *ie, uint8 **tlvs, uint *tlvs_len);
static void wlc_write_ucode(wlc_info_t *wlc, const uint32 ucode[], const uint nbytes);
static void wlc_write_pcm(wlc_info_t *wlc, const uint32 pcm[], const uint nbytes);
static void wlc_set_txant(wlc_info_t *wlc, int txant);
static void wlc_fifoerrors(wlc_info_t *wlc);
static bool wlc_validboardtype(wlc_info_t *wlc);
static bool wlc_isgoodchip(wlc_info_t* wlc);

static void wlc_txq_enq(void *ctx, struct scb *scb, void *sdu, uint prec);

static void wlc_pkt_callback(wlc_info_t *wlc, void *pkt, uint16 tx_status);
static void wlc_tx_prec_map_init(wlc_info_t *wlc);
static void wlc_switch_shortslot(wlc_info_t *wlc, bool shortslot);
static void wlc_update_slot_timing(wlc_info_t *wlc, bool shortslot);
static void wlc_watchdog(void *arg);
static bool wlc_update_gbss_modes(wlc_info_t *wlc);
static int wlc_set_rateset(wlc_info_t *wlc, struct rateset *rs_arg);
static int wlc_iovar_rangecheck(wlc_info_t *wlc, uint32 val, const wlc_iovar_t *vi);
int wlc_doiovar(void *hdl, const wlc_iovar_t *vi, uint32 actionid, const char *name,
	void *params, uint p_len, void *arg, int len, int val_size, struct wlc_if *wlcif);
static const wlc_iovar_t* wlc_iovar_lookup(const wlc_iovar_t *table, const char *name);
static void wlc_ap_upd(wlc_info_t* wlc);

static char* wlc_dump_bsscfg(wlc_info_t *wlc, wlc_bsscfg_t *cfg, int bsscfg_idx, char *buf);

/* send and receive */
static bool wlc_txstatus(wlc_info_t *wlc, uint bound);
static void wlc_dotxstatus(wlc_info_t *wlc, tx_status_t *txs);
static bool wlc_tx_fifo_suspended(wlc_info_t *wlc, uint tx_fifo);
static void wlc_tx_suspend(wlc_info_t *wlc);
static bool wlc_tx_suspended(wlc_info_t *wlc);
static void wlc_tx_resume(wlc_info_t *wlc);
static void wlc_tx_fifo_suspend(wlc_info_t *wlc, uint tx_fifo);
static void wlc_tx_fifo_resume(wlc_info_t *wlc, uint tx_fifo);
static void wlc_flushqueues(wlc_info_t *wlc);
static void* wlc_allocfrag(osl_t *osh, void *sdu, uint offset, uint headroom, uint frag_length,
	uint tailroom);
static void wlc_dofrag(wlc_info_t *wlc, void *p, uint frag, uint nfrags, uint next_payload_len,
	osl_t *osh, wlc_bsscfg_t *cfg, struct scb *scb, bool is8021x,
	uint fifo, wsec_key_t *key, uint8 prio);

static bool sstlookup(wlc_info_t *wlc, uint16 proto);

static uint16 wlc_compute_airtime(wlc_info_t *wlc, bool is_ofdm, uint phy_rate, uint length);
static void wlc_compute_cck_plcp(ratespec_t rate, uint length, uint8 *plcp);
static void wlc_compute_ofdm_plcp(ratespec_t rate, uint length, uint8 *plcp);
static uint16 wlc_d11hdrs(wlc_info_t *wlc, void *p, struct scb *scb, int short_preamble,
	uint frag, uint nfrags, uint queue, uint next_frag_len, wsec_key_t *key,
	ratespec_t rate_override);
static uint16 wlc_compute_rtscts_dur(wlc_info_t *wlc, bool cts_only, ratespec_t  rts_rate,
	ratespec_t frame_rate, int short_preamble, uint frame_len);
static uint16 wlc_compute_frame_dur(wlc_info_t *wlc, ratespec_t rate, int short_preamble,
	uint next_frag_len);

static bool wlc_recv(wlc_info_t *wlc, uint fifo, uint bound);

static uint16 wlc_recvfilter(wlc_info_t *wlc, struct dot11_header *h, d11rxhdr_t *rxh,
	struct scb **pscb);
static bool wlc_mcastfilter(wlc_info_t *wlc, struct dot11_header *h);
static void wlc_recvctl(wlc_info_t *wlc, osl_t *osh, d11rxhdr_t *rxh, void *p);
static void wlc_process_beacon(wlc_info_t *wlc, d11rxhdr_t *rxh, uint8 *plcp,
	struct dot11_management_header *hdr, int frame_len);

static void wlc_frameaction_specmgmt(uint action_id, wlc_info_t *wlc,
	struct dot11_management_header *hdr, d11rxhdr_t *rxh,
	uint8 *plcp, uint8 *body, int body_len);
static void wlc_recv_tpc_request(wlc_info_t *wlc, d11rxhdr_t *rxh, void *plcp,
	struct dot11_management_header *hdr, uint8 *body, int body_len);
static void wlc_recv_measure_request(wlc_info_t *wlc, struct dot11_management_header *hdr,
	uint8 *body, int body_len);
static void wlc_recv_measure_report(wlc_info_t *wlc, struct dot11_management_header *hdr,
	uint8 *body, int body_len);
static void wlc_recv_csa_action(wlc_info_t *wlc, struct dot11_management_header *hdr, uint8 *body,
	int body_len);

static void wlc_send_tpc_report(wlc_info_t *wlc, struct ether_addr *da, struct ether_addr *bssid,
	uint8 token, int8 txpwr, int8 link_margin);
static void wlc_send_measure_report(wlc_info_t *wlc, struct ether_addr *da,
	struct ether_addr *bssid, uint8 token, uint8 *report, uint report_len);

static void wlc_frameaction_wme(uint action_id, wlc_info_t *wlc,
	struct dot11_management_header *hdr, uint8 *body, int body_len);


static void wlc_appendfrag(wlc_info_t *wlc, void *fragbuf, uint *fragresid, uchar *body,
	uint body_len, void* osh);
static void wlc_set_mac(wlc_info_t *wlc, struct ether_addr *addr);
static void wlc_set_bssid(wlc_info_t *wlc);
static void wlc_clear_bssid(wlc_info_t *wlc);

static uint wlc_calc_frame_len(wlc_info_t *wlc, ratespec_t  rate, int short_preamble, uint dur);
static uint wlc_calc_ack_time(wlc_info_t *wlc, ratespec_t  rate, int short_preamble);
static uint wlc_calc_cts_time(wlc_info_t *wlc, ratespec_t  rate, int short_preamble);

static bool wlc_valid_rate(wlc_info_t *wlc, ratespec_t rate, int band, bool verbose);
static void wlc_mod_prb_rsp_rate_table(wlc_info_t *wlc, uint frame_len);
static uint16 wlc_compute_bcntsfoff(wlc_info_t *wlc, bool is_ofdm, uint phy_rate,
	uint short_preamble, bool phydelay);
static bool wlc_nonerp_find(wlc_info_t *wlc, void *body, uint body_len, uint8 **erp, int *len);
static bool wlc_erp_find(wlc_info_t *wlc, void *body, uint body_len, uint8 **erp, int *len);
static void wlc_bcn_prb_template(wlc_info_t *wlc, uint type, ratespec_t bcn_rate, wlc_bsscfg_t *cfg,
	uint16 *buf, int *len);
static uint8* wlc_copy_info_elt(uint8 *cp, const void* data);
static void wlc_bcn_prb_body(wlc_info_t *wlc, uint type, wlc_bsscfg_t *cfg, uint8 *bcn, int *len);
static void wlc_process_brcm_ie(wlc_info_t *wlc, struct scb *scb, brcm_ie_t *brcm_ie);
static void *wlc_sendauth(wlc_info_t *wlc, struct ether_addr *ea, struct ether_addr *bssid,
                          struct scb *scb,
                          int auth_alg, int auth_seq, int auth_status, wsec_key_t *scb_key,
                          uint8 *challenge_text, int short_preamble);

static void *wlc_senddisassoc(wlc_info_t *wlc, const struct ether_addr *da,
                              const struct ether_addr *bssid,
                              struct scb *scb, uint16 reason_code);
static uint32 wlc_mhfdef(wlc_info_t *wlc);
static void wlc_write_mhf(wlc_info_t *wlc, uint32 val);

static bool wlc_mac_request_entry(wlc_info_t* wlc, int req);
static void wlc_reprate_init(wlc_info_t *wlc);
static uint wlc_get_reprate(wlc_info_t *wlc);
static int wlc_parse_rates(wlc_info_t *wlc, uchar *tlvs, uint tlvs_len, wlc_bss_info_t *bi,
	struct rateset *rates);
static void wlc_update_probe_resp(wlc_info_t *wlc, wlc_bsscfg_t *cfg, bool suspend);
static void wlc_update_all_probe_resp(wlc_info_t *wlc, bool suspend);

extern uint16 wlc_read_ihr(wlc_info_t *wlc, uint offset);
extern void wlc_write_ihr(wlc_info_t *wlc, uint offset, uint16 v);

/* channel */
static bool wlc_radar_channel(wlc_info_t *wlc, uint channel);



/* interrupt, up/down, band */
static uint32 wlc_setband_inact(wlc_info_t *wlc, uint bandunit);
static void wlc_bsinit(wlc_info_t *wlc);
static uint32 wlc_wlintrsoff(wlc_info_t *wlc);
static void wlc_wlintrsrestore(wlc_info_t *wlc, uint32 macintmask);

static bool wlc_radio_read_hwdisabled(wlc_info_t* wlc);
static bool wlc_radio_monitor_start(wlc_info_t *wlc);
static bool wlc_radio_monitor_stop(wlc_info_t *wlc);
static void wlc_radio_timer(void *arg);
static void wlc_radio_disable(wlc_info_t *wlc);
static void wlc_radio_enable(wlc_info_t *wlc);
static void wlc_radio_upd(wlc_info_t* wlc);
static void wlc_down_led_upd(wlc_info_t *wlc);

static void wlc_led_gcclear(wlc_info_t *wlc);


static void wlc_timer_led(void *arg);
static void wlc_timer_led_blink(void *arg);

static void wlc_coreinit(wlc_info_t *wlc);
static void wlc_gpio_init(wlc_info_t *wlc);

/* scan, association, BSS */
static void wlc_scan(wlc_info_t *wlc, int bss_type, const struct ether_addr* bssid,
	const uchar ssid[], int ssid_len, int scan_type,
	int nprobes, int active_time, int passive_time, int home_time,
	const uint16* channel_list, int channel_num,
	bool save_prb, cb_fn_t fn, void* arg);
static void wlc_scan_abort(wlc_info_t *wlc);
static void wlc_scantimer(void *arg);
static void wlc_scan_radar_clear(wlc_info_t *wlc);
static void wlc_scan_channels(wlc_info_t *wlc, uint8 *channel_list, int *pchannel_num,
	int channel_max, uint channel_start, int channel_type);
static void wlc_scan_parse(wlc_info_t *wlc, d11rxhdr_t *rxh, uint8 *plcp,
	struct dot11_management_header *hdr, uint8 *body, int body_len);

static wlc_bss_info_t *wlc_BSSlookup(wlc_info_t *wlc, uchar *bssid, uint channel, uchar ssid[],
	uint ssid_len);
static int wlc_parse_bcn_prb(wlc_info_t *wlc, d11rxhdr_t *rxh,
	struct dot11_management_header *hdr, void *body, uint body_len, wlc_bss_info_t *bi);


static int wlc_bss2wl_bss(wlc_bss_info_t *bi, wl_bss_info_t *to_bi, int to_bi_len);
static void wlc_BSSinit(wlc_info_t *wlc, wlc_bss_info_t *bi, wlc_bsscfg_t *cfg, int start);

static int wlc_wpa_cap(wlc_info_t *wlc, wlc_bsscfg_t *cfg, uint8 *cap, int len);

static int wlc_set_rate_override(wlc_info_t *wlc, int band, int32 phy_rate, bool mcast);
static void wlc_11h_do_chanswitch(wlc_info_t *wlc, uint8 newchan);
static void wlc_war16165(wlc_info_t *wlc, bool tx);




/* ** STA-only routines ** */
/* STA association/roam scan */
static void wlc_setssid_disassoc_complete(wlc_info_t *wlc, uint txstatus, void *ignore);
static void wlc_roam_start(wlc_info_t *wlc, uint reason);
static void wlc_assoc_scan_start(wlc_info_t *wlc, struct ether_addr *bssid);
static void wlc_assoc_scan_complete(void *arg);

/* STA association/roam join */
static void wlc_cook_join_targets(wlc_info_t *wlc, bool roam, int cur_rssi);
static void wlc_join_start_ibss(wlc_info_t *wlc, const uchar ssid[], int ssid_len);
static void wlc_join_attempt(wlc_info_t *wlc);
static bool wlc_join_basicrate_supported(wlc_info_t *wlc, rateset_t *rs, int band);
static void wlc_join_BSS(wlc_info_t *wlc, wlc_bss_info_t* bi);
static void wlc_join_adopt_bss(wlc_info_t *wlc);
static void wlc_tsf_adopt_bcn(wlc_info_t *wlc, struct dot11_bcn_prb *bcn);
static void wlc_join_complete(wlc_info_t *wlc, struct dot11_bcn_prb *bcn, int bcn_len);
static void wlc_merge_bcn_prb(wlc_info_t *wlc,
	struct dot11_bcn_prb *p1, int p1_len,
	struct dot11_bcn_prb *p2, int p2_len,
	struct dot11_bcn_prb **merged, int *merged_len);
static int wlc_merged_ie_len(wlc_info_t *wlc, uint8 *tlvs1, int tlvs1_len, uint8 *tlvs2,
	int tlvs2_len);
static bool wlc_find_ie_match(bcm_tlv_t *ie, bcm_tlv_t *ies, int len);
static void wlc_merge_ies(uint8 *tlvs1, int tlvs1_len, uint8 *tlvs2, int tlvs2_len, uint8* merge);
static void wlc_assoc_timeout(void *arg);

static void wlc_ibss_suspend(wlc_info_t *wlc);
static void wlc_ibss_resume(wlc_info_t *wlc);
static void *wlc_sendassocreq(wlc_info_t *wlc, wlc_bss_info_t *bi, struct scb *scb, bool reassoc);
static void wlc_authresp_client(wlc_info_t *wlc, d11rxhdr_t *rxh,
	struct dot11_management_header *hdr,
	void *body, uint len);

/* power save and tbtt sync */
static void wlc_tbtt_adopt(wlc_info_t *wlc, struct dot11_bcn_prb *bcn);
static void wlc_tbtt_align(wlc_info_t *wlc, struct dot11_bcn_prb *bcn, bool adopt, bool is_ofdm,
	uint phy_rate, uint short_preamble);
static int wlc_bcn_tsf_later(wlc_info_t* wlc, d11rxhdr_t *rxh, void* bcn_plcp);
static void wlc_ps_null_complete(wlc_info_t *wlc, uint txstatus, void *arg);
static void wlc_auth_complete(wlc_info_t *wlc, uint txstatus, void *arg);
static void wlc_assocreq_complete(wlc_info_t *wlc, uint txstatus, void *arg);
static bool wlc_sendpspoll(wlc_info_t *wlc);



static void wlc_rateprobe_complete(wlc_info_t *wlc, uint txstatus, void *ea_arg);
static void wlc_rateprobe_scan(wlc_info_t *wlc);
static void wlc_tkip_countermeasures(wlc_info_t *wlc, uint txstatus, void *dummy);
static void wlc_11h_channel_switch(wlc_info_t *wlc, dot11_channel_switch_t *tag);
static void wlc_11h_quiet(wlc_info_t *wlc, dot11_quiet_t *tag, struct dot11_bcn_prb *bcn);
static void wlc_start_quiet0(wlc_info_t *wlc);
static void wlc_start_quiet1(wlc_info_t *wlc);
static void wlc_start_quiet2(wlc_info_t *wlc);
static void wlc_start_quiet3(wlc_info_t *wlc);
static void wlc_quiet_timer(void *arg);

static void wlc_freqtrack(wlc_info_t *wlc);
static void wlc_freqtrack_reset(wlc_info_t *wlc);
static void wlc_freqtrack_verify(wlc_info_t *wlc);


static void wlc_parse_11h(wlc_info_t *wlc, uint8 *params, int len, struct dot11_bcn_prb *bcn);
static void *wlc_frame_get_ps_ctl(wlc_info_t *wlc, const struct ether_addr *bssid);
static void wlc_wme_setup_resp(wlc_info_t *wlc, struct dot11_management_header *hdr,
	uint8 *body, int body_len);





/* the unit for HW fifo size is 256byte(one block), total size is HW dependent 
 * ucode has default fifo partition, sw can overwrite if necessary
 */
static uint8 hwfifo_blk4[] = {
	15,	15,	15,	15,	15,	2
	/* 3584, 3584, 3584, 3584, 3584, 512 */
};

static uint8 hwfifo_blk5_8[] = {
	9,	13,	10,	8,	13,	1
	/* 2304, 3328, 2560, 2048, 3328, 256 */
};

static uint8 hwfifo_blk9_11[] = {
	10,	14,	11,	9,	14,	2
	/* 2560, 3584, 2816, 2304, 3584, 512 */
};

static void wlc_d11fifo_alloc(wlc_info_t *wlc);


/* Some helper functions for wlc_phy.c */
bool
wlc_scaninprog(wlc_info_t *wlc) {
	return SCAN_IN_PROGRESS(wlc);
}

bool
wlc_rminprog(wlc_info_t *wlc) {
	return FALSE;
}

bool
wlc_ofdm_restrict(wlc_info_t *wlc) {
	return (wlc_check_locale_flags(wlc->band->bandtype, WLC_NO_OFDM));
}

int
wlc_cur_bandtype(wlc_info_t *wlc) {
	return (wlc->band->bandtype);
}

void *
wlc_cur_phy(wlc_info_t *wlc) {
	return ((void *)wlc->band->pi);
}

static INLINE uint
wlc_frag(wlc_info_t *wlc, struct scb *scb, uint fifo, uint plen, uint *flen)
{
	uint payload, thresh, nfrags;

	plen -= ETHER_HDR_LEN;

	ASSERT(fifo < NFIFO);

	thresh = wlc->fragthresh[fifo];

	/* optimize for non-fragmented case */
	if (plen < (thresh - (DOT11_A4_HDR_LEN + DOT11_QOS_LEN +
		DOT11_FCS_LEN + ETHER_ADDR_LEN))) {
		*flen = plen;
		return (1);
	}

	/* account for 802.11 MAC header */
	thresh -= DOT11_A3_HDR_LEN + DOT11_FCS_LEN;

	/* WME: account for QoS Control Field */
	if (SCB_WME(scb))
		thresh -= DOT11_QOS_LEN;

	/*
	 * Spec says broadcast and multicast frames are not fragmented.
	 * LLC/SNAP considered part of packet payload.
	 * Fragment length must be even per 9.4 .
	 */
	if ((plen > thresh) && !SCB_ISMULTI(scb)) {
		*flen = payload = thresh & ~1;
		nfrags = (plen + payload - 1) / payload;
	} else {
		*flen = plen;
		nfrags = 1;
	}

	ASSERT(nfrags <= DOT11_MAXNUMFRAGS);

	return (nfrags);
}

void
wlc_clkctl_clk(wlc_info_t *wlc, uint mode)
{

	/* wake the mac */
	if (wlc->pub.up) {
		wlc->wakeforclkctl = TRUE;
		wlc_set_ps_ctrl(wlc);
	}

	/* set the clk mode and check if that mode forces the fast clk */
	if ((wlc->forcefastclk = sb_clkctl_clk(wlc->pub.sbh, mode)))
		wlc_mhf(wlc, MHF_FORCEFASTCLK, MHF_FORCEFASTCLK, TRUE);
	else
		wlc_mhf(wlc, MHF_FORCEFASTCLK, 0, TRUE);

	/* ok to clear the wakeup now */
	if (wlc->pub.up) {
		wlc->wakeforclkctl = FALSE;
		wlc_set_ps_ctrl(wlc);
	}
}

void
BCMINITFN(wlc_reset)(wlc_info_t *wlc)
{
	int i;

	WL_TRACE(("wl%d: wlc_reset\n", wlc->pub.unit));

	WLCNTINCR(wlc->pub._cnt.reset);

	/* slurp up hw mac counters before core reset */
	if ((WLC_UPDATE_STATS(wlc)) && (sb_iscoreup(wlc->pub.sbh))) {
		wlc_statsupd(wlc);
	}

	/* clear any pending pkt callbacks */
	bzero((char*)&wlc->pkt_callback, sizeof(wlc->pkt_callback));
	WLCNTSET(wlc->pub._cnt.pkt_callback_reg_fail, 0);

	wlc->check_for_unaligned_tbtt = FALSE;

	/* reset the core */
	if (!DEVICEREMOVED(wlc))
		wlc_corereset(wlc);

	/* purge the pio queues or dma rings */
	wlc_flushqueues(wlc);

	for (i = 0; i < NFIFO; i++)
		wlc->core->txpktpend[i] = 0;
	wlc->txretried = 0;

	wlc->core->tsf_timerlow = 0;

	/* reset our delta macstat counters */
	bzero((char*)&wlc->core->macstat_delta, sizeof(macstat32_t));
}

void
BCMINITFN(wlc_init)(wlc_info_t *wlc)
{
	d11regs_t *regs;
	uint32 macintmask;

	WL_TRACE(("wl%d: wlc_init\n", wlc->pub.unit));

	regs = wlc->regs;

	/* disable interrupts */
	macintmask = wl_intrsoff(wlc->wl);

	/* do one-time phy inits and calibration */
	wlc_phy_cal_init(wlc->band->pi);

	/* core-specific initialization */
	wlc_coreinit(wlc);

	/* band-specific inits */
	wlc_bsinit(wlc);
	wlc->band->init = TRUE;

	/* Enable EDCF mode (while the MAC is suspended) */
	if (D11REV_GE(wlc->pub.corerev, 4) && WME_ENAB(wlc)) {
		wlc_mhf(wlc, MHF_EDCF, MHF_EDCF, TRUE);
		OR_REG(&wlc->regs->ifs_ctl, IFS_USEEDCF);
		wlc_wme_setparams(wlc, FALSE);
	}

	/* Init precedence maps for empty FIFOs */
	wlc_tx_prec_map_init(wlc);

	/* ..now really unleash hell (allow the MAC out of suspend) */
	wlc_enable_mac(wlc);

	/* restore macintmask */
	wl_intrsrestore(wlc->wl, macintmask);

	/* clear tx flow control */
	if (wlc->pub.txqstopped) {
		wlc->pub.txqstopped = OFF;
		wl_txflowcontrol(wlc->wl, OFF);
	}

	/* clear tx data fifo suspends */
	wlc->tx_suspended = FALSE;

	/* enable the RF Disable Delay timer */
	if (wlc->pub.corerev >= 10) {
		W_REG(&wlc->regs->rfdisabledly, RFDISABLE_DEFAULT);
	}



	/* Reinitialize the precedence map to syncup with FIFOs */
	wlc_tx_prec_map_init(wlc);
}

/* return initial host flags value */
static uint32
BCMINITFN(wlc_mhfdef)(wlc_info_t *wlc)
{
	uint32 mhf;

	mhf = 0;

	if (ISGPHY(wlc->band->pi))
		mhf |= MHF_SYMWAR;

	if (ISGPHY(wlc->band->pi) && GREV_IS(wlc->band->pi->phy_rev, 1))
		mhf |= MHF_DCFILTWAR;

	if (ISGPHY(wlc->band->pi) && (wlc->pub.boardflags & BFL_PACTRL) && !wlc->dbgsel)
		mhf |= MHF_OFDMPWR;

	/* enable CCK power boost in ucode but not for 4318/20 */
	if (ISGPHY(wlc->band->pi) && (wlc->band->pi->phy_rev < 3)) {
		if (mhf & MHF_OFDMPWR)
			WL_ERROR(("wl%d: Cannot support simultaneous MHF_OFDMPWR & MHF_CCKPWR\n",
				wlc->pub.unit));
		else
			mhf |= MHF_CCKPWR;
	}

	/* prohibit use of slowclock on multifunction boards */
	if (wlc->pub.boardflags & BFL_NOPLLDOWN)
		mhf |= MHF_FORCEFASTCLK;

	if ((wlc->band->pi->radioid == BCM2050_ID) && (wlc->band->pi->radiorev < 6))
		mhf |= MHF_SYNTHPUWAR;

	if (WLC_WAR16165(wlc))
		mhf |= MHF_PCISLOWCLKWAR;

	return (mhf);
}

/* set or clear ucode host flag bits and return new value
 * it has an optimization for no-change write
 * it only writes through shared memory when the core has clock;
 * pre-CLK changes should use wlc_write_mhf to get around the optimization
 *
 * it change both bands' shadow variables if allbands is TRUE
 * return value is valid only if wlc->band has been initialized
 */
uint32
wlc_mhf(wlc_info_t *wlc, uint32 mask, uint32 val, bool allbands)
{
	uint32 save;

	ASSERT((val & ~mask) == 0);

	if (wlc->band) {
		save = wlc->band->mhf;
		wlc->band->mhf = (wlc->band->mhf & ~mask) | val;

		/* optimization: only write through if changed */
		if (wlc->core->clk && (wlc->band->mhf != save)) {
			wlc_write_shm(wlc, M_HOST_FLAGS, (uint16)wlc->band->mhf);
			wlc_write_shm(wlc, M_HOST_FLAGS2, (uint16)(wlc->band->mhf >>
				M_HOST_FLAGS_SZ));
		}
	}

	if (allbands) {
		wlc->bandstate[0].mhf = (wlc->bandstate[0].mhf & ~mask) | val;
		wlc->bandstate[1].mhf = (wlc->bandstate[1].mhf & ~mask) | val;
	}

	return (wlc->band ? wlc->band->mhf : 0);
}

static void
wlc_write_mhf(wlc_info_t *wlc, uint32 val)
{
	wlc_write_shm(wlc, M_HOST_FLAGS, (uint16)val);
	wlc_write_shm(wlc, M_HOST_FLAGS2, (uint16)(val >> M_HOST_FLAGS_SZ));
}

/* set or clear maccontrol bits and return new value */
void wlc_mctrl(wlc_info_t *wlc, uint32 mask, uint32 val)
{
	ASSERT((val & ~mask) == 0);

	wlc->maccontrol = (R_REG(&wlc->regs->maccontrol) & ~mask) | val;
	W_REG(&wlc->regs->maccontrol, wlc->maccontrol);
}

void
wlc_mac_bcn_promisc(wlc_info_t *wlc)
{
	/* default behavior: on for Gmode in BSS/IBSS) || (A/B mode in IBSS)
	 *		     off for other cases
	 */
	if (wlc->band->gmode || wlc->bcnmisc_ibss || wlc->bcnmisc_scan ||
	    wlc->bcnmisc_monitor)
		wlc_mctrl(wlc, MCTL_BCNS_PROMISC, MCTL_BCNS_PROMISC);
	else
		wlc_mctrl(wlc, MCTL_BCNS_PROMISC, 0);
}

/* set or clear maccontrol bits MCTL_PROMISC and MCTL_KEEPCONTROL */
void
wlc_mac_promisc(wlc_info_t *wlc)
{
	uint32 promisc_bits = 0;

	/* promiscuous mode just sets MCTL_PROMISC
	 * Note: APs get all BSS traffic without the need to set the MCTL_PROMISC bit
	 * since all BSS data traffic is directed at the AP
	 */
	if (wlc->pub.promisc)
		promisc_bits |= MCTL_PROMISC;

	/* monitor mode needs both MCTL_PROMISC and MCTL_KEEPCONTROL
	 * Note: monitor mode also needs MCTL_BCNS_PROMISC, but that is
	 * handled in wlc_mac_bcn_promisc()
	 */
	if (wlc->monitor)
		promisc_bits |= MCTL_PROMISC | MCTL_KEEPCONTROL;

	wlc_mctrl(wlc, MCTL_PROMISC | MCTL_KEEPCONTROL, promisc_bits);
}

/* push sw hps and wake state through hardware */
void
wlc_set_ps_ctrl(wlc_info_t *wlc)
{
	uint32 v1, v2;
	bool hps, wake;
	bool awake_before;

	hps = PS_ALLOWED(wlc);
	wake = STAY_AWAKE(wlc);

	if (!hps)
		wake = TRUE;

	WL_TRACE(("wl%d: wlc_set_ps_ctrl: hps %d wake %d\n", wlc->pub.unit, hps, wake));

	v1 = v2 = R_REG(&wlc->regs->maccontrol);

	if (hps)
		v2 |= MCTL_HPS;
	else
		v2 &= ~MCTL_HPS;
	if (wake)
		v2 |= MCTL_WAKE;
	else
		v2 &= ~MCTL_WAKE;

	/* just return if no change */
	if ((v1 & (MCTL_WAKE | MCTL_HPS)) == (v2 & ((MCTL_WAKE | MCTL_HPS))))
		return;

	if ((v1 & MCTL_HPS) && !hps)
		WL_PS(("wl%d: PS mode disable \n", wlc->pub.unit));
	if (!(v1 & MCTL_HPS) && hps)
		WL_PS(("wl%d: PS mode enable \n", wlc->pub.unit));

	if (hps) {
		if ((v1 & MCTL_WAKE) && !wake)
			WL_PS(("wl%d: PS-MODE: sleep\n", wlc->pub.unit));
		if (!(v1 & MCTL_WAKE) && wake)
			WL_PS(("wl%d: PS-MODE: wake\n", wlc->pub.unit));
	}

	wlc_mctrl(wlc, ~0, v2);

	awake_before = ((v1 & MCTL_WAKE) || ((v1 & MCTL_HPS) == 0));

	/* readback */
	if (wake && !awake_before) {
		if (D11REV_LT(wlc->pub.corerev, 5)) /* no slowclock */
			OSL_DELAY(5);
		else {
			if ((ISGPHY(wlc->band->pi)) && (D11REV_IS(wlc->pub.corerev, 5)))
				OSL_DELAY(2000);	/* delay before first read */
			else
				OSL_DELAY(40);	/* delay before first read */

			SPINWAIT((wlc_read_shm(wlc, M_UCODE_DBGST) == DBGST_ASLEEP), (8 *
				MIN_SLOW_CLK));
		}
		ASSERT(wlc_read_shm(wlc, M_UCODE_DBGST) != DBGST_ASLEEP);
	}
}

/*
 * Write STA MAC address to core.
 * Updates RXE match engine and tx template addr fields.
 */
static void
wlc_set_mac(wlc_info_t *wlc, struct ether_addr *addr)
{
	d11regs_t *regs;
	char template_buf[(T_RAM_ACCESS_SZ*2)];

	WL_TRACE(("wl%d: wlc_set_mac\n", wlc->pub.unit));

	regs = wlc->regs;

	/* enter the MAC addr into the RXE match registers */
	wlc_set_addrmatch(wlc, RCM_MAC_OFFSET, addr);

	/* fill out a buffer with a multiple of 4 length, starting at
	 * a 4 byte boundary offset for wlc_write_template_ram, 6 bytes
	 * of MAC addr and the first 2 bytes of our beaconing BSSID.
	 */
	bcopy((char*)wlc->pub.cur_etheraddr.octet, template_buf, ETHER_ADDR_LEN);
	bcopy((char*)wlc->announce_BSSID.octet, template_buf + ETHER_ADDR_LEN, 2);


	/* The A2 (SA) MAC addr appears at bytes [16, 21] in the 802.11 frames
	 * in the templates.
	 * 6 bytes PLCP
	 * 2 bytes FC
	 * 2 bytes DUR/ID
	 * 6 bytes A1
	 * 6 bytes A2 (SA)
	 * 6 bytes A3 (BSSID)
	 */

	/* update the BEACON templates */
	wlc_write_template_ram(wlc, (T_BCN0_TPL_BASE + 16), (T_RAM_ACCESS_SZ*2), template_buf);
	wlc_write_template_ram(wlc, (T_BCN1_TPL_BASE + 16), (T_RAM_ACCESS_SZ*2), template_buf);

}

static void
wlc_clear_bssid(wlc_info_t *wlc)
{
	W_REG(&wlc->regs->rcm_ctl, RCM_BSSID_OFFSET | RCM_INC_DATA);
	W_REG(&wlc->regs->rcm_mat_data, 0);
	W_REG(&wlc->regs->rcm_mat_data, 0);
	W_REG(&wlc->regs->rcm_mat_data, 0);
}

/* Write BSSID address to core (set_bssid in d11procs.tcl).
 * Updates RXE match engine and tx template addr fields.
 */
static void
wlc_set_bssid(wlc_info_t *wlc)
{
	d11regs_t *regs;
	char template_buf[(T_RAM_ACCESS_SZ*2)];

	WL_TRACE(("wl%d: wlc_set_bssid\n", wlc->pub.unit));

	regs = wlc->regs;

	/* enter the BSSID addr into the RXE match registers */
	wlc_set_addrmatch(wlc, RCM_BSSID_OFFSET, &wlc->BSSID);

	/* The A3 (BSSID) MAC addr appears at bytes [22, 27] in the 802.11 frames
	 * in the templates.
	 * 6 bytes PLCP
	 * 2 bytes FC
	 * 2 bytes DUR/ID
	 * 6 bytes A1
	 * 6 bytes A2 (SA)
	 * 6 bytes A3 (BSSID)
	 */

	/* fill out a buffer with a multiple of 4 length, starting at
	 * a 4 byte boundary offset for wlc_write_template_ram, last 2
	 * bytes of MAC addr and the all 6 bytes our beaconing BSSID
	 */
	bcopy((char*)wlc->pub.cur_etheraddr.octet + 4, template_buf, 2);
	bcopy((char*)wlc->announce_BSSID.octet, template_buf + 2, ETHER_ADDR_LEN);

	/* update the BEACON templates */
	wlc_write_template_ram(wlc, (T_BCN0_TPL_BASE + 20), (T_RAM_ACCESS_SZ*2), template_buf);
	wlc_write_template_ram(wlc, (T_BCN1_TPL_BASE + 20), (T_RAM_ACCESS_SZ*2), template_buf);

}

/*
 * Write a MAC address to the rcmta structure
 */
void
wlc_set_rcmta(wlc_info_t *wlc, int idx, const struct ether_addr *addr)
{
	d11regs_t *regs = wlc->regs;
	volatile uint16* objdata16 = (uint16*)&regs->objdata;
	uint32 mac_hm;
	uint16 mac_l;

	WL_TRACE(("wl%d: %s\n", wlc->pub.unit, __FUNCTION__));

	ASSERT(wlc->pub.corerev > 4);
	ASSERT((idx >= 0) && (idx < (WSEC_MAX_RCMTA_KEYS - WSEC_MAX_DEFAULT_KEYS)));

	mac_hm = (addr->octet[3] << 24) | (addr->octet[2] << 16) | (addr->octet[1] << 8) |
		addr->octet[0];
	mac_l = (addr->octet[5] << 8) | addr->octet[4];

	W_REG(&regs->objaddr, (OBJADDR_RCMTA_SEL | (idx * 2)));
	W_REG(&regs->objdata, mac_hm);
	W_REG(&regs->objaddr, (OBJADDR_RCMTA_SEL | ((idx * 2) + 1)));
	W_REG(objdata16, mac_l);
}

/*
 * Write a MAC address to the given match reg offset in the RXE match engine.
 */
void
wlc_set_addrmatch(wlc_info_t *wlc, int match_reg_offset, const struct ether_addr *addr)
{
	d11regs_t *regs;
	uint16 mac_l;
	uint16 mac_m;
	uint16 mac_h;

	WL_TRACE(("wl%d: wlc_set_addrmatch\n", wlc->pub.unit));

	ASSERT((match_reg_offset < RCM_WEP_TA0_OFFSET) || (wlc->pub.corerev < 5));

	regs = wlc->regs;
	mac_l = addr->octet[0] | (addr->octet[1] << 8);
	mac_m = addr->octet[2] | (addr->octet[3] << 8);
	mac_h = addr->octet[4] | (addr->octet[5] << 8);


	/* enter the MAC addr into the RXE match registers */
	W_REG(&regs->rcm_ctl, RCM_INC_DATA | match_reg_offset);
	W_REG(&regs->rcm_mat_data, mac_l);
	W_REG(&regs->rcm_mat_data, mac_m);
	W_REG(&regs->rcm_mat_data, mac_h);

}

void
wlc_write_template_ram(wlc_info_t *wlc, int offset, int len, void *buf)
{
	d11regs_t *regs;
	uint32 word;
	bool be_bit;

	WL_TRACE(("wl%d: wlc_write_template_ram\n", wlc->pub.unit));

	regs = wlc->regs;

	ASSERT(ISALIGNED(offset, sizeof(uint32)));
	ASSERT(ISALIGNED(len, sizeof(uint32)));
	ASSERT((offset & ~0xffff) == 0);

	W_REG(&regs->tplatewrptr, offset);


	/* if MCTL_BIGEND bit set in mac control register,
	 * the chip swaps data in fifo, as well as data in
	 * template ram
	 */
	be_bit = (R_REG(&regs->maccontrol) & MCTL_BIGEND) != 0;

	while (len > 0) {
		bcopy((uint8*)buf, &word, sizeof(uint32));

		if (be_bit)
			word = hton32(word);
		else
			word = htol32(word);
		W_REG(&regs->tplatewrdata, word);

		buf = (uint8*)buf + sizeof(uint32);
		len -= sizeof(uint32);
	}
}

/*
 * Suspend the the MAC and update the slot timing
 * for standard 11b/g (20us slots) or shortslot 11g (9us slots).
 */
static void
wlc_switch_shortslot(wlc_info_t *wlc, bool shortslot)
{
	ASSERT(wlc->band->gmode);

	/* use the override if it is set */
	if (wlc->shortslot_override != WLC_SHORTSLOT_AUTO)
		shortslot = (wlc->shortslot_override == WLC_SHORTSLOT_ON);

	if (wlc->shortslot == shortslot)
		return;

	wlc->shortslot = shortslot;

	/* update the capability based on current shortslot mode */
	wlc->pub.current_bss.capability &= ~DOT11_CAP_SHORTSLOT;
	if (wlc->shortslot)
		wlc->pub.current_bss.capability |= DOT11_CAP_SHORTSLOT;

	wlc_suspend_mac_and_wait(wlc);
	wlc_update_slot_timing(wlc, shortslot);
	wlc_enable_mac(wlc);
}

/*
 * Update the slot timing for standard 11b/g (20us slots)
 * or shortslot 11g (9us slots)
 * The PSM needs to be suspended for this call.
 */
static void
wlc_update_slot_timing(wlc_info_t *wlc, bool shortslot)
{
	d11regs_t *regs;

	regs = wlc->regs;

	if (shortslot) {
		/* 11g short slot: 11a timing */
		W_REG(&regs->ifs_slot, 0x0207);	/* APHY_SLOT_TIME */
		wlc_write_shm(wlc, M_DOT11_SLOT, APHY_SLOT_TIME);
	} else {
		/* 11g long slot: 11b timing */
		W_REG(&regs->ifs_slot, 0x0212);	/* BPHY_SLOT_TIME */
		wlc_write_shm(wlc, M_DOT11_SLOT, BPHY_SLOT_TIME);
	}
}

void
wlc_set_cwmin(wlc_info_t *wlc, uint16 newmin)
{
	wlc->band->CWmin = newmin;
	W_REG(&wlc->regs->objaddr, OBJADDR_SCR_SEL | S_DOT11_CWMIN);
	W_REG(&wlc->regs->objdata, newmin);
}

void
wlc_set_cwmax(wlc_info_t *wlc, uint16 newmax)
{
	wlc->band->CWmax = newmax;
	W_REG(&wlc->regs->objaddr, OBJADDR_SCR_SEL | S_DOT11_CWMAX);
	W_REG(&wlc->regs->objdata, newmax);
}


void
wlc_update_txpwr_shmem(wlc_pub_t *pub)
{
	wlc_info_t *wlc = (wlc_info_t *)pub;
	uint8	r;
	bool	war = FALSE;


	WL_TRACE(("wl%d: %s\n", pub->unit, __FUNCTION__));

	if (wlc->core->clk) {
		if (wlc->pub.associated)
			r = wlc->pub.current_bss.rateset.rates[0];
		else
			r = wlc->default_bss.rateset.rates[0];
		if (ISGPHY(wlc->band->pi) &&
		    ((r != 0x82) && (r != 0x84) && (r != 0x8b) && (r != 0x96)))
			war = TRUE;
		wlc_phy_update_txpwr_shmem(wlc->band->pi, war);
	}
}

static void
wlc_scan(
	wlc_info_t *wlc,
	int bss_type,
	const struct ether_addr* bssid,
	const uchar ssid[],
	int ssid_len,
	int scan_type,
	int nprobes,
	int active_time,
	int passive_time,
	int home_time,
	const uint16* channel_list,
	int channel_num,
	bool save_prb,
	cb_fn_t fn, void* arg)
{
	bool scan_in_progress;
	bool scan_timer_set;
	int i;

	ASSERT(ssid_len >= 0);
	ASSERT(ssid_len <= DOT11_MAX_SSID_LEN);
	ASSERT(bss_type == DOT11_BSSTYPE_INFRASTRUCTURE || bss_type == DOT11_BSSTYPE_INDEPENDENT ||
		bss_type == DOT11_BSSTYPE_ANY);

	WL_INFORM(("wl%d: %s: for SSID \"%s\"\n",
		wlc->pub.unit, __FUNCTION__, (wlc_format_ssid(ssidbuf, ssid, ssid_len), ssidbuf)));

	scan_in_progress = SCAN_IN_PROGRESS(wlc);
	scan_timer_set = (scan_in_progress &&
		0 == (wlc->scan.state & (SCAN_STATE_WSUSPEND | SCAN_STATE_PSPEND)));

	/* clear or set optional params to default */
	wlc->scan.state &= SCAN_STATE_SUPPRESS;	/* keep persistent scan suppress flag */
	wlc->scan.nprobes = wlc->scan.defaults.nprobes;
	if (wlc->pub.associated) {
		wlc->scan.active_time = wlc->scan.defaults.assoc_time;
		wlc->scan.home_time = wlc->scan.defaults.home_time;
	} else {
		wlc->scan.active_time = wlc->scan.defaults.unassoc_time;
		wlc->scan.home_time = 0;
	}
	wlc->scan.passive_time = wlc->scan.defaults.passive_time;
	if (wlc->scan.defaults.passive)
		wlc->scan.state |= SCAN_STATE_PASSIVE;

	wlc_scan_default_channels(wlc, wlc->scan.channel_list, &wlc->scan.channel_num);

	/* set required and optional params */
	/* If IBSS Lock Out feature is turned on, set the scan type to BSS only */
	wlc->scan.bss_type = (wlc->ibss_allowed == FALSE)?DOT11_BSSTYPE_INFRASTRUCTURE:bss_type;
	bcopy((char*)bssid, (char*)&wlc->scan.bssid, ETHER_ADDR_LEN);
	wlc->scan.ssid_len = ssid_len;
	bzero(wlc->scan.ssid, DOT11_MAX_SSID_LEN);
	if (ssid_len > 0)
		bcopy(ssid, wlc->scan.ssid, ssid_len);
	if (scan_type == DOT11_SCANTYPE_ACTIVE) {
		wlc->scan.state &= ~SCAN_STATE_PASSIVE;
	} else if (scan_type == DOT11_SCANTYPE_PASSIVE) {
		wlc->scan.state |= SCAN_STATE_PASSIVE;
	}
	/* passive scan always has nprobes to 1 */
	if (wlc->scan.state & SCAN_STATE_PASSIVE) {
		wlc->scan.nprobes = 1;
	}
	if (active_time > 0)
		wlc->scan.active_time = (uint16)active_time;

	if (passive_time > 0)
		wlc->scan.passive_time = (uint16)passive_time;

	if (home_time >= 0 && wlc->pub.associated)
		wlc->scan.home_time = (uint16)home_time;
	if (nprobes > 0 && (wlc->scan.state & SCAN_STATE_PASSIVE) == 0)
		wlc->scan.nprobes = (uint8)nprobes;
	if (save_prb)
		wlc->scan.state |= SCAN_STATE_SAVE_PRB;
	wlc->scan.cb = fn;
	wlc->scan.cb_arg = arg;

	WL_INFORM(("wl%d: %s: wlc_scan params: nprobes %d dwell active/passive %dms/%dms home %dms"
		" flags %d\n",
		wlc->pub.unit, __FUNCTION__, wlc->scan.nprobes, wlc->scan.active_time,
		wlc->scan.passive_time,
		wlc->scan.home_time, wlc->scan.state));

	/* channel list validation */
	if (channel_num > MAXCHANNEL) {
		WL_ERROR(("wl%d: %s: wlc_scan bad param channel_num %d greater than max %d\n",
			wlc->pub.unit, __FUNCTION__, channel_num, MAXCHANNEL));
		channel_num = 0;
	}
	if (channel_num > 0 && channel_list == NULL) {
		WL_ERROR(("wl%d: %s: wlc_scan bad param channel_list was NULL with channel_num ="
			" %d\n",
			wlc->pub.unit, __FUNCTION__, channel_num));
		channel_num = 0;
	}
	for (i = 0; i < channel_num; i++) {
		if (!VALID_CHANNEL_DB(wlc, channel_list[i] & WL_CHANSPEC_CHAN_MASK)) {
			channel_num = 0;
		}
	}

	if (channel_num > 0) {
		for (i = 0; i < channel_num; i++)
			wlc->scan.channel_list[i] = channel_list[i] & WL_CHANSPEC_CHAN_MASK;
		wlc->scan.channel_num = channel_num;
	}
	if ((wlc->scan.state & SCAN_STATE_SUPPRESS) || (!wlc->scan.channel_num)) {
		WL_INFORM(("wl%d: %s: scan.state %d scan.channel_num %d\n",
			wlc->pub.unit, __FUNCTION__, wlc->scan.state, wlc->scan.channel_num));

		if (scan_in_progress)
			wlc_scan_abort(wlc);

		/* no scan now, but free any earlier leftovers */
		wlc_bss_list_free(wlc, &wlc->scan_results);
		if (fn)
			(fn)(arg);

		return;
	}

	if (scan_in_progress && wlc->assoc_state == AS_IDLE)
		wlc_mac_event(wlc, WLC_E_SCAN_COMPLETE, NULL, WLC_E_STATUS_ABORT, 0, 0, 0, 0);

	/* start the scan with the results cleared */
	wlc->scan.away_count = 0;
	wlc->scan.away_limit = MAX(1, WLC_SCAN_AWAY_LIMIT / wlc->scan.active_time);
	wlc->scan.channel_idx = 0;
	wlc->scan.pass = WLC_SCAN_START;
	/* ...and free any leftover responses from before */
	wlc_bss_list_free(wlc, &wlc->scan_results);


	/* keep core awake to receive solicited probe responses, SCAN_IN_PROGRESS is TRUE */
	ASSERT(STAY_AWAKE(wlc));
	wlc_set_ps_ctrl(wlc);

	if (wlc->pub.associated) {
		/* return to the current BSS channel if associated */
		wlc->scan.channel_return = wlc->pub.current_bss.channel;
	} else if (!scan_in_progress) {
		/* not associated and no scan in progress, so just return to the current channel */
		if (VALID_CHANNEL_DB(wlc, wlc->band->pi->radio_channel)) {
			wlc->scan.channel_return = (uint8)wlc->band->pi->radio_channel;
		} else {
			wlc->scan.channel_return = wlc_next_channel(wlc,
				(uint8)wlc->band->pi->radio_channel, CHAN_TYPE_ANY, 1);
		}
	}
	/* else, we are interrupting a scan, but not associated, so use the original return channel
	 */

	if (!scan_timer_set) {
		/* call wlc_scantimer to get the scan state machine going */
		/* DUALBAND - Don't call wlc_scantimer() directly from DPC... */
		wl_add_timer(wlc->wl, wlc->scan.timer, 0, 0);
	}
	/* if a scan is in progress, allow the next callback to restart the scan state machine */
}

static void
wlc_scan_abort(wlc_info_t *wlc)
{
	if (!SCAN_IN_PROGRESS(wlc))
		return;

	/* abort the current scan, and return to home channel */
	WL_INFORM(("wl%d: %s: aborting scan in progress\n", wlc->pub.unit, __FUNCTION__));

	if (wlc->assoc_state == AS_IDLE)
		wlc_mac_event(wlc, WLC_E_SCAN_COMPLETE, NULL, WLC_E_STATUS_ABORT, 0, 0, 0, 0);

	wlc_bss_list_free(wlc, &wlc->scan_results);

	/* Move to abort state for wlc_scantimer to clean up */
	wlc->scan.pass = WLC_SCAN_ABORT;

	/* Change the radio channel to the return channel */
	if (wlc->band->pi->radio_channel != wlc->scan.channel_return) {
		wlc_suspend_mac_and_wait(wlc);
		wlc_set_channel(wlc, wlc->scan.channel_return);
		wlc_enable_mac(wlc);
	}
	/* un-block PSPoll operations and restore PS state */
	if (wlc->PMblocked) {
		wlc->PMblocked = FALSE;
		/* come out of PS mode if appropriate */
		if (wlc->PM != PM_MAX || wlc->WME_PM_blocked)
			wlc_set_pmstate(wlc, FALSE);
	}
	/* Clear the tssi values since we may have collected
	 * info from other channels during the scan
	 */
	wlc_clear_tssi(wlc->band->pi);

	/* un-suspend the data fifos now that we are back on the home channel */
	wlc_tx_resume(wlc);
	wlc->block_datafifo &= ~DATA_BLOCK_SCAN;
	/* enable CFP update */
	wlc_mhf(wlc, MHF_SKIP_CFP_UPDATE, 0, TRUE);

	/* check for states that indicate the scan timer is not scheduled */
	if (wlc->scan.state & (SCAN_STATE_WSUSPEND | SCAN_STATE_PSPEND)) {
		/* wlc_scantimer would have been scheduled by either a tx fifo
		 * suspend or PS indication, both of which we are canceling.
		 * Call wlc_scantimer directly to abort.
		 */
		wlc->scan.state &= ~(SCAN_STATE_WSUSPEND | SCAN_STATE_PSPEND);
		wlc_scantimer(wlc);
	}
}

static void
wlc_scantimer(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;
	uint channel;
	int next_channel;
	int8 passes;

	if (!wlc->pub.up)
		return;

	if (DEVICEREMOVED(wlc)) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}

	if (wlc->scan.pass == WLC_SCAN_ABORT) {
		WL_ASSOC(("SCAN: wlc_scantimer: aborted scan for SSID \"%s\"\n", ssidbuf));
		wlc->scan.channel_idx = -1;
		wlc_set_ps_ctrl(wlc);
		return;
	}

	/* check if we are still waiting on an event to continue */
	if (wlc->scan.state & SCAN_STATE_SUSPEND_REQ) {
		wlc->scan.state &= ~SCAN_STATE_SUSPEND_REQ;
		if (!wlc_tx_suspended(wlc)) {
			wlc_tx_suspend(wlc);
			wlc->scan.state |= SCAN_STATE_WSUSPEND;
			return;
		}
	}
	/* still waiting for tx data fifo suspended indication */
	if (wlc->scan.state & SCAN_STATE_WSUSPEND)
		return;

	wlc->scan.pass++;
	channel = wlc->scan.channel_list[wlc->scan.channel_idx];
	passes = QUIET_CHANNEL(wlc, channel) ? 1 : wlc->scan.nprobes;

	if (wlc->scan.pass > passes) {
		if (wlc->scan.pass == passes + 1) {
			/* scan passes complete for the current channel */
			WL_INFORM(("wl%d: %s: %sscanned channel %d, %d total responses for SSID"
				" \"%s\"\n",
				wlc->pub.unit, __FUNCTION__,
				((QUIET_CHANNEL(wlc, channel) && !(wlc->scan.state &
				SCAN_STATE_RADAR_CLEAR)) ? "passively ":""),
				channel, wlc->scan_results.count, ssidbuf));
			/* reset the radar clear flag since we will be leaving the channel */
			wlc->scan.state &= ~SCAN_STATE_RADAR_CLEAR;
		}

		if (wlc->scan.channel_idx < wlc->scan.channel_num - 1)
			next_channel = wlc->scan.channel_list[wlc->scan.channel_idx + 1];
		else
			next_channel = -1;

		/* keep track of the number of channels scanned since the last
		 * time we returned to the home channel
		 */
		if (wlc->band->pi->radio_channel != wlc->scan.channel_return)
			wlc->scan.away_count++;
		else
			wlc->scan.away_count = 0;

		/* If the home_time is non-zero,
		 * and there are more channels to scan,
		 * and we reached the away channel limit or the channel
		 *	we just scanned or are about to scan is a passive scan channel,
		 * return to the home channel before scanning the next channel
		 */
		if (wlc->scan.pass == (passes + 1) &&
		    wlc->scan.home_time > 0 &&
		    next_channel != -1 &&
		    (wlc->scan.away_count >= wlc->scan.away_limit ||
		     QUIET_CHANNEL(wlc, channel) || QUIET_CHANNEL(wlc, next_channel))) {
			/* Change the radio channel to associated BSS */
			wlc_suspend_mac_and_wait(wlc);
			wlc_set_channel(wlc, wlc->scan.channel_return);
			if (!wlc->pub.BSS)
				wlc_ibss_resume(wlc);
			/* Restore promisc behavior for beacons and probes */
			wlc->bcnmisc_scan = FALSE;
			wlc_mac_bcn_promisc(wlc);

			wlc_enable_mac(wlc);

			/* Clear the tssi values */
			wlc_clear_tssi(wlc->band->pi);
			/* un-block PSPoll operations and restore PS state */
			if (wlc->PMblocked) {
				wlc->PMblocked = FALSE;
				/* come out of PS mode if appropriate */
				if (wlc->PM != PM_MAX || wlc->WME_PM_blocked)
					wlc_set_pmstate(wlc, FALSE);
			}
			/* un-suspend the DATA fifo now that we are back on the home channel */
			wlc_tx_resume(wlc);

			/* re-enable txq processing now that we are back on the home channel */
			wlc->block_datafifo &= ~DATA_BLOCK_SCAN;

			/* enable CFP update */
			wlc_mhf(wlc, MHF_SKIP_CFP_UPDATE, 0, TRUE);

			/* Allow normal traffic before next channel scan */
			wl_add_timer(wlc->wl, wlc->scan.timer, wlc->scan.home_time, 0);

			/* wlc_scantimer called back when wlc->scan.timer fires */
			return;
		}

		/* if there are more channels to scan ... */
		if (next_channel != -1) {
			/* ... continue scanning */
			wlc->scan.channel_idx++;
			channel = (uint)next_channel;
			wlc->scan.pass = WLC_SCAN_CHANNEL_PREP;
		}
		/* ... otherwise the scan is done,
		 * scan.pass > passes and we fall through to the end of this function
		 */
	} /* wlc->scan.pass > passes */

	if (wlc->scan.pass == WLC_SCAN_CHANNEL_PREP) {
		/* do off-home-channel setup if we are associated and on the home channel */
		if (wlc->pub.associated &&
			wlc->band->pi->radio_channel == wlc->scan.channel_return) {
			/* announce PS mode to the AP if we are not already in PS mode */
			if (wlc->pub.BSS && WLC_BSS_CONNECTED(wlc) && !wlc->PMenabled) {
				/* set our PS callback flag so that wlc_scantimer is called
				 * when the PM State Null Data packet completes
				 */
				wlc->scan.state |= SCAN_STATE_PSPEND;
				wlc_set_pmstate(wlc, TRUE);
				/* clear our callback flag if PMpending is false, indicating
				 * that no PM State Null Data packet was sent by
				 * wlc_set_pmstate()
				 */
				if (!wlc->PMpending)
					wlc->scan.state &= ~SCAN_STATE_PSPEND;
			}
			/* block any PSPoll operations since we are just holding off AP
			 * traffic
			 */
			if (wlc->pub.BSS && wlc->PMenabled) {
				wlc->PMblocked = TRUE;
			}
			/* Must leave IBSS first before starting a scan */
			if (!wlc->pub.BSS) {
				wlc_ibss_suspend(wlc);
			}
			/* block any tx traffic */
			if (!wlc_tx_suspended(wlc)) {
				if ((wlc->scan.state & SCAN_STATE_PSPEND) == 0) {
					/* suspend the data fifos to avoid sending data packets
					 * during scan
					 */
					wlc_tx_suspend(wlc);
					wlc->scan.state |= SCAN_STATE_WSUSPEND;
				} else {
					/* wlc_scantimer() will do the wlc_tx_suspend() call
					 * when the PS announce completes
					 */
					wlc->scan.state |= SCAN_STATE_SUSPEND_REQ;
				}
			}
			/* if there is any prep work, return here and let wlc_scantimer be called
			 * back
			 */
			if (wlc->scan.state & (SCAN_STATE_PSPEND | SCAN_STATE_WSUSPEND))
				return;
		}
		wlc->scan.pass = 1;
	}

	/* scan the channel */
	if (wlc->scan.pass >= 1 && wlc->scan.pass <= passes) {

		/* skip CFP update if not in home channel */
		if (channel != wlc->scan.channel_return)
			wlc_mhf(wlc, MHF_SKIP_CFP_UPDATE, MHF_SKIP_CFP_UPDATE, TRUE);

		if (channel != wlc->band->pi->radio_channel) {
			wlc_suspend_mac_and_wait(wlc);
			wlc_set_channel(wlc, channel);
			wlc_enable_mac(wlc);
		}

		/*
		 * disable txq processing to avoid using wrong rateset and
		 * posting packets to wrong ring during scan on "other" band
		 */
		if (wlc->pub.associated && !VALID_CHANNEL(wlc, wlc->scan.channel_return))
			wlc->block_datafifo |= DATA_BLOCK_SCAN;
		else
			wlc->block_datafifo &= ~DATA_BLOCK_SCAN;

		if ((wlc->scan.state & SCAN_STATE_PASSIVE) == 0 && !QUIET_CHANNEL(wlc, channel)) {
			/* Only send probes if in active scan mode, and this is not a quiet
			 * channel.
			 */
			wlc_sendprobe(wlc, wlc->scan.ssid, wlc->scan.ssid_len);
		} else {
			/* Otherwise, make sure we are promisc for beacons and probes for a passive
			 * scan
			 */
			wlc->bcnmisc_scan = TRUE;
			wlc_mac_bcn_promisc(wlc);
		}

		/* Wait for responses to come back */
		if ((wlc->scan.state & SCAN_STATE_PASSIVE) || QUIET_CHANNEL(wlc, channel)) {
			uint32 dummy_tsf_h;

			/* use passive time for a passive scan or for a quiet channel in a active
			 * scan
			 */
			wl_add_timer(wlc->wl, wlc->scan.timer, wlc->scan.passive_time, 0);

			/* record the start time in case we switch to an active scan in
			 * wlc_scan_radar_clear()
			 */
			wlc_read_tsf(wlc, &wlc->scan.start_tsf, &dummy_tsf_h);
		} else {
			/* for an active scan on a non-quiet channel, use channel active dwell time
			 * divided by the number of probes we need to send
			 */
			wl_add_timer(wlc->wl, wlc->scan.timer,
				wlc->scan.active_time/wlc->scan.nprobes, 0);
		}

		/* wlc_scantimer called back when wlc->scan.timer fires */

		/* record phy noise for the scan channel */
		wlc->phy_noise_list[channel] = wlc_compute_phy_noise(wlc->band->pi);

		return;
	}

	/*
	 * wraps up scan process.
	 */
	WL_INFORM(("wl%d: %s: %s scan done, %d total responses for SSID \"%s\"\n",
		wlc->pub.unit, __FUNCTION__,
		((wlc->scan.state & SCAN_STATE_PASSIVE) ? "Passive" : "Active"),
		wlc->scan_results.count, ssidbuf));

	wlc->scan.channel_idx = -1;

	/* Return to the original core and change the radio channel to the return channel */
	wlc_suspend_mac_and_wait(wlc);
	wlc_set_channel(wlc, wlc->scan.channel_return);
	if (!wlc->pub.BSS && wlc->pub.associated)
		wlc_ibss_resume(wlc);
	/* Restore promisc behavior for beacons and probes */
	wlc->bcnmisc_scan = FALSE;
	wlc_mac_bcn_promisc(wlc);

	wlc_enable_mac(wlc);

	/* Clear the tssi values */
	wlc_clear_tssi(wlc->band->pi);
	/* un-block PSPoll operations and restore PS state */
	if (wlc->PMblocked) {
		wlc->PMblocked = FALSE;
		/* come out of PS mode if appropriate */
		if (wlc->PM != PM_MAX || wlc->WME_PM_blocked)
			wlc_set_pmstate(wlc, FALSE);
	}
	/* un-suspend the DATA fifo now that we are done with the scan */
	wlc_tx_resume(wlc);

	/* re-enable txq processing now that we are done with the scan */
	wlc->block_datafifo &= ~DATA_BLOCK_SCAN;

	/* enable CFP update */
	wlc_mhf(wlc, MHF_SKIP_CFP_UPDATE, 0, TRUE);

	/* allow core to sleep again (no more solicited probe responses) */
	wlc_set_ps_ctrl(wlc);

	if (wlc->scan.cb) {
		cb_fn_t cb = wlc->scan.cb;
		void *cb_arg = wlc->scan.cb_arg;

		wlc->scan.cb = NULL;
		wlc->scan.cb_arg = NULL;

		(cb)(cb_arg);
		if (wlc->assoc_state == AS_IDLE)
			wlc_mac_event(wlc, WLC_E_SCAN_COMPLETE, NULL, WLC_E_STATUS_SUCCESS,
			              0, 0, 0, 0);
	}

	wlc_bss_list_free(wlc, &wlc->scan_results);

	WL_ASSOC(("SCAN: wlc_scantimer done\n"));
}

static void
wlc_scan_radar_clear(wlc_info_t *wlc)
{
	uint32 cur_l, cur_h;
	uint32 elapsed_time, remaining_time, active_time;

	/* if we are not on a radar quiet channel,
	 * or a passive scan was requested,
	 * or we already processed the radar clear signal,
	 * then do nothing
	 */
	if (!QUIET_CHANNEL(wlc, wlc->band->pi->radio_channel) ||
	    (wlc->scan.state & (SCAN_STATE_PASSIVE | SCAN_STATE_RADAR_CLEAR)))
		return;

	/* if we are not in the channel scan portion of the scan, do nothing */
	if (wlc->scan.pass != 1)
		return;

	/* if there is not enough time remaining for a probe, do nothing */
	wlc_read_tsf(wlc, &cur_l, &cur_h);
	elapsed_time = (cur_l - wlc->scan.start_tsf) / 1000;
	if (elapsed_time > wlc->scan.passive_time)
		remaining_time = 0;
	else
		remaining_time = wlc->scan.passive_time - elapsed_time;

	if (remaining_time < WLC_SCAN_MIN_PROBE_TIME)
		return;

	/* everything is ok to switch to an active scan */
	wlc->scan.state |= SCAN_STATE_RADAR_CLEAR;

	active_time = MIN(remaining_time, wlc->scan.active_time);

	wl_del_timer(wlc->wl, wlc->scan.timer);
	wl_add_timer(wlc->wl, wlc->scan.timer, active_time, 0);

	wlc_mute(wlc, OFF);
	wlc_sendprobe(wlc, wlc->scan.ssid, wlc->scan.ssid_len);

	WL_REGULATORY(("wl%d: wlc_scan_radar_clear: rcvd beacon on radar channel %d, converting to"
		       " active scan, %d ms left\n",
		       wlc->pub.unit, wlc->band->pi->radio_channel, active_time));
}

void
wlc_scan_default_channels(wlc_info_t *wlc, uint8 *channel_list, int *pchannel_num)
{
	uint channel_start;
	int num;

	if (wlc->pub.associated) {
		/* start the scan on the current BSS channel */
		channel_start = wlc->pub.current_bss.channel;
	} else if (SCAN_IN_PROGRESS(wlc)) {
		/* interrupting a scan, so use the original return channel */
		channel_start = wlc->scan.channel_return;
	} else {
		/* not associated, so just start on the current channel */
		channel_start = wlc->band->pi->radio_channel;
	}
	ASSERT(channel_start < MAXCHANNEL);

	/* enumerate all the active (non-quiet) channels first */
	wlc_scan_channels(wlc, channel_list, &num, MAXCHANNEL, channel_start, CHAN_TYPE_CHATTY);
	*pchannel_num = num;

	/* enumerate all the passive (quiet) channels second */
	wlc_scan_channels(wlc, (channel_list + num), &num, (MAXCHANNEL - num), channel_start,
		CHAN_TYPE_QUIET);
	*pchannel_num += num;
}

static void
wlc_scan_channels(wlc_info_t *wlc, uint8 *channel_list, int *pchannel_num, int channel_max,
	uint channel_start, int channel_type)
{
	uint bandunit;
	uint channel;
	int num = 0;
	uint i;

	ASSERT(channel_start < MAXCHANNEL);
	bandunit = CHANNEL_BANDUNIT(wlc, channel_start);
	for (i = 0; i < NBANDS(wlc); i++) {
		/* find all valid channels for this locale in bandunit's band */
		channel = channel_start;
		while (num < channel_max) {
			if (VALID_CHANNEL_IN_BAND(wlc, bandunit, channel) &&
			    ((channel_type == CHAN_TYPE_CHATTY && !QUIET_CHANNEL(wlc, channel)) ||
			     (channel_type == CHAN_TYPE_QUIET && QUIET_CHANNEL(wlc, channel))))
				channel_list[num++] = (uint8)channel;
			channel = (channel + 1) % MAXCHANNEL;
			if (channel == channel_start)
				break;
		}

		/* only find channels for one band */
		if (!IS_MBAND_UNLOCKED(wlc))
			break;

		/* prepare to find the other band's channels */
		bandunit = ((bandunit == 1) ? 0 : 1);
		channel_start = 0;
	}

	*pchannel_num = num;
}


void
wlc_set_ssid(wlc_info_t *wlc, uchar SSID[], int len)
{
	wlc_bsscfg_t *cfg;

	ASSERT(wlc->pub.up);

	cfg = &wlc->cfg;

	ASSERT(BSSCFG_STA(cfg));

	if (len == 0)
		WL_ASSOC(("SCAN: wlc_set_ssid, Setting SSID to NULL...\n"));
	else
		WL_ASSOC(("SCAN: wlc_set_ssid, Setting SSID to \"%s\"...\n",
			(wlc_format_ssid(ssidbuf, SSID, len), ssidbuf)));

	/* abort any association state machine in process */
	if ((wlc->assoc_state != AS_IDLE) && (wlc->assoc_state != AS_WAIT_DISASSOC))
		wlc_assoc_abort(wlc);

	/* adopt the default BSS params as the target */
	wlc->target_bss = wlc->default_bss;

	/* update the target_bss and default config with the ssid */
	wlc->target_bss.SSID_len = (uint8) len;
	wlc->cfg.SSID_len = (uint8)len;
	if (len > 0) {
		bcopy(SSID, (char*)wlc->target_bss.SSID, len);
		bcopy(SSID, (char*)wlc->cfg.SSID, len);
	}

	/* this is STA only, no aps_associated issues */
	wlc->cfg.enable = TRUE;
	wlc->cfg.up = TRUE;

#ifdef BCMWPA2
	/* clear PMKID cache */
	if (cfg->WPA_auth == WPA2_AUTH_UNSPECIFIED) {
		WL_WSEC(("wl%d: clearing PMKID cache and candidate list\n",
			wlc->pub.unit));
		wlc->pub.npmkid = 0;
		wlc->pub.npmkid_cand = 0;
        }
#endif /* BCMWPA2 */

	/* initialize ucode default key flag MHF_DEFKEYVALID */
	wlc_mhf(wlc, MHF_DEFKEYVALID, wlc_key_defkeyflag(wlc), TRUE);

	if (wlc->assoc_state == AS_WAIT_DISASSOC) {
		/* we are interrupting an earlier wlc_set_ssid call and wlc_ssid_scan_start is
		 * scheduled to run as the wlc_disassociate_client callback. The target_bss has
		 * been updated above so just allow the wlc_ssid_scan_start callback to pick up
		 * the set_ssid work with the new target.
		 */
	} else {
		wlc->assoc_type = AS_ASSOCIATION;
		if (wlc->pub.associated &&
		    (len != wlc->pub.current_bss.SSID_len ||
		     bcmp(SSID, (char*)wlc->pub.current_bss.SSID, len))) {
			/* abort any current scan, and return to home channel */
			wlc_scan_abort(wlc);

			WL_ASSOC(("SCAN: wlc_set_ssid, Disassociating from %s first\n",
				bcm_ether_ntoa(&wlc->prev_BSSID, eabuf)));
			wlc_assoc_change_state(wlc, AS_WAIT_DISASSOC);
			wlc_disassociate_client(wlc, TRUE, wlc_setssid_disassoc_complete, NULL);
		} else {
			/* make sure the association retry timer is not pending */
			wl_del_timer(wlc->wl, wlc->assoc_timer);
			/* continue the association process by starting the association scan */
			wlc_assoc_scan_start(wlc, NULL);
		}
	}
}

static void
wlc_setssid_disassoc_complete(wlc_info_t *wlc, uint txstatus, void *ignore)
{
	WL_ASSOC(("SCAN: disassociation complete, start assoc scan\n"));

	/* continue with the association process by starting the association scan */
	wlc_assoc_scan_start(wlc, NULL);

	return;
}

static bool
wlc_rsn_ucast_lookup(struct rsn_parms *rsn, uint8 auth)
{
	uint i;

	for (i = 0; i < rsn->ucount; i++)
		if (rsn->unicast[i] == auth)
			return TRUE;

	return FALSE;
}

static void
wlc_cook_join_targets(wlc_info_t *wlc, bool roam, int cur_rssi)
{
	int i, j, k;
	wlc_bss_info_t **bip, *tmp_bi;
	uint orig_roam_delta, roam_delta = 0;
	bool done;
	uint32 *join_pref_score;
	uint32 join_pref_score_size = wlc->join_targets.count * sizeof(uint32);
	uint32 tmp_score;

	WL_ASSOC(("wl%d: %s: cur_rssi 0x%x targets count 0x%x",
		WLCWLUNIT(wlc), __FUNCTION__,
		cur_rssi, wlc->join_targets.count));

	join_pref_score = (uint32 *)MALLOC(wlc->pub.osh, join_pref_score_size);
	if (!join_pref_score) {
		ASSERT(0);
		return;
	}

	bip = wlc->join_targets.ptrs;

	for (i = 0; i < (int)wlc->join_targets_last; i++) {
		join_pref_score[i] = bip[i]->RSSI;
		WL_ASSOC(("wl%d: RSSI 0x%x in BSS %s score 0x%x aac 0x%x chan_free 0x%x\n",
			WLCWLUNIT(wlc), bip[i]->RSSI,
			bcm_ether_ntoa(&bip[i]->BSSID, eabuf), join_pref_score[i],
				bip[i]->qbss_load_aac, bip[i]->qbss_load_chan_free));
	}

	/* no need to sort if we only see one AP */
	if (wlc->join_targets.count > 1) {
		/* sort join_targets by join preference score in increasing order */
		for (k = (int)wlc->join_targets.count; --k >= 0;) {
			done = TRUE;
			for (j = 0; j < k; j++) {
				if (join_pref_score[j] > join_pref_score[j+1]) {
					/* swap score */
					tmp_score = join_pref_score[j];
					join_pref_score[j] = join_pref_score[j+1];
					join_pref_score[j+1] = tmp_score;
					/* swap bip */
					tmp_bi = bip[j];
					bip[j] = bip[j+1];
					bip[j+1] = tmp_bi;
					done = FALSE;
				}
			}
			if (done)
				break;
		}
	}
	if (roam) {
		/*
		 * Placeholder for roaming support:
		 * - We're here because our current AP's JSSI fell below some threshold
		 *   or we haven't heard from him for a while.
		 */

		/*
		 * Use input cur_rssi if we didn't get a probe response because we were
		 * unlucky or we're out of contact; otherwise, use the RSSI of the probe
		 * response. The channel of the probe response/beacon is checked in case
		 * the AP switched channels. In that case we do not want to update cur_rssi
		 * and instead consider the AP as a roam target.
		 */
		for (i = 0; i < (int)wlc->join_targets.count; i++) {
			if (WLC_IS_CURRENT_BSSID(wlc, &bip[i]->BSSID) &&
			    bip[i]->channel == wlc->pub.current_bss.channel) {
				cur_rssi = bip[i]->RSSI;
				break;
			}
		}

		WL_ASSOC(("ROAM: wlc_cook_join_targets, roam_metric[%s] = %d\n",
			bcm_ether_ntoa(&wlc->BSSID, eabuf), cur_rssi));

		/* Prune candidates not "significantly better" than our current AP.
		 * Notes:
		 * - The definition of "significantly better" may not be the same for
		 *   all candidates.  For example, Jason suggests using a different
		 *   threshold for our previous AP, if we roamed before.
		 * - The metric will undoubtedly need to change.  First pass, just use
		 *   the RSSI of the probe response.
		 */

		/* Try to make roam_delta adaptive. lower threshold when our AP is very weak,
		 *  falling the edge of sensitivity. Hold threshold when our AP is relative strong.
		 * It should really be a lookup table when segment gets "tiny".
		 *
		 *	roam_trigger+10		roam_trigger	roam_trigger-10	roam_trigger-20
		 *		|               ||		|		|
		 *   roam_delta	| roam_delta    |   roam_delta	| roam_delta-10 |roam_delta-10
		 */
		orig_roam_delta = wlc->band->roam_delta;

		if (cur_rssi > wlc->band->roam_trigger - 10)
			roam_delta = orig_roam_delta;
		else
			roam_delta =
				(orig_roam_delta > 10) ? (orig_roam_delta - 10) : orig_roam_delta;

		/* TODO
		 * It may be advisable not to apply roam delta-based prune once
		 * preference score is above a certain threshold.
		 */

		WL_ASSOC(("ROAM: wlc_cook_join_targets, roam_delta = %d\n", roam_delta));
		/* find cutoff point for pruning. 
		 * already sorted in increasing order of join_pref_score
		 */
		for (i = 0; i < (int)wlc->join_targets.count; i++) {
			if ((int)join_pref_score[i] - cur_rssi > (int)roam_delta)       {
				break;
			}
		}
		/* Prune, finally.
		 * - move qualifying APs to the beginning of list
		 * - note the boundary of the qualifying AP list
		 */
		for (j = 0; i < (int)wlc->join_targets.count; i++, j++) {
			/* move join_pref_score[i] to join_pref_score[j] */
			join_pref_score[j] = join_pref_score[i];
			/* swap bip[i] with bip[j]
			 * moving bip[i] to bip[j] alone without swapping causes memory leak.
			 * by swapping, the ones below threshold are still left in bip
			 * so that they can be freed at later time.
			 */
			tmp_bi = bip[j];
			bip[j] = bip[i];
			bip[i] = tmp_bi;
			WL_ASSOC(("ROAM: cook_join_targets, after prune, roam_metric[%s] = %d\n",
				bcm_ether_ntoa(&bip[j]->BSSID, eabuf),
				join_pref_score[j]));
		}
		wlc->join_targets_last = j;
	} /* end of if (roam) */

	MFREE(wlc->pub.osh, join_pref_score, join_pref_score_size);
}

static void
wlc_assoc_scan_start(wlc_info_t *wlc, struct ether_addr *bssid)
{
	bool assoc = (wlc->assoc_type == AS_ASSOCIATION);
	uchar *ssid;
	int ssid_len;
	int bss_type;
	uint16 *channel_list = NULL;
	int channel_num = 0;

	/* specific bssid is optional */
	if (bssid == NULL)
		bssid = (struct ether_addr*)&ether_bcast;

	if (assoc) {
		/* standard association */
		ssid = wlc->target_bss.SSID;
		ssid_len = wlc->target_bss.SSID_len;
		if (wlc->target_bss.infra == 0)
			bss_type = DOT11_BSSTYPE_INDEPENDENT;
		else if (wlc->target_bss.infra == 1)
			bss_type = DOT11_BSSTYPE_INFRASTRUCTURE;
		else
			bss_type = DOT11_BSSTYPE_ANY;
	} else {
		/* roaming */
		ssid = wlc->pub.current_bss.SSID;
		ssid_len = wlc->pub.current_bss.SSID_len;
		bss_type = DOT11_BSSTYPE_INFRASTRUCTURE;
	}

	WL_ASSOC(("SCAN: wlc_assoc_scan_start for (%s%s) starting an SSID scan for %s\n",
		msg_pref, msg_name, (wlc_format_ssid(ssidbuf, ssid, ssid_len), ssidbuf)));

	wlc_assoc_change_state(wlc, AS_SCAN);

	if (wlc->pub.up)
		/* kick off a scan for the requested SSID, possibly broadcast SSID. */
		wlc_scan(wlc, bss_type, bssid, ssid, ssid_len, -1, -1, -1, -1, -1,
			channel_list, channel_num, TRUE, wlc_assoc_scan_complete, wlc);


	/* when the scan is done, wlc_assoc_scan_complete() will be called to copy
	 * scan_results to join_targets and continue the association process
	 */

	return;
}

#ifdef	BCMWPA2
void
wlc_pmkid_build_cand_list(wlc_info_t *wlc, bool check_SSID)
{
	uint i, j, mark;
	wlc_bss_info_t *bi;

	WL_WSEC(("wl%d: building PMKID candidate list\n", wlc->pub.unit));

	/* Merge scan results and pmkid cand list */
	for (i = 0; i < wlc->scan_results.count; i++) {

		bi = wlc->scan_results.ptrs[i];
		mark = MAXPMKID;

		/* right network?  if not, move on */
		if (check_SSID &&
			((bi->SSID_len != wlc->target_bss.SSID_len) ||
			bcmp(bi->SSID, wlc->target_bss.SSID, bi->SSID_len)))
			continue;

		WL_WSEC(("wl%d: wlc_build_pmkid_cand_list(): processing %s...",
			wlc->pub.unit, bcm_ether_ntoa(&bi->BSSID, eabuf)));

		/* already in candidate list? */
		for (j = 0; j < wlc->pub.npmkid_cand; j++) {
			if (bcmp((void *)&bi->BSSID, (void *)&wlc->pub.pmkid_cand[j].BSSID,
				ETHER_ADDR_LEN) == 0) {
				WL_WSEC(("already in candidate list\n"));

				mark = j;
				break;
			}
		}

		/* already in candidate list at the end, move on */
		if (mark != MAXPMKID && mark != (wlc->pub.npmkid_cand - 1))
			continue;

		/* not already in candidate list, add */
		if (mark == MAXPMKID) {
			WL_WSEC(("add to candidate list\n"));

			if (wlc->pub.npmkid_cand == (MAXPMKID - 1)) {
				WL_WSEC(("wl%d: wlc_build_pmkid_cand_list(): no room..."
					 "replace oldest\n", wlc->pub.unit));
				mark = 0;
			} else
				wlc->pub.npmkid_cand++;
		}

		/* bubble each item up, overwriting item at mark */
		for (j = mark + 1; j < wlc->pub.npmkid_cand; j++)
			wlc->pub.pmkid_cand[j - 1] = wlc->pub.pmkid_cand[j];

		/* new or updated entry gets added at or moved to end of list */
		bcopy(bi->BSSID.octet,
		      wlc->pub.pmkid_cand[wlc->pub.npmkid_cand - 1].BSSID.octet,
		      ETHER_ADDR_LEN);
		wlc->pub.pmkid_cand[wlc->pub.npmkid_cand - 1].preauth =
			(((bi->wpa2.flags & RSN_FLAGS_PREAUTH) != 0) ? 1 : 0);
	}

	/* if port's open, request PMKID cache plumb */
	ASSERT(wlc->cfg.WPA_auth == WPA2_AUTH_UNSPECIFIED);
	if (wlc->wsec_portopen && wlc->pub.npmkid_cand != 0) {
		WL_WSEC(("wl%d: wlc_build_pmkid_cand_list(): requesting PMKID cache plumb...\n",
			wlc->pub.unit));
			wlc_pmkid_event(wlc);
	}
}
#endif	/* BCMWPA2 */

static void
wlc_assoc_scan_complete(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;
	bool roam = (wlc->assoc_type == AS_ROAM);

	WL_ASSOC(("SCAN: wlc_assoc_scan_complete\n"));

#ifdef	BCMWPA2
	/* register scan results as possible PMKID candidates */
	if (wlc->cfg.WPA_auth == WPA2_AUTH_UNSPECIFIED)
		wlc_pmkid_build_cand_list(wlc, FALSE);
#endif	/* BCMWPA2 */

	wlc_bss_list_xfer(&wlc->scan_results, &wlc->join_targets);
	wlc->join_targets_last = wlc->join_targets.count;
	/* directed scans should have at most one result */
	ASSERT(ETHER_ISMULTI(&wlc->scan.bssid) || (wlc->join_targets.count <= 1));

	if (wlc->join_targets.count > 0) {
		WL_ASSOC(("SCAN for %s%s: SSID scan complete, %d results for \"%s\"\n",
			msg_pref, msg_name, wlc->join_targets.count, ssidbuf));
	} else if (wlc->target_bss.SSID_len > 0) {
		WL_ASSOC(("SCAN for %s%s: SSID scan complete, no matching SSIDs found for \"%s\"\n",
			msg_pref, msg_name, ssidbuf));
	} else {
		WL_ASSOC(("SCAN for %s: SSID scan complete, no SSIDs found for broadcast scan\n",
			msg_name));
	}

	/* sort join targets by signal strength */
	/* for roam, prune targets if they do not have signals stronger than our current AP */
	if (wlc->join_targets.count > 0) {
		if (roam) {
			wlc_cook_join_targets(wlc, TRUE, wlc->roam_metric);
			WL_ASSOC(("ROAM: %d roam target%s after pruning\n", wlc->join_targets_last,
				(wlc->join_targets_last == 1) ? "" : "s"));
		} else {
			wlc_cook_join_targets(wlc, FALSE, 0);
		}
	}

	if (wlc->join_targets_last > 0) {
		wlc_join_attempt(wlc);
	} else if (wlc->target_bss.infra != 1 &&
		wlc->target_bss.SSID_len > 0) {
		/* Create an IBSS if we are IBSS or AutoUnknown mode,
		 * and we had a non-Null SSID,
		 * (that is, we are in regulatory mode and do not have channel info yet)
		 */
		WL_ASSOC(("JOIN: creating an IBSS with SSID \"%s\"\n", ssidbuf));

		wlc_join_start_ibss(wlc, wlc->target_bss.SSID, wlc->target_bss.SSID_len);

		wlc_link(wlc, TRUE, &wlc->BSSID, wlc->bsscfg[0]);
		wlc_mac_event(wlc, WLC_E_SET_SSID, &wlc->BSSID, WLC_E_STATUS_SUCCESS, 0, 0, 0, 0);
	} else {
		/* no join targets */
		if (roam)
			wlc_mac_event(wlc, WLC_E_ROAM, NULL, WLC_E_STATUS_NO_NETWORKS, 0, 0, 0, 0);
		else
			wlc_mac_event(wlc, WLC_E_SET_SSID, NULL, WLC_E_STATUS_NO_NETWORKS, 0, 0, 0,
				0);
	}

	wlc_led_event(wlc->ledh);
}



static void
wlc_custom_scan_complete(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;

	/* release old BSS information */
	wlc_bss_list_free(wlc, &wlc->custom_scan_results);

	/* copy scan results to custom_scan_results for reporting via ioctl */
	wlc_bss_list_xfer(&wlc->scan_results, &wlc->custom_scan_results);
}


int
wlc_scan_request(
	wlc_info_t *wlc,
	int bss_type,
	const struct ether_addr* bssid,
	const uchar ssid[],
	int ssid_len,
	int scan_type,
	int nprobes,
	int active_time,
	int passive_time,
	int home_time,
	const uint16* channel_list,
	int channel_num,
	bool save_prb,
	cb_fn_t fn, void* arg)
{
	int err = 0;
	ASSERT(ssid_len <= DOT11_MAX_SSID_LEN);

	WL_ASSOC(("wl%d: %s:\n", wlc->pub.unit, __FUNCTION__));

	if (!wlc->pub.up) {
		WL_ERROR(("wl%d: wlc_scan_request, can not scan while driver is down\n",
			wlc->pub.unit));
		err = BCME_NOTUP;
	} else if (!wlc_mac_request_entry(wlc, WLC_ACTION_SCAN)) {
		if (!SCAN_IN_PROGRESS(wlc))
			err = BCME_ERROR;
	} else
		wlc_scan(wlc, bss_type, bssid, ssid, ssid_len,
			scan_type, nprobes, active_time, passive_time, home_time,
			channel_list, channel_num, save_prb, fn, arg);

	return err;
}


/* cases to roam
 * - wlc_watchdog() found beacon lost for too long
 * - AP DEAUTH/DISASSOC sta
 * it will end up in wlc_assoc_scan_complete and WLC_E_ROAM
 */
int
wlc_roam_scan(wlc_info_t *wlc, uint reason)
{
	if (!wlc_mac_request_entry(wlc, WLC_ACTION_ROAM))
		return 1;

	wlc->assoc_type = AS_ROAM;
	wlc->roam_reason = reason;

	/* start the re-association process by starting the association scan */
	wlc_assoc_scan_start(wlc, NULL);

	return 0;
}

/* do some pre-processing, then call wlc_roam_scan, called by receiving FC_DISASSOC or FC_DEAUTH */
static void
wlc_roam_start(wlc_info_t *wlc, uint reason)
{

	wlc->dtim_programmed = FALSE;
	wlc_set_ps_ctrl(wlc);

	bzero(wlc->BSSID.octet, ETHER_ADDR_LEN);
	bzero(wlc->pub.current_bss.BSSID.octet, ETHER_ADDR_LEN);

	/* Roaming, roaming, roaming... */
	wlc->roam_metric = WLC_RSSI_MINVAL;

	wlc_roam_scan(wlc, (reason == WLC_E_DEAUTH_IND) ? WLC_E_REASON_DEAUTH :
		WLC_E_REASON_DISASSOC);
}

static void
wlc_join_attempt(wlc_info_t *wlc)
{
	wlc_bss_info_t *bi;
	bool prune;
	uint i;
	wlcband_t *target_band;

	wlc_bsscfg_t *cfg = &wlc->cfg;
	/* keep core awake until join finishes, wlc->assoc_state != AS_IDLE */
	ASSERT(STAY_AWAKE(wlc));
	wlc_set_ps_ctrl(wlc);

	/* walk the list until there is a valid join target */
	for (; wlc->join_targets_last > 0; wlc->join_targets_last--) {
		struct rateset rateset;

		bi = wlc->join_targets.ptrs[wlc->join_targets_last - 1];

		/* check Privacy (encryption) in target BSS */
		/*
		 * WiFi: A STA with WEP off should never attempt to associate with
		 * an AP that has WEP on
		 */
		if ((bi->capability & DOT11_CAP_PRIVACY) && !WSEC_ENABLED(cfg->wsec)) {
			WL_ASSOC(("JOIN: Skipping BSSID %s, encryption mandatory in BSS, but"
				" encryption off for us.\n",
				  bcm_ether_ntoa(&bi->BSSID, eabuf)));
			wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0, WLC_E_PRUNE_ENCR_MISMATCH,
				bi->channel, 0, 0);
			continue;
		}
		/* skip broadcast bssid */
		if (ETHER_ISBCAST(bi->BSSID.octet)) {
			WL_ASSOC(("JOIN: Skipping BSS with broadcast BSSID\n"));
			wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0, WLC_E_PRUNE_BCAST_BSSID,
				bi->channel, 0, 0);
			continue;
		}
		/* prune join_targets based on allow/deny list */
		if (wlc->macmode) {
			/* check the allow/deny list */
			for (i = 0; i < wlc->nmac; i++) {
				if (bcmp(bi->BSSID.octet, (void*)&wlc->maclist[i],
					ETHER_ADDR_LEN) == 0)
					break;
			}
			/* prune if target is on the deny list or not on the allow list */
			if ((wlc->macmode == WLC_MACMODE_DENY && i < wlc->nmac) ||
			    (wlc->macmode == WLC_MACMODE_ALLOW && i >= wlc->nmac)) {
				if (wlc->macmode == WLC_MACMODE_DENY) {
					WL_ASSOC(("JOIN: pruning BSSID %s because it was on the MAC"
						" Deny list\n",
						bcm_ether_ntoa(&bi->BSSID, eabuf)));
					wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0,
						WLC_E_PRUNE_MAC_DENY, bi->channel, 0, 0);
				} else {
					WL_ASSOC(("JOIN: pruning BSSID %s because it was not on the"
						" MAC Allow list\n",
						bcm_ether_ntoa(&bi->BSSID, eabuf)));
					wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0,
						WLC_E_PRUNE_MAC_NA, bi->channel, 0, 0);
				}
				continue;
			}
		}

		target_band = &wlc->bandstate[CHANNEL_BANDUNIT(wlc, bi->channel)];

		/* prune if we are in strict SpectrumManagement mode, the AP is not advertising
		 * support,
		 * and the locale requires 802.11h SpectrumManagement
		 */
		if ((WL11H_ENAB(wlc) == SPECT_MNGMT_STRICT_11H) &&
		    bi->infra && (bi->capability & DOT11_CAP_SPECTRUM) == 0 &&
		    (wlc_check_locale_flags(target_band->bandtype, WLC_DFS_TPC))) {
			WL_ASSOC(("JOIN: pruning AP %s (SSID %s). requires spectrum management but AP does not have SpectrumManagement. Channel %d\n",
			          bcm_ether_ntoa(&bi->BSSID, eabuf), ssidbuf, bi->channel));
			wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0, WLC_E_PRUNE_SPCT_MGMT,
				bi->channel, 0, 0);
			continue;
		}
		/* prune if we are in SpectrumManagement mode, the IBSS is on a radar channel,
		 * and the locale requires 802.11h SpectrumManagement
		 */
		if ((WL11H_ENAB(wlc) != SPECT_MNGMT_OFF) &&
		    (bi->infra == 0) &&
		    (wlc_radar_channel(wlc, bi->channel) == TRUE) &&
		    (wlc_check_locale_flags(target_band->bandtype, WLC_DFS_TPC))) {
			WL_ASSOC(("JOIN: pruning IBSS \"%s\" since channel %d is a radar channel in"
			          " the Current locale\n",
			          ssidbuf, bi->channel));
			wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0, WLC_E_PRUNE_RADAR,
			              bi->channel, 0, 0);
			continue;
		}
		WL_ERROR(("wpa_auth is %d and wses enabled is %d\n", cfg->WPA_auth, WSEC_ENABLED(cfg->wsec)));
		/* prune join targets based on security settings */
		if (cfg->WPA_auth != WPA_AUTH_DISABLED && WSEC_ENABLED(cfg->wsec)) {
			struct rsn_parms *rsn;

			/* check authentication mode */
			if (IS_WPA_AUTH(cfg->WPA_auth) &&
				(bi->wpa.flags & RSN_FLAGS_SUPPORTED))
				rsn = &(bi->wpa);
			else
#ifdef BCMWPA2
			if (IS_WPA2_AUTH(cfg->WPA_auth) &&
				(bi->wpa2.flags & RSN_FLAGS_SUPPORTED))
				rsn = &(bi->wpa2);
			else
#endif /* BCMWPA2 */
			{
				WL_ASSOC(("JOIN: BSSID %s pruned for security reasons\n",
					bcm_ether_ntoa(&bi->BSSID, eabuf)));
				wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0, WLC_E_RSN_MISMATCH,
					0, 0, 0);
				continue;
			}

			/*
			 * check encryption settings:
			 *
			 * if (WSEC_AES_ENABLED(cfg->wsec)) {
			 *    if (((AP ucast is None) && (AP mcast is AES)) ||
			 *         (!(AP mcast is None) && (AP ucast includes AES))) {
			 *         keep();
			 *     }
			 * } else if (WSEC_TKIP_ENABLED(cfg->wsec)) {
			 *     if (((AP ucast is None) && (AP mcast is TKIP)) ||
			 *         (!(AP mcast is None) && (AP ucast includes TKIP))) {
			 *         keep();
			 *     }
			 * } else if (WSEC_WEP_ENABLED(cfg->wsec)) {
			 *     if ((AP ucast is None) && (AP mcast is WEP)) {
			 *         keep();
			 *     }
			 * }
			 * prune();
			 *
			 * TKIP countermeasures:
			 * - prune non-WPA
			 * - prune WPA with encryption <= TKIP
			 */

#define UCAST_NONE(prsn)	(((prsn)->ucount == 1) && \
			 ((prsn)->unicast[0] == WPA_CIPHER_NONE))
#define UCAST_AES(prsn)		(wlc_rsn_ucast_lookup(prsn, WPA_CIPHER_AES_CCM))
#define UCAST_TKIP(prsn)	(wlc_rsn_ucast_lookup(prsn, WPA_CIPHER_TKIP))

#define MCAST_NONE(prsn)	((prsn)->multicast == WPA_CIPHER_NONE)
#define MCAST_AES(prsn)		((prsn)->multicast == WPA_CIPHER_AES_CCM)
#define MCAST_TKIP(prsn)	((prsn)->multicast == WPA_CIPHER_TKIP)
#define MCAST_WEP(prsn)		((prsn)->multicast == WPA_CIPHER_WEP_40 || \
			 (prsn)->multicast == WPA_CIPHER_WEP_104)

			prune = TRUE;
			if (WSEC_AES_ENABLED(cfg->wsec)) {
				if ((UCAST_NONE(rsn) && MCAST_AES(rsn)) ||
					(MCAST_AES(rsn) && UCAST_AES(rsn)) ||
					((cfg->tk_cm_bt == 0) &&
					(!MCAST_NONE(rsn) && UCAST_AES(rsn))))
					prune = FALSE;
				else {
					WL_ASSOC(("JOIN: BSSID %s AES: no AES support or TKIP cm bt"
						" %d\n",
						bcm_ether_ntoa(&bi->BSSID, eabuf),
						cfg->tk_cm_bt));
					wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0,
						WLC_E_PRUNE_ENCR_MISMATCH, bi->channel, 0, 0);
				}
			} else if (WSEC_TKIP_ENABLED(cfg->wsec)) {
				if ((cfg->tk_cm_bt == 0) &&
					((UCAST_NONE(rsn) && MCAST_TKIP(rsn)) ||
					(!MCAST_NONE(rsn) && UCAST_TKIP(rsn))))
					prune = FALSE;
				else {
					WL_ASSOC(("JOIN: BSSID %s TKIP: no TKIP support or TKIP cm"
						" bt %d\n",
						bcm_ether_ntoa(&bi->BSSID, eabuf),
						cfg->tk_cm_bt));
					wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0,
						WLC_E_PRUNE_ENCR_MISMATCH, bi->channel, 0, 0);
				}
			} else
			{
				if ((cfg->tk_cm_bt == 0) &&
					(UCAST_NONE(rsn) && MCAST_WEP(rsn)))
					prune = FALSE;
				else {
					WL_ASSOC(("JOIN: BSSID %s no WEP support or TKIP cm %d\n",
						bcm_ether_ntoa(&bi->BSSID, eabuf),
						cfg->tk_cm_bt));
					wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0,
						WLC_E_PRUNE_ENCR_MISMATCH, bi->channel, 0, 0);
				}
			}

			if (prune) {
				WL_ASSOC(("JOIN: BSSID %s pruned for security reasons\n",
					bcm_ether_ntoa(&bi->BSSID, eabuf)));
				continue;
			}
		}

		/* find out if we have rates in common with the BSS/IBSS, bail out if no common
		 * rates
		 */

		/* copy rateset to preserve original (CCK only if LegacyB) */
		if (ISGPHY(target_band->pi) && (target_band->gmode == GMODE_LEGACY_B))
			wlc_rateset_filter(&bi->rateset, &rateset, FALSE, TRUE, RATE_MASK_FULL);
		else
			wlc_rateset_filter(&bi->rateset, &rateset, FALSE, FALSE, RATE_MASK_FULL);

		/* filter-out unsupported rates */
		wlc_sort_rates(&rateset, &target_band->hw_rateset);

		if (rateset.count == 0) {
			WL_ASSOC(("JOIN: BSSID %s pruned because we don't have any rates in common"
				" with the BSS\n",
				bcm_ether_ntoa(&bi->BSSID, eabuf)));
			wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0, WLC_E_PRUNE_NO_COMMON_RATES,
				0, 0, 0);
			continue;
		}

		/* skip any IBSS having basic rates which we do not support */
		if (bi->infra == 0 &&
		    !wlc_join_basicrate_supported(wlc, &bi->rateset, (bi->channel <=
		    WLC_MAX_2G_CHANNEL) ? WLC_BAND_2G : WLC_BAND_5G)) {
			WL_ASSOC(("JOIN: BSSID %s pruned because we do not support all Basic Rates"
				" of the BSS\n",
				bcm_ether_ntoa(&bi->BSSID, eabuf)));
			wlc_mac_event(wlc, WLC_E_PRUNE, &bi->BSSID, 0, WLC_E_PRUNE_BASIC_RATES, 0,
				0, 0);
			continue;
		}
		/* here if the BSS passed all the pruning tests, so break the loop */
		break;
	}

	if (wlc->join_targets_last > 0) {
		bi = wlc->join_targets.ptrs[--wlc->join_targets_last];
		WL_ASSOC(("JOIN: attempting BSSID: %s\n", bcm_ether_ntoa(&bi->BSSID, eabuf)));

		wlc_join_BSS(wlc, bi);
		return;
	}

	WL_ASSOC(("JOIN: no more join targets available\n"));

	/* handle reassociation failure */
	if (wlc->pub.associated) {
		wlc_mac_event(wlc, WLC_E_ROAM, NULL, WLC_E_STATUS_FAIL, 0, 0, 0, 0);
	} else {
		wlc_mac_event(wlc, WLC_E_SET_SSID, NULL, WLC_E_STATUS_FAIL, 0, 0, 0, 0);
	}

	wlc->block_datafifo &= ~DATA_BLOCK_JOIN;
}

static void
wlc_join_BSS(wlc_info_t *wlc, wlc_bss_info_t* bi)
{
	struct scb *scb;
	void *pkt;
	WL_ASSOC(("JOIN: wlc_BSS_join, selected BSSID: %s\n", bcm_ether_ntoa(&bi->BSSID,
		eabuf)));

	wlc->block_datafifo |= DATA_BLOCK_JOIN;


	/* Change the radio channel to match the target_bss */
	if (wlc->band->pi->radio_channel != bi->channel) {
		/* clear the quiet bit on the dest channel */
		CLRQUIET(wlc, bi->channel);
		wlc_suspend_mac_and_wait(wlc);
		wlc_set_channel(wlc, bi->channel);
		wlc_enable_mac(wlc);
	} else if (QUIET_CHANNEL(wlc, bi->channel)) {
		/* clear the quiet bit on our channel and un-quiet */
		CLRQUIET(wlc, bi->channel);
		wlc_mute(wlc, OFF);
	}

	/* select target bss info */
	bcopy((char*)bi, (char*)&wlc->target_bss, sizeof(wlc_bss_info_t));

	/* filter-out unsupported rates */
	wlc_sort_rates(&wlc->target_bss.rateset, &wlc->band->hw_rateset);

	/* Keep only CCK if gmode == GMODE_LEGACY_B */
	if (ISGPHY(wlc->band->pi) && wlc->band->gmode == GMODE_LEGACY_B)
		wlc_rateset_filter(&wlc->target_bss.rateset, &wlc->target_bss.rateset,
			FALSE, TRUE, RATE_MASK_FULL);

	wlc_rate_lookup_init(wlc, &wlc->target_bss.rateset);

	/* attempt to associate with the selected BSS */
	if (bi->infra == 0) {
		/* IBSS join */
		/*
		 * nothing more to do from a protocol point of view...
		 * join the BSS when the next beacon is received...
		 */
		WL_ASSOC(("JOIN: IBSS case, wait for a beacon\n"));
		/* enter the BSSID addr into the RXE match registers for STA w/o promisc beacon */
		wlc_set_addrmatch(wlc, RCM_BSSID_OFFSET, &bi->BSSID);
		wlc_assoc_change_state(wlc, AS_WAIT_RCV_BCN);
	} else {
		/* BSS join */

		/* running out of scbs is fatal here */
		if (!(scb = wlc_scblookup(wlc, &bi->BSSID))) {
			WL_ERROR(("wl%d: wlc_BSS_join: out of scbs\n", wlc->pub.unit));
			return;
		}

		/* the cap and additional IE are only in the bcn/prb response pkts, 
		 * when joining a bss parse the bcn_prb to check for these IE's.
		*/

		/* just created or assigned an SCB for the AP, flag as MYAP */
		ASSERT(!(SCB_ASSOCIATED(scb) && !(scb->flags & SCB_MYAP)));
		scb->flags |= SCB_MYAP;

		/* replace any old scb rateset with new target rateset */
		wlc_rateset_filter(&wlc->target_bss.rateset, &scb->rateset, FALSE, FALSE,
			RATE_MASK);
		wlc_scb_rate_init(wlc, scb);

		wl_del_timer(wlc->wl, wlc->assoc_timer);

		/* send authentication request */
		if (!(scb->state & (PENDING_AUTH | PENDING_ASSOC))) {
			int auth = wlc->cfg.auth;
			WL_ASSOC(("JOIN: BSS case, sending AUTH REQ alg=%d ...\n", auth));
			wlc_scb_setstatebit(scb, PENDING_AUTH | PENDING_ASSOC);

			wlc_assoc_change_state(wlc, AS_SENT_AUTH_1);
			pkt = wlc_sendauth(wlc, &scb->ea, &bi->BSSID, scb, auth, 1,
			                   DOT11_STATUS_SUCCESS, NULL, NULL,
			                   bi->capability & DOT11_CAP_SHORT);
			if (pkt != NULL)
				wlc_pkt_callback_register(wlc, wlc_auth_complete, NULL, pkt);
			else
				wl_add_timer(wlc->wl, wlc->assoc_timer, WECA_ASSOC_TIMEOUT + 10, 0);
		}
	}
}

static void
wlc_assoc_timeout(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;
	WL_TRACE(("wl%d: wlc_assoc_timer", wlc->pub.unit));

	if (!wlc->pub.up)
		return;

	if (DEVICEREMOVED(wlc)) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}

	if (wlc->assoc_state == AS_SENT_AUTH_1 ||
		wlc->assoc_state == AS_SENT_AUTH_3) {
		wlc_mac_event(wlc, WLC_E_AUTH, &wlc->target_bss.BSSID, WLC_E_STATUS_TIMEOUT, 0,
			wlc->cfg.auth, 0, 0);
	} else if (wlc->assoc_state == AS_SENT_ASSOC) {
		wlc_mac_event(wlc, (wlc->assoc_type == AS_ASSOCIATION) ? WLC_E_ASSOC :
			WLC_E_REASSOC,
			&wlc->target_bss.BSSID, WLC_E_STATUS_TIMEOUT, 0, 0, 0, 0);
	} else if (wlc->assoc_state == AS_IDLE && !wlc->pub.associated && wlc->sta_retry_time &&
	           BSSCFG_STA(wlc->bsscfg[0]) && wlc->bsscfg[0]->enable) {
		WL_ASSOC(("wl%d: Retrying failed association\n", wlc->pub.unit));
		wlc_bsscfg_up(wlc, wlc->bsscfg[0]);
	}

	return;
}

void
wlc_assoc_abort(wlc_info_t *wlc)
{
	struct scb *scb;

	if (wlc->assoc_state == AS_IDLE)
		return;

	if (wlc->assoc_type == AS_ASSOCIATION) {
		WL_ASSOC(("JOIN: Aborting association in process\n"));
	} else {
		WL_ASSOC(("ROAM: Aborting roam in process\n"));
	}

	if (SCAN_IN_PROGRESS(wlc)) {
		WL_ASSOC(("wlc_assoc_abort: aborting association scan in process\n"));
		wlc_scan_abort(wlc);
	}

	if (wlc->assoc_state == AS_SENT_AUTH_1 ||
	    wlc->assoc_state == AS_SENT_AUTH_3 ||
	    wlc->assoc_state == AS_SENT_ASSOC)
		wl_del_timer(wlc->wl, wlc->assoc_timer);

	if (wlc->assoc_type == AS_ASSOCIATION) {
		if (wlc->assoc_state == AS_SENT_AUTH_1 ||
		    wlc->assoc_state == AS_SENT_AUTH_3) {
			wlc_mac_event(wlc, WLC_E_AUTH, &wlc->target_bss.BSSID, WLC_E_STATUS_ABORT,
			              0, wlc->cfg.auth, 0, 0);
		} else if (wlc->assoc_state == AS_SENT_ASSOC) {
			wlc_mac_event(wlc, (wlc->assoc_type == AS_ASSOCIATION) ? WLC_E_ASSOC :
			              WLC_E_REASSOC, &wlc->target_bss.BSSID, WLC_E_STATUS_ABORT,
			              0, 0, 0, 0);
		}
		wlc_mac_event(wlc, WLC_E_SET_SSID, &wlc->target_bss.BSSID, WLC_E_STATUS_ABORT,
		              0, 0, 0, 0);
	}

	/* Clear pending bits */
	if ((scb = wlc_scbfind(wlc, &wlc->target_bss.BSSID)))
		wlc_scb_clearstatebit(scb, PENDING_AUTH | PENDING_ASSOC);

	wlc_bss_list_free(wlc, &wlc->join_targets);

	wlc->assoc_type = AS_IDLE;
	wlc_assoc_change_state(wlc, AS_IDLE);

	wlc_set_ps_ctrl(wlc);
	WL_ERROR(("%s: %d: Assoctype state is %d, scan->chan_idx is %d\n",  __FUNCTION__, __LINE__, wlc->assoc_type, wlc->scan.channel_idx ));
}

void
wlc_assoc_change_state(wlc_info_t *wlc, uint newstate)
{
	if (newstate > AS_WAIT_DISASSOC) {
		WL_ERROR(("wlc_assoc_change_state: wrong assoc state %d\n", newstate));
		ASSERT(0);
	}
	wlc->assoc_state = newstate;
}

static void
wlc_join_adopt_bss(wlc_info_t *wlc)
{
	wlcband_t *band;

	if (wlc->target_bss.infra) {
		WL_ASSOC(("JOIN: join BSS \"%s\" on channel %d\n", ssidbuf,
			wlc->target_bss.channel));
	} else {
		WL_ASSOC(("JOIN: join IBSS \"%s\" on channel %d\n", ssidbuf,
			wlc->target_bss.channel));
	}

	wlc->block_datafifo &= ~DATA_BLOCK_JOIN;

	if (wlc->assoc_type == AS_ASSOCIATION) {
		wlc->roam_reason = WLC_E_REASON_INITIAL_ASSOC;
		WL_ASSOC(("wl%d: ROAM: roam_reason cleared to 0x%x\n",
			wlc->pub.unit, WLC_E_REASON_INITIAL_ASSOC));
	}

	/* restore roam parameters to default */
	band = wlc->band;
	band->roam_trigger = band->roam_trigger_def;
	band->roam_delta = band->roam_delta_def;
	if (NBANDS(wlc) > 1) {
		band = &wlc->bandstate[OTHERBANDUNIT(wlc)];
		band->roam_trigger = band->roam_trigger_def;
		band->roam_delta = band->roam_delta_def;
	}

	/* suspend the MAC and join the BSS */
	wlc_suspend_mac_and_wait(wlc);
	wlc_BSSinit(wlc, &wlc->target_bss, wlc->bsscfg[0], WLC_BSS_JOIN);
	wlc_enable_mac(wlc);

	wlc->pub.associated = TRUE;

	wlc_link(wlc, TRUE, &wlc->BSSID, wlc->bsscfg[0]);


	if (wlc->target_bss.infra) {
		/* infrastructure join needs a BCN for TBTT coordination */
		WL_ASSOC(("JOIN: BSS case...waiting for the next beacon...\n"));
		wlc_assoc_change_state(wlc, AS_WAIT_RCV_BCN);
	}

	wlc_led_event(wlc->ledh);
}

/* return true if we support all of the target basic rates */
static bool
wlc_join_basicrate_supported(wlc_info_t *wlc, rateset_t *rs, int band)
{
	uint i;
	uint8 rate;
	bool only_cck = FALSE;
	wlcband_t* pband;

	/* determine if we need to do addtional gmode == GMODE_LEGACY_B checking of rates */
	if (band == WLC_BAND_2G) {
		if (ISGPHY(wlc->band->pi))
			pband = wlc->band;
		else if (wlc->bandstate[OTHERBANDUNIT(wlc)].pi->phy_type == PHY_TYPE_G)
			pband = &wlc->bandstate[OTHERBANDUNIT(wlc)];
		else
			pband = NULL;

		if (pband && pband->gmode == GMODE_LEGACY_B)
			only_cck = TRUE;
	}

	for (i = 0; i < rs->count; i++)
		if (rs->rates[i] & WLC_RATE_FLAG) {
			rate = rs->rates[i] & RATE_MASK;
			if (!wlc_valid_rate(wlc, rate, band, FALSE))
				return (FALSE);
			if (only_cck && !IS_CCK(rate))
				return (FALSE);
		}
	return (TRUE);
}


/* return true if the rateset contains an OFDM rate */
bool
wlc_rateset_isofdm(uint count, uint8 *rates)
{
	int i;

	for (i = count - 1; i >= 0; i--)
		if (IS_OFDM(rates[i]))
			return (TRUE);
	return (FALSE);
}

static uint
wlc_bcn_rate(wlc_info_t *wlc)
{
	uint8 bcn_rate;
	uint i;

	/* Use the lowest basic rate for beacons */
	bcn_rate = wlc->pub.current_bss.rateset.rates[0] & RATE_MASK;
	for (i = 0; i < wlc->pub.current_bss.rateset.count; i++) {
		if (wlc->pub.current_bss.rateset.rates[i] & WLC_RATE_FLAG) {
			bcn_rate = wlc->pub.current_bss.rateset.rates[i] & RATE_MASK;
			break;
		}
	}

	ASSERT(VALID_RATE_DBG(wlc, bcn_rate));
	return (bcn_rate);
}

static void
wlc_BSSinit(wlc_info_t *wlc, wlc_bss_info_t *bi, wlc_bsscfg_t *cfg, int create_bss)
{
	d11regs_t *regs;
	uint16 bcn[BCN_TMPL_LEN/2];
	int len = BCN_TMPL_LEN;
	uint bcnint, bcnint_us, val, cfpp = 0;
	uint16 bcntsfoff, tim_offset, phyctl;
	uint i;


	WL_TRACE(("wl%d: %s:\n", wlc->pub.unit, __FUNCTION__));

	regs = wlc->regs;

	/* update our infrastructure mode */
	wlc->pub.BSS = (bi->infra == 1);

	/* clear out bookkeeping */
	/* Only clear when STA (re)joins a BSS */
	if (BSSCFG_STA(cfg)) {
		wlc->time_since_bcn = 0;
		wlc->bcns_lost = FALSE;
		wlc->tsf_l = wlc->tsf_h = 0;
	}
	if (!wlc->aps_associated || !wlc->pub.BSS) {
		wlc->nonerp_assoc = FALSE;
		wlc->ofdm_assoc = FALSE;
		wlc->nonabcap_assoc = FALSE;
		wlc->longslot_assoc = FALSE;
		wlc->longpre_assoc = FALSE;
		wlc->nonerp_ibss_timeout = 0;
		wlc->nonerp_ovlp_timeout = 0;
		wlc->ofdm_ibss_timeout = 0;
		wlc->ofdm_ovlp_timeout = 0;
		wlc->longpre_detect_timeout = 0;
		wlc->barker_detect_timeout = 0;
		wlc->protection_detect_timeout = 0;
		wlc->n_protection_ibss_detect_timeout = 0;

		/* reset BSS local power limits */
		wlc->pub.txpwr_local_max = WLC_TXPWR_MAX;
		wlc->pub.txpwr_local_constraint = 0;
	}

	/* Reset the frequency tracking b/w to default values */
	wlc_freqtrack_reset(wlc);

	/* update current BSS information */
	if (wlc->pub.current_bss.bcn_prb)
		MFREE(wlc->pub.osh, wlc->pub.current_bss.bcn_prb,
			wlc->pub.current_bss.bcn_prb_len);
	bcopy((char*)bi, (char*)&wlc->pub.current_bss, sizeof(wlc_bss_info_t));
	if (bi->bcn_prb) {
		/* need to copy bcn_prb, too */
		wlc->pub.current_bss.bcn_prb = (struct dot11_bcn_prb *) MALLOC(wlc->pub.osh,
			bi->bcn_prb_len);
		if (wlc->pub.current_bss.bcn_prb) {
			bcopy((char*)bi->bcn_prb, (char*)wlc->pub.current_bss.bcn_prb,
				bi->bcn_prb_len);
		} else {
			WL_ERROR(("wl%d: wlc_BSSinit: out of memory, malloced %d bytes\n",
				wlc->pub.unit, MALLOCED(wlc->pub.osh)));
			wlc->pub.current_bss.bcn_prb_len = 0;
		}
	}

	/* Validate current_bss.channel in case the locale has changed */
	if (!VALID_CHANNEL_DB(wlc, wlc->pub.current_bss.channel)) {
		uint8 chan;
		ASSERT(!wlc->pub.associated);
		chan = wlc_next_channel(wlc, 0, CHAN_TYPE_ANY, 0);
		ASSERT(chan != INVCHANNEL);
		wlc->pub.current_bss.channel = chan;
	}

	/*
	 * Set the radio channel to current_bss.channel.
	 * Must do this before the rate calculations below.
	 */
	if (wlc->band->pi->radio_channel != wlc->pub.current_bss.channel) {
		wlc_set_channel(wlc, wlc->pub.current_bss.channel);
	}

	/* filter-out unsupported rates */
	wlc_sort_rates(&wlc->pub.current_bss.rateset, &wlc->band->hw_rateset);

	/* Keep only CCK if gmode == GMODE_LEGACY_B */
	if (ISGPHY(wlc->band->pi) && (wlc->band->gmode == GMODE_LEGACY_B))
		wlc_rateset_filter(&wlc->pub.current_bss.rateset, &wlc->pub.current_bss.rateset,
			FALSE, TRUE, RATE_MASK_FULL);

	/* if empty or no basic rates - use driver default rateset */
	/* ??? should fixup all WLC_RATE_FLAG reference to remove this requirement */
	val = 0;
	for (i = 0; i < wlc->pub.current_bss.rateset.count; i++)
		if (wlc->pub.current_bss.rateset.rates[i] & WLC_RATE_FLAG)
			val = 1;

	if (val == 0) {
		wlc_default_rateset(wlc, &wlc->pub.current_bss.rateset);
		/* Keep only CCK if gmode == GMODE_LEGACY_B */
		if (ISGPHY(wlc->band->pi) && wlc->band->gmode == GMODE_LEGACY_B)
			wlc_rateset_filter(&wlc->pub.current_bss.rateset,
				&wlc->pub.current_bss.rateset,
				FALSE, TRUE, RATE_MASK_FULL);
	}

	wlc_rate_lookup_init(wlc, &wlc->pub.current_bss.rateset);

	/* Validate beacon interval, we should never create a 0 beacon period BSS/IBSS */
	if (create_bss) {
		ASSERT(wlc->pub.current_bss.beacon_period);
	}
	/* don't adopt a illegal beacon_period IBSS */
	if (wlc->pub.current_bss.beacon_period == 0)
		wlc->pub.current_bss.beacon_period = BEACON_INTERVAL_DEFAULT;

	/* initialize global BSSID information */
	wlc->BSSID = wlc->pub.current_bss.BSSID;
	wlc->prev_BSSID = wlc->pub.current_bss.BSSID;
	wlc->announce_BSSID = wlc->pub.BSS ? wlc->pub.cur_etheraddr : bi->BSSID;

	/* clear all old non-permanent scbs for IBSS */
	if (!wlc->pub.BSS)
		wlc_scbclear(wlc, FALSE);

	/* write the BSSID (to RXE) */
	wlc_set_bssid(wlc);

	/* write to SHM to adjust the ucode ACK/CTS rate tables... */
	wlc_set_ratetable(wlc);

	/* the world is new again, so is our reported rate */
	if (!wlc->pub.associated)
		wlc_reprate_init(wlc);

	/* update the CCK preamble mode for STAs joining an AP */
	if (BSSCFG_STA(cfg) && wlc->pub.BSS)
		wlc->shortpreamble = ((wlc->pub.current_bss.capability & DOT11_CAP_SHORT) != 0);

	if (wlc->band->gmode) {
		/* Update the gmode protection state */
		if (wlc->g_protection_override == WLC_PROTECTION_AUTO) {
			wlc->g_protection = FALSE;
		} else {
			wlc->g_protection = (wlc->g_protection_override == WLC_PROTECTION_ON);
		}
		wlc->erp_ie_use_protection = wlc->g_protection;

		/* update shortslot state now before founding or joining a BSS */
		if (wlc->shortslot_override == WLC_SHORTSLOT_AUTO) {
			if (!create_bss) /* STA JOIN: adopt the 11g shortslot mode of the network */
				wlc->shortslot = (wlc->pub.current_bss.capability &
					DOT11_CAP_SHORTSLOT) != 0;
			else	/* IBSS START: depending on user setting in advanced tab. */
				wlc->shortslot = (wlc->default_bss.capability &
					DOT11_CAP_SHORTSLOT) != 0;
		} else {
			/* Switch to long or short slot mode based on the override */
			wlc->shortslot = (wlc->shortslot_override == WLC_SHORTSLOT_ON);
		}
		wlc_update_slot_timing(wlc, wlc->shortslot);

	}
	if (!wlc->pub.BSS) {
		/* Create a Beacon and Probe Response template */
		uint bcn_rate, phy_rate;

		/* update our capability */
		wlc->pub.current_bss.capability &= ~DOT11_CAP_SHORT;
		/* use IBSSGmode setting for bcn and prb cap */
		if (wlc->default_bss.capability & DOT11_CAP_SHORT)
			wlc->pub.current_bss.capability |= DOT11_CAP_SHORT;

		/* update the capability based on current shortslot mode */
		wlc->pub.current_bss.capability &= ~DOT11_CAP_SHORTSLOT;
		if (wlc->shortslot && wlc->band->gmode)
			wlc->pub.current_bss.capability |= DOT11_CAP_SHORTSLOT;

		/* default ERP bits */
		wlc->erp_ie_nonerp = FALSE;
		wlc->barker_preamble = WLC_BARKER_SHORT_ALLOWED;

		/* Use the lowest basic rate for beacons */
		bcn_rate = wlc_bcn_rate(wlc);

		/* create a beacon template */
		len = BCN_TMPL_LEN;
		wlc_bcn_prb_template(wlc, FC_BEACON, bcn_rate, cfg, bcn, &len);

		/* set a default bcn_prb for IBSS */
		if (BSSCFG_STA(cfg) && wlc->pub.current_bss.bcn_prb == NULL) {
			uint hdr_len = D11_PHY_HDR_LEN + DOT11_MAC_HDR_LEN;
			wlc->pub.current_bss.bcn_prb = (struct dot11_bcn_prb *) MALLOC(wlc->pub.osh,
				len - hdr_len);
			if (wlc->pub.current_bss.bcn_prb) {
				bcopy((char*)bcn + hdr_len, (char*)wlc->pub.current_bss.bcn_prb, len
					- hdr_len);
				wlc->pub.current_bss.bcn_prb_len = len - hdr_len;
			} else {
				WL_ERROR(("wl%d: wlc_BSSinit: out of memory, malloced %d bytes\n",
					wlc->pub.unit, MALLOCED(wlc->pub.osh)));
				wlc->pub.current_bss.bcn_prb_len = 0;
			}
		}

		/* write template to hardware if this is the designated beaconing bsscfg */
		if (cfg == wlc->bsscfg[wlc->bsscfg_bcn_idx]) {
			/* copy beacon into both template 0 and 1, since we don't know which the
			 * ucode will use next
			 */
			wlc_write_template_ram(wlc, T_BCN0_TPL_BASE, (len + 3) & ~3, bcn);
			wlc_write_template_ram(wlc, T_BCN1_TPL_BASE, (len + 3) & ~3, bcn);
			/* write beacon length to SCR */
			ASSERT(len < 65536);
			wlc_write_shm(wlc, M_BCN0_FRM_BYTESZ, (uint16)len);
			wlc_write_shm(wlc, M_BCN1_FRM_BYTESZ, (uint16)len);

			/* mark beacon 0 valid */
			OR_REG(&regs->maccommand, MCMD_BCN0VLD);

			/* write beacon timestamp tx tsf offset, long preamble */
			phy_rate = PLCP_PHYRATE(bcn, IS_OFDM(bcn_rate));
			bcntsfoff = wlc_compute_bcntsfoff(wlc, IS_OFDM(bcn_rate), phy_rate, 0,
				TRUE);
			wlc_write_shm(wlc, M_BCN_TXTSF_OFFSET, bcntsfoff);

			/* Write the phycontrol OFDM bit for the beacon */
			phyctl = wlc_read_shm(wlc, M_BCN_PCTLWD);
			phyctl &= ~PHY_TXC_OFDM;

			if (IS_OFDM(bcn_rate)) phyctl |= PHY_TXC_OFDM;

			/* If ever beacons use short headers or phy pwr override,
			 * add the proper bits here.
			 */
			wlc_write_shm(wlc, M_BCN_PCTLWD, phyctl);
			WL_GMODE(("wl%d: wlc_BSSinit: Beacon shm phyctl 0x%x\n", wlc->pub.unit,
				phyctl));

			if (wlc->pub.BSS) {
				uint8* cp;

				/* find the TIM elt offset in the bcn template */
				cp = (uint8*)bcn + D11_PHY_HDR_LEN + DOT11_MAC_HDR_LEN +
					DOT11_BCN_PRB_LEN;
				cp = (uint8*)bcm_parse_tlvs(cp, len, DOT11_MNG_TIM_ID);
				ASSERT(cp); /* catch preparation bug, the bcn should have a TIM */
				tim_offset = (uint16)(cp - (uint8*)bcn);

				wlc_write_shm(wlc, M_TIMBPOS_INBEACON, tim_offset);

				/* set DTIM count to 0 in SCR */
				W_REG(&regs->objaddr, OBJADDR_SCR_SEL + S_DOT11_DTIMCOUNT);
				W_REG(&regs->objdata, 0);

				/* set DTIM period */
				/* using default_bss works for any AP mode (APONLY) */
				wlc_write_shm(wlc, M_DOT11_DTIMPERIOD,
					wlc->default_bss.dtim_period);
			}
		}

		wlc_update_probe_resp(wlc, cfg, FALSE);
	}

	if (BSSCFG_STA(cfg)) {
		if (!wlc->pub.BSS) {
			/* program the ATIM window */
			if (wlc->pub.current_bss.atim_window) {
				/* note that CFP is present */
				cfpp = CFPREP_CFPP;
			}
			/* set CFP duration */
			W_REG(&regs->tsf_cfpmaxdur32,
			      (uint32)wlc->pub.current_bss.atim_window);

			/* set initial RSSI value */
			wlc->rssi = WLC_RSSI_EXCELLENT;
		} else {
			/* set initial RSSI value */
			wlc->rssi = WLC_RSSI_EXCELLENT;
		}

		wlc_phy_BSSinit(wlc->band->pi, wlc->rssi);
	}

	/* First AP starting a BSS, or STA starting an IBSS */
	if (create_bss && !wlc->aps_associated) {

		/* Program the next TBTT and adjust TSF timer.
		 * If we are not starting the BSS, then the TSF timer adjust is done
		 * at beacon reception time.
		 */
		/* Using upstream beacon period will make 11H easier */
		bcnint = wlc->pub.current_bss.beacon_period;

		bcnint_us = ((uint32)bcnint << 10);

		/* reset the TSF timer */
		W_REG(&regs->tsf_timerlow, 0);
		W_REG(&regs->tsf_timerhigh, 0);

		/* write the beacon interval (to TSF) */
		W_REG(&regs->tsf_cfprep, (bcnint_us << CFPREP_CBI_SHIFT) | cfpp);

		/* CFP start is the next beacon interval after timestamp */
		W_REG(&regs->tsf_cfpstart, bcnint_us);
	}

	/* set mode bits in MACCONTROL */
	val = R_REG(&regs->maccontrol);

	/* infrastructure vs ad-hoc, STA vs AP */
	val &= ~(MCTL_INFRA | MCTL_AP);
	if (wlc->pub.BSS) {
		val |= MCTL_INFRA;
		/* Set AP if this BSS is AP, or an AP BSS is up
		 * As PMQ entries are currently used only by AP for detecting PM Mode transitions
		 * for STA, instruct ucode not to fill up entries in PMQ
		 * CAUTION: For STA, this bit will need to be cleared if we ever support IBSS using
		 * PMQ.
		 */
		val |= MCTL_DISCARD_PMQ;
	}
	/* if this is the bsscfg for ucode probe responses, indicate whether this is a closed
	 * network
	 */
	if (cfg == wlc->bsscfg[wlc->bsscfg_prb_idx]) {
		if (cfg->closednet)
			val |= MCTL_CLOSED_NETWORK;
		else
			val &= ~MCTL_CLOSED_NETWORK;
	}

	wlc_mctrl(wlc, ~0, val);

	/* Entering IBSS: make sure bcn promisc is on */
	if (!wlc->pub.BSS) {
		wlc->bcnmisc_ibss = TRUE;
		wlc_mac_bcn_promisc(wlc);
	}

	/* update bcast/mcast rateset */
	wlc_rateset_filter(&wlc->pub.current_bss.rateset,
		&wlc->band->bcmc_scb->rateset,
		FALSE, FALSE, RATE_MASK);

	/* update PRETBTT */
	if (wlc->pub.BSS)
		phyctl = (ISAPHY(wlc->band->pi)) ? PRETBTT_APHY_US : PRETBTT_BPHY_US;
	else
		phyctl = 2;
	W_REG(&wlc->regs->tsf_cfppretbtt, phyctl);	/* IHR */
	wlc_write_shm(wlc, M_PRETBTT, phyctl);		/* SHM */

}

static void
wlc_tsf_adopt_bcn(wlc_info_t *wlc, struct dot11_bcn_prb *bcn)
{
	d11regs_t *regs = wlc->regs;
	uint32 bi;
	uint32 tsf_l, tsf_h;

	tsf_l = ltoh32(bcn->timestamp[0]);
	tsf_h = ltoh32(bcn->timestamp[1]);

	bi = ltoh16(bcn->beacon_interval);
	if (bi == 0)	/* bad AP, change to a valid value */
		bi = 1024;

	/* write beacon's timestamp directly to TSF timer */

	/* Hold TBTTs */
	wlc_mctrl(wlc, MCTL_TBTT_HOLD, MCTL_TBTT_HOLD);

	/* low first, then high so the 64bit write is atomic */
	W_REG(&regs->tsf_timerlow, tsf_l);
	W_REG(&regs->tsf_timerhigh, tsf_h);

	/* write the beacon interval (to TSF) */
	W_REG(&regs->tsf_cfprep, ((uint32)bi << NBITS(uint16)));

	/* update the TBTT in the core */
	wlc_tbtt_adopt(wlc, bcn);

	/* Release hold on TBTTs */
	wlc_mctrl(wlc, MCTL_TBTT_HOLD, 0);
}

static void
wlc_join_complete(wlc_info_t *wlc, struct dot11_bcn_prb *bcn, int bcn_len)
{
	uint8* tlvs;
	int tlvs_len;
	bcm_tlv_t *tim;
	struct dot11_bcn_prb *merged;
	int merged_len;


	wlc_tsf_adopt_bcn(wlc, bcn);
	tlvs = (uint8*)bcn + DOT11_BCN_PRB_LEN;
	tlvs_len = bcn_len - DOT11_BCN_PRB_LEN;

	if (wlc->target_bss.infra == 1) {
		/* adopt DTIM period for PS-mode support */
		tim = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_TIM_ID);

		if (tim) {
			wlc->pub.current_bss.dtim_period = tim->data[DOT11_MNG_TIM_DTIM_PERIOD];
			wlc_write_shm(wlc, M_DOT11_DTIMPERIOD, wlc->pub.current_bss.dtim_period);
			wlc->dtim_programmed = TRUE;
		} else {
			/* illed AP, prevent going to power-save mode */
			wlc->PM_override = TRUE;
		}

	} else {
		/* adopt the IBSS parameters */
		wlc_join_adopt_bss(wlc);
		wlc->time_since_bcn = 1;
		wlc->bcns_lost = TRUE;
	}

	/* Merge the current saved probe response IEs with the beacon's in case the
	 * beacon is missing some info.
	 */
	wlc_merge_bcn_prb(wlc, wlc->pub.current_bss.bcn_prb, wlc->pub.current_bss.bcn_prb_len,
	                  bcn, bcn_len, &merged, &merged_len);

	/* save bcn's fixed and tagged parameters in current_bss */
	if (merged != NULL) {
		if (wlc->pub.current_bss.bcn_prb)
			MFREE(wlc->pub.osh, wlc->pub.current_bss.bcn_prb,
				wlc->pub.current_bss.bcn_prb_len);
		wlc->pub.current_bss.bcn_prb = merged;
		wlc->pub.current_bss.bcn_prb_len = (uint16)merged_len;
	} else {
		WL_ERROR(("wl%d: wlc_join_complete: out of memory, malloced %d bytes\n",
			wlc->pub.unit, MALLOCED(wlc->pub.osh)));
	}



	/* 802.11 PM can be entered now that we have synchronized with
	 * the BSS's TSF and DTIM count.
	 */
	if (PS_ALLOWED(wlc))
		wlc_set_pmstate(wlc, TRUE);

	/* send event to indicate final step in joining BSS */
	wlc_mac_event(wlc, WLC_E_JOIN, &wlc->BSSID, WLC_E_STATUS_SUCCESS, 0, 0, 0, 0);
}

static void
wlc_merge_bcn_prb(wlc_info_t *wlc,
	struct dot11_bcn_prb *p1, int p1_len,
	struct dot11_bcn_prb *p2, int p2_len,
	struct dot11_bcn_prb **merged, int *merged_len)
{
	uint8 *tlvs1, *tlvs2;
	int tlvs1_len, tlvs2_len;

	*merged = NULL;
	*merged_len = 0;

	if (p1) {
		ASSERT(p1_len >= DOT11_BCN_PRB_LEN);
		/* fixup for non-assert builds */
		if (p1_len < DOT11_BCN_PRB_LEN)
			p1 = NULL;
	}
	if (p2) {
		ASSERT(p2_len >= DOT11_BCN_PRB_LEN);
		/* fixup for non-assert builds */
		if (p2_len < DOT11_BCN_PRB_LEN)
			p2 = NULL;
	}

	/* check for simple cases of one or the other of the source packets being null */
	if (p1 == NULL && p2 == NULL) {
		return;
	} else if (p2 == NULL) {
		*merged = (struct dot11_bcn_prb *) MALLOC(wlc->pub.osh, p1_len);
		if (*merged != NULL) {
			*merged_len = p1_len;
			bcopy((char*)p1, (char*)*merged, p1_len);
		}
		return;
	} else if (p1 == NULL) {
		*merged = (struct dot11_bcn_prb *) MALLOC(wlc->pub.osh, p2_len);
		if (*merged != NULL) {
			*merged_len = p2_len;
			bcopy((char*)p2, (char*)*merged, p2_len);
		}
		return;
	}

	/* both source packets are present, so do the merge work */
	tlvs1 = (uint8*)p1 + DOT11_BCN_PRB_LEN;
	tlvs1_len = p1_len - DOT11_BCN_PRB_LEN;

	tlvs2 = (uint8*)p2 + DOT11_BCN_PRB_LEN;
	tlvs2_len = p2_len - DOT11_BCN_PRB_LEN;

	/* allocate a buffer big enough for the merged ies */
	*merged_len = DOT11_BCN_PRB_LEN + wlc_merged_ie_len(wlc, tlvs1, tlvs1_len, tlvs2,
		tlvs2_len);
	*merged = (struct dot11_bcn_prb *) MALLOC(wlc->pub.osh, *merged_len);
	if (*merged == NULL) {
		*merged_len = 0;
		return;
	}

	/* copy the fixed portion of the second packet so the latest TSF, cap, etc is kept */
	bcopy(p2, *merged, DOT11_BCN_PRB_LEN);

	/* merge the ies from both packets */
	wlc_merge_ies(tlvs1, tlvs1_len, tlvs2, tlvs2_len, (uint8*)*merged + DOT11_BCN_PRB_LEN);
}

static int
wlc_merged_ie_len(wlc_info_t *wlc, uint8 *tlvs1, int tlvs1_len, uint8 *tlvs2, int tlvs2_len)
{
	bcm_tlv_t *tlv1 = (bcm_tlv_t*)tlvs1;
	bcm_tlv_t *tlv2 = (bcm_tlv_t*)tlvs2;
	int total;
	int len;

	/* treat an empty list or malformed list as empty */
	if (!bcm_valid_tlv(tlv1, tlvs1_len)) {
		tlv1 = NULL;
		tlvs1 = NULL;
	}
	if (!bcm_valid_tlv(tlv2, tlvs2_len)) {
		tlv2 = NULL;
		tlvs2 = NULL;
	}

	total = 0;

	len = tlvs1_len;
	while (tlv1) {
		total += TLV_HDR_LEN + tlv1->len;
		tlv1 = bcm_next_tlv(tlv1, &len);
	}

	len = tlvs2_len;
	while (tlv2) {
		if (!wlc_find_ie_match(tlv2, (bcm_tlv_t*)tlvs1, tlvs1_len))
			total += TLV_HDR_LEN + tlv2->len;

		tlv2 = bcm_next_tlv(tlv2, &len);
	}

	return total;
}

static bool
wlc_find_ie_match(bcm_tlv_t *ie, bcm_tlv_t *ies, int len)
{
	uint8 ie_len;

	ie_len = ie->len;

	while (ies) {
		if (ie_len == ies->len && !bcmp(ie, ies, ie_len)) {
			return TRUE;
		}
		ies = bcm_next_tlv(ies, &len);
	}

	return FALSE;
}

static void
wlc_merge_ies(uint8 *tlvs1, int tlvs1_len, uint8 *tlvs2, int tlvs2_len, uint8* merge)
{
	bcm_tlv_t *tlv1 = (bcm_tlv_t*)tlvs1;
	bcm_tlv_t *tlv2 = (bcm_tlv_t*)tlvs2;
	int len;

	/* treat an empty list or malformed list as empty */
	if (!bcm_valid_tlv(tlv1, tlvs1_len)) {
		tlv1 = NULL;
		tlvs1 = NULL;
	}
	if (!bcm_valid_tlv(tlv2, tlvs2_len)) {
		tlv2 = NULL;
		tlvs2 = NULL;
	}

	/* copy in the ies from the second set */
	len = tlvs2_len;
	while (tlv2) {
		bcopy(tlv2, merge, TLV_HDR_LEN + tlv2->len);
		merge += TLV_HDR_LEN + tlv2->len;

		tlv2 = bcm_next_tlv(tlv2, &len);
	}

	/* merge in the ies from the first set */
	len = tlvs1_len;
	while (tlv1) {
		if (!wlc_find_ie_match(tlv1, (bcm_tlv_t*)tlvs2, tlvs2_len)) {
			bcopy(tlv1, merge, TLV_HDR_LEN + tlv1->len);
			merge += TLV_HDR_LEN + tlv1->len;
		}
		tlv1 = bcm_next_tlv(tlv1, &len);
	}
}

/* low-level band switch utility routine */
void
WLBANDINITFN(wlc_setxband)(wlc_info_t *wlc, uint bandunit)
{
	uint coreunit;

	WL_TRACE(("wl%d: wlc_setxband: bandunit %d\n", wlc->pub.unit, bandunit));

	wlc->band = &wlc->bandstate[bandunit];

	/* switch cores if necessary */
	coreunit = BAND2CORE(wlc, bandunit);
	if (wlc->core != &wlc->corestate[coreunit]) {
		wlc->core = &wlc->corestate[coreunit];
		if (wlc->sbclk)
			sb_coreflags(wlc->pub.sbh, SBTML_GM, 0);
		wlc->regs = (d11regs_t*)sb_setcore(wlc->pub.sbh, SB_D11, coreunit);
	}

	/* set gmode core flag */
	if (wlc->sbclk)
		sb_coreflags(wlc->pub.sbh, SBTML_GM, ((bandunit == 0)? SBTML_GM : 0));
}

/* band-specific init */
static void
WLBANDINITFN(wlc_bsinit)(wlc_info_t *wlc)
{
	uint16 v;

	WL_TRACE(("wl%d: wlc_bsinit: bandunit %d\n", wlc->pub.unit, wlc->band->bandunit));

	/* sanity check */
	ASSERT(PHY_TYPE(R_REG(&wlc->regs->phyversion)) == wlc->band->pi->phy_type);

	/* init microcode host flags */
	wlc_write_mhf(wlc, wlc->band->mhf);

	/* do band-specific ucode IHR, SHM, and SCR inits */
	if (D11REV_GE(wlc->pub.corerev, 11)) {
		WL_ERROR(("wl%d: corerev >= 11 \n", wlc->pub.unit));
	} else if (D11REV_GE(wlc->pub.corerev, 5)) {
		if (ISAPHY(wlc->band->pi)) {
			if (sb_coreflagshi(wlc->pub.sbh, 0, 0) & SBTMH_GA)
				wlc_write_inits(wlc, d11a0g1bsinitvals5);
			else
				wlc_write_inits(wlc, d11a0g0bsinitvals5);
		} else
			wlc_write_inits(wlc, d11b0g0bsinitvals5);
	} else if (D11REV_IS(wlc->pub.corerev, 4)) {
		if (ISAPHY(wlc->band->pi))
			wlc_write_inits(wlc, d11a0g0bsinitvals4);
		else
			wlc_write_inits(wlc, d11b0g0bsinitvals4);
	} else
		WL_ERROR(("wl%d: %s: corerev %d is invalid", wlc->pub.unit,
		          __FUNCTION__, wlc->pub.corerev));

	wlc_phy_analogon((void *)wlc->regs);

	/* radio on */
	wlc_switch_radio(wlc->band->pi, ON);

	/* phy init */
	wlc_phy_init(wlc->band->pi);

	/* apply tx antenna override */
	wlc_set_txant(wlc, wlc->txant);

	/* cwmin is band-specific, update hardware with value for current band */
	wlc_set_cwmin(wlc, wlc->band->CWmin);
	wlc_set_cwmax(wlc, wlc->band->CWmax);

	wlc_update_slot_timing(wlc, ISAPHY(wlc->band->pi) ? TRUE : wlc->shortslot);

	/* write ucode ACK/CTS rate table */
	wlc_set_ratetable(wlc);

	/* write phytype and phyvers */
	wlc_write_shm(wlc, M_PHYTYPE, (uint16)wlc->band->pi->phy_type);
	wlc_write_shm(wlc, M_PHYVER, (uint16)wlc->band->pi->phy_rev);

	/* set or clear maccontrol.infra */
	if (wlc->pub.associated && (wlc->pub.current_bss.channel == wlc->band->pi->radio_channel)) {
		if (wlc->pub.current_bss.infra) {
			wlc->bcnmisc_ibss = FALSE;
			wlc_mctrl(wlc, MCTL_INFRA, MCTL_INFRA);
		} else {
			wlc->bcnmisc_ibss = TRUE;
			wlc_mctrl(wlc, MCTL_INFRA, 0);
		}
	} else {
		wlc->bcnmisc_ibss = FALSE;
		wlc_mctrl(wlc, MCTL_INFRA, MCTL_INFRA);
	}

	/* update the various promisc bits */
	wlc_mac_bcn_promisc(wlc);
	wlc_mac_promisc(wlc);

	/* set or clear the RXE BSSID */
	if (wlc->pub.associated && (wlc->pub.current_bss.channel == wlc->band->pi->radio_channel)) {
		wlc_set_bssid(wlc);
	} else {
		wlc_clear_bssid(wlc);
	}

	if (D11REV_LT(wlc->pub.corerev, 5) && ISAPHY(wlc->band->pi))
		v = 0x1d;
	else
		v = 0x1e;
	wlc_write_shm(wlc, M_SEC_DEFIVLOC, v);

	/* update SYNTHPU_DLY */
	if (wlc->pub.BSS)
		v = (ISAPHY(wlc->band->pi)) ? SYNTHPU_DLY_APHY_US : SYNTHPU_DLY_BPHY_US;
	else
		v = 500;

	if ((wlc->band->pi->radioid == BCM2050_ID) &&
	    (wlc->band->pi->radiorev == 8)) {
		if (v < 2400)
			v = 2400;
	}

	wlc_write_shm(wlc, M_SYNTHPU_DLY, v);	/* SHM */

	/* If we just switched to the B/G band and we had enabled wideband frequency
	 * tracking OR if wideband frequency tracking is always on, then ensure that
	 * the PHY registers have the right settings
	 */
	if ((wlc->freqtrack || (wlc->freqtrack_override == FREQTRACK_ON)) &&
	    ISGPHY(wlc->band->pi)) {
		wlc_phy_freqtrack_start(wlc->band->pi);
	}
}

/* switch to new band but leave it inactive */
static uint32
WLBANDINITFN(wlc_setband_inact)(wlc_info_t *wlc, uint bandunit)
{
	uint32 macintmask;

	ASSERT(bandunit != wlc->band->bandunit);
	ASSERT(sb_iscoreup(wlc->pub.sbh));
	ASSERT((R_REG(&wlc->regs->maccontrol) & MCTL_EN_MAC) == 0);

	/* disable interrupts */
	macintmask = wl_intrsoff(wlc->wl);

	/* radio off */
	wlc_switch_radio(wlc->band->pi, OFF);

	/* clear activity LED */
	if (wlc->ncores > 1)
		wlc_led_activityset(wlc->ledh, OFF);

	/* clear gmode bit, put phy into reset */
	sb_coreflags(wlc->pub.sbh, (SBTML_GM | SBTML_PR | SBTML_FGC), (SBTML_PR | SBTML_FGC));
	OSL_DELAY(1);
	sb_coreflags(wlc->pub.sbh, (SBTML_PR | SBTML_FGC), SBTML_PR);
	OSL_DELAY(1);

	wlc_setxband(wlc, bandunit);

	return (macintmask);
}

/* switch to and initialize new band */
void
WLBANDINITFN(wlc_setband)(wlc_info_t *wlc, uint bandunit)
{
	uint32 macintmask;
	uint phy_type;

	ASSERT(NBANDS(wlc) > 1);
	ASSERT(!wlc->bandlocked);
	ASSERT(bandunit != wlc->band->bandunit);

	WL_DUAL(("DUALBAND: wlc_setband: bandunit %d\n", bandunit));

	phy_type = wlc->band->pi->phy_type;

	/* Save tsf_timerlow of the old core if this results in a coreswitch with ncores > 1 */
	if (wlc->ncores > 1)
		wlc->core->tsf_timerlow = R_REG(&wlc->regs->tsf_timerlow);

	macintmask = wlc_setband_inact(wlc, bandunit);

	if (!wlc->pub.up)
		return;

	ASSERT(sb_iscoreup(wlc->pub.sbh));

	/* FREF: switch the pll frequency reference */
	if (D11REV_GE(wlc->pub.corerev, 5)) {
		uint cnt = 0;

		ASSERT(sb_coreflagshi(wlc->pub.sbh, 0, 0) & SBTMH_FCA);
		while (cnt++ < 5) {
			sb_coreflags(wlc->pub.sbh, SBTML_FS,
				((PHYTYPE_IS(phy_type, PHY_TYPE_A)) ? 0 : SBTML_FS));
			OSL_DELAY(2 * MIN_SLOW_CLK);	/* 2 slow clock tick (worst case 32khz) */
			SPINWAIT(!(sb_coreflagshi(wlc->pub.sbh, 0, 0) & SBTMH_FCA),
			         2 * FREF_DELAY);

			if (D11REV_GE(wlc->pub.corerev, 8) ||
			    (sb_coreflagshi(wlc->pub.sbh, 0, 0) & SBTMH_FCA))
				break;

			sb_coreflags(wlc->pub.sbh, SBTML_FS,
				(PHYTYPE_IS(phy_type, PHY_TYPE_A) ? SBTML_FS : 0));
			OSL_DELAY(FREF_DELAY);
		}
		ASSERT(sb_coreflagshi(wlc->pub.sbh, 0, 0) & SBTMH_FCA);
	}

	/* take phy out of reset */
	sb_coreflags(wlc->pub.sbh, (SBTML_PR | SBTML_FGC), SBTML_FGC);
	OSL_DELAY(1);
	sb_coreflags(wlc->pub.sbh, SBTML_FGC, 0);
	OSL_DELAY(1);

	/* Flag: wait for at least one beacon before entering sleeping state */
	wlc->PMawakebcn = TRUE;
	wlc_set_ps_ctrl(wlc);

	/* band-specific initializations */
	wlc_bsinit(wlc);

	/* update hw keys if out of date */
	wlc_key_bsupdate(wlc);

	/*
	 * If there are any pending software interrupt bits,
	 * then replace these with a harmless nonzero value
	 * so wlc_dpc() will re-enable interrupts when done.
	 */
	if (wlc->macintstatus)
		wlc->macintstatus = MI_DMAINT;

	/* restore macintmask */
	wl_intrsrestore(wlc->wl, macintmask);

	/* Save tsf_timerlow of the new core if this results in a coreswitch with ncores > 1 */
	if (wlc->ncores > 1)
		wlc->core->tsf_timerlow = R_REG(&wlc->regs->tsf_timerlow);

	/* ucode should still be suspended.. */
	ASSERT((R_REG(&wlc->regs->maccontrol) & MCTL_EN_MAC) == 0);
}

/*
 * Check if this TLVS entry is a WME IE or params IE entry.
 * If not, move the tlvs pointer/length to indicate the next TLVS entry.
 */
static bool
wlc_is_wme_ie(wlc_info_t *wlc, uint8 *ie, uint8 **tlvs, uint *tlvs_len)
{
	/* If the contents match the WPA_OUI, type=2, and supported subtype */
	if ((ie[TLV_LEN_OFF] == WME_IE_LEN &&
	     !bcmp(&ie[TLV_BODY_OFF], WME_OUI "\x02\x00", 5)) ||	/* WMM IE */
	    (ie[TLV_LEN_OFF] == WME_PARAM_IE_LEN &&
	     !bcmp(&ie[TLV_BODY_OFF], WME_OUI "\x02\x01", 5))) {	/* WMM PE */
		return TRUE;
	}

	/* point to the next ie */
	ie += ie[TLV_LEN_OFF] + TLV_HDR_LEN;
	/* calculate the length of the rest of the buffer */
	*tlvs_len -= (uint)(ie - *tlvs);
	/* update the pointer to the start of the buffer */
	*tlvs = ie;

	return FALSE;
}

/*
 * Extract the AC info from the wme params IE, then set the
 * info in the appropriate AC shm table locations
 *	(per Ling's ecdf_spec.txt rev1.8)
 */
static void
wlc_wme_setparams(wlc_info_t *wlc, bool suspend)
{
	uint aci, i, j;
	wme_acparam_t *acp_ie;
	shm_acparams_t acp_shm;
	uint16 *shm_entry;

	WL_INFORM(("wl%d: %s\n", wlc->pub.unit, __FUNCTION__));

	/* Only apply params if the core is out of reset and has clocks */
	if (!wlc->core->clk)
		return;

	if (suspend)
		wlc_suspend_mac_and_wait(wlc);

	/*
	 * AP uses AC params from wme_param_ie_ap.
	 * AP advertises AC params from wme_param_ie.
	 * STA uses AC params from wme_param_ie.
	 */

		acp_ie = (wme_acparam_t *)&wlc->wme_param_ie.acparam[0];

	wlc->wme_admctl = 0;

	for (i = 0; i < AC_COUNT; i++, acp_ie++) {
		bzero((char*)&acp_shm, sizeof(shm_acparams_t));
		/* find out which ac this set of params applies to */
		aci = (acp_ie->ACI & WME_ACI_MASK) >> WME_ACI_SHIFT;
		ASSERT(aci < AC_COUNT);
		/* set the admission control policy for this AC */
		if (acp_ie->ACI & WME_ACM_MASK)
			wlc->wme_admctl |= 1 << aci;
		/* fill in shm ac params struct */
		acp_shm.txop = ltoh16(acp_ie->TXOP);
		/* convert from units of 32us to us for ucode */
		wlc->wme_txop[aci] = acp_shm.txop = WME_TXOP2USEC(acp_shm.txop);
		acp_shm.aifs = (acp_ie->ACI & WME_AIFSN_MASK);
		if (acp_shm.aifs < WME_AIFSN_MIN || acp_shm.aifs > WME_AIFSN_MAX) {
			WL_ERROR(("wl%d: wlc_wme_setparams: bad aifs %d\n", wlc->pub.unit,
				acp_shm.aifs));
			continue;
		}


		/* CWmin = 2^(ECWmin) - 1 */
		acp_shm.cwmin = WME_ECW2CW(acp_ie->ECW & WME_ECWMIN_MASK);
		/* CWmax = 2^(ECWmax) - 1 */
		acp_shm.cwmax = WME_ECW2CW((acp_ie->ECW & WME_ECWMAX_MASK) >> WME_ECWMAX_SHIFT);
		acp_shm.cwcur = acp_shm.cwmin;
		acp_shm.bslots = R_REG(&wlc->regs->tsf_random) & acp_shm.cwcur;
		acp_shm.reggap = acp_shm.bslots + acp_shm.aifs;
		/* Indicate the new params to the ucode */
		acp_shm.status = wlc_read_shm(wlc, (M_EDCF_QINFO +
		                                    wme_shmemacindex[aci] * M_EDCF_QLEN +
		                                    M_EDCF_STATUS_OFF));
		acp_shm.status |= WME_STATUS_NEWAC;

		/* Fill in shm acparam table */
		shm_entry = (uint16*)&acp_shm;
		for (j = 0; j < (int)sizeof(shm_acparams_t); j += 2)
			wlc_write_shm(wlc,
			              M_EDCF_QINFO + wme_shmemacindex[aci] * M_EDCF_QLEN + j,
			              *shm_entry++);

		WL_INFORM(("wl%d: Setting %s: txop 0x%x cwmin 0x%x cwmax 0x%x "
		           "cwcur 0x%x aifs 0x%x bslots 0x%x reggap 0x%x status 0x%x\n",
		           wlc->pub.unit, aci_names[aci], acp_shm.txop, acp_shm.cwmin,
		           acp_shm.cwmax, acp_shm.cwcur, acp_shm.aifs,
		           acp_shm.bslots, acp_shm.reggap, acp_shm.status));
	}

	if (suspend)
		wlc_enable_mac(wlc);
}

static bool
BCMINITFN(wlc_isgoodchip)(wlc_info_t *wlc)
{
	/* reject some 4306 package/device combinations */
	if ((wlc->pub.sbh->chip == BCM4306_CHIP_ID) && (wlc->pub.sbh->chiprev > 2)) {
		/* 4309 is recognized by a pkg option */
		if (((wlc->deviceid == BCM4306_D11A_ID) ||
		     (wlc->deviceid == BCM4306_D11DUAL_ID)) &&
		    (wlc->pub.sbh->chippkg != BCM4309_PKG_ID))
			return FALSE;
	}

	/* reject unsupported corerev */
	if (!VALID_COREREV(wlc->pub.corerev)) {
		WL_ERROR(("unsupported core rev %d\n", wlc->pub.corerev));
		return FALSE;
	}

	return TRUE;
}

static bool
BCMINITFN(wlc_validboardtype)(wlc_info_t *wlc)
{
	bool goodboard = TRUE;
	int boardtype = wlc->pub.sbh->boardtype;
	int boardrev = wlc->pub.boardrev;

	if (wlc->pub.sbh->boardvendor != VENDOR_BROADCOM)
		return goodboard;

	if ((boardtype == BCM94306MP_BOARD) || (boardtype == BCM94306CB_BOARD)) {
		if (boardrev < 0x40)
			goodboard = FALSE;
	}
	else if (boardtype == BCM94309MP_BOARD) {
		goodboard = FALSE;
	}
	else if (boardtype == BCM94309G_BOARD) {
		if (boardrev < 0x51)
			goodboard = FALSE;
	}
	return goodboard;
}

char *
wlc_get_macaddr(wlc_info_t *wlc)
{
	char varname[32];
	char *macaddr = NULL;
	/*
	 * MAC addresses can not change during sw upgrades!
	 *
	 * 4309B0 dualband:  il0macaddr
	 * other  dualband:  et1macaddr
	 * uniband-A cards:  et1macaddr
	 * else:             il0macaddr
	 */
	if ((wlc->pub.sbh->chip == BCM4306_CHIP_ID) &&
	    (wlc->pub.sbh->chiprev == 2) && (NBANDS(wlc) > 1))
		strcpy(varname, "il0macaddr");
	else if (NBANDS(wlc) > 1)
		strcpy(varname, "et1macaddr");
	else if (ISAPHY(wlc->band->pi))
		strcpy(varname, "et1macaddr");
	else
		strcpy(varname, "il0macaddr");

	/* init perm_etheraddr and cur_etheraddr */
	if ((macaddr = getvar(wlc->pub.vars, varname)) == NULL) {
		WL_ERROR(("wl%d: wlc_get_macaddr: macaddr getvar(%s) not found\n",
			wlc->pub.unit, varname));
	}
	return macaddr;
}

bool
BCMINITFN(wlc_timers_init)(wlc_info_t *wlc, int unit)
{
	if (!(wlc->wdtimer = wl_init_timer(wlc->wl, wlc_watchdog))) {
		WL_ERROR(("wl%d: wlc_timers_init: wl_init_timer for wdtimer failed\n", unit));
		goto fail;
	}

	if (!(wlc->radio_timer = wl_init_timer(wlc->wl, wlc_radio_timer))) {
		WL_ERROR(("wl%d: wlc_timers_init: wl_init_timer for radio_timer failed\n", unit));
		goto fail;
	}

	wlc->country_list_extended = FALSE;


	if (!(wlc->led_timer = wl_init_timer(wlc->wl, wlc_timer_led))) {
		WL_ERROR(("wl%d: wlc_timers_init: wl_init_timer for led_timer failed\n", unit));
		goto fail;
	}
	if (!(wlc->pub.led_blink_timer = wl_init_timer(wlc->wl, wlc_timer_led_blink))) {
		WL_ERROR(("wl%d: wlc_timers_init: wl_init_timer for led_blink_timer failed\n",
			unit));
		goto fail;
	}

	if (!(wlc->scan.timer = wl_init_timer(wlc->wl, wlc_scantimer))) {
		WL_ERROR(("wl%d: wlc_timers_init: wl_init_timer for scantimer failed\n", unit));
		goto fail;
	}

	if (!(wlc->quiet.timer = wl_init_timer(wlc->wl, wlc_quiet_timer))) {
		WL_ERROR(("wl%d: wlc_timers_init: wl_init_timer for quiet-timer failed\n", unit));
		goto fail;
	}

	if (!(wlc->assoc_timer = wl_init_timer(wlc->wl, wlc_assoc_timeout))) {
		WL_ERROR(("wl%d: wlc_timers_init: wl_init_timer for assoc_timer failed\n", unit));
		goto fail;
	}

	return TRUE;

fail:
	return FALSE;
}

/* set wlc_info structure default values */
void
BCMINITFN(wlc_info_init)(wlc_info_t *wlc, bool wme, int unit)
{
	int i;
	brcm_ie_t *brcm_ie;

	/* Initialize default bsscfg structure */
	wlc->bsscfg[0] = &wlc->cfg;
	wlc_bsscfg_init(wlc, wlc->bsscfg[0]);

	/*
	 * Initialize wlc_info default values ...
	 * may get overrides later in this function
	 */

	/* initialize CCK preamble mode to unassociated state */
	wlc->shortpreamble = FALSE;

	/* various 802.11g modes */
	wlc->shortslot_override = WLC_SHORTSLOT_AUTO;
	wlc->shortslot = FALSE;
	wlc->g_protection_override = WLC_PROTECTION_AUTO;
	wlc->g_protection_control = WLC_PROTECTION_CTL_OFF;
	wlc->g_protection_cts = TRUE;
	wlc->g_protection = FALSE;

	wlc->barker_overlap_control = TRUE;
	wlc->barker_preamble = WLC_BARKER_SHORT_ALLOWED;

	wlc->n_protection_override = WLC_PROTECTION_AUTO;
	wlc->n_protection_control = WLC_PROTECTION_CTL_OFF;
	wlc->n_protection = FALSE;

	/* 802.11g draft 4.0 NonERP elt advertisement */
	wlc->include_legacy_erp = TRUE;

	wlc->pub.antdiv_override = -1;
	wlc->txant = TXANT_DEF;
	wlc->prb_resp_timeout = WLC_PRB_RESP_TIMEOUT;

	for (i = 0; i < NFIFO; i++)
		wlc->fragthresh[i] = DOT11_DEFAULT_FRAG_LEN;
	wlc->RTSThresh = DOT11_DEFAULT_RTS_LEN;


	/* set the mode to INFRA */
	wlc->pub.BSS = TRUE;

	/* default mac retry limits */
	wlc->SRL = RETRY_SHORT_DEF;
	wlc->LRL = RETRY_LONG_DEF;

	/* default rate fallback retry limits */
	wlc->SFBL = RETRY_SHORT_FB;
	wlc->LFBL = RETRY_LONG_FB;

	/* init PM state */
	wlc->PM = PM_OFF; /* User's setting of PM mode through IOCTL */
	wlc->PM_override = FALSE; /* Prevents from going to PM if our AP is 'ill' */
	wlc->PMenabled = FALSE; /* Current PM state */
	wlc->PMpending = FALSE; /* Tracks whether STA indicated PM in the last attempt */
	wlc->PMblocked = FALSE; /* To allow blocking going into PM during RM and scans */

	/* In WMM Auto mode, PM is allowed if association is a UAPSD association */
	wlc->WME_PM_blocked = FALSE;

	/* Overrides for the core to stay awake under zillion conditions
	 * Look for STAY_AWAKE
	 */
	wlc->wake = FALSE;
	wlc->PSpoll = FALSE; /* Are we waiting for a response to PS-Poll that we sent */

	/* APSD defaults */
	wlc->wme_apsd = TRUE;
	wlc->apsd_sta_usp = FALSE;

	/* VLAN Mode default */
	wlc->vlan_mode = AUTO;

	/* set default sw macintmask value */
	wlc->defmacintmask = DEF_MACINTMASK;

	/* Set flag to indicate that hw keys should be used when available. */
	wlc->wsec_swkeys = FALSE;

	/* init the 4 static WEP default keys */
	for (i = 0; i < WSEC_MAX_DEFAULT_KEYS; i++) {
		wlc->wsec_keys[i] = &wlc->wsec_def_keys[i];
		wlc->wsec_keys[i]->idx = (uint8)i;
	}

	wlc->_regulatory_domain = FALSE;	/* 802.11d */

	/* WME QoS mode is Auto by default */
	wlc->pub._wme = AUTO;

	/* enable all event callbacks by default */
	memset(wlc->eventing, 0xff, sizeof(wlc->eventing));

	/* scan inits */
	wlc->scan.channel_idx = -1;
	wlc->scan.defaults.assoc_time = WLC_SCAN_ASSOC_TIME;
	wlc->scan.defaults.unassoc_time = WLC_SCAN_UNASSOC_TIME;
	wlc->scan.defaults.home_time = WLC_SCAN_HOME_TIME;
	wlc->scan.defaults.passive_time = WLC_SCAN_PASSIVE_TIME;
	wlc->scan.defaults.nprobes = WLC_SCAN_NPROBES;
	wlc->scan.defaults.passive = FALSE;
	/* roam scan inits */
	wlc->roam_scan_block = 0;
	wlc->roam_scan_period = WLC_ROAM_SCAN_PERIOD;

	/* set default power output percentage to 100 percent */
	wlc->pub.txpwr_percent = 100;

	wlc->pub.bcmerror = 0;

	/* initialize our proprietary elt */
	brcm_ie = (brcm_ie_t *)&wlc->brcm_ie[0];
	bzero((char*)brcm_ie, sizeof(brcm_ie_t));
	brcm_ie->id = DOT11_MNG_PROPR_ID;
	brcm_ie->len = BRCM_IE_LEN - TLV_HDR_LEN;
	bcopy(BRCM_OUI, &brcm_ie->oui[0], DOT11_OUI_LEN);
	brcm_ie->ver = BRCM_IE_VER;

	wlc->ibss_allowed = TRUE;

	wlc->assoc_elts = NULL;
	wlc->assoc_elts_len = 0;

}

/*
 * Existing error codes should not be changed. New error codes should
 * be unique. Error code assignment should move with its respective code.
 */
void*
BCMINITFN(wlc_attach)(void *wl, uint16 vendor, uint16 device, uint unit, bool piomode,
	osl_t *osh, void *regsva, uint bustype, void *btparam, uint *perr)
{
	wlc_info_t *wlc;
	d11regs_t *regs;
	char name[8];
	int i, aa;
	char *macaddr = NULL;
	uint err = 0;
	uint j;
	int dma_attach_err = 0;
	bool wme = FALSE;
	char *var;

	WL_TRACE(("wl%d: wlc_attach: vendor 0x%x device 0x%x\n", unit, vendor, device));

	ASSERT(WSEC_MAX_SEC_KEYS <= WSEC_MAX_KEYS);
	ASSERT(WSEC_MAX_RCMTA_KEYS <= WSEC_MAX_KEYS);
	ASSERT(WSEC_MAX_DEFAULT_KEYS == WLC_DEFAULT_KEYS);

	/* some code depends on packed structures */
	ASSERT(sizeof(struct ether_addr) == ETHER_ADDR_LEN);
	ASSERT(sizeof(struct ether_header) == ETHER_HDR_LEN);
	ASSERT(sizeof(d11regs_t) == SB_CORE_SIZE);
	ASSERT(sizeof(ofdm_phy_hdr_t) == D11_PHY_HDR_LEN);
	ASSERT(sizeof(cck_phy_hdr_t) == D11_PHY_HDR_LEN);
	ASSERT(sizeof(d11txh_t) == D11_TXH_LEN);
	ASSERT(sizeof(d11rxhdr_t) == RXHDR_LEN);
	ASSERT(sizeof(struct dot11_llc_snap_header) == DOT11_LLC_SNAP_HDR_LEN);
	ASSERT(sizeof(struct dot11_header) == DOT11_A4_HDR_LEN);
	ASSERT(sizeof(struct dot11_rts_frame) == DOT11_RTS_LEN);
	ASSERT(sizeof(struct dot11_cts_frame) == DOT11_CTS_LEN);
	ASSERT(sizeof(struct dot11_ack_frame) == DOT11_ACK_LEN);
	ASSERT(sizeof(struct dot11_ps_poll_frame) == DOT11_PS_POLL_LEN);
	ASSERT(sizeof(struct dot11_cf_end_frame) == DOT11_CS_END_LEN);
	ASSERT(sizeof(struct dot11_management_header) == DOT11_MGMT_HDR_LEN);
	ASSERT(sizeof(struct dot11_auth) == DOT11_AUTH_FIXED_LEN);
	ASSERT(sizeof(struct dot11_bcn_prb) == DOT11_BCN_PRB_LEN);
	ASSERT(sizeof(tx_status_t) == TXSTATUS_LEN);
	ASSERT(sizeof(brcm_ie_t) == BRCM_IE_LEN);
	ASSERT(OFFSETOF(wl_scan_params_t, channel_list) == WL_SCAN_PARAMS_FIXED_SIZE);
	ASSERT(TKIP_MIC_SIZE == (2 * sizeof(uint32)));	/* some tkip code expects this */
	ASSERT(ISALIGNED(OFFSETOF(wsec_key_t, data), sizeof(uint32)));

	/*
	 * Number of replay counters value used in WPA IE must match # rxivs
	 * supported in wsec_key_t struct. See 802.11i/D3.0 sect. 7.3.2.17
	 * 'RSN Information Element' figure 8 for this mapping.
	 */
	ASSERT((WPA_CAP_16_REPLAY_CNTRS == WLC_REPLAY_CNTRS_VALUE && 16 == WLC_NUMRXIVS) ||
		(WPA_CAP_4_REPLAY_CNTRS == WLC_REPLAY_CNTRS_VALUE && 4 == WLC_NUMRXIVS));

	/* allocate wlc_info_t state structure */
	if ((wlc = (wlc_info_t*) MALLOC(osh, sizeof(wlc_info_t))) == NULL) {
		WL_ERROR(("wl%d: wlc_attach: wlc_info_t out of memory, malloced %d bytes\n",
			unit, MALLOCED(wlc->pub.osh)));
		err = 10;
		goto fail;
	}
	bzero((char*)wlc, sizeof(wlc_info_t));

	/* populate wlc_info_t with default values  */
	wlc_info_init(wlc, wme, unit);

	/* construct mapping from AC bitmap to precedence bitmap */
	for (i = 0; i < 16; i++) {
		wlc_acbitmap2precbitmap[i] = 0;
		if (AC_BITMAP_TST(i, AC_BE))
			wlc_acbitmap2precbitmap[i] |= WLC_PREC_BMP_AC_BE;
		if (AC_BITMAP_TST(i, AC_BK))
			wlc_acbitmap2precbitmap[i] |= WLC_PREC_BMP_AC_BK;
		if (AC_BITMAP_TST(i, AC_VI))
			wlc_acbitmap2precbitmap[i] |= WLC_PREC_BMP_AC_VI;
		if (AC_BITMAP_TST(i, AC_VO))
			wlc_acbitmap2precbitmap[i] |= WLC_PREC_BMP_AC_VO;
	}

	/* register a module (to handle iovars) */
	wlc_module_register(&wlc->pub, wlc_iovars, "wlc_iovars", wlc, wlc_doiovar, NULL, NULL);

	wlc->pub.osh = osh;
	wlc->btparam = btparam;

	/* inti band and core */
	wlc->band = &wlc->bandstate[0];
	wlc->core = &wlc->corestate[0];


	regs = (d11regs_t*) regsva;

	/*
	 * Do the hardware portion of the attach.
	 * Also initialize software state that depends on the particular hardware
	 * we are running.
	 */
	wlc->pub.sbh = sb_attach((uint)device, osh, regsva, bustype, btparam,
	                                  &wlc->pub.vars, &wlc->vars_size);
	if (wlc->pub.sbh == NULL) {
		WL_ERROR(("wl%d: wlc_attach: sb_attach failed\n", unit));
		err = 11;
		goto fail;
	}
	/*
	 * Get vendid/devid nvram overwrites, which could be different
	 */
#ifdef BCMBUSTYPE
	bustype = BCMBUSTYPE;
#endif	/* BCMBUSTYPE */

	if ((var = getvar(wlc->pub.vars, "vendid")))
		vendor = (uint16)bcm_strtoul(var, NULL, 0);
	if ((var = getvar(wlc->pub.vars, "devid")))
		device = (uint16)bcm_strtoul(var, NULL, 0);

	/* verify again the device is supported */
	if (!wlc_chipmatch(vendor, device)) {
		WL_ERROR(("wl%d: wlc_attach: Unsupported vendor/device (0x%x/0x%x)\n",
			unit, vendor, device));
		err = 12;
		goto fail;
	}

	wlc->vendorid = vendor;
	wlc->deviceid = device;

	/* initialize power control registers and force fastclock for the rest of attach */
	sb_clkctl_init(wlc->pub.sbh);
	sb_clkctl_clk(wlc->pub.sbh, CLK_FAST);

	/* set bar0 window to point at D11 core */
	regs = (d11regs_t *)sb_setcore(wlc->pub.sbh, SB_D11, 0);

	wlc->pub.corerev = sb_corerev(wlc->pub.sbh);

	/* validate chip, chiprev and corerev */
	if (!wlc_isgoodchip(wlc)) {
		err = 14;
		goto fail;
	}

	/* update sta/ap related parameters */
	wlc_ap_upd(wlc);

	wlc->wl = wl;
	wlc->pub.unit = unit;
	wlc->_piomode = piomode;
	wlc->regs = regs;


	/* reset the d11 core */
	wlc_corereset(wlc);

	if (!wlc_validate_chip_access(wlc)) {
		WL_ERROR(("wl%d: wlc_attach: validate_chip_access failed\n", unit));
		err = 21;
		goto fail;
	}

	/* get the board rev, used just below */
	wlc->pub.boardrev = (uint8)getintvar(wlc->pub.vars, "boardrev");
	/* promote srom boardrev of 0xFF to 1 */
	if (wlc->pub.boardrev == BOARDREV_PROMOTABLE)
		wlc->pub.boardrev = BOARDREV_PROMOTED;
	if (!(wlc->pub.boardrev > 0)) {
		WL_ERROR(("wl%d: wlc_attach: Unsupported board revision level: 0x%x\n",
			unit, wlc->pub.boardrev));
		ASSERT(wlc->pub.boardrev > 0);
		err = 15;
		goto fail;
	}
	if (!wlc_validboardtype(wlc)) {
		WL_ERROR(("wl%d: wlc_attach: Unsupported Broadcom board type (0x%x) or revision"
			" level (0x%x)\n",
			unit, wlc->pub.sbh->boardtype, wlc->pub.boardrev));
		err = 16;
		goto fail;
	}

	/* get srom revision */
	wlc->pub.sromrev = (uint8)getintvar(wlc->pub.vars, "sromrev");

	/* get board flags */
	wlc->pub.boardflags = (uint32)getintvar(wlc->pub.vars, "boardflags");

	if (D11REV_GE(wlc->pub.corerev, 5) && !(wlc->pub.boardflags & BFL_NOPLLDOWN))
		wlc->plldown = TRUE;

	if (sb_war16165(wlc->pub.sbh))
		wlc->war16165 = TRUE;

	/* name and offsets for dma_attach */
	sprintf(name, "wl%d", unit);

	/* special "multiband" pci device id */
	if ((device == BCM4306_D11DUAL_ID) ||
	    (device == BCM4318_D11DUAL_ID) ||
	    (device == BCM4311_D11DUAL_ID)) {
		/* Dualband boards */
		wlc->pub._nbands = 2;
	} else
		wlc->pub._nbands = 1;

	/* wlc->ncores is the # of D11 cores present whether we use them or not */
	if (((device == BCM4306_D11A_ID) ||
	     (device == BCM4318_D11A_ID) ||
	     (device == BCM4311_D11A_ID) ||
	     (NBANDS(wlc) == 2)) &&
	     (D11REV_IS(wlc->pub.corerev, 4))) {
		 /* Dual core devices */
		 wlc->ncores = 2;
	} else
		 wlc->ncores = 1;

	/* alloc and save pointer to shared phy state area */
	wlc->phy_sh = wlc_phy_shared_attach(&wlc->pub);

	/* initialize software state for each core and band */
	for (j = 0; j < NBANDS(wlc); j++) {
		/*
		 * band0 is always 2.4Ghz
		 * band1, if present, is 5Ghz
		 */

		/* So if this is a single band 11a card, use band 1 */
		if (IS_SINGLEBAND_5G(device))
			j = BAND_5G_INDEX;

		wlc_setxband(wlc, j);

		wlc->band->bandunit = j;
		wlc->band->bandtype = j ? WLC_BAND_5G : WLC_BAND_2G;
		wlc->core->coreunit = sb_coreunit(wlc->pub.sbh);
		wlc->core->coreidx = sb_coreidx(wlc->pub.sbh);

		/* init tx fifo size */
		wlc_d11fifo_alloc(wlc);

		/* Get a phy for this band */
		WL_NONE(("wl%d: %s: bandunit %d bandtype %d coreunit %d coreidx %d\n", unit,
		         __FUNCTION__, wlc->band->bandunit, wlc->band->bandtype,
		         wlc->core->coreunit, wlc->core->coreidx));
		if ((wlc->band->pi = wlc_phy_attach(&wlc->pub, (void *)regs,
		                                             wlc->band->bandtype,
		                                             (void *)wlc, wlc->phy_sh)) == NULL) {
			WL_ERROR(("wl%d: wlc_attach: wlc_phy_attach failed\n", unit));
			err = 12;
			goto fail;
		}

		/* init core's pio or dma channels */
		if (PIO_ENAB(wlc)) {
			/* skip if already initialized, for dual band, but single core */
			if (wlc->core->pio[0] == 0) {
				for (i = 0; i < NFIFO; i++) {
					wlc->core->pio[i] = wlc_pio_attach(&wlc->pub, wlc, i,
						PIOREG(wlc, PIO_TX, i),
						PIOREG(wlc, PIO_RX, i));
					if (wlc->core->pio[i] == NULL) {
						WL_ERROR(("wlc_attach: pio_attach failed\n"));
						err = 20;
						goto fail;
					}
				}
			}

		} else if (wlc->core->di[0] == 0) {	/* Init FIFOs */
			uint addrwidth;

			/* Find out the DMA addressing capability and let OS know
			 * All the channels within one DMA core have 'common-minimum' same
			 * capability
			 */
			addrwidth = dma_addrwidth(wlc->pub.sbh, DMAREG(wlc, DMA_TX, 0));
			OSL_DMADDRWIDTH(osh, addrwidth);

			/*
			 * FIFO 0
			 * TX: TX_AC_BK_FIFO (TX AC Background data packets)
			 * RX: RX_FIFO (RX data packets)
			 */
			ASSERT(TX_AC_BK_FIFO == 0);
			ASSERT(RX_FIFO == 0);
			dma_attach_err |= !(wlc->core->di[0] =
			                    dma_attach(osh, name,
			                               wlc->pub.sbh,
			                               (wme ? DMAREG(wlc, DMA_TX, 0) : NULL),
			                               DMAREG(wlc, DMA_RX, 0),
			                               (wme ? NTXD : 0), NRXD,
			                               RXBUFSZ, NRXBUFPOST, HWRXOFF,
			                               &wl_msg_level));

			/*
			 * FIFO 1
			 * TX: TX_AC_BE_FIFO (TX AC Best-Effort data packets
			 *   (legacy) TX_DATA_FIFO (TX data packets)
			 * RX: UNUSED
			 */
			ASSERT(TX_AC_BE_FIFO == 1);
			ASSERT(TX_DATA_FIFO == 1);
			dma_attach_err |= !(wlc->core->di[1] =
			                    dma_attach(osh, name,
			                               wlc->pub.sbh,
			                               DMAREG(wlc, DMA_TX, 1), NULL,
			                               NTXD, 0,
			                               0, 0, 0, &wl_msg_level));

			/*
			 * FIFO 3
			 * TX: TX_AC_VO_FIFO (TX AC Voice data packets
			 *   (legacy) TX_CTL_FIFO (TX control & mgmt packets)
			 * RX: RX_TXSTATUS_FIFO (transmit-status packets)
			 *	for corerev < 5 only
			 */
			ASSERT(TX_AC_VO_FIFO == 3);
			ASSERT(TX_CTL_FIFO == 3);
			if (D11REV_LT(wlc->pub.corerev, 5)) {
				ASSERT(RX_TXSTATUS_FIFO == 3);
				dma_attach_err |= !(wlc->core->di[3] =
				                    dma_attach(osh, name, wlc->pub.sbh,
				                               DMAREG(wlc, DMA_TX, 3),
				                               DMAREG(wlc, DMA_RX, 3),
				                               NTXD, NRXD,
				                               sizeof(tx_status_t), NRXBUFPOST, 0,
				                               &wl_msg_level));
			} else
				dma_attach_err |= !(wlc->core->di[3] =
				                    dma_attach(osh, name,
				                               wlc->pub.sbh,
				                               DMAREG(wlc, DMA_TX, 3), NULL,
				                               NTXD, 0,
				                               0, 0, 0, &wl_msg_level));


			if (dma_attach_err) {
				WL_ERROR(("wl%d: wlc_attach: dma_attach failed\n", unit));
				err = 20;
				goto fail;
			}

			/* get pointer to dma engine tx flow control variable */
			for (i = 0; i < NFIFO; i++)
				if (wlc->core->di[i])
					wlc->core->txavail[i] =
						(uint*)dma_getvar(wlc->core->di[i], "&txavail");
		}

		/* default contention windows size limits */
		wlc->band->CWmin = APHY_CWMIN;
		wlc->band->CWmax = PHY_CWMAX;

		/* verify good phy_type & supported phy revision */
		if (ISAPHY(wlc->band->pi)) {
			if (ACONF_HAS(wlc->band->pi->phy_rev))
				goto good_phy;
			else
				goto bad_phy;
		} else if (ISGPHY(wlc->band->pi)) {
			if (GCONF_HAS(wlc->band->pi->phy_rev))
				goto good_phy;
			else
				goto bad_phy;
		} else {
bad_phy:
			WL_ERROR(("wl%d: wlc_attach: unsupported phy type/rev (%d/%d)\n",
				unit, wlc->band->pi->phy_type, wlc->band->pi->phy_rev));
			err = 22;
			goto fail;
		}

good_phy:
		WL_INFORM(("wl%d: %s: chiprev %d coreunit %d corerev %d band %sG, phy_type %d "
		           "phy_rev %d ana_rev %d\n",
		           wlc->pub.unit, __FUNCTION__, wlc->pub.sbh->chiprev, wlc->core->coreunit,
		           wlc->pub.corerev, wlc->band->bandtype == WLC_BAND_2G ? "2.4" : "5",
		           wlc->band->pi->phy_type, wlc->band->pi->phy_rev,
		           wlc->band->pi->ana_rev));

		/* set default gmode value */
		if (ISGPHY(wlc->band->pi)) {
			wlc->band->gmode = GMODE_AUTO;
		}

		/* set default roam parameters */
		switch (wlc->band->pi->radioid) {
		case BCM2050_ID:
			if (wlc->band->pi->radiorev == 1) {
				wlc->band->roam_trigger_def = WLC_2053_ROAM_TRIGGER;
				wlc->band->roam_delta_def = WLC_2053_ROAM_DELTA;
			} else {
				wlc->band->roam_trigger_def = WLC_2050_ROAM_TRIGGER;
				wlc->band->roam_delta_def = WLC_2050_ROAM_DELTA;
			}
			if (wlc->pub.boardflags & BFL_EXTLNA) {
				wlc->band->roam_trigger_def -= 2;
			}
			break;
		case BCM2060_ID:
			wlc->band->roam_trigger_def = WLC_2060WW_ROAM_TRIGGER;
			wlc->band->roam_delta_def = WLC_2060WW_ROAM_DELTA;
			break;
		}

		wlc->band->roam_trigger = wlc->band->roam_trigger_def;
		wlc->band->roam_delta = wlc->band->roam_delta_def;

		/* get antennas available */
		aa = (int8)getintvar(wlc->pub.vars, (j ? "aa1" : "aa0"));
		if ((aa < 1) || (aa > 3)) {
			WL_ERROR(("wl%d: wlc_attach: Invalid antennas available in srom (0x%x).\n",
				unit, aa));
			err = 24;
			goto fail;
		}

		/* reset the defaults if we have a single antenna */
		if (aa == 1) {
			WL_INFORM(("wl%d: %s: Single antenna available (aa = %d) forcing antenna"
				" 0(main)\n",
				unit, __FUNCTION__, aa));
			wlc->pub.antdiv_override =  ANTDIV_FORCE_0;
			wlc->txant = TXANT_0;
		} else if (aa == 2) {
			WL_INFORM(("wl%d: %s: Single antenna available (aa = %d) forcing antenna"
				" 1(aux)\n",
				unit, __FUNCTION__, aa));
			wlc->pub.antdiv_override = ANTDIV_FORCE_1;
			wlc->txant = TXANT_1;
		}

		/* Compute Antenna Gain */
		wlc->band->antgain = (int8)getintvar(wlc->pub.vars, (j ? "ag1" : "ag0"));

		if ((wlc->band->antgain == -1) && (wlc->pub.sromrev == 1)) {
			/* default antenna gain for srom rev 1 is 2 dBm (8 qdbm) */
			wlc->band->antgain = 8;
		} else {
			int8 gain, fract;
			/* Older sroms specified gain in whole dbm only.  In order
			 * be able to specify qdbm granularity and remain backward compatible
			 * the whole dbms are now encoded in only low 6 bits and remaining qdbms
			 * are encoded in the hi 2 bits. 6 bit signed number ranges from
			 * -32 - 31. Examples: 0x1 = 1 db,
			 * 0xc1 = 1.75 db (1 + 3 quarters),
			 * 0x3f = -1 (-1 + 0 quarters),
			 * 0x7f = -.75 (-1 in low 6 bits + 1 quarters in hi 2 bits) = -3 qdbm.
			 * 0xbf = -.50 (-1 in low 6 bits + 2 quarters in hi 2 bits) = -2 qdbm.
			 */
			gain = wlc->band->antgain & 0x3f;
			gain <<= 2;	/* Sign extend */
			gain >>= 2;
			fract = (wlc->band->antgain & 0xc0) >> 6;
			wlc->band->antgain = 4 * gain + fract;
		}

		/* init per-band default rateset */
		wlc_default_rateset(wlc, &wlc->band->defrateset);

		/* fill in hw_rateset (used early by WLC_SET_RATESET) */
		wlc_rateset_filter(&wlc->band->defrateset, &wlc->band->hw_rateset,
			FALSE, FALSE, RATE_MASK);

		/* initial ucode host flags */
		wlc->band->mhf = wlc_mhfdef(wlc);

		/* disable core to match driver "down" state */
		wlc_coredisable(wlc);
	}


	/* register sb interrupt callback functions */
	sb_register_intr_callback(wlc->pub.sbh, wlc_wlintrsoff, wlc_wlintrsrestore, NULL, wlc);

	ASSERT(!wlc->sbclk || !sb_taclear(wlc->pub.sbh));

	/* turn off pll and xtal to match driver "down" state */
	wlc_xtal(wlc, OFF);

	/* *********************************************************************
	 * The hardware is in the DOWN state at this point. D11 core
	 * or cores are in reset with clocks off, and the board PLLs
	 * are off if possible.
	 *
	 * Beyond this point, wlc->sbclk == FALSE and chip registers
	 * should not be touched.
	 *********************************************************************
	 */

	/* Now we are ready to mark valid channels */
	if (wlc_channels_init(wlc)) {
		WL_ERROR(("wl%d: wlc_attach: Invalid Channel set\n", unit));
		goto fail;
	}


	/* init default, target, and current BSS with some sane initial values */
	wlc_default_bss(wlc);

	err = 30;

	if ((macaddr = wlc_get_macaddr(wlc)) == NULL) {
		WL_ERROR(("wl%d: wlc_attach: macaddr not found\n", unit));
		goto fail;
	}
	bcm_ether_atoe(macaddr, &wlc->perm_etheraddr);
	if (ETHER_ISBCAST((char*)&wlc->perm_etheraddr) ||
		ETHER_ISNULLADDR((char*)&wlc->perm_etheraddr)) {
		WL_ERROR(("wl%d: wlc_attach: bad macaddr %s\n", unit, macaddr));
		goto fail;
	}

	bcopy((char *)&wlc->perm_etheraddr, (char *)&wlc->pub.cur_etheraddr, ETHER_ADDR_LEN);

	if ((wlc->scbstate = wlc_scb_attach(&wlc->pub)) == NULL) {
		WL_ERROR(("wl%d: wlc_attach: wlc_scb_attach failed\n", unit));
		err = 37;
		goto fail;
	}

	/* allocate the led info structure */
	if ((wlc->ledh = wlc_led_attach(&wlc->pub, wlc->core->coreunit, (void *)wlc)) == NULL) {
		WL_ERROR(("wl%d: wlc_attach: wlc_led_attach failed\n", unit));
		err = 31;
		goto fail;
	}

	err = 34;

	if (!wlc_timers_init(wlc, unit)) {
		WL_ERROR(("wl%d: wlc_attach: wlc_init_timer failed\n", unit));
		goto fail;
	}

	wlc->pub.txmaxpkts = MAXTXPKTS;

	WLCNTSET(wlc->pub._cnt.version, WL_CNT_T_VERSION);
	WLCNTSET(wlc->pub._cnt.length, sizeof(wlc->pub._cnt));

	WLCNTSET(wlc->pub._wme_cnt.version, WL_WME_CNT_VERSION);
	WLCNTSET(wlc->pub._wme_cnt.length, sizeof(wlc->pub._wme_cnt));

	/*
	 * Complete the wlc default state initializations..
	 */
	/* Get notified when an SCB is freed */
	wlc_scb_cubby_reserve(wlc, 0, wlc_txq_scb_init, wlc_txq_scb_free_notify, NULL, wlc);
	wlc_txmod_fn_register(wlc, SCB_TRANSMIT, wlc_txq_enq, wlc, NULL, NULL);


	/* Have enough room for control packets along with HI watermark */
	/* Also, add room to txq for total psq packets if all the SCBs leave PS mode */
	/* The watermark for flowcontrol to OS packets will remain the same */
	pktq_init(&wlc->txq, WLC_PREC_COUNT, (2 * WLC_DATAHIWAT) + PKTQ_LEN_DEFAULT +
	          wlc->pub.psq_pkts_total);

	WL_INFORM(("wl%d: %s: board 0x%x rev %d.%d macaddr: %s\n",
		unit, __FUNCTION__, wlc->pub.sbh->boardtype,
		((wlc->pub.boardrev >> 4) & 0xf), (wlc->pub.boardrev & 0xf), macaddr));

	if (perr)
		*perr = 0;

	return ((void*)wlc);

fail:
	WL_ERROR(("wl%d: wlc_attach: failed\n", unit));
	if (wlc)
		wlc_detach(wlc);

	if (perr)
		*perr = err;
	return (NULL);
}

static void
wlc_d11fifo_alloc(wlc_info_t *wlc)
{
	uint8 *fifoarray;
	uint i;

	if (D11REV_GE(wlc->pub.corerev, 9)) {
		fifoarray = hwfifo_blk9_11;
	} else if (D11REV_GE(wlc->pub.corerev, 5)) {
		fifoarray = hwfifo_blk5_8;
	} else
		fifoarray = hwfifo_blk4;

	for (i = 0; i < NFIFO; i++) {
		wlc->core->xmtfifo_limit[i] = fifoarray[i];
	}
}

/*
 * Return a count of the number of driver callbacks still pending.
 */
uint
BCMUNINITFN(wlc_detach)(wlc_info_t *wlc)
{
	uint i, j;
	uint callbacks = 0;
	wlcband_t *band;

	if (wlc == NULL)
		return 0;

	WL_TRACE(("wl%d: %s\n", wlc->pub.unit, __FUNCTION__));

	ASSERT(!wlc->pub.up);

	/* make sure LEDs are cleared */
	wlc_led_gcclear(wlc);

	if (!wlc_radio_monitor_stop(wlc)) {
		callbacks++;
	}

	/* free all scbs, including permanent ones */
	wlc_scbclear(wlc, TRUE);


	/* Free per-band state */
	band = wlc->band;
	for (i = 0; i < NBANDS(wlc); i++) {

		if (band->pi) {
			/* Detach this band's phy */
			wlc_phy_detach(band->pi);
			band->pi = NULL;
		}
		band = &wlc->bandstate[OTHERBANDUNIT(wlc)];
	}

	/* Free shared phy state */
	wlc_phy_shared_detach(&wlc->pub, wlc->phy_sh);


	/* free timer state */
	if (wlc->wdtimer) {
		wl_free_timer(wlc->wl, wlc->wdtimer);
		wlc->wdtimer = NULL;
	}
	if (wlc->radio_timer) {
		wl_free_timer(wlc->wl, wlc->radio_timer);
		wlc->radio_timer = NULL;
	}
	if (wlc->led_timer) {
		wl_free_timer(wlc->wl, wlc->led_timer);
		wlc->led_timer = NULL;
	}
	if (wlc->pub.led_blink_timer) {
		wl_free_timer(wlc->wl, wlc->pub.led_blink_timer);
		wlc->pub.led_blink_timer = NULL;
	}
	if (wlc->scan.timer) {
		wl_free_timer(wlc->wl, wlc->scan.timer);
		wlc->scan.timer = NULL;
	}
	if (wlc->quiet.timer) {
		wl_free_timer(wlc->wl, wlc->quiet.timer);
		wlc->quiet.timer = NULL;
	}
	if (wlc->assoc_timer) {
		wl_free_timer(wlc->wl, wlc->assoc_timer);
		wlc->assoc_timer = NULL;
	}

	/* free association state */
	if (wlc->assoc_elts)
		MFREE(wlc->pub.osh, wlc->assoc_elts, wlc->assoc_elts_len);
	if (wlc->assoc_req)
		MFREE(wlc->pub.osh, wlc->assoc_req, wlc->assoc_req_len);
	if (wlc->assoc_resp)
		MFREE(wlc->pub.osh, wlc->assoc_resp, wlc->assoc_resp_len);

	/* free bcn_prb in wlc_bss_info_t in case they haven't been */
	wlc_bss_list_free(wlc, &wlc->scan_results);
	wlc_bss_list_free(wlc, &wlc->join_targets);
	wlc_bss_list_free(wlc, &wlc->custom_scan_results);


	/* free dma/pio resources */
	FOR_ACTIVE_D11_CORES(i, wlc->deviceid, wlc->ncores, NBANDS(wlc)) {
		for (j = 0; j < NFIFO; j++) {
			if (!PIO_ENAB(wlc)) {
				if (wlc->corestate[i].di[j]) {
					dma_detach(wlc->corestate[i].di[j]);
					wlc->corestate[i].di[j] = NULL;
				}

			} else {
				if (wlc->corestate[i].pio[j]) {
					wlc_pio_detach(wlc->corestate[i].pio[j]);
					wlc->corestate[i].pio[j] = NULL;
				}
			}
		}
	}
	wlc_led_detach(wlc->ledh);
	wlc->ledh = NULL;

	/* free scb state */
	if (wlc->scbstate) {
		wlc_scb_detach(wlc->scbstate);
		wlc->scbstate = NULL;
	}

	/* free sb handle */
	if (wlc->pub.sbh) {
		sb_detach(wlc->pub.sbh);
		wlc->pub.sbh = NULL;
	}

	/* free vars */
	if (wlc->pub.vars) {
		MFREE(wlc->pub.osh, wlc->pub.vars, wlc->vars_size);
		wlc->pub.vars = NULL;
	}


	/* Detach from iovar manager */
	wlc_module_unregister(&wlc->pub, "wlc_iovars", wlc);

	/*
	* consistency check: wlc_module_register/wlc_module_unregister calls
	* should match therefore nothing should be left here.
	*/
	for (i = 0; i < WLC_MAXMODULES; i ++)
		ASSERT(wlc->modulecb[i].name[0] == '\0');

	WL_MPC(("wl%d: DETACHED, callbacks %d\n", wlc->pub.unit, callbacks));

	/* free the wlc */
	MFREE(wlc->pub.osh, wlc, sizeof(wlc_info_t));
	return (callbacks);
}

bool
wlc_chipmatch(uint16 vendor, uint16 device)
{
	if (vendor != VENDOR_BROADCOM)
		return (FALSE);

	if (device == BCM4306_D11G_ID)
		return (TRUE);
	if (device == BCM4306_D11G_ID2)
		return (TRUE);
	if (device == BCM4306_D11A_ID)
		return (TRUE);
	if (device == BCM4306_D11DUAL_ID)
		return (TRUE);
	if (device == BCM4318_D11G_ID)
		return (TRUE);
	if (device == BCM4318_D11DUAL_ID)
		return (TRUE);
	if (device == BCM4318_D11A_ID)
		return (TRUE);
	if (device == BCM4311_D11G_ID)
		return (TRUE);
	if (device == BCM4311_D11A_ID)
		return (TRUE);
	if (device == BCM4311_D11DUAL_ID)
		return (TRUE);

	return (FALSE);
}

/* update state that depends on the current value of "ap" */
static void
wlc_ap_upd(wlc_info_t* wlc)
{
	wlc->PLCPHdr_override = WLC_PLCP_SHORT;	/* STA-BSS; short capable */
}

/*
 * Return TRUE if radio is disabled, otherwise FALSE.
 * hw radio disable signal is an external pin, users activate it asynchronously
 * this function could be called when driver is down and w/o clock
 * it operates on different registers depending on corerev and boardflag.
 */
static bool
wlc_radio_read_hwdisabled(wlc_info_t* wlc)
{
	bool v, clk;

	/* this routine requires chip clocks - refer to wlc_xtal() for details */
	ASSERT(wlc->sbclk);

	/* may need to take core out of reset first */
	clk = wlc->core->clk;
	if (!clk) {
		sb_core_reset(wlc->pub.sbh, 0);
		/* Ensure IHR accesses are enabled */
		wlc_mctrl(wlc, MCTL_IHR_EN, MCTL_IHR_EN);
	}
	v = wlc_phy_radio_disabled((void *)wlc->regs);

	/* put core back into reset */
	if (!clk)
		sb_core_disable(wlc->pub.sbh, 0);

	return (v);
}

/* read hwdisable state and propagate to wlc flag */
void
wlc_radio_hwdisable_upd(wlc_info_t* wlc)
{
	bool xtal_set = FALSE;

	if (!wlc->sbclk) {
		wlc_xtal(wlc, ON);
		xtal_set = TRUE;
	}

	if (wlc_radio_read_hwdisabled(wlc))
		mboolset(wlc->pub.radio_disabled, WL_RADIO_HW_DISABLE);
	else
		mboolclr(wlc->pub.radio_disabled, WL_RADIO_HW_DISABLE);

	if (xtal_set)
		wlc_xtal(wlc, OFF);
}

/*
 * centralized radio disable/enable function,
 * invoke radio enable/disable after updating hwradio status
 */
static void
wlc_radio_upd(wlc_info_t* wlc)
{
	WL_MPC(("wl%d, wlc_radio_upd, radio_disabled 0x%x\n", wlc->pub.unit,
		wlc->pub.radio_disabled));
	if (wlc->pub.radio_disabled)
		wlc_radio_disable(wlc);
	else
		wlc_radio_enable(wlc);
}


/* maintain LED behavior in down state */
static void
wlc_down_led_upd(wlc_info_t *wlc)
{
	bool xtal_set = FALSE;

	ASSERT(!wlc->pub.up);

	/* maintain LEDs while in down state, turn on sbclk if not available yet */
	/* turn on sbclk if necessary */

	if (!wlc->sbclk) {
		wlc_xtal(wlc, ON);
		xtal_set = TRUE;
	}

	wlc_led_event(wlc->ledh);

	if (xtal_set)
		wlc_xtal(wlc, OFF);
}

static void
wlc_radio_disable(wlc_info_t *wlc)
{
	if (!wlc->pub.up) {
		wlc_down_led_upd(wlc);
		return;
	}

	WL_MPC(("wl%d: wlc_radio_disable, radio off\n", wlc->pub.unit));
	wlc_radio_monitor_start(wlc);
	wl_down(wlc->wl);
}

static void
wlc_radio_enable(wlc_info_t *wlc)
{
	if (wlc->pub.up)
		return;

	if (DEVICEREMOVED(wlc))
		return;

	if (!wlc->down_override) {	/* imposed by wl down/out ioctl */
		WL_MPC(("wl%d: wlc_radio_enable, radio on\n", wlc->pub.unit));
		/* wlc_up->wlc_gpio_init will take care of GPIOs control */
		wl_up(wlc->wl);
	}
}

/* periodical query hw radio button while driver is "down" */
static void
wlc_radio_timer(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;

	if (DEVICEREMOVED(wlc)) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}

	/* validate all the reasons driver could be down and running this radio_timer */
	ASSERT(wlc->pub.radio_disabled || wlc->down_override);
	wlc_radio_hwdisable_upd(wlc);
	wlc_radio_upd(wlc);
}

static bool
wlc_radio_monitor_start(wlc_info_t *wlc)
{
	if (wlc->radio_monitor)
		return TRUE;
	wlc->radio_monitor = TRUE;
	wl_add_timer(wlc->wl, wlc->radio_timer, TIMER_INTERVAL_RADIOCHK, TRUE);
	return TRUE;
}

static bool
wlc_radio_monitor_stop(wlc_info_t *wlc)
{
	if (!wlc->radio_monitor)
		return TRUE;
	wlc->radio_monitor = FALSE;
	return (wl_del_timer(wlc->wl, wlc->radio_timer));
}

static void
wlc_timer_led(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;
	if (!wlc->pub.up) {
		return;
	}

	if (DEVICEREMOVED(wlc)) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}

	wlc_led_timer(wlc->ledh);
}

static void
wlc_timer_led_blink(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;
	if (!wlc->pub.up) {
		return;
	}

	if (DEVICEREMOVED(wlc)) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}

	wlc_led_blink_timer(wlc->ledh);
}



/* common watchdog code */
static void
wlc_watchdog(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;
	d11regs_t *regs;
	int i;

	WL_TRACE(("wl%d: wlc_watchdog\n", wlc->pub.unit));

	if (!wlc->pub.up)
		return;

	if (DEVICEREMOVED(wlc)) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}

	ASSERT(!wlc->sbclk || !sb_taclear(wlc->pub.sbh));

	regs = wlc->regs;

	/* increment second count */
	wlc->pub.now++;

	/* Check for FIFO error interrupts */
	wlc_fifoerrors(wlc);

	/* radio sync: sw/hw --> radio_disable/radio_enable */
	wlc_radio_hwdisable_upd(wlc);
	wlc_radio_upd(wlc);
	/* if radio is disable, driver may be down, quit here */
	if (wlc->pub.radio_disabled)
		return;

	/* make sure RX dma has buffers */
	if (!PIO_ENAB(wlc)) {
		dma_rxfill(wlc->core->di[RX_FIFO]);
		if (D11REV_LT(wlc->pub.corerev, 5)) {
			dma_rxfill(wlc->core->di[RX_TXSTATUS_FIFO]);
		}
	}


	/* occasionally sample mac stat counters to detect 16-bit counter wrap */
	if ((WLC_UPDATE_STATS(wlc)) && (!(wlc->pub.now % SW_TIMER_MAC_STAT_UPD)))
		wlc_statsupd(wlc);



	{
		/* Ensure that the PM state is in sync. between the STA and AP */
		if (wlc->PMpending && PS_ALLOWED(wlc))
			/* send another NULL data frame to communicate PM state */
			if (!(wlc_sendnulldata(wlc, &wlc->BSSID, 0, 0, PRIO_8021D_BE)))
				WL_ERROR(("wl%d: wlc_watchdog: PM state pkt failed\n",
					wlc->pub.unit));
		/* Monitor connection to a network */
		if (wlc->pub.associated) {
			if (wlc->pub.BSS) {
				/* STA: We have previously received a CSA from the AP with the
				 * channel switch scheduled for wlc->channel_sw.secs seconds in the
				 * future. Decrement seconds until it reaches 0 and then switch
				 * channels.
				 */
				if (WL11H_ENAB(wlc) && wlc->channel_sw.secs) {
						if (--wlc->channel_sw.secs == 0) {
						WL_REGULATORY(("wl%d: %s: Honoring pending channel"
							" switch %d\n",
							wlc->pub.unit, __FUNCTION__,
							wlc->channel_sw.channel));
						wlc_11h_do_chanswitch(wlc, wlc->channel_sw.channel);
					}
				}

				/* If we had increased the frequency tracking b/w to track beacons,
				 * verify that it is working
				 */
				if ((wlc->freqtrack_attempts > 0) &&
					(wlc->freqtrack_attempts <= WLC_FREQTRACK_MIN_ATTEMPTS))
					wlc_freqtrack_verify(wlc);

				/*
				 * Too many lost beacons. Perhaps, this is one of those old
				 * 802.11b Linksys APs which has a carrier frequency jitter.
				 * If we are currently associated with a B/G band AP and the
				 * link hasn't gone down yet, increase the frequency tracking
				 * bandwidth of the 802.11b demodulator and check if we get
				 * any beacons.
				 */
				if (wlc->time_since_bcn > WLC_FREQTRACK_THRESHOLD &&
					!wlc->bcns_lost)
					wlc_freqtrack(wlc);

				/* If the link was down and we got a beacon, mark it 'up' */
				if (wlc->time_since_bcn == 0) {
					/* bcns_lost indicates whether the link is marked 'down' or
					 * not
					 */
					if (wlc->bcns_lost == TRUE) {
						wlc->bcns_lost = FALSE;
						bcopy(wlc->prev_BSSID.octet, wlc->BSSID.octet,
							ETHER_ADDR_LEN);
						bcopy(wlc->prev_BSSID.octet,
							wlc->pub.current_bss.BSSID.octet,
							ETHER_ADDR_LEN);
						wlc_link(wlc, TRUE, &wlc->BSSID, wlc->bsscfg[0]);
						WL_ASSOC(("ROAM: new beacon: called link_up() \n"));
					}
				}

				/* beacon lost: link down ? roam ?
				 *  time_since_bcn is reset to 0 in BSSinit, receive AP beacon or
				 *  direct traffic
				 */
				if (wlc->time_since_bcn > 0) {

					/* No beacon for too long time. indicate the link is down */
					if (wlc->time_since_bcn > WLC_BCN_TIMEOUT &&
					    wlc->bcns_lost == FALSE) {
						/*
						 * Clear software BSSID information so that we do
						 * not process future
						 * beacons from this AP so that the roam code will
						 * continue attempting to join
						 */
						bzero(wlc->BSSID.octet, ETHER_ADDR_LEN);
						bzero(wlc->pub.current_bss.BSSID.octet,
							ETHER_ADDR_LEN);
#if ACONF     /* RADAR */
						/* if we are in spectrum management mode and this
						 * channel is subject
						 * to radar detection rules, restore the channel's
						 * quiet bit until
						 * we reestablish contact with our AP
						 */
						if (WL11H_ENAB(wlc)) {
							if (wlc_radar_channel(wlc,
							   (uint)wlc->pub.current_bss.channel) ==
							    TRUE)
								SETQUIET(wlc,
								      wlc->pub.current_bss.channel);
						}
#endif /* ACONF */
						/* Restore the frequency tracking bandwidth if we
						 * lost our AP
						 */
						wlc_freqtrack_reset(wlc);

						WL_ASSOC(("ROAM: time_since_bcn %d >"
							" WLC_BCN_TIMEOUT: link_down() \n",
							wlc->time_since_bcn));

						wlc_link(wlc, FALSE, &wlc->prev_BSSID,
						         wlc->bsscfg[0]);
						wlc->bcns_lost = TRUE;
					}

					/* no beacon for one tick, check unaligned beacon */
					if (wlc->time_since_bcn == 1) {

						if (wlc->PMenabled && !wlc->PM_override &&
							!ETHER_ISNULLADDR(&wlc->BSSID)) {
							WL_ASSOC(("ROAM: check for unaligned"
								" TBTT\n"));
							wlc->check_for_unaligned_tbtt = TRUE;
							ASSERT(STAY_AWAKE(wlc));
							wlc_set_ps_ctrl(wlc);
						}

					/* no beacon for more than one tick, start roam */
					} else {

						if (wlc->check_for_unaligned_tbtt) {
							wlc->check_for_unaligned_tbtt = FALSE;
							wlc_set_ps_ctrl(wlc);
						}

						if (!wlc->roam_scan_block) {
							WL_ASSOC(("ROAM: wlc->time_since_bcn"
							          " %d, request roam scan\n",
							          wlc->time_since_bcn));

							wlc->roam_metric = WLC_RSSI_MINVAL;
							wlc_roam_scan(wlc, WLC_E_REASON_BCNS_LOST);
						} 
					}
				}

				if (wlc->roam_scan_block)
					wlc->roam_scan_block--;

			} else {	/* IBSS link state */
				/* if no beacon for too long time, indicate the link is down */
				if (wlc->time_since_bcn > WLC_BCN_TIMEOUT && wlc->bcns_lost ==
					FALSE) {
					wlc_mac_event(wlc, WLC_E_GOT_BEACONS, NULL,
						WLC_E_STATUS_FAIL, 0, 0, 0, 0);
					wlc->bcns_lost = TRUE;
				}
			}

			/* Increment time since last bcn but avoid wrapping. */
			if (wlc->time_since_bcn <= WLC_BCN_TIMEOUT)
				wlc->time_since_bcn++;

		}

		/* periodic IBSS gmode rate probing */
		if (!wlc->pub.BSS && wlc->band->gmode &&
		    ((wlc->pub.now % SW_TIMER_IBSS_GMODE_RATEPROBE) == 0))
			wlc_rateprobe_scan(wlc);
	} 

	/* decrement beacon detection timeouts */

	/* overlapping NonERP (legacy, non-11g) bcn */
	if (wlc->nonerp_ovlp_timeout)
		wlc->nonerp_ovlp_timeout--;
	/* member NonERP IBSS bcn */
	if (wlc->nonerp_ibss_timeout)
		wlc->nonerp_ibss_timeout--;
	/* overlapping OFDM BSS */
	if (wlc->ofdm_ovlp_timeout)
		wlc->ofdm_ovlp_timeout--;
	/* member OFDM IBSS bcn */
	if (wlc->ofdm_ibss_timeout)
		wlc->ofdm_ibss_timeout--;
	/* Beacon from a long-only preamble STA detected */
	if (wlc->longpre_detect_timeout)
		wlc->longpre_detect_timeout--;
	/* Beacon signaling Barker long preamble only mode detected */
	if (wlc->barker_detect_timeout)
		wlc->barker_detect_timeout--;
	/* Beacon signaling Use_Protection detected */
	if (wlc->protection_detect_timeout)
		wlc->protection_detect_timeout--;
	/* Beacon in IBSS signaling Use_OFDM_Protection detected */
	if (wlc->n_protection_ibss_detect_timeout)
		wlc->n_protection_ibss_detect_timeout--;


	if (!wlc->pub.BSS && wlc->pub.associated) {
		if (wlc->band->gmode)
			wlc_update_gbss_modes(wlc);
	}


	/* Manage TKIP countermeasures timers */
	for (i = 0; i < WLC_MAXBSSCFG; i++) {
		wlc_bsscfg_t *cfg = wlc->bsscfg[i];
		if (cfg == NULL)
			continue;

		if (cfg->tk_cm_dt) {
			cfg->tk_cm_dt--;
			WL_WSEC(("wl%d: wlc_watchdog: TKIP countermeasures: cfg(%d) detect timer ="
				" %d\n",
				wlc->pub.unit, i, cfg->tk_cm_dt));
		}
		if (cfg->tk_cm_bt) {
			cfg->tk_cm_bt--;
			WL_WSEC(("wl%d: wlc_watchdog: TKIP countermeasures: cfg(%d) blocking timer"
				" = %d\n",
				wlc->pub.unit, i, cfg->tk_cm_bt));
		}
	}


	/* Call any registered watchdog handlers */
	for (i = 0; i < WLC_MAXMODULES; i ++) {
		if (wlc->modulecb[i].watchdog_fn)
			wlc->modulecb[i].watchdog_fn(wlc->modulecb[i].hdl);
	}

	ASSERT(!wlc->sbclk || !sb_taclear(wlc->pub.sbh));

	/* Verify that tx_prec_map and fifos are in sync to avoid lock ups */
	ASSERT(wlc_tx_prec_map_verify(wlc));

}


static bool
wlc_update_gbss_modes(wlc_info_t *wlc)
{
	bool update = FALSE;
	bool nonerp;
	bool erp_ie_use_protection;
	int8 barker_preamble;
	bool shortslot = wlc->shortslot;

	ASSERT(wlc->band->gmode);

	nonerp = (wlc->nonerp_assoc || wlc->nonerp_ibss_timeout > 0);

	if (wlc->erp_ie_nonerp != nonerp) {
		wlc->erp_ie_nonerp = nonerp;
		update = TRUE;
	}

	/* Update current 11g protection */
	if (wlc->g_protection_override == WLC_PROTECTION_AUTO) {
		erp_ie_use_protection = FALSE;
		if ((wlc->g_protection_control == WLC_PROTECTION_CTL_LOCAL ||
		     wlc->g_protection_control == WLC_PROTECTION_CTL_OVERLAP) &&
		    nonerp) {
			/* Turn 11g protection on if NonERP STAs associated */
			erp_ie_use_protection = TRUE;
		} else if (wlc->g_protection_control == WLC_PROTECTION_CTL_OVERLAP &&
			wlc->nonerp_ovlp_timeout != 0) {
			/* Turn 11g protection on if legacy BSSs on our channel */
			erp_ie_use_protection = TRUE;
		}
		/* Use 11g protection if we are advertising protection or
		 * we have seen Use_Protection signaled in our IBSS
		 */
		wlc->g_protection = (erp_ie_use_protection || wlc->protection_detect_timeout);
	} else {
		/* an override has been specified for protection mode */
		erp_ie_use_protection = (wlc->g_protection_override == WLC_PROTECTION_ON);
		wlc->g_protection = erp_ie_use_protection;
	}

	if (wlc->erp_ie_use_protection != erp_ie_use_protection) {
		wlc->erp_ie_use_protection = erp_ie_use_protection;
		update = TRUE;
	}

	/* Update current 11g Barker Preamble Mode (short/long CCK preamble indications) */
	if (wlc->longpre_assoc || wlc->longpre_detect_timeout > 0)
		barker_preamble = WLC_BARKER_LONG_ONLY;
	else
		barker_preamble = WLC_BARKER_SHORT_ALLOWED;

	if (wlc->barker_preamble != barker_preamble) {
		wlc->barker_preamble = barker_preamble;
		update = TRUE;
	}

	if (!wlc->pub.BSS) {	/* IBSS shortslot setting */
		/* apply shortslot setting if override */
		if (wlc->shortslot_override != WLC_SHORTSLOT_AUTO)
			shortslot = (wlc->shortslot_override == WLC_SHORTSLOT_ON);
	}

	if (wlc->shortslot != shortslot) {
			wlc_switch_shortslot(wlc, shortslot);
			update = TRUE;
	}

	if (update) {
		wlc_update_brcm_ie(wlc);
		wlc_update_beacon(wlc);
		wlc_update_all_probe_resp(wlc, TRUE);
	}

	if ((wlc->barker_preamble == WLC_BARKER_LONG_ONLY) || (wlc->barker_detect_timeout > 0))
		wlc->shortpreamble = FALSE;
	else
		wlc->shortpreamble = TRUE;

	return (update);
}

/* make interface operational */
int
BCMINITFN(wlc_up)(wlc_info_t *wlc)
{
	uint j;
	uint coremask;
	bool rst[MAXCORES];
	struct rateset default_rateset;

	uint parkband;
	bool bandlock_orig;

	rst[0] = rst[1] = FALSE;

	WL_TRACE(("wl%d: %s:\n", wlc->pub.unit, __FUNCTION__));

	ASSERT(wlc->macintmask == 0);

	/*
	 * The chip state at this point is unknown.  We've allocated our
	 * software resources in wlc_attach() but the chip may be off
	 * and cores in reset (from wlc_down()) or fresh from a
	 * power-on (resume from windows hibernate) .  Assume nothing.
	 */

	/*
	 * Enable pll and xtal, initialize the power control registers,
	 * and force fastclock for the remainder of wlc_up().
	 */
	wlc_xtal(wlc, ON);
	sb_clkctl_init(wlc->pub.sbh);
	wlc_clkctl_clk(wlc, CLK_FAST);

	/*
	 * Configure pci/pcmcia here instead of in wlc_attach()
	 * to allow mfg hotswap:  down, hotswap (chip power cycle), up.
	 */
	coremask = 0;
	FOR_ACTIVE_D11_CORES(j, wlc->deviceid, wlc->ncores, NBANDS(wlc))
		coremask |= (1 << wlc->corestate[j].coreidx);

	if (BUSTYPE(wlc->pub.sbh->bustype) == PCI_BUS)
		sb_pci_setup(wlc->pub.sbh, coremask);
	ASSERT(sb_coreid(wlc->pub.sbh) == SB_D11);

	/* Init BTC related GPIOs to clean state on power up as well. This must
	 * be done here as even if radio is disabled, driver needs to
	 * make sure that output GPIO is lowered
	 */
	wlc_btc_gpio_disable(wlc);

	wlc_radio_monitor_stop(wlc);

	/*
	 * Need to read the hwradio status here to cover the case where the system
	 * is loaded with the hw radio disabled. We do not want to bring the driver up in this case.
	 * if radio is disabled, abort up, lower power, start radio timer and return 0(for NDIS)
	 * don't call radio_update to avoid looping wlc_up
	 */
	wlc_radio_hwdisable_upd(wlc);

	if (wlc->pub.radio_disabled) {
		wlc_radio_monitor_start(wlc);
		wlc_xtal(wlc, OFF);
		return 0;
	}

	/*
	 * We might have been bandlocked during down and the chip power-cycled (hibernate).
	 * figure out the right band to park on
	 */
	if (wlc->bandlocked)
		parkband = wlc->band->bandunit;	
	else if (IS_SINGLEBAND_5G(wlc->deviceid))
		parkband = 1;	/* If this is a single band 5G card, use band 1 */
	else
		parkband = 0;	/* single band g or dualband unlocked */

	/* save and release wlc->bandlocked since we are going to init both bands */
	bandlock_orig = wlc->bandlocked;
	wlc->bandlocked = FALSE;

	/* make each band operational */
	for (j = 0; j < NBANDS(wlc); j++) {
		struct ether_addr ether_local = {{2, 0, 0, 0, 0, 0}};

		/* uniband a or b/g, switch to that band directly */
		if (NBANDS(wlc) <= 1)
			j = parkband;

		wlc_setxband(wlc, j);

		/* enable bluetooth coexistance (can't be done in wlc_attach).
		 * Do it only for 2G band
		 */
		if (BAND_2G(wlc->band->bandtype))
			wlc_mhf(wlc, MHF_BTCMOD | MHF_BTCOEXIST, wlc->btc_mhf, FALSE);

		/* allocate the perm scbs after everybody has reserved their cubby */
		if (!wlc->band->bcmc_scb) {
			wlc->band->bcmc_scb = wlc_scblookup(wlc, &ether_bcast);
			if (!wlc->band->bcmc_scb)
				return 0;
			wlc->band->bcmc_scb->permanent = TRUE;
		}

		if (!wlc->band->hwrs_scb) {
			wlc->band->hwrs_scb = wlc_scblookup(wlc, &ether_local);
			if (!wlc->band->hwrs_scb)
				return 0;
			wlc->band->hwrs_scb->permanent = TRUE;
			bcopy(&wlc->band->hw_rateset, &wlc->band->hwrs_scb->rateset,
				sizeof(rateset_t));
		}

		/* each core must be reset */
		if (!rst[wlc->core->coreunit]) {
			wlc_corereset(wlc);
			rst[wlc->core->coreunit] = TRUE;
		}

		wlc_default_rateset(wlc, &default_rateset);

		/* fill in hw_rate */
		wlc_rateset_filter(&default_rateset, &wlc->band->hw_rateset,
			FALSE, FALSE, RATE_MASK);

		/* init basic rate lookup */
		wlc_rate_lookup_init(wlc, &default_rateset);


		/* init per-band channel set and initial radio channel */
		wlc_channels_init(wlc);

		/* try to restore default channel selection */
		wlc->band->pi->radio_channel = wlc->default_bss.channel;

		/* validate channel for locale/band */
		if (!VALID_CHANNEL(wlc, wlc->band->pi->radio_channel)) {
			uint8 chan;
			if ((chan = wlc_next_channel(wlc, 0, CHAN_TYPE_ANY, 0)) == INVCHANNEL) {
				chan = wlc_band_first_channel(wlc->band->pi, wlc->band->bandtype);
			}
			wlc->band->pi->radio_channel = chan;
		}

		/* Set txpwr limit */
		wlc_adjust_txpwr_limit(wlc->band->pi, wlc->band->pi->radio_channel);

		wl_init(wlc->wl);

		/* dualband: suspend and turn radio off in current band */
		if (NBANDS(wlc) > 1) {
			wlc_suspend_mac_and_wait(wlc);
			wlc_switch_radio(wlc->band->pi, OFF);
		}

	}

	wlc->pub.up = TRUE;

	/* restore parkband */
	if (NBANDS(wlc) > 1) {
		/*
		 * call wlc_setband for full band initialization
		 * it only works on dual band and requires current band being different
		 */
		if (wlc->band->bandunit == parkband)
			wlc_setxband(wlc, OTHERBANDUNIT(wlc));
		wlc_setband(wlc, parkband);
		wlc_enable_mac(wlc);
	}

	/* restore bandlock */
	wlc->bandlocked = bandlock_orig;


	/* sanitize any existing scb rates */
	wlc_scblist_validaterates(wlc);

	/* start one second watchdog */
	wl_add_timer(wlc->wl, wlc->wdtimer, TIMER_INTERVAL_WATCHDOG, TRUE);


	/* start led timer */
	wl_add_timer(wlc->wl, wlc->led_timer, LED_TIME, TRUE);

	wlc_led_event(wlc->ledh);

	/* ensure the brcm ie is up to date */
	if (wlc_update_brcm_ie(wlc)) {
		wlc_update_beacon(wlc);
	}
	/* enable dynamic power control */
	wlc_clkctl_clk(wlc, CLK_DYNAMIC);

	/*
	 * If the user wants wideband freq tracking to be always ON,
	 * set the appropriate PHY regs
	 */
	if ((wlc->freqtrack_override == FREQTRACK_ON) &&
	    ISGPHY(wlc->band->pi))
		wlc_phy_freqtrack_start(wlc->band->pi);

	ASSERT(wlc->macintmask == 0);

	ASSERT(!wlc->sbclk || !sb_taclear(wlc->pub.sbh));

	/* enable interrupts last */
	wl_intrson(wlc->wl);

	WL_MPC(("wl%d: UP\n", wlc->pub.unit));
	return (0);
}	/* wlc_up() */

/* Initialize the base precedence map for dequeueing from txq based on WME settings */
static void
BCMINITFN(wlc_tx_prec_map_init)(wlc_info_t *wlc)
{
	wlc->tx_prec_map = WLC_PREC_BMP_ALL;
	bzero(wlc->fifo2prec_map, sizeof(uint16) * NFIFO);

	/* For non-WME, both fifos have overlapping MAXPRIO. So just disable all precedences
	 * if either is full.
	 */
	if (!WME_ENAB(wlc)) {
		wlc->fifo2prec_map[TX_DATA_FIFO] = WLC_PREC_BMP_ALL;
		wlc->fifo2prec_map[TX_CTL_FIFO] = WLC_PREC_BMP_ALL;
	} else {
		wlc->fifo2prec_map[TX_AC_BK_FIFO] = WLC_PREC_BMP_AC_BK;
		wlc->fifo2prec_map[TX_AC_BE_FIFO] = WLC_PREC_BMP_AC_BE;
		wlc->fifo2prec_map[TX_AC_VI_FIFO] = WLC_PREC_BMP_AC_VI;
		wlc->fifo2prec_map[TX_AC_VO_FIFO] = WLC_PREC_BMP_AC_VO;
	}
}

/*
 * Mark the interface nonoperational, stop the software mechanisms,
 * disable the hardware, free any transient buffer state.
 * Return a count of the number of driver callbacks still pending.
 */
uint
BCMUNINITFN(wlc_down)(wlc_info_t *wlc)
{
	uint callbacks;
	uint j, i;
	bool dev_gone;

	WL_TRACE(("wl%d: %s:\n", wlc->pub.unit, __FUNCTION__));

	callbacks = 0;

	/* check if we are already in the going down path */
	if (wlc->going_down) {
		WL_ERROR(("wl%d: %s: Driver going down so return\n", wlc->pub.unit, __FUNCTION__));
		return 0;
	}
	if (!wlc->pub.up)
		return (callbacks);

	/* in between someone could try to bring down again.. */
	wlc->going_down = TRUE;

	dev_gone = DEVICEREMOVED(wlc);

	/* disable interrupts */
	if (dev_gone)
		wlc->macintmask = 0;
	else {
		/* abort any scan in progress */
		wlc_scan_abort(wlc);

		/* now disable interrupts */
		wl_intrsoff(wlc->wl);

		/* ensure we're running on the pll clock again */
		wlc_clkctl_clk(wlc, CLK_FAST);

		/* Perform STA down operations if needed */
		if (BSSCFG_STA(&wlc->cfg)) {
			/* abort any association in progress */
			wlc_assoc_abort(wlc);


			/* if config is enabled, politely try to disassociate */
			/* disable the config (STA requires active restart) */
			if (wlc->cfg.enable) {
				wlc_bsscfg_disable(wlc, &wlc->cfg);
				/* allow time for dissassociation packet to
				 * notify associated AP of our departure
				 */
				OSL_DELAY(4 * 1000);
			}
		}
	}

	/* clear the scan results.*/
	WL_ERROR(("Cleaning out the scan results\n"));
	wlc_bss_list_free(wlc, &wlc->scan_results);
	wlc_bss_list_free(wlc, &wlc->join_targets);
	wlc_bss_list_free(wlc, &wlc->custom_scan_results);

	if (wlc->pub.current_bss.bcn_prb != NULL) {
		MFREE(wlc->pub.osh, wlc->pub.current_bss.bcn_prb,
			wlc->pub.current_bss.bcn_prb_len);
		wlc->pub.current_bss.bcn_prb = NULL;
		wlc->pub.current_bss.bcn_prb_len = 0;
	}

	/* Clear out association states */
	wlc->pub.associated = FALSE;

	/* Call any registered down handlers */
	for (i = 0; i < WLC_MAXMODULES; i ++) {
		if (wlc->modulecb[i].down_fn)
			wlc->modulecb[i].down_fn(wlc->modulecb[i].hdl);
	}
	/* cancel the watchdog timer */
	if (!wl_del_timer(wlc->wl, wlc->wdtimer))
		callbacks++;

	/* cancel the led timer */
	if (!wl_del_timer(wlc->wl, wlc->led_timer))
		callbacks++;


	/* cancel any led blink timer */
	if (wlc->pub.led_blink_run) {
		if (!wl_del_timer(wlc->wl, wlc->pub.led_blink_timer))
			callbacks++;
		wlc->pub.led_blink_run = FALSE;
	}

	/* cancel any active scan */
	if (!wl_del_timer(wlc->wl, wlc->scan.timer))
		callbacks++;
	wlc->scan.channel_idx = -1;

	/* cancel quiet period */
	if (!wl_del_timer(wlc->wl, wlc->quiet.timer))
		callbacks++;
	wlc->quiet._state = 0;

	/* cancel any association timer */
	if (!wl_del_timer(wlc->wl, wlc->assoc_timer))
		callbacks++;


	wlc->pub.up = FALSE;

	/* LED cleanup */
	wlc_led_event(wlc->ledh);

	/* clear txq flow control */
	if (wlc->pub.txqstopped) {
		wlc->pub.txqstopped = OFF;
		wl_txflowcontrol(wlc->wl, OFF);
	}

	/* flush txq */
	pktq_flush(wlc->pub.osh, &wlc->txq, TRUE);
	ASSERT(pktq_empty(&wlc->txq));


	if (dev_gone) {
		/* got clock?  not if device is gone */
		wlc->sbclk = FALSE;
		wlc->corestate[0].clk = wlc->corestate[1].clk = FALSE;

		/* reclaim any posted packets */
		for (j = 0; j < NBANDS(wlc); j++) {
			wlc_flushqueues(wlc);
			if (NBANDS(wlc) > 1) {
				uint coreunit = BAND2CORE(wlc, OTHERBANDUNIT(wlc));
				wlc->core = &wlc->corestate[coreunit];
			}
		}

		goto done;
	}

	/* interrupt must have been blocked */
	ASSERT((wlc->macintmask == 0) || !wlc->pub.up);

	/* Disable GPIOs related to BTC returning the control to chipcommon */
	wlc_btc_gpio_disable(wlc);

	/*
	 * Reset and disable all cores (intentionally ignore bandlocked).
	 * Leave with the same band selected as we entered.
	 */
	for (j = 0; j < NBANDS(wlc); j++) {
		if (sb_iscoreup(wlc->pub.sbh)) {
			if (R_REG(&wlc->regs->maccontrol) & MCTL_EN_MAC)
				wlc_suspend_mac_and_wait(wlc);
			callbacks += wl_reset(wlc->wl);
			wlc_coredisable(wlc);
		}
		wlc->band->init = FALSE;
		if (NBANDS(wlc) > 1)
			wlc_setxband(wlc, OTHERBANDUNIT(wlc));
	}

done:
	/* turn off primary xtal and pll */
	if (!dev_gone)
		wlc_xtal(wlc, OFF);

	ASSERT(!wlc->sbclk || !sb_taclear(wlc->pub.sbh));

	WL_MPC(("wl%d: DOWN  callbacks %d\n", wlc->pub.unit, callbacks));
	wlc->going_down = FALSE;
	return (callbacks);
}

static int
wlc_set_rateset(wlc_info_t *wlc, struct rateset *rs_arg)
{
	struct rateset rs, new;
	uint bandunit;

	bcopy((char*)rs_arg, (char*)&rs, sizeof(struct rateset));

	/* check for bad count value */
	if ((rs.count == 0) || (rs.count > WLC_NUMRATES))
		return BCME_BADRATESET;

	/* try the current band */
	bandunit = wlc->band->bandunit;
	bcopy((char*)&rs, (char*)&new, sizeof(struct rateset));
	if (wlc_valid_rateset(&new, &wlc->bandstate[bandunit].hw_rateset))
		goto good;

	/* try the other band */
	if (IS_MBAND_UNLOCKED(wlc)) {
		bandunit = OTHERBANDUNIT(wlc);
		bcopy((char*)&rs, (char*)&new, sizeof(struct rateset));
		if (wlc_valid_rateset(&new, &wlc->bandstate[bandunit].hw_rateset))
			goto good;
	}

	return BCME_ERROR;

good:
	/* apply new rateset */
	bcopy((char*)&new, (char*)&wlc->default_bss.rateset, sizeof(struct rateset));
	bcopy((char*)&new, (char*)&wlc->bandstate[bandunit].defrateset, sizeof(struct rateset));
	return (0);
}

/* return WPA capabilities */
static int
wlc_wpa_cap(wlc_info_t *wlc, wlc_bsscfg_t *cfg, uint8 *cap, int len)
{
	ASSERT(len >= WPA_CAP_LEN);

	if (len < WPA_CAP_LEN)
		return -1;

	/* RSN capabilities */
	cap[0] = ((WME_ENAB(wlc) ? WLC_REPLAY_CNTRS_VALUE : 0) << WPA_CAP_REPLAY_CNTR_SHIFT);
	cap[1] = 0;
	return 0;
}

/* simplified integer set interface for common ioctl handler */
int
wlc_set(wlc_info_t *wlc, int cmd, int arg)
{
	return wlc_ioctl(wlc, cmd, (void *)&arg, sizeof(arg), NULL);
}

/* simplified integer get interface for common ioctl handler */
int
wlc_get(wlc_info_t *wlc, int cmd, int *arg)
{
	return wlc_ioctl(wlc, cmd, arg, sizeof(int), NULL);
}

/* common ioctl handler. return: 0=ok, -1=error, positive=particular error */
int
wlc_ioctl(wlc_info_t *wlc, int cmd, void *arg, int len, wlc_if_t *wlcif)
{
	int val, *pval;
	bool bool_val;
	int bcmerror;
	d11regs_t *regs;
	uint i;
	bool ta_ok;
	wlc_bsscfg_t *bsscfg;

	/* initialize the following to get rid of compiler warning */
	ta_ok = FALSE;

	if (DEVICEREMOVED(wlc)) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return 0;
	}

	/* check for default interface */
	if (wlcif == NULL) {
		wlcif = &wlc->bsscfg[0]->wlcif;
	}
	bsscfg = (wlcif->type == WLC_IFTYPE_BSS) ? wlcif->u.bsscfg : wlc->bsscfg[0];


	/* default argument is generic integer */
	pval = arg ? (int *) arg : NULL;
	val = pval ? *pval : 0;
	/* bool conversion to avoid duplication below */
	bool_val = val != 0;

	WL_TRACE(("wl%d: wlc_ioctl: interface %s cmd %d val 0x%x (%d) len %d\n",
		wlc->pub.unit, wl_ifname(wlc->wl, wlcif->wlif), cmd, (uint)val, val, len));
	WL_OID(("wl%d: wlc_ioctl: interface %s cmd %d val 0x%x (%d) len %d\n",
		wlc->pub.unit, wl_ifname(wlc->wl, wlcif->wlif), cmd, (uint)val, val, len));

	bcmerror = 0;
	regs = wlc->regs;



	/* All the others do */
	if ((arg == NULL) || (len <= 0)) {
		WL_ERROR(("wl%d: %s: Command %d needs buffers\n",
			wlc->pub.unit, __FUNCTION__, cmd));
		bcmerror = BCME_ERROR;
		goto done;
	}

	switch (cmd) {

	case WLC_GET_MAGIC:
		*pval = WLC_IOCTL_MAGIC;
		break;

	case WLC_GET_VERSION:
		*pval = WLC_IOCTL_VERSION;
		break;

	case WLC_GET_SSID: {
		wlc_ssid_t *ssid = (wlc_ssid_t *) arg;
		bzero((char*)ssid, sizeof(wlc_ssid_t));
		if (wlc->pub.current_bss.SSID_len) {
			ssid->SSID_len = wlc->pub.current_bss.SSID_len;
			bcopy(wlc->pub.current_bss.SSID, ssid->SSID, wlc->pub.current_bss.SSID_len);
		}
		else if (bsscfg->SSID_len) {
			ssid->SSID_len = bsscfg->SSID_len;
			bcopy( (char*)bsscfg->SSID, (char*)ssid->SSID, ssid->SSID_len);
		}
		else 
			WL_INFORM(("SSID is not yet configured\n"));
			
		break;
	}

	case WLC_SET_SSID: {
		wlc_ssid_t *ssid = (wlc_ssid_t *) arg;
		if (ssid->SSID_len > DOT11_MAX_SSID_LEN) {
			bcmerror = BCME_BADSSIDLEN;
			break;
		}
		bsscfg->SSID_len = (uint8)ssid->SSID_len;
		if (ssid->SSID_len)
			bcopy((char*)ssid->SSID, (char*)bsscfg->SSID, ssid->SSID_len);

		/* clear the stale results... */
		wlc_bss_list_free(wlc, &wlc->custom_scan_results);
		break;
	}

	case WLC_GET_CHANNEL: {
		channel_info_t *ci = (channel_info_t*)arg;
		ci->hw_channel = wlc->band->pi->radio_channel;

		if (wlc->pub.associated)
			ci->target_channel = wlc->pub.current_bss.channel;
		else
			ci->target_channel = wlc->default_bss.channel;

		ci->scan_channel = 0;

		if (SCAN_IN_PROGRESS(wlc)) {
			ci->scan_channel = wlc->scan.channel_list[wlc->scan.channel_idx];
		}
		break;
	}

	case WLC_SET_CHANNEL:
		if (val < 0 || val > MAXCHANNEL) {
			bcmerror = BCME_OUTOFRANGECHAN;
			break;
		}

		if (!VALID_CHANNEL_DB(wlc, val)) {
			bcmerror = BCME_BADCHAN;
			break;
		}

		wlc->default_bss.channel = (uint8) val;

		/* wlc_BSSinit() will sanitize the rateset before using it.. */

		if (wlc->pub.up && !wlc->pub.associated &&
		    (wlc->band->pi->radio_channel != (uint)val)) {
			wlc_suspend_mac_and_wait(wlc);
			wlc_set_channel(wlc, val);
			wlc_enable_mac(wlc);
		}
		break;

	case WLC_GET_INFRA:
		*pval = wlc->pub.BSS;
		break;

	case WLC_SET_INFRA:
		wlc->default_bss.infra = val?1:0;
		break;

	case WLC_GET_VALID_CHANNELS: {
		wl_uint32_list_t *list = (wl_uint32_list_t *)arg;
		uint8 channels[MAXCHANNEL];
		uint count;

		/* make sure the io buffer has at least room for a uint32 count */
		if (len < (int)sizeof(uint32)) {
			bcmerror = BCME_BUFTOOSHORT;
			break;
		}
		/* trim count to the buffer size in case it is bogus */
		if (list->count > (len - sizeof(uint32))/sizeof(uint32))
			list->count = (len - sizeof(uint32))/sizeof(uint32);

		/* find all valid channels */
		count = 0;
		for (i = 0; i < MAXCHANNEL; i++)
			if (VALID_CHANNEL_DB(wlc, i))
				channels[count++] = (uint8)i;

		/* check for buffer size */
		if (list->count < count) {
			/* too short, need this much */
			list->count = count;
			bcmerror = BCME_BUFTOOSHORT;
			break;
		}
		/* return the valid channels */
		for (i = 0; i < count; i++) {
			list->element[i] = channels[i];
		}
		list->count = count;
		break;
	}

	case WLC_GET_CURR_RATESET:
		if (wlc->pub.associated)
			bcopy((char*)&wlc->pub.current_bss.rateset, (char*)arg,
			      sizeof(struct rateset));
		else
			bcopy((char*)&wlc->default_bss.rateset, (char*)arg, sizeof(struct rateset));
		break;

	case WLC_GET_PHYTYPE:
		*pval = WLC_PHYTYPE(wlc->band->pi->phy_type);
		break;
	/* (re)assoc request.... */
	case WLC_REASSOC:
		/* check if STA is already associated, if it is this is considered as a Reasoc req */
		if (wlc->pub.associated) {
			/* May be a Reassoc */
			if (!((ETHER_ISBCAST((char*)arg) || ETHER_ISNULLADDR((char*)arg)))) {
				WL_ASSOC(("REASSOC: Attempting Reassociation to %s\n", bcm_ether_ntoa((struct ether_addr *)arg, eabuf)));
				if (wlc_mac_request_entry(wlc, WLC_ACTION_SCAN)) {
					wlc->assoc_type = AS_ROAM;
	WL_ERROR(("%s: %d: Assoctype state is %d\n",  __FUNCTION__, __LINE__, wlc->assoc_type ));
					wlc->roam_metric = WLC_RSSI_MINVAL;
					if (!bcmp(wlc->BSSID.octet, (char*)arg, ETHER_ADDR_LEN)) {
						/* Clear the current BSSID info to allow a roam-to-self */
						bzero(wlc->BSSID.octet, ETHER_ADDR_LEN);
						bzero(wlc->pub.current_bss.BSSID.octet, ETHER_ADDR_LEN);
					}
					wlc_assoc_scan_start(wlc, (struct ether_addr*)arg);
				} else {
					WL_ASSOC(("REASSOC: Blocked by other activity\n"));
					bcmerror = BCME_BUSY;
				}
			}
			/* ignore the bssid if it is a brodcast address or a NULL address */
		}
		else {
			if (BSSCFG_STA(bsscfg))
				bcmerror = wlc_bsscfg_enable(wlc, bsscfg);
		}
		
		break;

	case WLC_GET_BSSID:
		/* Report on primary config */
		if ((BSSCFG_STA(&wlc->cfg) && !wlc->pub.associated)) {
			bcmerror = BCME_NOTASSOCIATED;
		}
		else
			bcopy((char*)wlc->BSSID.octet, (char*)arg, ETHER_ADDR_LEN);
		break;

	case WLC_SCAN: {
		wl_scan_params_t params;
		wlc_ssid_t ssid;
		int bss_type = DOT11_BSSTYPE_ANY;
		const struct ether_addr* bssid = &ether_bcast;
		int scan_type = -1;
		int nprobes = -1;
		int active_time = -1;
		int passive_time = -1;
		int home_time = -1;
		const uint16* channel_list = NULL;
		int channel_num = 0;

		bzero(&ssid, sizeof(ssid));


		if (len >= WL_SCAN_PARAMS_FIXED_SIZE) {
			/* full wl_scan_params_t provided */
			bcopy(arg, &params, WL_SCAN_PARAMS_FIXED_SIZE);
			bcopy(&params.ssid, &ssid, sizeof(ssid));
			bssid = &params.bssid;
			bss_type = (int)params.bss_type;
			scan_type = (int)params.scan_type;
			nprobes = (int)params.nprobes;
			active_time = (int)params.active_time;
			passive_time = (int)params.passive_time;
			home_time = (int)params.home_time;
			channel_num = (int)params.channel_num;
			if ((int)(WL_SCAN_PARAMS_FIXED_SIZE + channel_num * sizeof(uint16)) > len) {
				bcmerror = BCME_BUFTOOSHORT; /* arg buffer too short */
				break;
			}
			if (channel_num > 0) {
				channel_list = (uint16*)((int8*)arg + OFFSETOF(wl_scan_params_t,
					channel_list));
				if ((uintptr)channel_list & 1) {
					/* alignment problem, bail */
					bcmerror = BCME_BADARG;
					break;
				}
			}
		} else if (len >= (int)sizeof(uint32)) {
			/* just an SSID provided */
			bcopy(arg, &ssid, MIN((int)sizeof(ssid), len));
		}
		WL_INFORM(("wl%d: Initiating Custom Scan\n", wlc->pub.unit));
		/* clean custom scan results... */
		bcmerror = wlc_scan_request(wlc, bss_type, bssid, ssid.SSID, ssid.SSID_len,
			scan_type, nprobes, active_time, passive_time, home_time,
			channel_list, channel_num,
			TRUE, wlc_custom_scan_complete, wlc);
		break;
	}

	case WLC_SCAN_RESULTS: {
		/* IO buffer has a uint32 followed by varible data.
		 * The uint32 declares the size of the buffer on input and
		 * size written on output (or required if insufficient buf space).
		 * If successful, return the bss count in uint32, followed by
		 * varible length wl_bss_info_t structs.
		 */
		wl_bss_info_t *wl_bi;
		wl_scan_results_t *iob = (wl_scan_results_t*)arg;
		int buflen;
		uint32 totallen;
		uint32 datalen;
		wlc_bss_info_t **scan_results = 0;
		uint scan_results_num = 0;

		scan_results_num = wlc->custom_scan_results.count;

		scan_results = wlc->custom_scan_results.ptrs;

		/* calc buffer length: fixed + IEs */
		totallen = WL_SCAN_RESULTS_FIXED_SIZE +
			scan_results_num * sizeof(wl_bss_info_t);
		for (i = 0; i < scan_results_num; i ++) {
			ASSERT(scan_results[i] != NULL);
			if (scan_results[i]->bcn_prb_len > DOT11_BCN_PRB_LEN)
				totallen += ROUNDUP(scan_results[i]->bcn_prb_len -
					DOT11_BCN_PRB_LEN, 4);
		}

		/* convert each wlc_bss_info_t, writing into buffer */
		iob->version = WL_BSS_INFO_VERSION;
		buflen = iob->buflen - WL_SCAN_RESULTS_FIXED_SIZE;
		datalen = WL_SCAN_RESULTS_FIXED_SIZE;
		wl_bi = iob->bss_info;
		for (i = 0; i < scan_results_num; i++) {
			if (wlc_bss2wl_bss(scan_results[i], wl_bi, buflen) == BCME_BUFTOOSHORT)
				break;
			datalen += wl_bi->length;
			buflen -= wl_bi->length;
			wl_bi = (wl_bss_info_t *)((char *)wl_bi + wl_bi->length);
		}

		/* buffer too short */
		if (iob->buflen < totallen) {
			iob->buflen = totallen;
			bcmerror = BCME_BUFTOOSHORT;
		} else
			iob->buflen = datalen;
		iob->count = i;
		break;
	}

	case WLC_SET_RATESET:
		bcmerror = wlc_set_rateset(wlc, (struct rateset *)arg);
		break;
	case WLC_GET_RATE:
                /* return rate in units of 500 Kbit/s */
		*pval = wlc_get_reprate(wlc);
		break;

	case WLC_SET_RADIO: { /* 32 bits input, higher 16 bits are mask, lower 16 bits are value to
			       * set
			       */
		uint16 radiomask, radioval;
		uint validbits = WL_RADIO_SW_DISABLE | WL_RADIO_HW_DISABLE;

		radiomask = (val & 0xffff0000) >> 16;
		radioval  = val & 0x0000ffff;

		if ((radiomask == 0) || (radiomask & ~validbits) || (radioval & ~validbits) ||
		    ((radioval & ~radiomask) != 0)) {
			WL_ERROR(("SET_RADIO with wrong bits 0x%x\n", val));
			bcmerror = BCME_RANGE;
			break;
		}

		wlc->pub.radio_disabled = (wlc->pub.radio_disabled & ~radiomask) | radioval;
		WL_MPC(("wl%d: SET_RADIO, radio_disable vector 0x%x\n", wlc->pub.unit,
			wlc->pub.radio_disabled));

		wlc_radio_hwdisable_upd(wlc);
		wlc_radio_upd(wlc);
		break;
	}

	case WLC_GET_RADIO:	/* use mask if don't want to expose some internal bits */
		*pval = wlc->pub.radio_disabled;
		break;

	case WLC_GET_LRL:
		*pval = wlc->LRL;
		break;

	case WLC_SET_LRL:
		if (!wlc->core->clk) {
			bcmerror = BCME_NOCLK;
			break;
		}

		if (val >= 1 && val <= 255) {
			wlc->LRL = (uint16)val;
			/* write retry limit to SCR, shouldn't need to suspend */
			W_REG(&regs->objaddr, OBJADDR_SCR_SEL | S_DOT11_LRC_LMT);
			W_REG(&regs->objdata, wlc->LRL);
		} else
			bcmerror = BCME_RANGE;
		break;

	case WLC_GET_SRL:
		*pval = wlc->SRL;
		break;

	case WLC_SET_SRL:
		if (!wlc->core->clk) {
			bcmerror = BCME_NOCLK;
			break;
		}

		if (val >= 1 && val <= RETRY_SHORT_MAX) {
			wlc->SRL = (uint16)val;
			/* write retry limit to SCR, shouldn't need to suspend */
			W_REG(&regs->objaddr, OBJADDR_SCR_SEL | S_DOT11_SRC_LMT);
			W_REG(&regs->objdata, wlc->SRL);
		} else
			bcmerror = BCME_RANGE;
		break;

	case WLC_GET_KEY_PRIMARY:
		if ((val >= 0) && (val < WSEC_MAX_DEFAULT_KEYS)) {
			*pval = (val == bsscfg->wsec_index) ? 1 : 0;
		} else {
			bcmerror = BCME_BADKEYIDX;
		}
		break;

	case WLC_SET_KEY_PRIMARY: {
		wsec_key_t *key;

		if ((val >= 0) && (val < WSEC_MAX_DEFAULT_KEYS) &&
		    (key = WLC_BSSCFG_WEPKEY(bsscfg, val)) != NULL) {
			if (WSEC_BSS_DEFAULT_KEY(bsscfg)) {
				WSEC_BSS_DEFAULT_KEY(bsscfg)->flags &= ~WSEC_PRIMARY_KEY;
			}
			key->flags |= WSEC_PRIMARY_KEY;
			bsscfg->wsec_index = val;
		} else {
			bcmerror = BCME_BADKEYIDX;
		}
		break;
	}

	case WLC_SET_KEY:
		bcmerror = wlc_iovar_op(wlc, "wsec_key", NULL, 0, arg, len, IOV_SET, wlcif);
		break;

	case WLC_GET_AUTH:
		*pval = (int)bsscfg->auth;
		break;

	case WLC_SET_AUTH:
		WL_ERROR(("Setting the D11 auth to %d\n", val));
		bsscfg->auth = val;
		break;


	case WLC_GET_WSEC:
		bcmerror = wlc_iovar_op(wlc, "wsec", NULL, 0, arg, len, IOV_GET, wlcif);
		break;

	case WLC_SET_WSEC:
		bcmerror = wlc_iovar_op(wlc, "wsec", NULL, 0, arg, len, IOV_SET, wlcif);
		break;

	case WLC_GET_PM:
		*pval = wlc->PM;
		break;

	case WLC_SET_PM:
		if ((val >= PM_OFF) && (val <= PM_FAST)) {
			WL_INFORM(("PS-MODE: setting PS-MODE to %d\n", val));
			wlc->PM = (uint8)val;
			/* Set pmstate only if coming out of PM mode or
			 * PM is allowed
			 */
			if (!wlc->WME_PM_blocked || val == OFF)
				wlc_set_pmstate(wlc, bool_val);
		}
		else
			bcmerror = BCME_NOTSTA;
		break;

	case WLC_GET_PROMISC:
		*pval = (int)wlc->pub.promisc;
		break;

	case WLC_SET_PROMISC:
		wlc->pub.promisc = (val != 0);

		if (!wlc->core->clk) {
			bcmerror = 0;
			break;
		}
		wlc_mac_promisc(wlc);
		break;

	case WLC_SET_VAR:
	case WLC_GET_VAR: {
		char *name;

		/* validate the name value */
		name = (char*)arg;
		for (i = 0; i < (uint)len && *name != '\0'; i++, name++)
			;

		if (i == (uint)len) {
			bcmerror = BCME_BUFTOOSHORT;
			break;
		}
		i++; /* include the null in the string length */


		if (cmd == WLC_GET_VAR)
			bcmerror = wlc_iovar_op(wlc, arg, (void*)((int8*)arg + i), len - i, arg,
				len, IOV_GET, wlcif);
		else
			bcmerror = wlc_iovar_op(wlc, arg, NULL, 0, (void*)((int8*)arg + i), len - i,
				IOV_SET, wlcif);

		break;
	}

	case WLC_DISASSOC:
		if (!wlc->core->clk) {
			bcmerror = BCME_NOCLK;
		} else {
			/* translate the WLC_DISASSOC command to disabling the config
			 * disassoc packet will be sent if currently associated
			*/
			WL_INFORM(("wl%d: WLC_DISASSOC -> wlc_bsscfg_disable()\n", wlc->pub.unit));
			wlc_bsscfg_disable(wlc, &wlc->cfg);
                }
                break;

	case WLC_SCB_DEAUTHENTICATE_FOR_REASON: {
		/* point arg at MAC addr */
		struct scb *scb;
		scb_val_t *scbval = (scb_val_t *)arg;
		struct ether_addr *apmac = &scbval->ea;
		if (ETHER_ISMULTI(apmac)) {
			WL_ERROR(("wl%d: deauth to AP,  Invalid Mac address, so BSSID\n", wlc->pub.unit));
			apmac = &wlc->BSSID;
		}
		if ((scb = wlc_scbfind(wlc, apmac)) && (scb->flags & SCB_MYAP))
			wlc_scb_resetstate(scb);
		wlc_disassociate_client(wlc, FALSE, NULL, &val);
		(void)wlc_senddeauth(wlc, apmac, apmac, scb, (uint16)scbval->val);
		WL_INFORM(("wl%d: SCB DEAUTH: wlc_bsscfg_disable()\n", wlc->pub.unit));
		wlc_bsscfg_disable(wlc, bsscfg);
		break;
	}

	case WLC_LAST:
	default:
		/* returns BCME_UNSUPPORTED if phy does not recognise the cmd */
		bcmerror = wlc_phy_ioctl(wlc->band->pi, cmd, len, pval, bool_val, arg, &ta_ok);
	}
done:

	if (bcmerror) {
		if (VALID_BCMERROR(bcmerror)) {
			wlc->pub.bcmerror = bcmerror;
		}
		else
			bcmerror = 0;

	}

	/* The sequence is significant.
	 *   Only if sbclk is TRUE, we can proceed with register access.
	 *   Even though ta_ok is TRUE, we still want to check(and clear) target abort
	 *   sb_taclear returns TRUE if there was a target abort, In this case, ta_ok must be TRUE
	 *   to avoid assert
	 *   ASSERT and sb_taclear are both under ifdef BCMDBG
	 */
	ASSERT(!wlc->sbclk || !sb_taclear(wlc->pub.sbh) || ta_ok);

	return (bcmerror);
}


int
wlc_iocbandchk(wlc_info_t *wlc, int *arg, int len, int *bands, bool clkchk)
{
	int band;

	/* get optional band */
	band = (len < (int)(2 * sizeof(int))) ? WLC_BAND_AUTO : arg[1];

	/* if band is specified, it must be either the current band or "all" */
	if ((band != WLC_BAND_AUTO) && (band != WLC_BAND_ALL) &&
	    (band != wlc->band->bandtype))
		return (BCME_BADBAND);

	/* if multiband and band is not specified, band must be locked */
	if ((band == WLC_BAND_AUTO) && IS_MBAND_UNLOCKED(wlc))
		return (BCME_NOTBANDLOCKED);

	/* no need clocks */
	if (clkchk && !wlc->core->clk)
		return (BCME_NOCLK);

	if (bands)
		*bands = band;
	return (0);
}


/* Look up the given var name in the given table */
static const wlc_iovar_t*
wlc_iovar_lookup(const wlc_iovar_t *table, const char *name)
{
	const wlc_iovar_t *vi;
	const char *lookup_name;

	/* skip any ':' delimited option prefixes */
	lookup_name = strrchr(name, ':');
	if (lookup_name != NULL)
		lookup_name++;
	else
		lookup_name = name;

	ASSERT(table);

	for (vi = table; vi->name; vi++) {
		if (!strcmp(vi->name, lookup_name))
			return vi;
	}
	/* ran to end of table */

	return NULL; /* var name not found */
}

/* simplified integer get interface for common WLC_GET_VAR ioctl handler */
int
wlc_iovar_getint(wlc_info_t *wlc, const char *name, int *arg)
{
	return wlc_iovar_op(wlc, name, NULL, 0, arg, sizeof(int32), IOV_GET, NULL);
}

/* simplified integer set interface for common WLC_SET_VAR ioctl handler */
int
wlc_iovar_setint(wlc_info_t *wlc, const char *name, int arg)
{
	return wlc_iovar_op(wlc, name, NULL, 0, (void *)&arg, sizeof(arg), IOV_SET, NULL);
}

/* simplified int8 get interface for common WLC_GET_VAR ioctl handler */
int
wlc_iovar_getint8(wlc_info_t *wlc, const char *name, int8 *arg)
{
	int iovar_int;
	int err;

	err = wlc_iovar_op(wlc, name, NULL, 0, &iovar_int, sizeof(iovar_int), IOV_GET, NULL);
	if (!err)
		*arg = (int8)iovar_int;

	return err;
}

/* simplified bool get interface for common WLC_GET_VAR ioctl handler */
int
wlc_iovar_getbool(wlc_info_t *wlc, const char *name, bool *arg)
{
	int iovar_int;
	int err;

	err = wlc_iovar_op(wlc, name, NULL, 0, &iovar_int, sizeof(iovar_int), IOV_GET, NULL);
	if (!err)
		*arg = (bool)iovar_int;

	return err;
}

/*
* register iovar table, watchdog and down handlers.
* calling function must keep 'iovars' until wlc_module_unregister is called.
* 'iovar' must have the last entry's name field being NULL as terminator.
*/
int
wlc_module_register(wlc_pub_t *pub, const wlc_iovar_t *iovars,
	char *name, void *hdl, iovar_fn_t i_fn, watchdog_fn_t w_fn, down_fn_t d_fn)
{
	wlc_info_t *wlc = (wlc_info_t *)pub;
	int i;

	ASSERT(name);
	ASSERT(i_fn || w_fn || d_fn);

	/* find an empty entry and just add, no duplication check! */
	for (i = 0; i < WLC_MAXMODULES; i ++) {
		if (wlc->modulecb[i].name[0] == '\0') {
			strcpy(wlc->modulecb[i].name, name);
			wlc->modulecb[i].iovars = iovars;
			wlc->modulecb[i].hdl = hdl;
			wlc->modulecb[i].iovar_fn = i_fn;
			wlc->modulecb[i].watchdog_fn = w_fn;
			wlc->modulecb[i].down_fn = d_fn;
			return 0;
		}
	}

	/* it is time to increase the capacity */
	ASSERT(i < WLC_MAXMODULES);
	return BCME_NORESOURCE;
}

/* unregister module callbacks */
int
wlc_module_unregister(wlc_pub_t *pub, const char *name, void *hdl)
{
	wlc_info_t *wlc = (wlc_info_t *)pub;
	int i;

	ASSERT(name);

	for (i = 0; i < WLC_MAXMODULES; i ++) {
		if (!strcmp(wlc->modulecb[i].name, name) &&
		    (wlc->modulecb[i].hdl == hdl)) {
			bzero(&wlc->modulecb[i], sizeof(modulecb_t));
			return 0;
		}
	}

	/* table not found! */
	return BCME_NOTFOUND;
}

/* Get or set an iovar.  The params/p_len pair specifies any additional
 * qualifying parameters (e.g. an "element index") for a get, while the
 * arg/len pair is the buffer for the value to be set or retrieved.
 * Operation (get/set) is specified by the last argument.
 * interface context provided by wlcif
 *
 * All pointers may point into the same buffer.
 */
int
wlc_iovar_op(wlc_info_t *wlc, const char *name,
	void *params, int p_len, void *arg, int len,
	bool set, struct wlc_if *wlcif)
{
	int err = 0;
	int val_size;
	const wlc_iovar_t *vi = NULL;
	uint32 actionid;
	int i;

	ASSERT(name);

	ASSERT(len >= 0);

	/* Get MUST have return space */
	ASSERT(set || (arg && len));

	/* Set does NOT take qualifiers */
	ASSERT(!set || (!params && !p_len));

	/* find the given iovar name */
	for (i = 0; i < WLC_MAXMODULES; i ++) {
		if (!wlc->modulecb[i].iovars)
			continue;
		if ((vi = wlc_iovar_lookup(wlc->modulecb[i].iovars, name)))
			break;
	}
	/* iovar name not found */
	if (i >= WLC_MAXMODULES) {
		err = BCME_UNSUPPORTED;
		goto exit;
	}

	WL_OID(("wl%d: table \"%s\" iovar \"%s\" %s len %d p_len %d\n", wlc->pub.unit,
		wlc->modulecb[i].name, name, (set ? "set" : "get"), len, p_len));

	/* set up 'params' pointer in case this is a set command so that
	 * the convenience int and bool code can be common to set and get
	 */
	if (params == NULL) {
		params = arg;
		p_len = len;
	}

	if (vi->type == IOVT_VOID)
		val_size = 0;
	else if (vi->type == IOVT_BUFFER)
		val_size = len;
	else
		/* all other types are integer sized */
		val_size = sizeof(int);

	/* Do the actual parameter implementation */
	actionid = set ? IOV_SVAL(vi->varid) : IOV_GVAL(vi->varid);
	err = wlc->modulecb[i].iovar_fn(wlc->modulecb[i].hdl, vi, actionid,
		name, params, p_len, arg, len, val_size, wlcif);

exit:
	return err;
}

int
wlc_iovar_check(wlc_pub_t *pub, const wlc_iovar_t *vi, void *arg, int len, bool set)
{
	wlc_info_t *wlc = (wlc_info_t *)pub;
	int err = 0;
	int32 int_val = 0;

	/* check generic condition flags */
	if (set) {
		if (((vi->flags & IOVF_SET_DOWN) && wlc->pub.up) ||
		    ((vi->flags & IOVF_SET_UP) && !wlc->pub.up)) {
			err = (wlc->pub.up ? BCME_NOTDOWN : BCME_NOTUP);
		}
		else if ((vi->flags & IOVF_SET_BAND) && IS_MBAND_UNLOCKED(wlc)) {
			err = BCME_NOTBANDLOCKED;
		}
		else if ((vi->flags & IOVF_SET_CLK) && !wlc->core->clk) {
			err = BCME_NOCLK;
		}
	} else {
		if ((vi->flags & IOVF_GET_BAND) && IS_MBAND_UNLOCKED(wlc)) {
			err = BCME_NOTBANDLOCKED;
		}
		else if ((vi->flags & IOVF_GET_CLK) && !wlc->core->clk) {
			err = BCME_NOCLK;
		}
	}
	if (err)
		goto exit;

	/* length check on io buf */
	switch (vi->type) {
	case IOVT_BOOL:
	case IOVT_INT8:
	case IOVT_INT16:
	case IOVT_INT32:
	case IOVT_UINT8:
	case IOVT_UINT16:
	case IOVT_UINT32:
		/* all integers are int32 sized args at the ioctl interface */
		if (len < (int)sizeof(int)) {
			err = BCME_BUFTOOSHORT;
		}
		/* on set, check value ranges for integer types */
		else if (set) {
			bcopy(arg, &int_val, sizeof(int));
			err = wlc_iovar_rangecheck(wlc, int_val, vi);
		}
		break;

	case IOVT_BUFFER:
		if (len < vi->minlen) {
			err = BCME_BUFTOOSHORT;
		}
		break;

	case IOVT_VOID:
		if (!set) {
			/* Cannot return nil... */
			err = BCME_UNSUPPORTED;
		} else if (len) {
			/* Set is an action w/o parameters */
			err = BCME_BUFTOOLONG;
		}
		break;

	default:
		/* unknown type for length check in iovar info */
		ASSERT(0);
		err = BCME_UNSUPPORTED;
	}

exit:
	return err;
}

/* handler for iovar table wlc_iovars */
/*
 * IMPLEMENTATION NOTE: In order to avoid checking for get/set in each
 * iovar case, the switch statement maps the iovar id into separate get
 * and set values.  If you add a new iovar to the switch you MUST use
 * IOV_GVAL and/or IOV_SVAL in the case labels to avoid conflict with
 * another case.
 */
int
wlc_doiovar(void *hdl, const wlc_iovar_t *vi, uint32 actionid, const char *name,
	void *params, uint p_len, void *arg, int len, int val_size, struct wlc_if *wlcif)
{
	wlc_info_t *wlc = hdl;
	wlc_bsscfg_t *bsscfg = &wlc->cfg;
	int err = 0;
	int32 bsscfg_idx = 0;
	int32 int_val = 0;
	int32 int_val2 = 0;
	bool bool_val;
	bool bool_val2;
	int  wsec_val;

	/* update bsscfg w/provided interface context */
	if (wlcif && (wlcif->type == WLC_IFTYPE_BSS))
		bsscfg = wlcif->u.bsscfg;

	/* handle the "bsscfg:" prefix and index arg */
	if (!strncmp("bsscfg:", name, 7)) {
		if (p_len < (int)sizeof(bsscfg_idx)) {
			err = BCME_BUFTOOSHORT;
			goto exit;
		}
		bcopy(params, &bsscfg_idx, sizeof(bsscfg_idx));

		if (IOV_ISSET(actionid))
			bsscfg = wlc_bsscfg_get(wlc, bsscfg_idx, &err);
		else
			bsscfg = wlc_bsscfg_find(wlc, bsscfg_idx, &err);

		if (err)
			goto exit;

		/* ajust the name and i/o pointers to skip over the bsscfg index
		 * parameters
		 */
		name += strlen("bsscfg:");
		params = (int8*)params + sizeof(bsscfg_idx);
		p_len -= sizeof(bsscfg_idx);
		if (IOV_ISSET(actionid)) {
			arg = (int8*)arg + sizeof(bsscfg_idx);
			len -= sizeof(bsscfg_idx);
		}
	}

	if ((err = wlc_iovar_check(&wlc->pub, vi, arg, len, IOV_ISSET(actionid))) != 0)
		return err;

	/* convenience int and bool vals for first 8 bytes of buffer */
	if (p_len >= (int)sizeof(int_val))
		bcopy(params, &int_val, sizeof(int_val));

	if (p_len >= (int)sizeof(int_val) * 2)
		bcopy((void*)((uintptr)params + sizeof(int_val)), &int_val2, sizeof(int_val));

	bool_val = (int_val != 0) ? TRUE : FALSE;
	bool_val2 = (int_val2 != 0) ? TRUE : FALSE;

	/* Do the actual parameter implementation */
	switch (actionid) {

	case IOV_GVAL(IOV_RTSTHRESH):
		int_val = (int32)wlc->RTSThresh;
		bcopy(&int_val, arg, val_size);
		break;

	case IOV_SVAL(IOV_RTSTHRESH):
		wlc->RTSThresh = (uint16)int_val;
		break;

	case IOV_GVAL(IOV_5G_RATE):
		int_val = RSPEC2RATE(wlc->bandstate[BAND_5G_INDEX].rspec_override);
		bcopy(&int_val, arg, val_size);
		break;

	case IOV_SVAL(IOV_5G_RATE):
		err = wlc_set_rate_override(wlc, WLC_BAND_5G, (int)int_val, FALSE);
		break;

	case IOV_GVAL(IOV_2G_RATE):
		int_val = RSPEC2RATE(wlc->bandstate[BAND_2G_INDEX].rspec_override);
		bcopy(&int_val, arg, val_size);
		break;

	case IOV_SVAL(IOV_2G_RATE):
		err = wlc_set_rate_override(wlc, WLC_BAND_2G, (int)int_val, FALSE);
		break;

	case IOV_GVAL(IOV_FRAGTHRESH):
		*((uint*)arg) = wlc->fragthresh[TX_DATA_FIFO];
		break;

	case IOV_SVAL(IOV_FRAGTHRESH):
	{
		int i;
		uint16 frag = (uint16)int_val;
		if (frag < DOT11_MIN_FRAG_LEN || frag > DOT11_MAX_FRAG_LEN) {
			err = BCME_RANGE;
		} else {
			for (i = 0; i < NFIFO; i++)
				wlc->fragthresh[i] = frag;
		}
		break;
	}
	case IOV_GVAL(IOV_PWSEC):
		*((uint*)arg) = bsscfg->pwsec;
		break;
	case IOV_GVAL(IOV_GWSEC):
		*((uint*)arg) = bsscfg->gwsec;
		break;
	case IOV_GVAL(IOV_WSEC):
		*((uint*)arg) = bsscfg->wsec;
		break;
	case IOV_SVAL(IOV_WSEC):
	case IOV_SVAL(IOV_PWSEC):
	case IOV_SVAL(IOV_GWSEC):
		if (actionid == IOV_SVAL(IOV_PWSEC))
			wsec_val = int_val | bsscfg->gwsec;
		else if (actionid == IOV_SVAL(IOV_GWSEC))
			wsec_val = int_val | bsscfg->pwsec;
		else 
			wsec_val = int_val;
		WL_ERROR(("setting the wsec value to %d, passed in val is %d\n", wsec_val, int_val));
		err = wlc_wsec(wlc, bsscfg, (uint32)wsec_val);
		if (!err) {
			if (actionid == IOV_SVAL(IOV_PWSEC))
				bsscfg->pwsec = int_val;
			else if (actionid == IOV_SVAL(IOV_GWSEC))
				bsscfg->gwsec = int_val;
			else {
				if (int_val == 0) {
					bsscfg->pwsec = 0;
					bsscfg->gwsec = 0;
				}
			}
		}
		break;
	case IOV_SVAL(IOV_WSEC_KEY): {
		wl_wsec_key_t key;
		bcopy(arg, &key, sizeof(wl_wsec_key_t));
		if (key.index >= WSEC_MAX_DEFAULT_KEYS) {
			WL_ERROR(("%s: Bad Key index\n", __FUNCTION__));
			err = BCME_BADKEYIDX;
			break;
		}

		if (key.len == 0) {
			WL_ERROR(("Trying to remove the key index %d\n", key.index));
			wlc_key_remove(wlc, bsscfg, &key);
		}
		else {
			wsec_iv_t key_iv;
			wsec_iv_t *initial_iv = NULL;
			if (key.iv_initialized) {
				key_iv.hi = key.rxiv.hi;
				key_iv.lo = key.rxiv.lo;
				initial_iv = &key_iv;
			}
			WL_ERROR(("Trying to add the key index %d, rxiv.hi 0x%x and rxiv.low 0x%x, key len is %d, key algo is %d\n", key.index, key_iv.hi, key_iv.lo, key.len, key.algo));
			err = wlc_key_insert(wlc, bsscfg, key.len, key.index, key.flags, key.algo,
				key.data, &key.ea, initial_iv);
		}
		break;
	}
	case IOV_GVAL(IOV_D11_AUTH):
		int_val = bsscfg->auth;
		bcopy(&int_val, arg, val_size);
		break;
	case IOV_SVAL(IOV_D11_AUTH):
		WL_ERROR(("%d: Setting the D11auth to %d\n", __LINE__, int_val));
		bsscfg->auth = int_val;
		break;
	case IOV_GVAL(IOV_WPA_AUTH):
		*((uint*)arg) = bsscfg->WPA_auth;
                break;
	case IOV_SVAL(IOV_WPA_AUTH):
		/* change of WPA_Auth modifies the PS_ALLOWED state */
		if (BSSCFG_STA(bsscfg)) {
			bool prev_psallowed = PS_ALLOWED(wlc);

			bsscfg->WPA_auth = (uint8) int_val;
			if (prev_psallowed !=  PS_ALLOWED(wlc))
				wlc_set_pmstate(wlc, wlc->PMenabled);
                }
		else 
			bsscfg->WPA_auth = (uint8)int_val;
		break;
	case IOV_GVAL(IOV_WPAIE):
		if (wlc->assoc_elts) {
			WL_ASSOC(("wl%d: copying from assocelts for WPAIE\n", wlc->pub.unit));
			len = MIN(len, wlc->assoc_elts_len);
			bcopy((const void *)wlc->assoc_elts, arg,  len);
		}
		else {
			WL_ASSOC(("wl%d: copying from wpa_info_elementj for WPAIE\n", wlc->pub.unit));
			len = MIN(len, sizeof(WPA_info_element));
			bcopy(WPA_info_element, arg,  len);
		}
		break;
	case IOV_SVAL(IOV_WPAIE):
		if (!wlc->assoc_elts) {
			WL_INFORM(("wl%d: Allocating memory for WPAIE\n", wlc->pub.unit));
			wlc->assoc_elts_len = len;
			wlc->assoc_elts = MALLOC(wlc->pub.osh, len);
		}
		if (wlc->assoc_elts)
			bcopy((const void *)arg, wlc->assoc_elts, len);
		else 
			WL_ERROR(("wl%d: SET_WPAIE: assoc_elts is null\n",wlc->pub.unit));
		break;

	case IOV_GVAL(IOV_WSEC_RESTRICT):
		*((uint*)arg) = bsscfg->wsec_restrict;
		break;

	case IOV_SVAL(IOV_WSEC_RESTRICT):
		bsscfg->wsec_restrict = bool_val;
		WL_INFORM(("wl%d: Set the wsec_restrict to %d\n", wlc->pub.unit, bsscfg->wsec_restrict));
		break;

	case IOV_GVAL(IOV_URXEAPOL_RESTRICT):
		*((uint*)arg) = bsscfg->rx_unencrypted_eapol;
		break;

	case IOV_SVAL(IOV_URXEAPOL_RESTRICT):
		bsscfg->rx_unencrypted_eapol = bool_val;
		WL_INFORM(("wl%d: Set the rx_unencrypted_eapol to %d\n", wlc->pub.unit, bsscfg->rx_unencrypted_eapol));
		break;

	case IOV_GVAL(IOV_TKIP_CMS):
		if (WSEC_TKIP_ENABLED(bsscfg->wsec))
			*((uint*)arg) = bsscfg->tkip_countermeasures;
		else 
			err = BCME_BADARG;
		break;

	case IOV_SVAL(IOV_TKIP_CMS):
		if (WSEC_TKIP_ENABLED(bsscfg->wsec))
			bsscfg->tkip_countermeasures = bool_val;
		else
			err = BCME_BADARG;
		WL_INFORM(("wl%d: Set the tkip_countermeasures to %d\n", wlc->pub.unit, bsscfg->tkip_countermeasures));
		break;
	default:
		err = BCME_UNSUPPORTED;
		break;
	}

exit:
	return err;
}


static int
wlc_iovar_rangecheck(wlc_info_t *wlc, uint32 val, const wlc_iovar_t *vi)
{
	int err = 0;
	uint32 min = 0;
	uint32 max = 0;

	/* Only ranged integers are checked */
	switch (vi->type) {
	case IOVT_INT32: max |= 0x7fffffff;
	case IOVT_INT16: max |= 0x00007fff;
	case IOVT_INT8:  max |= 0x0000007f;
		min = ~max;
		if (vi->flags & IOVF_NTRL)
			min = 1;
		else if (vi->flags & IOVF_WHL)
			min = 0;

		/* Signed values checked against above max and min */
		if ((int32)val < (int32)min || (int32)val > (int32)max) {
			err = BCME_RANGE;
		}
		break;

	case IOVT_UINT32: max |= 0xffffffff;
	case IOVT_UINT16: max |= 0x0000ffff;
	case IOVT_UINT8:  max |= 0x000000ff;

		if (vi->flags & IOVF_NTRL)
			min = 1;
		if ((val < min) || (val > max)) {
			err = BCME_RANGE;
		}
		break;
	}

	return err;
}

/* Convert a wlc_bss_info_t into an external wl_bss_info_t
 * Destination pointer does not need to be aligned.
 */
static int
wlc_bss2wl_bss(wlc_bss_info_t *bi, wl_bss_info_t *to_bi, int to_bi_len)
{
	wl_bss_info_t wl_bi;
	uint bi_length;
	int ie_length;
	uint8 *ie;

	/* user buffer should at least hold fixed portion of bss info */
	if (to_bi_len < (int) sizeof(wl_bss_info_t)) {
		WL_ERROR(("wlc_bss2wl_bss: to_bi_len %d too small\n", to_bi_len));
		return BCME_BUFTOOSHORT;
	}

	/* init ie length, point to IEs */
	if (bi->bcn_prb) {
		ie_length = bi->bcn_prb_len - DOT11_BCN_PRB_LEN;
		if (ie_length > 0)
			ie = (uint8 *)bi->bcn_prb + DOT11_BCN_PRB_LEN;
		else {
			ie_length = 0;
			ie = NULL;
		}
	}
	/* no IEs */
	else {
		ie_length = 0;
		ie = NULL;
	}

	/* check for user buffer again to see if it can take IEs */
	if (ie_length) {
		/* adjust bss info length to hold variable length IEs as well */
		bi_length = sizeof(wl_bss_info_t) + ROUNDUP(ie_length, 4);
		/* check again for user buffer length */
		if (to_bi_len < (int) bi_length) {
			bi_length = sizeof(wl_bss_info_t); /* just the fixed bss info */
			ie_length = 0;
			ie = NULL;
		}
	}
	else
		bi_length = sizeof(wl_bss_info_t);

	bzero(&wl_bi, sizeof(wl_bss_info_t));

	/* constant values for wl_bss_info */
	wl_bi.version		= WL_BSS_INFO_VERSION;
	wl_bi.length		= bi_length;

	/* simple copies from wlc_bss_info */
	wl_bi.beacon_period	= bi->beacon_period;
	wl_bi.capability	= bi->capability;
	wl_bi.channel		= bi->channel;
	wl_bi.atim_window	= bi->atim_window;
	wl_bi.dtim_period	= bi->dtim_period;
	wl_bi.RSSI		= bi->RSSI;
	wl_bi.phy_noise		= bi->phy_noise;

	bcopy(bi->BSSID.octet, wl_bi.BSSID.octet, ETHER_ADDR_LEN);

	wl_bi.SSID_len	= bi->SSID_len;
	bcopy(bi->SSID, wl_bi.SSID, wl_bi.SSID_len);

	/* limit rates to destination rate array size */
	wl_bi.rateset.count = MIN(bi->rateset.count, sizeof(wl_bi.rateset.rates));
	bcopy(bi->rateset.rates, wl_bi.rateset.rates, wl_bi.rateset.count);

	/* make capability match the infra value */
	wl_bi.capability &= ~(DOT11_CAP_ESS | DOT11_CAP_IBSS);
	if (bi->infra)
		wl_bi.capability |= DOT11_CAP_ESS;
	else
		wl_bi.capability |= DOT11_CAP_IBSS;

	/* ie length */
	wl_bi.ie_length = ie_length;

	/* copy fixed portion of the bss info structure to user buffer */
	bcopy(&wl_bi, to_bi, sizeof(wl_bss_info_t));

	/* append beacon/probe response IEs */
	if (ie_length)
		bcopy(ie, (uint8 *)to_bi + sizeof(wl_bss_info_t), ie_length);

	return 0;
}

/*
 * Read one mac counter, compare against the last value read, and
 * if it differs increment the software aggregate counter
 * and return the delta.
 */
uint
wlc_ctrupd(wlc_info_t *wlc, uint ucode_offset, uint offset)
{
#ifdef WLCNT
	uint32 v, delta;
	uint32 *total;
	uint32 *per_core;

	total = (uint32*)((char*)(&wlc->pub._cnt.txallfrm) + offset);
	per_core = (uint32*)((char*)&wlc->core->macstat_delta + offset);
	delta = 0;

	v = (uint32)wlc_read_shm(wlc, M_UCODE_MACSTAT + ucode_offset);
	if (v != *per_core) {
		if (v > *per_core)
			delta = v - *per_core;
		else
			delta = (0x10000 - *per_core) + v;
		*total += delta;
		*per_core = v;
	}
	return (delta);
#else
	return (0);
#endif /* WLCNT */
}

#define MACSTATOFF(name) ((uint)((char *)(&wlc->pub._cnt.name) - (char *)(&wlc->pub._cnt.txallfrm)))

void
wlc_statsupd(wlc_info_t *wlc)
{
	int i;

	/* if driver down, make no sense to update stats */
	if (!wlc->pub.up)
		return;


	/* update mac stats */
#ifdef WLCNT
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txallfrm), MACSTATOFF(txallfrm));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txrtsfrm), MACSTATOFF(txrtsfrm));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txctsfrm), MACSTATOFF(txctsfrm));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txackfrm), MACSTATOFF(txackfrm));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txdnlfrm), MACSTATOFF(txdnlfrm));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txbcnfrm), MACSTATOFF(txbcnfrm));
	for (i = 0; i < NFIFO; i++)
		wlc_ctrupd(wlc, OFFSETOF(macstat_t, txfunfl[i]), MACSTATOFF(txfunfl[i]));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txtplunfl), MACSTATOFF(txtplunfl));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txphyerr), MACSTATOFF(txphyerror));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxfrmtoolong), MACSTATOFF(rxfrmtoolong));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxfrmtooshrt), MACSTATOFF(rxfrmtooshrt));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxinvmachdr), MACSTATOFF(rxinvmachdr));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxbadfcs), MACSTATOFF(rxbadfcs));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxbadplcp), MACSTATOFF(rxbadplcp));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxcrsglitch), MACSTATOFF(rxcrsglitch));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxstrt), MACSTATOFF(rxstrt));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxdfrmucastmbss), MACSTATOFF(rxdfrmucastmbss));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxmfrmucastmbss), MACSTATOFF(rxmfrmucastmbss));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxcfrmucast), MACSTATOFF(rxcfrmucast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxrtsucast), MACSTATOFF(rxrtsucast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxctsucast), MACSTATOFF(rxctsucast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxackucast), MACSTATOFF(rxackucast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxdfrmocast), MACSTATOFF(rxdfrmocast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxmfrmocast), MACSTATOFF(rxmfrmocast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxcfrmocast), MACSTATOFF(rxcfrmocast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxrtsocast), MACSTATOFF(rxrtsocast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxctsocast), MACSTATOFF(rxctsocast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxdfrmmcast), MACSTATOFF(rxdfrmmcast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxmfrmmcast), MACSTATOFF(rxmfrmmcast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxcfrmmcast), MACSTATOFF(rxcfrmmcast));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxbeaconmbss), MACSTATOFF(rxbeaconmbss));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxdfrmucastobss), MACSTATOFF(rxdfrmucastobss));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxbeaconobss), MACSTATOFF(rxbeaconobss));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxrsptmout), MACSTATOFF(rxrsptmout));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, bcntxcancl), MACSTATOFF(bcntxcancl));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxf0ovfl), MACSTATOFF(rxf0ovfl));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxf1ovfl), MACSTATOFF(rxf1ovfl));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxf2ovfl), MACSTATOFF(rxf2ovfl));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txsfovfl), MACSTATOFF(txsfovfl));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, pmqovfl), MACSTATOFF(pmqovfl));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxcgprqfrm), MACSTATOFF(rxcgprqfrm));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxcgprsqovfl), MACSTATOFF(rxcgprsqovfl));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txcgprsfail), MACSTATOFF(txcgprsfail));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txcgprssuc), MACSTATOFF(txcgprssuc));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, prs_timeout), MACSTATOFF(prs_timeout));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, rxnack), MACSTATOFF(rxnack));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, frmscons), MACSTATOFF(frmscons));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txnack), MACSTATOFF(txnack));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txglitch_nack), MACSTATOFF(txglitch_nack));
	wlc_ctrupd(wlc, OFFSETOF(macstat_t, txburst), MACSTATOFF(txburst));
#endif /* WLCNT */


	/* dot11 counter update */

	WLCNTSET(wlc->pub._cnt.txrts, (wlc->pub._cnt.rxctsucast - wlc->pub._cnt.d11cnt_txrts_off));
	WLCNTSET(wlc->pub._cnt.rxcrc, (wlc->pub._cnt.rxbadfcs - wlc->pub._cnt.d11cnt_rxcrc_off));
	WLCNTSET(wlc->pub._cnt.txnocts, (wlc->pub._cnt.txrtsfrm - wlc->pub._cnt.rxctsucast) \
			- wlc->pub._cnt.d11cnt_txnocts_off);

	if (!PIO_ENAB(wlc)) {
		/* merge counters from dma module */
		for (i = 0; i < NFIFO; i++) {
			if (wlc->core->di[i]) {
				WLCNTADD(wlc->pub._cnt.txnobuf, (wlc->core->di[i])->txnobuf);
				WLCNTADD(wlc->pub._cnt.rxnobuf, (wlc->core->di[i])->rxnobuf);
				WLCNTADD(wlc->pub._cnt.rxgiant, (wlc->core->di[i])->rxgiants);
				dma_counterreset(wlc->core->di[i]);
			}
		}
	}

	/*
	 * Aggregate transmit and receive errors that probably resulted
	 * in the loss of a frame are computed on the fly.
	 */
	WLCNTSET(wlc->pub._cnt.txerror,  wlc->pub._cnt.txnobuf + wlc->pub._cnt.txnoassoc +
	       wlc->pub._cnt.txuflo + wlc->pub._cnt.txrunt + wlc->pub._cnt.dmade +
	       wlc->pub._cnt.dmada + wlc->pub._cnt.dmape);
	WLCNTSET(wlc->pub._cnt.rxerror, wlc->pub._cnt.rxoflo + wlc->pub._cnt.rxnobuf +
	       wlc->pub._cnt.rxfragerr + wlc->pub._cnt.rxrunt + wlc->pub._cnt.rxgiant +
	       wlc->pub._cnt.rxnoscb + wlc->pub._cnt.rxbadsrcmac);
	for (i = 0; i < NFIFO; i++)
		WLCNTADD(wlc->pub._cnt.rxerror, wlc->pub._cnt.rxuflo[i]);
}


void
wlc_tinydump(wlc_info_t *wlc, char *buf)
{
	char perm[32], cur[32];
	int i;

	buf += sprintf(buf, "\n");

	buf += sprintf(buf, "resets %d\n", WLCNTVAL(wlc->pub._cnt.reset));

	buf += sprintf(buf, "perm_etheraddr %s cur_etheraddr %s\n",
		bcm_ether_ntoa(&wlc->perm_etheraddr, (char*)perm),
		bcm_ether_ntoa(&wlc->pub.cur_etheraddr, (char*)cur));

	buf += sprintf(buf, "board 0x%x, board rev %d.%d", wlc->pub.sbh->boardtype,
		(wlc->pub.boardrev >> 4) & 0xf, wlc->pub.boardrev & 0xf);
	if (wlc->pub.boardrev == BOARDREV_PROMOTED)
		buf += sprintf(buf, " (actually 0x%02x)", BOARDREV_PROMOTABLE);
	buf += sprintf(buf, "\n");

	buf += sprintf(buf, "rate_override: A %d, B %d\n",
		wlc->bandstate[BAND_5G_INDEX].rspec_override,
		wlc->bandstate[BAND_2G_INDEX].rspec_override);

	buf += sprintf(buf, "antdiv_override %d", wlc->pub.antdiv_override);
	if (wlc->pub.antdiv_override == -1)
		buf += sprintf(buf, " (%d default)", ANTDIV_START_0);

	buf += sprintf(buf, " txant %d\n", wlc->txant);

	for (i = 0; i < WLC_MAXBSSCFG; i++) {
		if (wlc->bsscfg[i] == NULL)
			continue;
		buf += sprintf(buf, "\n");
		buf = wlc_dump_bsscfg(wlc, wlc->bsscfg[i], i, buf);
	}

	buf += sprintf(buf, "current_bss.BSSID %s\n",
		bcm_ether_ntoa(&wlc->pub.current_bss.BSSID, (char*)perm));

	buf += sprintf(buf, "current_bss.SSID \"");
	buf += wlc_format_ssid(buf, wlc->pub.current_bss.SSID, wlc->pub.current_bss.SSID_len);
	buf += sprintf(buf, "\"\n");

	buf += sprintf(buf, "assoc_state %d associated %d\n", wlc->assoc_state,
		wlc->pub.associated);

	return;
}


static char*
wlc_dump_bsscfg(wlc_info_t *wlc, wlc_bsscfg_t *cfg, int bsscfg_idx, char *buf)
{
	char ssidbuf[SSID_FMT_BUF_LEN];
	char ifname[32] = "";

	wlc_format_ssid(ssidbuf, cfg->SSID, cfg->SSID_len);

	strncpy(ifname, wl_ifname(wlc->wl, cfg->wlcif.wlif), 31);
	ifname[31] = '\0';

	buf += sprintf(buf, "BSS Config %d: \"%s\"\n", bsscfg_idx, ssidbuf);


	buf += sprintf(buf, "enable %d up %d wlif 0x%p \"%s\"\n",
		cfg->enable, cfg->up, cfg->wlcif.wlif, ifname);
	buf += sprintf(buf, "wsec 0x%x auth %d wsec_index %d wep_algo %d\n",
		cfg->wsec, cfg->auth, cfg->wsec_index,
		WSEC_BSS_DEFAULT_KEY(cfg) ? WSEC_BSS_DEFAULT_KEY(cfg)->algo : 0);

	return buf;
}

void
wlc_dump_txstatus(tx_status_t* txs)
{
}

#if defined(BCMDBG_DUMPS)
#define	PRVAL(name)	buf += sprintf(buf, "%s %d ", #name, WLCNTVAL(wlc->pub._cnt.name))
#define	PRNL()		buf += sprintf(buf, "\n")
#endif /* defined(BCMDBG) || defined(BCMDBG_DUMPS) */


int
wlc_format_ssid(char* buf, const uchar ssid[], int ssid_len)
{
	int i, c;
	char *p = buf;

	if (ssid_len > DOT11_MAX_SSID_LEN) ssid_len = DOT11_MAX_SSID_LEN;

	for (i = 0; i < ssid_len; i++) {
		c = (int)ssid[i];
		if (c == '\\') {
			*p++ = '\\';
			*p++ = '\\';
		} else if (bcm_isprint((uchar)c)) {
			*p++ = (char)c;
		} else {
			p += sprintf(p, "\\x%02X", c);
		}
	}
	*p = '\0';

	return (int)(p - buf);
}

void
BCMINITFN(wlc_corereset)(wlc_info_t *wlc)
{
	uint32	flags = 0;

	WL_TRACE(("wl%d: %s\n", wlc->pub.unit, __FUNCTION__));

	flags = (wlc->band->pi ? wlc->band->pi->coreflags : 0);

	wlc_corereset_flags(wlc, flags);
}

static bool
wlc_dma_rxreset(wlc_info_t *wlc, uint fifo)
{
	hnddma_t *di = wlc->core->di[fifo];

	if (D11REV_LT(wlc->pub.corerev, 12)) {
		bool rxidle = TRUE;
		uint16 rcv_frm_cnt = 0;

		W_REG(&wlc->regs->rcv_fifo_ctl, fifo << 8);
		SPINWAIT((!(rxidle = dma_rxidle(di))) &&
		         ((rcv_frm_cnt = R_REG(&wlc->regs->rcv_frm_cnt)) != 0), 50000);

		if (!rxidle && (rcv_frm_cnt != 0))
			WL_ERROR(("wl%d: %s: rxdma[%d] not idle && rcv_frm_cnt(%d) not zero\n",
			          wlc->pub.unit, __FUNCTION__, fifo, rcv_frm_cnt));

		OSL_DELAY(2000);
	}

	return (dma_rxreset(di));
}

void
BCMINITFN(wlc_corereset_flags)(wlc_info_t *wlc, uint32 flags)
{
	d11regs_t *regs;
	uint32	bits;
	uint i;

	WL_TRACE(("wl%d: %s\n", wlc->pub.unit, __FUNCTION__));

	regs = wlc->regs;

	/* reset the dma engines except first time thru */
	if (sb_iscoreup(wlc->pub.sbh)) {
		if (!PIO_ENAB(wlc)) {
			for (i = 0; i < NFIFO; i++)
				if ((wlc->core->di[i]) && (!dma_txreset(wlc->core->di[i]))) {
					WL_ERROR(("wl%d: %s: dma_txreset[%d]: cannot stop dma\n",
					          wlc->pub.unit, __FUNCTION__, i));
				}

			if ((wlc->core->di[RX_FIFO]) && (!wlc_dma_rxreset(wlc, RX_FIFO))) {
				WL_ERROR(("wl%d: %s: dma_rxreset[%d]: cannot stop dma\n",
				          wlc->pub.unit, __FUNCTION__, RX_FIFO));
			}

			if (D11REV_LT(wlc->pub.corerev, 5) && wlc->core->di[RX_TXSTATUS_FIFO] &&
			    (!wlc_dma_rxreset(wlc, RX_TXSTATUS_FIFO))) {
				WL_ERROR(("wl%d: %s: dma_rxreset[%d]: cannot stop dma\n",
				          wlc->pub.unit, __FUNCTION__, RX_TXSTATUS_FIFO));
			}
		} else {
			for (i = 0; i < NFIFO; i++)
				if (wlc->core->pio[i])
					wlc_pio_reset(wlc->core->pio[i]);
		}
	}

	bits = flags | SBTML_CE;

	bits |= SBTML_PR;


	/* reset the core */
	wlc->core->clk = FALSE;
	sb_core_reset(wlc->pub.sbh, bits);
	wlc->core->clk = TRUE;

	OSL_DELAY(PLL_DELAY);
	/* take phy out of reset */
	sb_coreflags(wlc->pub.sbh, (SBTML_PR | SBTML_FGC), SBTML_FGC);
	OSL_DELAY(1);
	sb_coreflags(wlc->pub.sbh, SBTML_FGC, 0);
	OSL_DELAY(1);

	if (wlc->band->pi)
		wlc_phy_analogon((void *)wlc->regs);

	/* IHR accesses are always enabled */
	wlc_mctrl(wlc, MCTL_IHR_EN, MCTL_IHR_EN);

	/* clear this bit for DEVICEREMOVE checking, it's set/clear once in wlc_coreinit() */
	wlc_mctrl(wlc, MCTL_PSM_JMP_0, 0);

	/* clear sw intstatus */
	wlc->macintstatus = 0;
}

/* If the ucode that supports corerev 5 is used for corerev 9 and above,
 * txfifo sizes needs to be modified(increased) since the newer cores
 * have more memory.
 */
static void
BCMINITFN(wlc_corerev_ge9_fifofixup)(wlc_info_t *wlc)
{
	d11regs_t *regs = wlc->regs;
	uint16 fifo_nu;
	uint16 txfifo_startblk, txfifo_endblk;
	uint16 txfifo_def;
	uint16 txfifo_cmd;

	/* tx fifos start at TXFIFO_START_BLK from the Base address */
	txfifo_startblk = TXFIFO_START_BLK;
	/* sequence of operations:  reset fifo, set fifo size, reset fifo */
	for (fifo_nu = 0; fifo_nu < NFIFO; fifo_nu++) {

		txfifo_endblk = txfifo_startblk + wlc->core->xmtfifo_limit[fifo_nu];
		txfifo_def = (txfifo_startblk) | ((txfifo_endblk - 1) << TXFIFO_FIFOTOP_SHIFT);
		txfifo_cmd = TXFIFOCMD_RESET_MASK | (fifo_nu << TXFIFOCMD_FIFOSEL_SHIFT);

		W_REG(&regs->xmtfifocmd, txfifo_cmd);
		W_REG(&regs->xmtfifodef, txfifo_def);
		W_REG(&regs->xmtfifocmd, txfifo_cmd);

		txfifo_startblk += wlc->core->xmtfifo_limit[fifo_nu];
	}

	/* need to propogate to shm location to be in sync since ucode/hw won't do this */
	wlc_write_shm(wlc, M_FIFOSIZE0, ((wlc->core->xmtfifo_limit[1] << 8) |
		wlc->core->xmtfifo_limit[0]));
	wlc_write_shm(wlc, M_FIFOSIZE1, ((wlc->core->xmtfifo_limit[3] << 8) |
		wlc->core->xmtfifo_limit[2]));
	wlc_write_shm(wlc, M_FIFOSIZE2, ((wlc->core->xmtfifo_limit[5] << 8) |
		wlc->core->xmtfifo_limit[4]));
	wlc_write_shm(wlc, M_FIFOSIZE3, 0x0);
}

/*
 * Initialize all the core registers.
 * (If dma mode, init tx and rx dma engines but leave the fifos disabled.)
 */
static void
BCMINITFN(wlc_coreinit)(wlc_info_t *wlc)
{
	d11regs_t *regs;
	uint bcnint, bcnint_us;
	uint i;
	uint8 buf[NFIFO];
	bool corege9_fifosz_fixup = FALSE;

	regs = wlc->regs;

	WL_TRACE(("wl%d: wlc_coreinit\n", wlc->pub.unit));

	/* do timeout fixup */
	sb_core_tofixup(wlc->pub.sbh);

	/* reset PSM */
	wlc_mctrl(wlc, ~0, (MCTL_IHR_EN | MCTL_PSM_JMP_0));

	/* write the ucode */
	if (D11REV_GE(wlc->pub.corerev, 11))
		wlc_write_ucode(wlc, d11ucode11, d11ucode11sz);
	else if (D11REV_GE(wlc->pub.corerev, 5)) {
		wlc_write_ucode(wlc, d11ucode5, d11ucode5sz);
		/* For newer cores using older ucode, fifo sizes should be fixed up */
		if (D11REV_GE(wlc->pub.corerev, 9))
			corege9_fifosz_fixup = TRUE;
	}
	else if (D11REV_IS(wlc->pub.corerev, 4))
		wlc_write_ucode(wlc, d11ucode4, d11ucode4sz);
	else
		WL_ERROR(("wl%d: %s: corerev %d is invalid", wlc->pub.unit,
		          __FUNCTION__, wlc->pub.corerev));

	/* write the PCM ucode for cores supporting AES */
	if (D11REV_IS(wlc->pub.corerev, 4))
		wlc_write_pcm(wlc, d11pcm4, d11pcm4sz);
	else
		wlc_write_pcm(wlc, d11pcm5, d11pcm5sz);

	/* let the PSM run to the suspended state, set mode to BSS STA */
	W_REG(&regs->macintstatus, -1);
	wlc_mctrl(wlc, ~0, (MCTL_IHR_EN | MCTL_INFRA | MCTL_PSM_RUN));

	/* wait for ucode to self-suspend after auto-init */
	SPINWAIT(((R_REG(&regs->macintstatus) & MI_MACSSPNDD) == 0), 1000 * 1000);
	if ((R_REG(&regs->macintstatus) & MI_MACSSPNDD) == 0)
		WL_ERROR(("wl%d: wlc_coreinit: ucode did not self-suspend!\n", wlc->pub.unit));

	wlc_gpio_init(wlc);

	/* init IHR, SHM, and SCR */
	if (D11REV_GE(wlc->pub.corerev, 11)) {
		WL_ERROR(("wl%d: corerev >= 11\n", wlc->pub.unit));
	} else if (D11REV_GE(wlc->pub.corerev, 5)) {
		if (ISAPHY(wlc->band->pi)) {
			if (sb_coreflagshi(wlc->pub.sbh, 0, 0) & SBTMH_GA)
				wlc_write_inits(wlc, d11a0g1initvals5);
			else

				wlc_write_inits(wlc, d11a0g0initvals5);
		} else
			wlc_write_inits(wlc, d11b0g0initvals5);
	} else if (D11REV_IS(wlc->pub.corerev, 4)) {
		if (ISAPHY(wlc->band->pi))
			wlc_write_inits(wlc, d11a0g0initvals4);
		else
			wlc_write_inits(wlc, d11b0g0initvals4);
	}

	/* For old ucode, txfifo sizes needs to be modified(increased) for Corerev >= 9 */
	if (corege9_fifosz_fixup == TRUE) {
		wlc_corerev_ge9_fifofixup(wlc);
	}

	/* check txfifo allocations match between ucode and driver */
	wlc_copyfrom_shm(wlc, M_FIFOSIZE0, buf, NFIFO);
	for (i = 0; i < NFIFO; i++) {
		if (buf[i] != wlc->core->xmtfifo_limit[i]) {
			WL_ERROR(("wlc_coreinit: txfifo mismattch: ucode %d, driver %d\n",
				buf[i], wlc->core->xmtfifo_limit[i]));
			ASSERT(0);
		}
	}

	/* make sure we can still talk to the mac */
	ASSERT(R_REG(&regs->maccontrol) != 0xffffffff);

	/* band-specific inits done by wlc_bsinit() */

	/* overwrite/ignore antenna swap threshold init value */
	wlc_write_shm(wlc, M_MAX_ANTCNT, ANTCNT);

	/* enable one rx interrupt per received frame */
	W_REG(&regs->intrcvlazy[0], (1 << IRL_FC_SHIFT));
	if (D11REV_LT(wlc->pub.corerev, 5))
		W_REG(&regs->intrcvlazy[3], (1 << IRL_FC_SHIFT));

	/* set the station mode (BSS STA) */
	wlc_mctrl(wlc, MCTL_AP, 0);
	wlc_mctrl(wlc, MCTL_INFRA, MCTL_INFRA);

	if (PIO_ENAB(wlc)) {
		/* set fifo mode for each VALID rx channel */
		dma_rxpiomode(&regs->fifo.f32regs.dmaregs[RX_FIFO].rcv);
		if (D11REV_LT(wlc->pub.corerev, 5)) {
			dma_rxpiomode(&regs->fifo.f32regs.dmaregs[RX_TXSTATUS_FIFO].rcv);
		}

		for (i = 0; i < NFIFO; i++)
			if (wlc->core->pio[i])
				wlc_pio_init(wlc->core->pio[i]);

		/*
		 * For D11 corerev less than 8, the h/w does not store the pad bytes in the
		 * Rx Data FIFO
		*/
		if (D11REV_LT(wlc->pub.corerev, 8))
			wlc_write_shm(wlc, M_RX_PAD_DATA_OFFSET, 0);

#ifdef IL_BIGENDIAN
		/* enable byte swapping */
		wlc_mctrl(wlc, MCTL_BIGEND, MCTL_BIGEND);
#endif /* IL_BIGENDIAN */
	}

	/* init probe response timeout */
	wlc_write_shm(wlc, M_PRS_MAXTIME, wlc->prb_resp_timeout);

	/* init bcast/mcast frameid */
	BCMCFID(wlc, INVALIDFID);

	/* overwrite default long slot timing */
	if (wlc->shortslot)
		wlc_update_slot_timing(wlc, wlc->shortslot);

	/* set up Beacon interval */
	bcnint = 0x8000;
	bcnint_us = bcnint << 10;
	W_REG(&regs->tsf_cfprep, (bcnint_us << CFPREP_CBI_SHIFT));
	W_REG(&regs->tsf_cfpstart, bcnint_us);
	W_REG(&regs->macintstatus, MI_GP1);

	/* write interrupt mask */
	W_REG(&regs->intctrlregs[RX_FIFO].intmask, DEF_RXINTMASK);
	if (D11REV_LT(wlc->pub.corerev, 5))
		W_REG(&regs->intctrlregs[RX_TXSTATUS_FIFO].intmask, DEF_RXINTMASK);

	/* allow the MAC to control the PHY clock (dynamic on/off) */
	sb_coreflags(wlc->pub.sbh, SBTML_MP, SBTML_MP);

	/* program dynamic clock control fast powerup delay register */
	if (D11REV_GE(wlc->pub.corerev, 5))
		W_REG(&regs->scc_fastpwrup_dly, sb_clkctl_fast_pwrup_delay(wlc->pub.sbh));

	/* read the ucode revision */
	wlc->ucode_rev = (wlc_read_shm(wlc, M_BOM_REV_MAJOR) << NBITS(uint16)) |
		wlc_read_shm(wlc, M_BOM_REV_MINOR);

	/* New txheader support started with BOM 297 */
	ASSERT((wlc->ucode_rev >> 16) >= 297);

	/* read the key table address */
	wlc->seckeys = wlc_read_shm(wlc, M_SECRXKEYS_PTR) * 2;

	/* tell the ucode the corerev */
	wlc_write_shm(wlc, M_MACHW_VER, (uint16)wlc->pub.corerev);

	/* write retry limits to SCR, this done after PSM init */
	W_REG(&regs->objaddr, OBJADDR_SCR_SEL | S_DOT11_SRC_LMT);
	W_REG(&regs->objdata, wlc->SRL);
	W_REG(&regs->objaddr, OBJADDR_SCR_SEL | S_DOT11_LRC_LMT);
	W_REG(&regs->objdata, wlc->LRL);

	/* write rate fallback retry limits */
	wlc_write_shm(wlc, M_SFRMTXCNTFBRTHSD, wlc->SFBL);
	wlc_write_shm(wlc, M_LFRMTXCNTFBRTHSD, wlc->LFBL);

	/* the world is new again, so is our reported rate */
	wlc_reprate_init(wlc);

	/* write ethernet address to core */
	wlc_set_mac(wlc, &wlc->pub.cur_etheraddr);
	wlc_set_bssid(wlc);

	wlc_key_hw_init_all(wlc);

	/* dma or pio initializations */
	if (!PIO_ENAB(wlc)) {
		wlc->txpend16165war = 0;

		/* init the tx dma engines */
		for (i = 0; i < NFIFO; i++) {
			if (wlc->core->di[i])
				dma_txinit(wlc->core->di[i]);
		}

		/* init the rx dma engine(s) and post receive buffers */
		dma_rxinit(wlc->core->di[RX_FIFO]);
		dma_rxfill(wlc->core->di[RX_FIFO]);
		if (D11REV_LT(wlc->pub.corerev, 5)) {
			dma_rxinit(wlc->core->di[RX_TXSTATUS_FIFO]);
			dma_rxfill(wlc->core->di[RX_TXSTATUS_FIFO]);
		}
	} else {
		for (i = 0; i < NFIFO; i++) {
			uint tmp = 0;
			if (wlc_pio_txdepthget(wlc->core->pio[i]) == 0) {
				wlc_pio_txdepthset(wlc->core->pio[i], (buf[i] << 8));

				tmp = wlc_pio_txdepthget(wlc->core->pio[i]);
				if ((D11REV_LE(wlc->pub.corerev, 7)) && tmp)
					wlc_pio_txdepthset(wlc->core->pio[i], tmp - 4);
			}
		}
	}
}

uint16
wlc_rcvfifo_limit_get(wlc_info_t *wlc)
{
	uint16 rcvfifo;

	/* determine rx fifo. no register/shm, it's hardwired in RTL */

	if (D11REV_LT(wlc->pub.corerev, 4)) {
		/* stubs for old cores, support is going away */
		rcvfifo = 1024;
	} else if (D11REV_IS(wlc->pub.corerev, 4)) {
		rcvfifo = 5 * 1024;
	} else if (D11REV_LT(wlc->pub.corerev, 8)) {
		rcvfifo = 2560;
	} else if (D11REV_LE(wlc->pub.corerev, 10)) {
		rcvfifo = 4608;
	} else if (D11REV_IS(wlc->pub.corerev, 11)) {
		rcvfifo = 10240;
	} else {
		/* chips doesn't exist yet */
		ASSERT(0);
		return 0;
	}
	return rcvfifo;
}

static void
BCMINITFN(wlc_gpio_init)(wlc_info_t *wlc)
{
	d11regs_t *regs;
	uint32 gc, gm = 0;

	regs = wlc->regs;

	/* use GPIO select 0 to get all gpio signals from the gpio out reg */
	wlc_mctrl(wlc, MCTL_GPOUT_SEL_MASK, 0);


	/*
	 * Common GPIO setup:
	 *	G0 = LED 0 = WLAN Activity
	 *	G1 = LED 1 = WLAN 2.4 GHz Radio State
	 *	G2 = LED 2 = WLAN 5 GHz Radio State
	 *	G4 = radio disable input (HI enabled, LO disabled)
	 * Boards that support BT Coexistence:
	 *	G7 = BTC
	 *	G8 = BTC
	 * Boards with chips that have fewer gpios and support BT Coexistence:
	 *	G4 = BTC
	 *	G5 = BTC
	 */
	wlc_led_init(wlc->ledh, &wlc->core->gpio_led_outen);


	/*
	 * set gpiocontrol ownership:
	 * gpio 0-3 are customer-defined led pins
	 */

	gc = gm;

	/* If board is BTC enabled and turned on, configure the pins
	 * but if btc is turned off, make sure that TX_WLAN is ACTIVE_LOW
	 */
	if (wlc->btc_mhf) {
		if (wlc->btc_mhf == MHF_BTCOEXIST) {
			gm |= gc |= BOARD_GPIO_BTC_OUT | BOARD_GPIO_BTC_IN;
			OR_REG(&regs->psm_gpio_oe, BOARD_GPIO_BTC_OUT);
		} else {
			gm |= gc |= BOARD_GPIO_BTCMOD_OUT | BOARD_GPIO_BTCMOD_IN;
			OR_REG(&regs->psm_gpio_oe, BOARD_GPIO_BTCMOD_OUT);
		}
	} else wlc_btc_gpio_disable(wlc);

	/* gpio 9 controls the PA.  ucode is responsible for wiggling out and oe */
	if (wlc->pub.boardflags & BFL_PACTRL)
		gm |= gc |= BOARD_GPIO_PACTRL;

	/* gpio 14(Xtal_up) and gpio 15(PLL_powerdown) are controlled in PCI config space */


	WL_INFORM(("wl%d: gpiocontrol mask 0x%x value 0x%x\n", wlc->pub.unit, gm, gc));

	/* apply to gpiocontrol register */
	sb_gpiocontrol(wlc->pub.sbh, gm, gc, GPIO_DRV_PRIORITY);
}

/* Turn off LEDs in the core controlling the LED gpios. */
static void
wlc_led_gcclear(wlc_info_t *wlc)
{
	bool xtal_set = FALSE;

	if (!wlc->sbclk) {
		wlc_xtal(wlc, ON);
		xtal_set = TRUE;
	}

	/* disable outen, clear out */
	if (wlc->pub.sbh) {
		sb_gpioouten(wlc->pub.sbh, wlc->core->gpio_led_outen, 0, GPIO_DRV_PRIORITY);
		sb_gpioout(wlc->pub.sbh, wlc->core->gpio_led_outen, 0, GPIO_DRV_PRIORITY);
	}

	if (xtal_set)
		wlc_xtal(wlc, OFF);
}


static void
BCMINITFN(wlc_write_ucode)(wlc_info_t *wlc, const uint32 ucode[], const uint nbytes)
{
	uint i;
	uint count;

	WL_TRACE(("wl%d: wlc_write_ucode\n", wlc->pub.unit));

	ASSERT(ISALIGNED(nbytes, sizeof(uint32)));

	count = (nbytes/sizeof(uint32));

	W_REG(&wlc->regs->objaddr, (OBJADDR_AUTO_INC | OBJADDR_UCM_SEL));
	for (i = 0; i < count; i++)
		W_REG(&wlc->regs->objdata, ucode[i]);

}

static void
BCMINITFN(wlc_write_pcm)(wlc_info_t *wlc, const uint32 pcm[], const uint nbytes)
{
	uint i;

	WL_TRACE(("wl%d: wlc_write_pcm\n", wlc->pub.unit));

	ASSERT(ISALIGNED(nbytes, sizeof(uint32)));

	W_REG(&wlc->regs->objaddr, (OBJADDR_IHR_SEL | ((WEP_PCMADDR - PIHR_BASE)/sizeof(uint16))));
	W_REG(&wlc->regs->objdata, (PCMADDR_INC | PCMADDR_UCM_SEL));
	W_REG(&wlc->regs->objaddr, (OBJADDR_IHR_SEL | ((WEP_PCMDATA - PIHR_BASE)/sizeof(uint16))));
	for (i = 0; i < (nbytes/sizeof(uint32)); i++)
		W_REG(&wlc->regs->objdata, pcm[i]);
}

static void
wlc_write_inits(wlc_info_t *wlc, const d11init_t *inits)
{
	int i;
	volatile uint8 *base;

	WL_TRACE(("wl%d: wlc_write_inits\n", wlc->pub.unit));

	base = (volatile uint8*)wlc->regs;

	for (i = 0; inits[i].addr != 0xffff; i++) {
		ASSERT((inits[i].size == 2) || (inits[i].size == 4));

		if (inits[i].size == 2)
			W_REG((uint16*)(base+inits[i].addr), inits[i].value);
		else if (inits[i].size == 4)
			W_REG((uint32*)(base+inits[i].addr), inits[i].value);
	}
}

static void
wlc_set_txant(wlc_info_t *wlc, int txant)
{
	uint16 phyctl;

	ASSERT(txant == TXANT_0 || txant == TXANT_1 || txant == TXANT_LAST_RX);

	if (txant == TXANT_0)
		wlc->phytxant = PHY_TXC_OLD_ANT_0;
	else if (txant == TXANT_1)
		wlc->phytxant = PHY_TXC_OLD_ANT_1;
	else
		wlc->phytxant = PHY_TXC_OLD_ANT_LAST;

	/* set the Response (ACK/CTS) frame phy control word */
	phyctl = wlc_read_shm(wlc, M_RSP_PCTLWD);
	phyctl = (phyctl & ~PHY_TXC_ANT_MASK) | wlc->phytxant;
	wlc_write_shm(wlc, M_RSP_PCTLWD, phyctl);

	/* set the Probe Response frame phy control word */
	phyctl = wlc_read_shm(wlc, M_CTXPRS_BLK + C_CTX_PCTLWD_POS);
	phyctl = (phyctl & ~PHY_TXC_ANT_MASK) | wlc->phytxant;
	wlc_write_shm(wlc, M_CTXPRS_BLK + C_CTX_PCTLWD_POS, phyctl);

	/* set the Beacon frame phy control word */
	phyctl = wlc_read_shm(wlc, M_BCN_PCTLWD);
	phyctl = (phyctl & ~PHY_TXC_ANT_MASK) | wlc->phytxant;
	wlc_write_shm(wlc, M_BCN_PCTLWD, phyctl);
}

static void
wlc_fifoerrors(wlc_info_t *wlc)
{
	bool fatal = FALSE;
	uint unit;
	uint intstatus, idx;
	d11regs_t *regs = wlc->regs;

	unit = wlc->pub.unit;

	for (idx = 0; idx < NFIFO; idx++) {
		/* read intstatus register and ignore any non-error bits */
		intstatus = R_REG(&regs->intctrlregs[idx].intstatus) & I_ERRORS;
		if (!intstatus)
			continue;

		WL_TRACE(("wl%d: wlc_fifoerrors: intstatus%d 0x%x\n", unit, idx, intstatus));

		if (intstatus & I_RO) {
			WL_ERROR(("wl%d: fifo %d: receive fifo overflow\n", unit, idx));
			WLCNTINCR(wlc->pub._cnt.rxoflo);
			fatal = TRUE;
		}

		if (intstatus & I_PC) {
			WL_ERROR(("wl%d: fifo %d: descriptor error\n", unit, idx));
			WLCNTINCR(wlc->pub._cnt.dmade);
			fatal = TRUE;
		}

		if (intstatus & I_PD) {
			WL_ERROR(("wl%d: fifo %d: data error\n", unit, idx));
			WLCNTINCR(wlc->pub._cnt.dmada);
			fatal = TRUE;
		}

		if (intstatus & I_DE) {
			WL_ERROR(("wl%d: fifo %d: descriptor protocol error\n", unit, idx));
			WLCNTINCR(wlc->pub._cnt.dmape);
			fatal = TRUE;
		}

		if (intstatus & I_RU) {
			WL_ERROR(("wl%d: fifo %d: receive descriptor underflow\n", unit, idx));
			WLCNTINCR(wlc->pub._cnt.rxuflo[idx]);
		}

		if (intstatus & I_XU) {
			WL_ERROR(("wl%d: fifo %d: transmit fifo underflow\n", idx, unit));
			WLCNTINCR(wlc->pub._cnt.txuflo);
			fatal = TRUE;
		}


		if (fatal) {
			wl_init(wlc->wl);	/* big hammer */
			break;
		}
		else
			W_REG(&regs->intctrlregs[idx].intstatus, intstatus);
	}
}

/* alloc fragment buf and fill with data from msdu input packet */
static void *
wlc_allocfrag(osl_t *osh, void *sdu, uint offset, uint headroom, uint frag_length, uint tailroom)
{
	void *p1;

	frag_length = MIN(frag_length, (pkttotlen(osh, sdu) - offset));

	/* one-copy: alloc fragment buffers and fill with data from msdu input pkt */


	/* alloc a new pktbuf */
	if ((p1 = PKTGET(osh, headroom + tailroom + frag_length, TRUE)) == NULL)
		return (NULL);
	PKTPULL(osh, p1, headroom);

	/* copy frags worth of data at offset into pktbuf */
	pktcopy(osh, sdu, offset, frag_length, (uchar*)PKTDATA(osh, p1));
	PKTSETLEN(osh, p1, frag_length);

	/* Currently our PKTXXX macros only handle contiguous pkts. */
	ASSERT(!PKTNEXT(osh, p1));

	return (p1);
}

/*
 * If a lifetime is configured, caculate the expiry time for
 * the packet and update the pkttag.
 */
static INLINE void
wlc_lifetime_set(wlc_info_t *wlc, struct scb *scb, void *sdu, uint32 lifetime)
{
	WLPKTTAG(sdu)->flags |= WLF_EXPTIME;
	WLPKTTAG(sdu)->exptime = R_REG(&wlc->regs->tsf_timerlow) + lifetime;

	/* In dual d11 core chips like 4306B0, the current core may be different
	 * from the target core and the tsf timers in the two cores could be
	 * out of sync. If so use the tsf timer value of the target core at band
	 * switch and adjust it for time since last band switch.
	 */
	if ((wlc->ncores > 1) && (wlc->core != &wlc->corestate[BAND2CORE(wlc, scb->bandunit)])) {
		WLPKTTAG(sdu)->exptime -= wlc->core->tsf_timerlow;
		WLPKTTAG(sdu)->exptime += wlc->corestate[OTHERBANDUNIT(wlc)].tsf_timerlow;
	}
}

/*
 * Attempts to queue a packet onto a multiple-precedence queue,
 * if necessary evicting a lower precedence packet from the queue.
 *
 * 'prec' is the precedence number that has already been mapped
 * from the packet priority.
 *
 * Returns TRUE if queued, FALSE if not.
 */
bool
wlc_prec_enq(wlc_info_t *wlc, struct pktq *q, void *pkt, int prec)
{
	void *p;
	int eprec = -1;		/* precedence to evict from */
	bool discard_oldest;



	/* Fast case, precedence queue is not full and we are also not
	 * exceeding total queue length
	 */
	if (!pktq_pfull(q, prec) && !pktq_full(q)) {
		pktq_penq(q, prec, pkt);
		return TRUE;
	}

	/* Determine precedence from which to evict packet, if any */
	if (pktq_pfull(q, prec))
		eprec = prec;
	else if (pktq_full(q)) {
		p = pktq_peek_tail(q, &eprec);
		ASSERT(p);
		if (eprec > prec)
			return FALSE;
	}

	/* Evict if needed */
	if (eprec >= 0) {
		/* Detect queueing to unconfigured precedence */
		ASSERT(!pktq_pempty(q, eprec));
		discard_oldest = AC_BITMAP_TST(wlc->wme_dp, eprec);
		if (eprec == prec && !discard_oldest)
			return FALSE;		/* refuse newer (incoming) packet */
		/* Evict packet according to discard policy */
		p = discard_oldest ? pktq_pdeq(q, eprec) : pktq_pdeq_tail(q, eprec);
		ASSERT(p);

		/* Increment wme stats */
		if (WME_ENAB(wlc)) {
			WLCNTINCR(wlc->pub._wme_cnt.tx_failed[WME_PRIO2AC(PKTPRIO(p))].packets);
			WLCNTADD(wlc->pub._wme_cnt.tx_failed[WME_PRIO2AC(PKTPRIO(p))].bytes,
			         pkttotlen(wlc->pub.osh, p));
		}

		wlc_pkt_flush(wlc, p, TX_STATUS_NO_ACK);
		WLCNTINCR(wlc->pub._cnt.txnobuf);
	}

	/* Enqueue */
	p = pktq_penq(q, prec, pkt);
	ASSERT(p);

	return TRUE;
}

static void
wlc_txq_enq(void *ctx, struct scb *scb, void *sdu, uint prec)
{
	wlc_info_t *wlc = (wlc_info_t *)ctx;
	ASSERT(pktq_max(&wlc->txq) >= WLC_DATAHIWAT);

	if (!wlc_prec_enq(wlc, &wlc->txq, sdu, prec)) {
		if (WME_ENAB(wlc))
			WL_INFORM(("wl%d: wlc_sendpkt: txq overflow\n", wlc->pub.unit));
		else
			WL_ERROR(("wl%d: wlc_sendpkt: txq overflow\n", wlc->pub.unit));
		wlc_pkt_flush(wlc, sdu, TX_STATUS_NO_ACK);
	}

	/* Check if flow control needs to be turned on after enqueing the packet
	 * Don't turn on flow control if WME is enabled. Driver would make the decision on what
	 * to drop instead of relying on stack to make the right decision
	 */
	if (!WME_ENAB(wlc)) {
		if (!wlc->pub.txqstopped && (pktq_len(&wlc->txq) >= WLC_DATAHIWAT)) {
			wlc->pub.txqstopped = ON;
			wl_txflowcontrol(wlc->wl, ON);
		}
	}
}

/*
 * Enqueues a packet on txq, then sends as many packets as possible.
 * Packets are enqueued to a maximum depth of WLC_DATAHIWAT*2.
 * tx flow control is turned on at WLC_DATAHIWAT, and off at WLC_DATAHIWAT/2.
 *
 * NOTE: it's important that the txq be able to accept at least
 * one packet after tx flow control is turned on, as this ensures
 * NDIS does not drop a packet.
 *
 * Returns TRUE if packet discarded and freed because txq was full, FALSE otherwise.
 */
bool
wlc_sendpkt(wlc_info_t *wlc, void *sdu, struct wlc_if *wlcif)
{
	struct scb *scb;
	osl_t *osh;
	struct ether_header *eh;
	struct ether_addr *dst;
	wlc_bsscfg_t *bsscfg;
	bool discarded = FALSE;
	void *pkt;

	uint32 lifetime;


	WL_TRACE(("wlc%d: wlc_sendpkt\n", wlc->pub.unit));

	/* sanity */
	ASSERT(pktq_max(&wlc->txq) >= WLC_DATAHIWAT);
	ASSERT(sdu);

	ASSERT(WLPKTTAG(sdu));

	/* figure out the bsscfg for this packet */
	if (!wlcif)
		bsscfg = &wlc->cfg;
	else if (wlcif->type == WLC_IFTYPE_BSS)
		bsscfg = wlcif->u.bsscfg;
	else {
		bsscfg = &wlc->cfg;
	}

	/* discard if we're not up or not yet part of a BSS */
	if ((!bsscfg->up || (BSSCFG_STA(bsscfg) && ETHER_ISNULLADDR(&wlc->BSSID)))) {
		WLCNTINCR(wlc->pub._cnt.txnoassoc);
		goto toss;
	}

	osh = wlc->pub.osh;
	eh = (struct ether_header*) PKTDATA(osh, sdu);

	dst = wlc->pub.BSS ? &wlc->BSSID : (struct ether_addr*)eh->ether_dhost;

	/* toss if station is not associated to the correct bsscfg. Make sure to use
	 * the band while looking up as we could be scanning on different band
	 */
	if (wlc->pub.BSS && !ETHER_ISMULTI(dst)) {
		/* Class 3 (BSS) frame */
		if (!(scb = wlc_scbfindband(wlc, dst,
		                            CHANNEL_BANDUNIT(wlc, wlc->pub.current_bss.channel))) ||
		    (scb->bsscfg != bsscfg)) {
			WL_INFORM(("wl%d: wlc_sendpkt: invalid class 3 frame to "
				"non-associated station %s\n", wlc->pub.unit,
				bcm_ether_ntoa(dst, eabuf)));
			WLCNTINCR(wlc->pub._cnt.txnoassoc);
			goto toss;
		}
	} else {
		/* Class 1 (IBSS) frame */
		if (!(scb = wlc_scblookupband(wlc, dst,
		                          CHANNEL_BANDUNIT(wlc, wlc->pub.current_bss.channel)))) {
			WL_ERROR(("wl%d: wlc_sendpkt: out of scbs", wlc->pub.unit));
			WLCNTINCR(wlc->pub._cnt.txnobuf);
			goto toss;
		}
	}

	pkt = wlc_hdr_proc(wlc, sdu, scb);

	/* Header conversion failed */
	if (pkt == NULL)
		goto toss;
	else
		sdu = pkt;

	WLPKTTAGSCBSET(sdu, scb);


	/* Set packet lifetime if configured */
	if ((lifetime = wlc->lifetime[(SCB_WME(scb) ? WME_PRIO2AC(PKTPRIO(sdu)) : AC_BE)]))
		wlc_lifetime_set(wlc, scb, sdu, lifetime);

	SCB_TX_NEXT(SCB_START, scb, sdu, WLC_PRIO_TO_PREC(PKTPRIO(sdu)));

	wlc_send_q(wlc, &wlc->txq);

	return (FALSE);

toss:
	/* Increment wme stats */
	if (WME_ENAB(wlc)) {
		WLCNTINCR(wlc->pub._wme_cnt.tx_failed[WME_PRIO2AC(PKTPRIO(sdu))].packets);
		WLCNTADD(wlc->pub._wme_cnt.tx_failed[WME_PRIO2AC(PKTPRIO(sdu))].bytes,
		         pkttotlen(wlc->pub.osh, sdu));
	}

	wlc_pkt_flush(wlc, sdu, TX_STATUS_NO_ACK);
	return (discarded);
}

/* allocate headroom buffer if necessary and convert ether frame to 8023 frame 
 * !! WARNING: HND WL driver only supports data frame of type ethernet, 802.1x or 802.3
 */
void*
wlc_hdr_proc(wlc_info_t *wlc, void *sdu, struct scb *scb)
{
	osl_t *osh;
	struct ether_header *eh;
	uint headroom;
	void *pkt;
	int prio;

	osh = wlc->pub.osh;

	/* allocate enough room once for all cases */
	headroom = TXOFF;


	prio = PKTPRIO(sdu);
	if ((uint)PKTHEADROOM(osh, sdu) < headroom || PKTSHARED(sdu)) {
		if ((pkt = PKTGET(osh, headroom, TRUE)) == NULL) {
			WL_ERROR(("PKTGET headroom %d failed\n", headroom));
			WLCNTINCR(wlc->pub._cnt.txnobuf);
			return NULL;
		}
		PKTPULL(osh, pkt, headroom);

		WLPKTTAG_INFO_MOVE(sdu, pkt);
		/* Transfer priority */
		PKTSETPRIO(pkt, prio);

		/* move ether_hdr from data buffer to header buffer */
		eh = (struct ether_header*) PKTDATA(osh, sdu);
		PKTPULL(osh, sdu, ETHER_HDR_LEN);
		PKTPUSH(osh, pkt, ETHER_HDR_LEN);
		bcopy((char*)eh, (char*)PKTDATA(osh, pkt), ETHER_HDR_LEN);

		/* chain original sdu onto newly allocated header */
		PKTSETNEXT(osh, pkt, sdu);
		sdu = pkt;
	}

	/*
	 * Optionally add an 802.1Q VLAN tag to convey non-zero priority for
	 * non-WMM associations.
	 */
	eh = (struct ether_header *)PKTDATA(osh, sdu);

	if (prio && !SCB_WME(scb)  && (wlc->vlan_mode != OFF) &&
	    (ntoh16(eh->ether_type) != ETHER_TYPE_8021Q)) {
		struct ethervlan_header *vh;
		struct ether_header da_sa;

		bcopy(eh, &da_sa, VLAN_TAG_OFFSET);
		vh = (struct ethervlan_header *)PKTPUSH(osh, sdu, VLAN_TAG_LEN);
		bcopy(&da_sa, vh, VLAN_TAG_OFFSET);

		vh->vlan_type = hton16(ETHER_TYPE_8021Q);
		vh->vlan_tag = hton16(prio << VLAN_PRI_SHIFT);	/* Priority-only Tag */
	}

	/*
	 * Original Ethernet (header length = 14):
	 * ----------------------------------------------------------------------------------------
	 * |                                                     |   DA   |   SA   | T |  Data... |
	 * ----------------------------------------------------------------------------------------
	 *                                                            6        6     2
	 *
	 * Conversion to 802.3 (header length = 22):
	 *                     (LLC includes ether_type in last 2 bytes):
	 * ----------------------------------------------------------------------------------------
	 * |                                      |   DA   |   SA   | L | LLC/SNAP | T |  Data... |
	 * ----------------------------------------------------------------------------------------
	 *                                             6        6     2       6      2
	 */

	eh = (struct ether_header *)PKTDATA(osh, sdu);
	if ((ntoh16(eh->ether_type) > ETHER_MAX_DATA)) {

		if (ntoh16(eh->ether_type) == ETHER_TYPE_802_1X)
			WLPKTTAG(sdu)->flags |= WLF_8021X;

		/* save original type in pkt tag */
		WLPKTTAG(sdu)->flags |= WLF_NON8023;
		wlc_ether_8023hdr(wlc, osh, eh, sdu);
	}

	return sdu;
}

void
wlc_send_q(wlc_info_t *wlc, struct pktq *q)
{
	void *pkt[DOT11_MAXNUMFRAGS];
	int prec;
	uint16 prec_map;
	int err, i, count;
	uint fifo;
	wlc_pkttag_t *pkttag;

	prec_map = wlc->tx_prec_map;

	/* Send all the enq'd pkts that we can.
	 * Dequeue packets with precedence with empty HW fifo only
	 */
	while (prec_map && (pkt[0] = pktq_mdeq(q, prec_map, &prec))) {
		/* 
		 * SDU using wlc_prep_sdu and PDU using wlc_prep_pdu followed by
		 * wlc_txfifo() for each of the fragments
		 */
		pkttag = WLPKTTAG(pkt[0]);
		if (pkttag->flags & WLF_MPDU) {
			count = 1;
			err = wlc_prep_pdu(wlc, pkt[0], &fifo);
		} else
			err = wlc_prep_sdu(wlc, pkt, &count, &fifo);

		/* transmit if no error */
		if (!err) {
			for (i = 0; i < count; i++)
				wlc_txfifo(wlc, fifo, pkt[i], TRUE);
		}

		if (err == BCME_BUSY) {
			pktq_penq_head(q, prec, pkt[0]);
			/* If send failed due to any other reason than a change in
			 * HW FIFO condition, quit. Otherwise, read the new prec_map!
			 */
			if (prec_map == wlc->tx_prec_map)
				break;
			prec_map = wlc->tx_prec_map;
		}
	}

	/* Check if flow control needs to be turned off after sending the packet */
	if (!(WME_ENAB(wlc))) {
		if (wlc->pub.txqstopped && (pktq_len(&wlc->txq) < WLC_DATAHIWAT / 2)) {
			wlc->pub.txqstopped = OFF;
			wl_txflowcontrol(wlc->wl, OFF);
		}
	}
}

void
wlc_txfifo(wlc_info_t *wlc, uint fifo, void *p, bool commit)
{


	/* Packet has been accepted for transmission, so clear scb pointer */
	WLPKTTAGSCBCLR(p);

	/* bump up pending count */
	if (commit)
		wlc->core->txpktpend[fifo]++;

	if (!PIO_ENAB(wlc)) {
		if (WLC_WAR16165(wlc))
			wlc_war16165(wlc, TRUE);
		(void)dma_txfast(wlc->core->di[fifo], p, commit);
	} else
		wlc_pio_tx(wlc->core->pio[fifo], p);
}

/* get original etype for converted ether frame or 8023 frame */
uint16
wlc_sdu_etype(wlc_info_t *wlc, void *sdu)
{
	uint16 etype;
	struct ether_header *eh;

	eh = (struct ether_header*) PKTDATA(wlc->pub.osh, sdu);
	if (WLPKTTAG(sdu)->flags & WLF_NON8023) {
		/* ether and llc/snap are in one continuous buffer */
		etype = *(uint16 *)((uint8*)eh + ETHER_HDR_LEN + DOT11_LLC_SNAP_HDR_LEN - 2);
		ASSERT(ntoh16(etype) > ETHER_MAX_DATA);
	} else {
		etype = eh->ether_type;
	}

	return etype;
}

/* get data/body pointer from 8023 frame(original or converted) */
uint8 *
wlc_sdu_data(wlc_info_t *wlc, void *sdu)
{
	uint body_offset;
	uint hdrlen;
	osl_t *osh;

	osh = wlc->pub.osh;

	body_offset = ETHER_HDR_LEN + DOT11_LLC_SNAP_HDR_LEN;

	/* handle chained hdr buffer and data buffer */
	hdrlen = PKTLEN(osh, sdu);
	if (body_offset >= hdrlen)
		return ((uint8*)PKTDATA(osh, PKTNEXT(osh, sdu)) + (body_offset - hdrlen));
	else
		return ((uint8*)PKTDATA(osh, sdu) + body_offset);
}

/*
 * Common transmit packet routine
 * Return 0 when a packet is accepted and should not be referenced again
 * by the caller -- wlc will free it.
 * Return an error code when we're flow-controlled and the caller should try again later..
 */

int
wlc_prep_sdu(wlc_info_t *wlc, void **pkts, int *npkts, uint *fifop)
{
	struct scb *scb;
	uint i, j, nfrags, frag_length, next_payload_len, fifo;
	void *sdu;
	uint offset;
	osl_t *osh;
	uint headroom, want_tailroom;
	wsec_key_t *key;
	uint pkt_length;
	uint8 prio = 0;
	struct ether_header *eh;
	bool is_8021x;
	bool fast_path;
	wlc_bsscfg_t *bsscfg;
	wlc_pkttag_t *pkttag;

	/* Make sure that we are passed in an SDU */
	sdu = pkts[0];
	pkttag = WLPKTTAG(sdu);
	ASSERT(sdu);
	ASSERT((pkttag->flags & WLF_MPDU) == 0);

	if (!wlc->pub.up) {
		WLCNTINCR(wlc->pub._cnt.txnoassoc);
		goto toss;
	}

	/* Something is blocking data packets */
	if (wlc->block_datafifo)
		return BCME_BUSY;

	osh = wlc->pub.osh;
	eh = (struct ether_header*) PKTDATA(osh, sdu);

	is_8021x = (WLPKTTAG(sdu)->flags & WLF_8021X);

	scb = WLPKTTAGSCBGET(sdu);

	bsscfg = SCB_BSSCFG(scb);

	/* toss if station is not associated to the correct bsscfg */
	if (wlc->pub.BSS  && !SCB_ISMULTI(scb)) {
		/* Class 3 (BSS) frame */
		if (!SCB_ASSOCIATED(scb)) {
			WL_INFORM(("wl%d: wlc_prep_sdu, invalid class 3 frame to "
				   "non-associated station %s\n", wlc->pub.unit,
				    bcm_ether_ntoa(&scb->ea, eabuf)));
			WLCNTINCR(wlc->pub._cnt.txnoassoc);
			goto toss;
		}
	}

	/* Toss the frame if scb's band does not match our current band
	 * this is possible if the STA has roam'ed while the packet was on txq
	 */
	if (scb->bandunit != CHANNEL_BANDUNIT(wlc, wlc->pub.current_bss.channel)) {
		WL_INFORM(("wl%d: frame destined to %s sent on incorrect band %d\n", wlc->pub.unit,
		           bcm_ether_ntoa(&scb->ea, eabuf),
		           scb->bandunit));
		WLCNTINCR(wlc->pub._cnt.txnoassoc);
		goto toss;
	}

	/*
	 * On the AP, bcast/mcast always goes through the bcmc fifo if bcmc_scb is in PS Mode.
	 * bcmc_scb is in PS Mode if at least one ucast STA is in PS.  SDUs are data
	 * packets and always go in BE_FIFO unless WME is enabled.
	 *
	 * Default prio has to be 0 for TKIP MIC calculation when sending packet on a non-WME
	 * link, as the receiving end uses prio of 0 for MIC calculation when QoS header
	 * is absent from the packet.
	 */

	prio = 0;
	fifo = TX_AC_BE_FIFO;

	if (SCB_WME(scb)) {
		prio = (uint8)PKTPRIO(sdu);
		ASSERT(prio <= MAXPRIO);
		fifo = wme_prio2fifo[prio];

		/*
		 * Apply AC admission control.  The policy for choosing an
		 * alternate AC is not defined in the WME spec.
		 */
		if (AC_BITMAP_TST(wlc->wme_admctl, wme_fifo2ac[fifo])) {
			do {
				if (fifo == 0) {
					WL_ERROR(("wl%d: wlc_prep_sdu: "
					          "no AC available; tossing pkt\n",
					          wlc->pub.unit));
					WLCNTINCR(wlc->pub._cnt.txnobuf);
					goto toss;
				}

				fifo--;
			} while (AC_BITMAP_TST(wlc->wme_admctl, wme_fifo2ac[fifo]));

			/*
			 * Note: prio in packet is NOT updated.
			 * Original prio is also used below to determine APSD trigger AC.
			 */
		}
	}

	*fifop = fifo;

	/*
	 * Apply FIFO admission control to reduce "rate lag"
	 * by limiting the amount of data queued to the hardware.
	 */
	if ((wlc->core->txpktpend[fifo] >= wlc->pub.txmaxpkts) && (fifo <= TX_AC_VO_FIFO)) {
		/* Mark precedences related to this FIFO, unsendable */
		WLC_TX_FIFO_CLEAR(wlc, fifo);
		return BCME_BUSY;
	}
	/* toss runt frames */
	pkt_length = pkttotlen(osh, sdu);
	if (pkt_length < ETHER_HDR_LEN + DOT11_LLC_SNAP_HDR_LEN) {
		WLCNTINCR(wlc->pub._cnt.txrunt);
		goto toss;
	}

	/* toss if station not yet authorized to receive non-802.1X frames */
	if (bsscfg->eap_restrict && !SCB_ISMULTI(scb) && !SCB_AUTHORIZED(scb)) {
		if (!is_8021x) {
			WL_ERROR(("wl%d: non-802.1X frame to unauthorized station %s\n",
				wlc->pub.unit, bcm_ether_ntoa(&scb->ea, eabuf)));
			goto toss;
		}
	}

	WL_PRUSR("tx", (uchar*)eh, ETHER_HDR_LEN);

	key = NULL;
	if (WSEC_ENABLED(bsscfg->wsec)) {
		if (is_8021x) {
			/*
			* use per scb WPA_auth to handle 802.1x frames differently
			* in WPA/802.1x mixed mode when having a pairwise key:
			*  - 802.1x frames are unencrypted in plain 802.1x mode
			*  - 802.1x frames are encrypted in WPA mode
			*/
			uint8 WPA_auth = scb->WPA_auth;

			if ((WPA_auth != WPA_AUTH_DISABLED) &&
			    scb->key) {
				WL_WSEC(("wl%d: wlc_prep_sdu: encrypting 802.1X frame using "
					"per-path key\n", wlc->pub.unit));
				key = scb->key;
			} else {
				/* Do not encrypt 802.1X packets with group keys */
				WL_WSEC(("wl%d: wlc_prep_sdu: not encrypting 802.1x frame\n",
					wlc->pub.unit));
				key = NULL;
			}
		} else {
			/* Use a paired key or primary group key if present, toss otherwise */
			if (scb->key) {
				key = scb->key;
				WL_WSEC(("wl%d: wlc_prep_sdu: using per-path key, algo %d\n",
					wlc->pub.unit, key->algo));
			} else if ((key = WSEC_BSS_DEFAULT_KEY(bsscfg)) != NULL) {
				WL_WSEC(("wl%d: wlc_prep_sdu: using group key at key-ID %d, "
				         "algo %d\n",
				         wlc->pub.unit, bsscfg->wsec_index, key->algo));
			} else {	
				WL_WSEC(("wl%d: wlc_prep_sdu: tossing unencryptable frame\n",
					wlc->pub.unit));
				goto toss;
			}
		}
	} else {	/* Do not encrypt packet */
		WL_WSEC(("wl%d: wlc_prep_sdu: not encrypting frame, encryption disabled\n",
			wlc->pub.unit));
		WL_WSEC(("wl%d: wlc_prep_sdu: wsec 0x%x \n", wlc->pub.unit, bsscfg->wsec));
		key = NULL;
	}

	/* TKIP MIC space reservation */
	if (key && (key->algo == CRYPTO_ALGO_TKIP))
		pkt_length += TKIP_MIC_SIZE;

	/* calculate how many frags needed */
	nfrags = wlc_frag(wlc, scb, fifo, pkt_length, &frag_length);
	if (!PIO_ENAB(wlc)) {
		/* bail if there is insufficient room in the ring to hold all fragments */
		if (*wlc->core->txavail[fifo] < (nfrags * 4)) {
			/* Mark precedences related to this FIFO, unsendable */
			WLC_TX_FIFO_CLEAR(wlc, fifo);
			return BCME_BUSY;
		}
	} else {
		uint sz;
		/* per frag header overhead */
		sz = D11_TXH_LEN + D11_PHY_HDR_LEN + DOT11_A3_HDR_LEN;
		/* WME: account for QoS Control Field */
		if (SCB_WME(scb))
			sz += DOT11_QOS_LEN;
		/* per frag encrypt overhead */
		if (key)
			sz += key->iv_len;

		/* total frag overhead */
		sz = sz * nfrags;
		sz += pkt_length - ETHER_HDR_LEN;
		/* bail if there is insufficient room in the fifo to hold all fragments */
		if (!wlc_pio_txavailable(wlc->core->pio[fifo], sz, nfrags)) {
			/* Mark precedences related to this FIFO, unsendable */
			WLC_TX_FIFO_CLEAR(wlc, fifo);
			return BCME_BUSY;
		}
	}

	fast_path = (nfrags == 1);
	want_tailroom = 0;

	if (key && key->algo == CRYPTO_ALGO_TKIP) {
		fast_path = FALSE;
		/* external tkip_encypt needs 4 and tkip_mic add needs 8, so 12 */
		want_tailroom = 12; 
	}

	/*
	 * Prealloc all fragment buffers for this frame.
	 * If any of the allocs fail, free them all and bail.
	 * p->data points to ETHER_HEADER, original ether_type is passed separately
	 */
	offset = ETHER_HDR_LEN;
	for (i = 0; i < nfrags; i++) {
		headroom = TXOFF;

		if (fast_path) {
			/* fast path for non fragmentation/non copy of pkts
			 * header buffer has been allocated in wlc_sendpkt()
			 * don't change ether_header at the front
			 */
			pkts[i] = sdu;

		} else {
			/* fragmentation: 
			 * realloc new buffer for each frag, copy over data,
			 * append original ether_header at the front
			 */
			pkts[i] = wlc_allocfrag(osh, sdu, offset, headroom, frag_length,
				want_tailroom);
			if (pkts[i] == NULL) {
				for (j = 0; j < i; j++)
					PKTFREE(osh, pkts[j], TRUE);
				WL_ERROR(("wl%d: wlc_prep_sdu: allocfrag failed\n", wlc->pub.unit));
				WLCNTINCR(wlc->pub._cnt.txnobuf);
				return BCME_BUSY;
			} else {
				PKTPUSH(osh, pkts[i], ETHER_HDR_LEN);
				bcopy((char*)eh, (char*)PKTDATA(osh, pkts[i]), ETHER_HDR_LEN);

				/* Transfer SDU's pkttag info to the last fragment */
				if (i == (nfrags - 1))
					WLPKTTAG_INFO_MOVE(sdu, pkts[i]);
			}
		}

		/* set fragment priority */
		PKTSETPRIO(pkts[i], prio);

		/* leading frag buf must be aligned 0-mod-2 */
		ASSERT(ISALIGNED((uintptr)PKTDATA(osh, pkts[i]), 2));
		offset += frag_length;
	}

	wlc->PMidlebcns = 0;

	if (wlc->PMenabled) {
		/* Leave PM if in fast ps mode */
		if ((wlc->PM == PM_FAST) && !wlc->PMblocked)
			wlc_set_pmstate(wlc, FALSE);
	}

	/* build and transmit each fragment */
	for (i = 0; i < nfrags; i++) {
		if (i < nfrags - 1)
			next_payload_len = pkttotlen(osh, pkts[i + 1]);
		else
			next_payload_len = 0;

		/* ether header is on front of each frag to provide the dhost and shost
		 *  ether_type may not be original, should not be used
		 */
		wlc_dofrag(wlc, pkts[i], i, nfrags, next_payload_len, osh, bsscfg, scb,
			is_8021x, fifo, key, prio);
	}

	/* Free the original SDU if not shared */
	if (pkts[nfrags - 1] != sdu)
		PKTFREE(wlc->pub.osh, sdu, TRUE);

	*npkts = nfrags;

	wlc->counter++;

	/* update stat counters */
	WLCNTINCR(wlc->pub._cnt.txframe);
	WLCNTADD(wlc->pub._cnt.txbyte, pkt_length);

	if (WME_ENAB(wlc)) {
		WLCNTINCR(wlc->pub._wme_cnt.tx[WME_PRIO2AC(prio)].packets);
		WLCNTADD(wlc->pub._wme_cnt.tx[WME_PRIO2AC(prio)].bytes, pkt_length);
	}
	return 0;
toss:
	*npkts = 0;
	if (WME_ENAB(wlc)) {
		WLCNTINCR(wlc->pub._wme_cnt.tx_failed[WME_PRIO2AC(prio)].packets);
		WLCNTADD(wlc->pub._wme_cnt.tx_failed[WME_PRIO2AC(prio)].bytes,
		         pkttotlen(wlc->pub.osh, sdu));
	}

	wlc_pkt_flush(wlc, sdu, TX_STATUS_NO_ACK);
	return BCME_ERROR;
}

/* construct and send one fragment */
static void
wlc_dofrag(wlc_info_t *wlc, void *p, uint frag, uint nfrags, uint next_payload_len,
	osl_t *osh, wlc_bsscfg_t *cfg, struct scb *scb, bool is8021x, uint fifo,
	wsec_key_t *key, uint8 prio)
{
	struct ether_header *eh;
	struct dot11_header *h;
	uint next_frag_len;
	uint16 frameid;

	ASSERT(scb);

	/*
	 * 802.3 (header length = 22):
	 *                     (LLC includes ether_type in last 2 bytes):
	 * ----------------------------------------------------------------------------------------
	 * |                                      |   DA   |   SA   | L | LLC/SNAP | T |  Data... |
	 * ----------------------------------------------------------------------------------------
	 *                                             6        6     2       6      2
	 *
	 * NON-WDS
	 *
	 * Conversion to 802.11 (header length = 32):
	 * ----------------------------------------------------------------------------------------
	 * |              | FC | D |   A1   |   A2   |   A3   | S | QoS | LLC/SNAP | T |  Data... |
	 * ----------------------------------------------------------------------------------------
	 *                   2   2      6        6        6     2    2        6      2
	 *
	 * Conversion to 802.11 (WEP, QoS):
	 * ----------------------------------------------------------------------------------------
	 * |         | FC | D |   A1   |   A2   |   A3   | S | QoS | IV | LLC/SNAP | T |  Data... |
	 * ----------------------------------------------------------------------------------------
	 *             2   2      6        6        6     2    2     4        6      2
	 *
	 */

	eh = (struct ether_header*)PKTDATA(osh, p);
	WL_WSEC(("wlc_dofrag: tx sdu, len %d(with 802.3 hdr)\n", pkttotlen(osh, p)));

	/*
	 * Convert 802.3 MAC header to 802.11 MAC header (data only)
	 * and add WEP IV information if enabled.
	 *  requires valid DA and SA, does not care ether_type
	 */
	ASSERT(scb);
	h = wlc_80211hdr(wlc, osh, p, cfg, scb, (bool)(frag != (nfrags - 1)), key,
	                 prio);

	WL_WSEC(("wlc_dofrag: tx sdu, len %d(w 80211hdr and iv, no d11hdr, icv and fcs)\n",
		pkttotlen(osh, p)));

	/* 802.3 header needs to be included in WEP-encrypted portion of frame, and
	 * we need the IV info in the frame, so we're forced to do WEP here
	 */
	if (key && (key->algo == CRYPTO_ALGO_TKIP)) {
		wl_tkip_micadd(wlc->wl, p, DOT11_A3_HDR_LEN); /* Fix the Header Len */
	}

	/* determine total MPDU length of next frag */
	next_frag_len = next_payload_len;
	if (next_frag_len > 0) {	/* there is a following frag */
		next_frag_len += DOT11_A3_HDR_LEN + DOT11_FCS_LEN;
		/* WME: account for QoS Control Field */
		if (SCB_WME(scb))
			next_frag_len += DOT11_QOS_LEN;
		if (key && (key->algo != CRYPTO_ALGO_TKIP))
			next_frag_len += key->iv_len + key->icv_len;
	}

	WL_WSEC(("wlc_dofrag: tx sdu, len %d(before d11hdrs)\n", pkttotlen(osh, p)));
	/* add d11 headers */
	frameid = wlc_d11hdrs(wlc, p, scb, ((scb && wlc->shortpreamble && (scb->flags &
		SCB_SHORTPREAMBLE) != 0) ? 1 : 0),
		frag, nfrags, fifo, next_frag_len, key, 0);

	WL_WSEC(("wlc_dofrag: tx sdu, len %d(after d11hdrs, before adjustmnet)\n", pkttotlen(osh, p)));

	if (key && (key->algo == CRYPTO_ALGO_TKIP)) {
		/* pull the frame such that the data points at Dot11 Header field now */
		PKTPULL(OSH, p, D11_PHY_HDR_LEN + D11_TXH_LEN + 8);
	WL_WSEC(("wlc_dofrag: tx sdu, len %d(after d11hdrs, before crypto)\n", pkttotlen(osh, p)));
		wl_tkip_encrypt(wlc->wl, p, 24); /* Fix the Header Len */
		PKTPUSH(OSH, p, D11_PHY_HDR_LEN + D11_TXH_LEN);
	}

	WL_WSEC(("wlc_dofrag: tx sdu, len %d(after crypto)\n", pkttotlen(osh, p)));

	if (cfg->tk_cm_activate && next_frag_len == 0 && is8021x) {
		WL_WSEC(("wl%d: wlc_dofrag: TKIP countermeasures: sending MIC failure"
			" report...registering pkt callback\n", wlc->pub.unit));

		if (frameid == 0xffff ||
		    wlc_pkt_callback_register(wlc, wlc_tkip_countermeasures, NULL, p)) {
			WL_ERROR(("wl%d: wlc_dofrag: could not register MIC failure packet"
			          " callback\n", wlc->pub.unit));
		}

		cfg->tk_cm_activate = FALSE;
	}
}

static bool
sstlookup(wlc_info_t *wlc, uint16 proto)
{
	return (proto == 0x80f3 || proto == 0x8137);
}

/* Convert Ethernet to 802.3 per 802.1H (use bridge-tunnel if type in SST) */
void
wlc_ether_8023hdr(wlc_info_t *wlc, osl_t *osh, struct ether_header *eh, void *p)
{
	struct ether_header *neh;
	struct dot11_llc_snap_header *lsh;
	uint16 plen, ether_type;

	ether_type = ntoh16(eh->ether_type);
	neh = (struct ether_header *)PKTPUSH(osh, p, DOT11_LLC_SNAP_HDR_LEN);

	/* 802.3 MAC header */
	bcopy((char*)eh->ether_dhost, (char*)neh->ether_dhost, ETHER_ADDR_LEN);
	bcopy((char*)eh->ether_shost, (char*)neh->ether_shost, ETHER_ADDR_LEN);
	plen = (uint16)pkttotlen(osh, p) - ETHER_HDR_LEN;
	neh->ether_type = hton16(plen);

	/* 802.2 LLC header */
	lsh = (struct dot11_llc_snap_header *)&neh[1];
	lsh->dsap = 0xaa;
	lsh->ssap = 0xaa;
	lsh->ctl = 0x03;

	/* 802.2 SNAP header Use RFC1042 or bridge-tunnel if type in SST per 802.1H */
	lsh->oui[0] = 0x00;
	lsh->oui[1] = 0x00;
	if (sstlookup(wlc, ether_type))
		lsh->oui[2] = 0xf8;
	else
		lsh->oui[2] = 0x00;
	lsh->type = hton16(ether_type);
}

/* Convert 802.3 back to ethernet per 802.1H (use bridge-tunnel if type in SST) */
void
wlc_8023_etherhdr(wlc_info_t *wlc, osl_t *osh, void *p)
{
	struct ether_header *neh, *eh;
	struct dot11_llc_snap_header *lsh;
	bool isrfc894;

	eh = (struct ether_header *) PKTDATA(osh, p);
	neh = (struct ether_header *) PKTPULL(osh, p, DOT11_LLC_SNAP_HDR_LEN);

	/*
	 * check conversion is necessary
	 * - if ([AA AA 03 00 00 00] and protocol is not in SST) or
	 *   if ([AA AA 03 00 00 F8] and protocol is in SST), convert to RFC894
	 * - otherwise,
	 *	 preserve 802.3 (including RFC1042 with protocol in SST)
	 */
	lsh = (struct dot11_llc_snap_header *)((uint8 *)eh + ETHER_HDR_LEN);
	isrfc894 = (lsh->dsap == 0xaa && lsh->ssap == 0xaa && lsh->ctl == 0x03 &&
		lsh->oui[0] == 0 && lsh->oui[1] == 0 &&
		((lsh->oui[2] == 0x00 && !sstlookup(wlc, ntoh16(lsh->type))) ||
		(lsh->oui[2] == 0xf8 && sstlookup(wlc, ntoh16(lsh->type)))));

	/* if it was not converted in transmitter, no need to convert it back at here receiver */
	if (!isrfc894)
		return;


	/* 802.3 MAC header */
	bcopy((char*)eh->ether_shost, (char*)neh->ether_shost, ETHER_ADDR_LEN);
	bcopy((char*)eh->ether_dhost, (char*)neh->ether_dhost, ETHER_ADDR_LEN);

	/* no change to the ether_type field */

	return;
}

/*
 * Convert 802.3 MAC header to 802.11 MAC header (data only)
 * and add WEP IV information if enabled.
 */
struct dot11_header *
wlc_80211hdr(wlc_info_t *wlc, osl_t *osh, void *p, wlc_bsscfg_t *bsscfg, struct scb *scb,
	bool MoreFrag, wsec_key_t *key, uint8 prio)
{
	struct ether_header *eh;
	struct dot11_header *h;
	struct ether_addr tmpaddr;
	uchar *iv_data;
	uint16 offset;
	struct ether_addr *ra;

	ra =  NULL;

	/* convert 802.3 header to 802.11 header */
	eh = (struct ether_header *) PKTDATA(osh, p);
	/* Make room for 802.11 header, add additional bytes if WEP enabled for IV */
	offset = DOT11_A3_HDR_LEN - ETHER_HDR_LEN;
	if (SCB_WME(scb))
		offset += DOT11_QOS_LEN;
	if (key && key->algo != CRYPTO_ALGO_TKIP)
		offset += key->iv_len;
	h = (struct dot11_header *) PKTPUSH(osh, p, offset);
	bzero((char*)h, offset);

	if (wlc->pub.BSS) {
		/* BSS STA: a1 = BSSID, a2 = SA, a3 = DA, ToDS = 1, FromDS = 0 */
		bcopy((char*)&wlc->BSSID, (char*)&h->a1, ETHER_ADDR_LEN);
		bcopy((char*)&eh->ether_dhost, (char*)&tmpaddr, ETHER_ADDR_LEN);
		bcopy((char*)&eh->ether_shost, (char*)&h->a2, ETHER_ADDR_LEN);
		bcopy((char*)&tmpaddr, (char*)&h->a3, ETHER_ADDR_LEN);
		h->fc = FC_TODS;
	} else {
		/* IBSS STA: a1 = DA, a2 = SA, a3 = BSSID, ToDS = 0, FromDS = 0 */
		bcopy((char*)&eh->ether_dhost, (char*)&h->a1, ETHER_ADDR_LEN);
		bcopy((char*)&eh->ether_shost, (char*)&h->a2, ETHER_ADDR_LEN);
		bcopy((char*)&wlc->BSSID, (char*)&h->a3, ETHER_ADDR_LEN);
	}

	/* WME: Fill QoS Control Field */
	if (SCB_WME(scb)) {
		uint16 qos, *pqos;

		/* Set fragment priority */
		qos = (prio << QOS_PRIO_SHIFT) & QOS_PRIO_MASK;

		qos |= (wlc->wme_noack << QOS_ACK_SHIFT) & QOS_ACK_MASK;


		pqos = (uint16 *)((uchar *)h + DOT11_A3_HDR_LEN);
		ASSERT(ISALIGNED((uintptr)pqos, sizeof(*pqos)));

		*pqos = htol16(qos);

		/* Set subtype to QoS Data */
		h->fc |= (FC_SUBTYPE_QOS_DATA << FC_SUBTYPE_SHIFT);
	}

	if (key && key->algo != CRYPTO_ALGO_TKIP) {
		bool update = TRUE;

		iv_data = (uchar *)h + DOT11_A3_HDR_LEN;
		if (SCB_WME(scb))
			iv_data += DOT11_QOS_LEN;
		wlc_key_iv_update(wlc, bsscfg, key, iv_data, update);
	}

	h->fc |= (FC_TYPE_DATA << FC_TYPE_SHIFT);

	/* Set MoreFrag, WEP, and Order fc fields */
	if (MoreFrag)
		h->fc |= FC_MOREFRAG;
	if (key)
		h->fc |= FC_WEP;

	h->fc = htol16(h->fc);

	return h;
}

static uint16
wlc_compute_airtime(wlc_info_t *wlc, bool is_ofdm, uint phy_rate, uint length)
{
	uint16 usec = 0;
	uint mac_rate;
	uint nsyms;

	if (is_ofdm) {
		mac_rate = OFDM_PHY2MAC_RATE(phy_rate);

		/* nsyms = Ceiling(Nbits / (Nbits/sym))
		 *
		 * Nbits = length * 8
		 * Nbits/sym = Mbps * 4 = mac_rate * 2
		 */
		nsyms = CEIL((length * 8), (mac_rate * 2));

		/* usec = symbols * usec/symbol */
		usec = (uint16) (nsyms * APHY_SYMBOL_TIME);
		return (usec);
	} else {
		switch (phy_rate) {
		case 10:
			usec = length << 3;
			break;
		case 20:
			usec = length << 2;
			break;
		case 55:
			usec = (length << 4)/11;
			break;
		case 110:
			usec = (length << 3)/11;
			break;
		default:
			WL_ERROR(("wl%d: wlc_compute_airtime: unsupported rate %d\n",
				wlc->pub.unit, phy_rate));
			ASSERT((char*)"Bad phy_rate" == NULL);
			break;
		}
	}

	return (usec);
}

void
wlc_compute_plcp(wlc_info_t *wlc, ratespec_t rspec, uint length, uint8 *plcp)
{
	WL_TRACE(("wl%d: wlc_compute_plcp: rate 0x%x, length %d\n", wlc->pub.unit, rspec, length));

	if (IS_OFDM(rspec)) {
		wlc_compute_ofdm_plcp(rspec, length, plcp);
	} else {
		wlc_compute_cck_plcp(rspec, length, plcp);
	}
	return;
}

/* Rate: 802.11 rate code, length: PSDU length in octets */
static void
wlc_compute_ofdm_plcp(ratespec_t rspec, uint32 length, uint8 *plcp)
{
	uint8 signal;
	uint32 tmp = 0;
	int rate = RSPEC2RATE(rspec);

	ASSERT(IS_OFDM(rspec));

	/* encode rate per 802.11a-1999 sec 17.3.4.1, with lsb transmitted first */
	signal = rate_info[rate] & RATE_MASK;
	ASSERT(signal != 0);

	bzero(plcp, D11_PHY_HDR_LEN);
	D11A_PHY_HDR_SRATE((ofdm_phy_hdr_t *)plcp, signal);

	tmp = (length & 0xfff) << 5;
	plcp[2] |= (tmp >> 16) & 0xff;
	plcp[1] |= (tmp >> 8) & 0xff;
	plcp[0] |= tmp & 0xff;

	return;
}

/* Rate: 802.11 rate code, length: PSDU length in octets */
static void
wlc_compute_cck_plcp(ratespec_t rspec, uint length, uint8 *plcp)
{
	uint16 usec = 0;
	uint8 le = 0;
	int rate = RSPEC2RATE(rspec);

	ASSERT(IS_CCK(rspec));

	switch (rate) {
	case 2:
		usec = length << 3;
		break;
	case 4:
		usec = length << 2;
		break;
	case 11:
		usec = (length << 4)/11;
		if ((length << 4) - (usec * 11) > 0)
			usec++;
		break;
	case 22:
		usec = (length << 3)/11;
		if ((length << 3) - (usec * 11) > 0) {
			usec++;
			if ((usec * 11) - (length << 3) >= 8)
				le = D11B_PLCP_SIGNAL_LE;
		}
		break;
	default:
		WL_ERROR(("wlc_compute_d11b_plcp: unsupported rate %d\n", rate));
		ASSERT((char*)"invalid rate" == NULL);
		break;
	}

	/* PLCP signal byte */
	plcp[0] = rate * 5; /* r (500kbps) * 5 == r (100kbps) */
	/* PLCP service byte */
	plcp[1] = (uint8)(le | D11B_PLCP_SIGNAL_LOCKED);
	/* PLCP length uint16, little endian */
	plcp[2] = usec & 0xff;
	plcp[3] = (usec >> 8) & 0xff;
	/* PLCP CRC16 */
	plcp[4] = 0;
	plcp[5] = 0;

	return;
}


/* wlc_compute_frame_dur()
 *
 * Calculate the 802.11 MAC header DUR field for MPDU
 * DUR for a single frame = 1 SIFS + 1 ACK
 * DUR for a frame with following frags = 3 SIFS + 2 ACK + next frag time
 *
 * rate			MPDU rate in unit of 500kbps
 * next_frag_len	next MPDU length in bytes
 * short_preamble	use short or long PLCP header
 */
static uint16
wlc_compute_frame_dur(wlc_info_t *wlc, ratespec_t rate, int short_preamble, uint next_frag_len)
{
	uint16 dur, sifs;

	sifs = SIFS(wlc->band);

	dur = sifs;
	dur += (uint16)wlc_calc_ack_time(wlc, rate, short_preamble);

	if (next_frag_len) {
		/* Double the current DUR to get 2 SIFS + 2 ACKs */
		dur *= 2;
		/* add another SIFS and the frag time */
		dur += sifs;
		dur += (uint16)wlc_calc_frame_time(wlc, rate, short_preamble, next_frag_len);
	}
	return (dur);
}

/* wlc_compute_rtscts_dur()
 *
 * Calculate the 802.11 MAC header DUR field for an RTS or CTS frame
 * DUR for normal RTS/CTS w/ frame = 3 SIFS + 1 CTS + next frame time + 1 ACK
 * DUR for CTS-TO-SELF w/ frame    = 2 SIFS         + next frame time + 1 ACK
 *
 * cts			cts-to-self or rts/cts
 * rts_rate		rts or cts rate in unit of 500kbps
 * rate			next MPDU rate in unit of 500kbps
 * frame_len		next MPDU frame length in bytes
 */
static uint16
wlc_compute_rtscts_dur(wlc_info_t *wlc, bool cts_only, ratespec_t rts_rate, ratespec_t frame_rate,
                       int short_preamble, uint frame_len)
{
	uint16 dur, sifs;

	sifs = SIFS(wlc->band);

	if (!cts_only) {	/* RTS/CTS */
		dur = 3 * sifs;
		dur += (uint16)wlc_calc_cts_time(wlc, rts_rate, short_preamble);
	} else {	/* CTS-TO-SELF */
		dur = 2 * sifs;
	}

	dur += (uint16)wlc_calc_frame_time(wlc, frame_rate, short_preamble, frame_len);
	dur += (uint16)wlc_calc_ack_time(wlc, frame_rate, short_preamble);
	return (dur);
}

/*
 * Add d11txh_t, cck_phy_hdr_t.
 *
 * 'p' data must start with 802.11 MAC header
 * 'p' must allow enough bytes of local headers to be "pushed" onto the packet
 *
 * headroom == D11_PHY_HDR_LEN + D11_TXH_LEN == 82 bytes
 *
 */
static uint16
wlc_d11hdrs(wlc_info_t *wlc, void *p, struct scb *scb, int short_preamble, uint frag,
	uint nfrags, uint queue, uint next_frag_len, wsec_key_t *key, ratespec_t rspec_override)
{
	struct dot11_header *h;
	d11txh_t *txh;
	uint8 *plcp, plcp_fallback[D11_PHY_HDR_LEN];
	osl_t *osh;
	int len, phylen, rts_phylen;
	uint16 fc, type, frameid, mch, phyctl,  mainrates, xfts;
	uint16 seq = 0, mcl = 0, status = 0;
	ratespec_t rspec, rspec_fallback, min_rspec;
	bool use_rts = FALSE;
	bool use_cts = FALSE;
	uint8 *rts_plcp, rts_plcp_fallback[D11_PHY_HDR_LEN];
	struct dot11_rts_frame *rts = NULL;
	ratespec_t rts_rspec = 0, rts_rspec_fallback = 0;
	bool  qos;
	uint ac;
	uint txant;

	ASSERT(scb);
	ASSERT(queue < NFIFO);

	osh = wlc->pub.osh;

	/* locate 802.11 MAC header */
	h = (struct dot11_header*) PKTDATA(osh, p);
	fc = ltoh16(h->fc);
	type = FC_TYPE(fc);
	qos = (type == FC_TYPE_DATA && FC_SUBTYPE_ANY_QOS(FC_SUBTYPE(fc)));

	/* compute length of frame in bytes for use in PLCP computations */
	len = pkttotlen(osh, p);
	phylen = len + DOT11_FCS_LEN;

	/* If WEP enabled, add room in phylen for the additional bytes of
	 * ICV which MAC generates.  We do NOT add the additional bytes to
	 * the packet itself, thus phylen = packet length + ICV_LEN + FCS_LEN
	 * in this case
	 */

	if (key)
		phylen += key->icv_len;

	/* Software TKIP module adds the extiv at the front */
	if (key && (key->algo == CRYPTO_ALGO_TKIP)) {
		PKTPUSH(osh, p, 8);
		phylen += 8;
	}
		
	/* add PLCP */
	plcp = PKTPUSH(osh, p, D11_PHY_HDR_LEN);

	/* add Broadcom tx descriptor header */
	txh = (d11txh_t*)PKTPUSH(osh, p, D11_TXH_LEN);
	bzero((char*)txh, D11_TXH_LEN);

	/* use software seqnum */
	if (SCB_WME(scb) && qos && !ETHER_ISMULTI(&h->a1)) {
		seq = SCB_SEQNUM(scb, PKTPRIO(p));
		/* Increment the frag number only for last fragment */
		if (frag == (nfrags - 1))
		    SCB_SEQNUM(scb, PKTPRIO(p))++;
	} else if (type != FC_TYPE_CTL)
		mcl |= TXC_HWSEQ;

	if (type != FC_TYPE_CTL) {
		seq = (seq << SEQNUM_SHIFT) | (frag & FRAGNUM_MASK);
		h->seq = htol16(seq);
	}

	/* setup frameid */
	seq = (wlc->counter << SEQNUM_SHIFT) | (frag & FRAGNUM_MASK);
	frameid = ((seq << TXFID_SEQ_SHIFT) & TXFID_SEQ_MASK) | (queue & TXFID_QUEUE_MASK);

	/* (1) RATE: determine and validate primary rate and fallback rates */

	/* rate_override takes precedence */
	if (!RSPC_ACTIVE(rspec_override))
		rspec_override = wlc->band->rspec_override;

	if (RSPC_ACTIVE(rspec_override))
		rspec = rspec_fallback = rspec_override;
	else if (RSPC_ACTIVE(wlc->band->mrspec_override) && ETHER_ISMULTI(&h->a1))
		rspec = rspec_fallback = wlc->band->mrspec_override;
	else if (ETHER_ISMULTI(&h->a1) || (type == FC_TYPE_MNG) || (type == FC_TYPE_CTL))
		rspec = rspec_fallback = scb->rateset.rates[0] & RATE_MASK;
	else {
		/* run rate algorithm for data frame only, a cookie will be deposited in frameid */
		wlc_rate_sel(scb->ratestate, scb->select_rateset.rates, scb->select_rateset.count,
			&frameid, &rspec, &rspec_fallback);

		/* note tx rate for reporting to os */
		if (frag == 0) {
			wlc->txrate[wlc->txrateidx][0] = rspec;
			wlc->txrate[wlc->txrateidx][1] = (uint8) nfrags;
			wlc->txrateidx = (wlc->txrateidx+1) % NTXRATE;
		}
	}

	/* (2) PROTECTION: set use_rts flag based on RTSThreshold */
	if (((type == FC_TYPE_DATA) || (type == FC_TYPE_MNG)) &&
		(phylen > wlc->RTSThresh) && !ETHER_ISMULTI(&h->a1))
		use_rts = TRUE;

	if (wlc->band->gmode && wlc->g_protection && IS_OFDM(rspec)) {
		if (nfrags > 1) {
			/* For a frag burst use the lower modulation rates for the entire frag burst
			 * instead of protection mechanisms.
			 * As per spec, if protection mechanism is being used, a fragment sequence
			 * may only employ ERP-OFDM modulation for the final fragment and control
			 * response. (802.11g Sec 9.10) For ease we send the *whole* sequence at the
			 * lower modulation instead of using a higher modulation for the last frag.
			 */

			/* downgrade the rate to CCK or OFDM */
			if (wlc->g_protection) {
				/* Use 11 Mbps as the rate and fallback. We should make sure that if
				 * we are downgrading an OFDM rate to CCK, we should pick a more
				 * robust rate.  6 and 9 Mbps are not usually selected by rate
				 * selection, but even if the OFDM rate we are downgrading is 6 or 9
				 * Mbps, 11 Mbps is more robust.
				 */
				rspec = rspec_fallback = WLC_RATE_11M;
			} else {
				rspec = rspec_fallback = WLC_RATE_24M;
			}
		} else {
			/* Use protection mechanisms on unfragmented frames */

			/* If this is a 11g protection, and we are using RTS/CTS instead
			 * of CTS-To-Self, then use RTS if unicast.
			 * Otherwise, use a CTS-To-Self if RTS/CTS is not already specified
			 * for this frame
			 */
			if (!wlc->g_protection_cts && wlc->g_protection) {
				if (!ETHER_ISMULTI(&h->a1))
					use_rts = TRUE;
			} else if (!use_rts) {
				use_cts = TRUE;
			}
		}
	}

	/* calculate minimum rate */
	ASSERT(RSPEC2RATE(rspec_fallback) <= RSPEC2RATE(rspec));
	min_rspec = rspec_fallback;

	ASSERT(VALID_RATE_DBG(wlc, rspec));
	ASSERT(VALID_RATE_DBG(wlc, rspec_fallback));

	/* (3) PLCP: determine PLCP header and MAC duration, fill d11txh_t */

	/* PLCP header */
	wlc_compute_plcp(wlc, rspec, phylen, plcp);

	/* fallback rate version of PLCP header */
	wlc_compute_plcp(wlc, rspec_fallback, phylen, plcp_fallback);
	bcopy(plcp_fallback, (char*)&txh->FragPLCPFallback, sizeof(txh->FragPLCPFallback));

	mainrates = IS_OFDM(rspec) ? D11A_PHY_HDR_GRATE((ofdm_phy_hdr_t *)plcp) : plcp[0];

	/* DUR field */
	if ((fc != FC_PS_POLL) && !ETHER_ISMULTI(&h->a1))
		h->durid = htol16(wlc_compute_frame_dur(wlc, rspec, short_preamble, next_frag_len));

	/* fallback rate version of DUR field */
	if (fc == FC_PS_POLL)
		txh->FragDurFallback = h->durid;
	else if (ETHER_ISMULTI(&h->a1))
		txh->FragDurFallback = 0;
	else
		txh->FragDurFallback = htol16(wlc_compute_frame_dur(wlc, rspec_fallback,
			short_preamble, next_frag_len));

	/* (4) MAC-HDR: MacTxControlLow */
	if (frag == 0)
		mcl |= TXC_STARTMSDU;

	if (!ETHER_ISMULTI(&h->a1) &&
	    (!(wlc->wme_noack && SCB_WME(scb))))
		mcl |= TXC_IMMEDACK;

	if (BAND_5G(wlc->band->bandtype))
		mcl |= TXC_FREQBAND_5G;

	txh->MacTxControlLow = htol16(mcl);

	/* MacTxControlHigh */
	mch = 0;
	if (key && (key->algo != CRYPTO_ALGO_TKIP)) {
		mch |= ((WSEC_KEY_INDEX(wlc, key) << TXC_SECKEY_SHIFT) & TXC_SECKEY_MASK);
		mch |= ((key->algo_hw << TXC_SECTYPE_SHIFT) & TXC_SECTYPE_MASK);
		WL_WSEC(("wl%d: wlc_d11hdrs: setting key info in mch: index %d type %d\n",
			wlc->pub.unit,
			(mch & TXC_SECKEY_MASK) >> TXC_SECKEY_SHIFT, mch & TXC_SECTYPE_MASK));
	}
	txh->MacTxControlHigh = htol16(mch);

	/* MacFrameControl */
	bcopy((char*)&h->fc, (char*)&txh->MacFrameControl, sizeof(uint16));

	txh->TxFesTimeNormal = htol16(0);

	txh->TxFesTimeFallback = htol16(0);

	/* SecurityInitVector (IV) */
	if (key && (key->algo != CRYPTO_ALGO_TKIP)) {
		char *IV;

		IV = (char*)h + DOT11_A3_HDR_LEN;
		if (SCB_WME(scb) && qos)
			IV += DOT11_QOS_LEN;

		/* WEP and AES */
		bcopy(IV, (char*)txh->IV, key->iv_len);
	}

	/* TxFrameRA */
	bcopy((char*)&h->a1, (char*)&txh->TxFrameRA, ETHER_ADDR_LEN);

	/* TxFrameID */
	txh->TxFrameID = htol16(frameid);

	/* TxStatus, Note the case of recreating the first frag of a suppressed frame
	 * then we may need to reset the retry cnt's via the status reg
	 */
	txh->TxStatus = htol16(status);


	/* (5) RTS/CTS: determine RTS/CTS PLCP header and MAC duration, furnish d11txh_t */

	/* RTS PLCP header and RTS frame */
	if (use_rts || use_cts) {
		if (wlc->band->gmode && wlc->g_protection && !IS_CCK(rspec)) {
			/* Use 11Mbps as the g protection RTS target rate and fallback.
			 * Use the WLC_BASIC_RATE() lookup to find the best basic rate under the
			 * target in case 11 Mbps is not Basic.
			 * 6 and 9 Mbps are not usually selected by rate selection, but even
			 * if the OFDM rate we are protecting is 6 or 9 Mbps, 11 is more robust.
			 */
			rts_rspec = WLC_BASIC_RATE(wlc, WLC_RATE_11M);
			rts_rspec_fallback = rts_rspec;
		} else {
			/* calculate RTS rate and fallback rate based on the frame rate
			 * RTS must be sent at a basic rate since it is a
			 * control frame, sec 9.6 of 802.11 spec
			 */
			rts_rspec = WLC_BASIC_RATE(wlc, rspec);
			rts_rspec_fallback = WLC_BASIC_RATE(wlc, rspec_fallback);
		}

		/* RTS/CTS additions to MacTxControlLow */
		if (use_cts) {
			txh->MacTxControlLow |= htol16(TXC_SENDCTS);
		} else {
			txh->MacTxControlLow |= htol16(TXC_SENDRTS);
			txh->MacTxControlLow |= htol16(TXC_LONGFRAME);
		}

		/* RTS PLCP header */
		ASSERT(ISALIGNED((uintptr)txh->RTSPhyHeader, sizeof(uint16)));
		rts_plcp = txh->RTSPhyHeader;
		if (use_cts)
			rts_phylen = DOT11_CTS_LEN + DOT11_FCS_LEN;
		else
			rts_phylen = DOT11_RTS_LEN + DOT11_FCS_LEN;

		wlc_compute_plcp(wlc, rts_rspec, rts_phylen, rts_plcp);

		/* fallback rate version of RTS PLCP header */
		wlc_compute_plcp(wlc, rts_rspec_fallback, rts_phylen, rts_plcp_fallback);
		bcopy(rts_plcp_fallback, (char*)&txh->RTSPLCPFallback,
			sizeof(txh->RTSPLCPFallback));

		/* RTS frame fields... */
		rts = (struct dot11_rts_frame*)&txh->rts_frame;

		rts->durid = htol16(wlc_compute_rtscts_dur(wlc, use_cts, rts_rspec,
				rspec, short_preamble, phylen));
		/* fallback rate version of RTS DUR field */
		txh->RTSDurFallback = htol16(wlc_compute_rtscts_dur(wlc, use_cts,
			rts_rspec_fallback, rspec_fallback, short_preamble, phylen));

		if (use_cts) {
			rts->fc = htol16(FC_CTS);
			bcopy((char*)&h->a2, (char*)&rts->ra, ETHER_ADDR_LEN);
		} else {
			rts->fc = htol16((uint16)FC_RTS);
			bcopy((char*)&h->a1, (char*)&rts->ra, 2*ETHER_ADDR_LEN);
		}

		/* mainrate 
		 *    low 8 bits: main frag rate/mcs,
		 *    high 8 bits: rts/cts rate/mcs
		 */
		mainrates |= (IS_OFDM(rts_rspec) ?
			D11A_PHY_HDR_GRATE((ofdm_phy_hdr_t *)rts_plcp) : rts_plcp[0]) << 8;

		/* recalculate minimum rate */
		ASSERT(RSPEC2RATE(rts_rspec_fallback) <= RSPEC2RATE(rts_rspec));
		if (RSPEC2RATE(min_rspec) > RSPEC2RATE(rts_rspec_fallback))
			min_rspec = rts_rspec_fallback;
	} else {
		bzero((char*)txh->RTSPhyHeader, D11_PHY_HDR_LEN);
		bzero((char*)&txh->rts_frame, sizeof(struct dot11_rts_frame));
		bzero((char*)txh->RTSPLCPFallback, sizeof(txh->RTSPLCPFallback));
		txh->RTSDurFallback = 0;
	}

	txh->MainRates = htol16(mainrates);

	/* XtraFrameTypes */
	xfts = FRAMETYPE(rspec_fallback, 0);
	xfts |= (FRAMETYPE(rts_rspec, 0) << XFTS_RTS_FT_SHIFT);
	xfts |= (FRAMETYPE(rts_rspec_fallback, 0) << XFTS_FBRRTS_FT_SHIFT);
	xfts |= (wlc->band->pi->radio_code << XFTS_CHANNEL_SHIFT);
        txh->XtraFrameTypes = htol16(xfts);

	/* PhyTxControlWord */
	phyctl = FRAMETYPE(rspec, 0);
	if (short_preamble) {
		if (((RSPEC2RATE(min_rspec) == WLC_RATE_1M) ||
		     (wlc->PLCPHdr_override == WLC_PLCP_LONG))) {
			/* recalculate duration fields...short assumed, but long used */
			if (h->durid)
				h->durid = htol16(wlc_compute_frame_dur(wlc, rspec, 0,
				                                        next_frag_len));
			if (txh->FragDurFallback)
				txh->FragDurFallback =
				        htol16(wlc_compute_frame_dur(wlc, rspec_fallback, 0,
				                                     next_frag_len));
			if (rts && rts->durid) {
				rts->durid =
				        htol16(wlc_compute_rtscts_dur(wlc, use_cts, rts_rspec,
				                                      rspec, 0, phylen));
				txh->RTSDurFallback =
				        htol16(wlc_compute_rtscts_dur(wlc, use_cts,
				                                      rts_rspec_fallback,
				                                      rspec_fallback, 0, phylen));
			}
		} else {
			/* Set the short preamble bit, this has no meaning for legacy OFDM frames
			 *                      CCK
			 * PHY_TXC_SHORT_HDR: Short/GreenField
			 *         otherwise: Long/Mixed-Mode
			 */
			phyctl |= PHY_TXC_SHORT_HDR;
			WLCNTINCR(wlc->pub._cnt.txprshort);
		}
	}

	/* phytxant has the txant properly bit shifted */
	txant = wlc->phytxant;
	phyctl |= txant & PHY_TXC_ANT_MASK;

	txh->PhyTxControlWord = htol16(phyctl);

	/* WME: Update TXOP threshold */
	if (SCB_WME(scb) && (frag == 0) && qos && wlc->wme_txop[(ac = wme_fifo2ac[queue])]) {
		uint frag_dur, dur;

		ASSERT(!ETHER_ISMULTI(&h->a1));

		frag_dur = wlc_calc_frame_time(wlc, rspec, short_preamble, phylen);

		if (rts) {
			/* 1 RTS or CTS-to-self frame */
			dur = wlc_calc_cts_time(wlc, rts_rspec, short_preamble);
			/* (SIFS + CTS) + SIFS + frame + SIFS + ACK */
			dur += ltoh16(rts->durid);
		} else {
			/* frame */
			dur = frag_dur;
			/* 1 SIFS + 1 ACK */
			dur += wlc_compute_frame_dur(wlc, rspec, short_preamble, 0);
		}

		/* update txop byte threshold (txop minus intraframe overhead) */
		if (wlc->wme_txop[ac] >= (dur - frag_dur)) {
			wlc->fragthresh[queue] = (uint16) wlc_calc_frame_len(wlc, rspec,
				short_preamble, (wlc->wme_txop[ac] - (dur - frag_dur)));
		} else
			WL_ERROR(("wl%d: %s txop invalid for rate %d\n",
				wlc->pub.unit, fifo_names[queue], RSPEC2RATE(rspec)));

		if (dur > wlc->wme_txop[ac])
			WL_ERROR(("wl%d: %s txop exceeded phylen %d/%d dur %d/%d\n", wlc->pub.unit,
				fifo_names[queue], phylen, wlc->fragthresh[queue],
				dur, wlc->wme_txop[ac]));
	}

	/* With d11 hdrs on, mark the packet as MPDU */
	WLPKTTAG(p)->flags |= WLF_MPDU;

	WL_PRHDRS(wlc, "txpkt hdr (MPDU)", (uint8*)plcp, txh, NULL, PKTLEN(osh, p));
	WL_PRPKT("txpkt body (MPDU)", (uchar *)txh, PKTLEN(osh, p));

	return (frameid);
}

void
wlc_processpmq(wlc_info_t *wlc)
{
	volatile uint32 *pmqhostdata;
	volatile uint16 *pmqctrlstatus;
	uint32 pmqdata;
	d11regs_t *regs = wlc->regs;

	pmqhostdata = &regs->pmqreg.pmqhostdata;
	pmqctrlstatus = &regs->pmqreg.w.pmqctrlstatus;

	/* Read and drain all the PMQ entries while not EMPTY.
	 * When PMQ handling is enabled (MCTL_DISCARD_PMQ in maccontrol is clear),
	 * one PMQ entry per packet received from a STA is created with corresponding 'ea' as key.
	 * AP reads the entry and handles the PowerSave mode transitions of a STA by
	 * comparing the PMQ entry with current PS-state of the STA. If PMQ entry is same as the
	 * driver state, it's ignored, else transition is handled.
	 */
	while ((pmqdata = R_REG(pmqhostdata)) & PMQH_NOT_EMPTY) {
	}

	/* delete all processed pmq entries */
	W_REG(pmqctrlstatus, PMQH_DEL_MULT);
}


/* Do multicast filtering
 * returns TRUE on success (reject frame).
 */
static bool
wlc_mcastfilter(wlc_info_t *wlc, struct dot11_header *h)
{
	unsigned int i;


	for (i = 0; i < wlc->pub.nmulticast; i++) {
		if (bcmp((void*)&(h->a1), (void*)&wlc->pub.multicast[i],
			ETHER_ADDR_LEN) == 0)
			return FALSE;
	}

	return TRUE;
}

void
wlc_recvdata(wlc_info_t *wlc, osl_t *osh, d11rxhdr_t *rxh, void *p)
{
	uchar *plcp;
	uint min_len;
	bool promisc_frame = FALSE;
	struct scb *scb = NULL;
	struct wlc_frminfo f;	/* frame info to be passed to intermediate functions */
	uint offset;

	WL_TRACE(("wl%d: wlc_recvdata\n", wlc->pub.unit));

	f.key = NULL;
	f.prio = 0;
	f.ac = 0;
	f.apsd_eosp = FALSE;
	f.ividx = 0;
	f.p = p;
	f.rxh = rxh;

	min_len = D11_PHY_HDR_LEN + DOT11_A3_HDR_LEN + DOT11_FCS_LEN;

	/* check for runts */
	if ((uint)PKTLEN(osh, f.p) < min_len) {
		WL_ERROR(("wl%d: wlc_recvdata: runt frame\n", wlc->pub.unit));
		WLCNTINCR(wlc->pub._cnt.rxrunt);
		goto toss;
	}

	plcp = PKTDATA(osh, p);
	f.h = (struct dot11_header *)(plcp + D11_PHY_HDR_LEN);

	WL_PRHDRS(wlc, "rxpkt hdr (data)", (uint8*)plcp, NULL, f.rxh, PKTLEN(osh, f.p));
	WL_PRPKT("rxpkt body (data)\n", plcp, PKTLEN(osh, f.p));

	WL_GMODE(("wl%d: Rx packet is %s\n", wlc->pub.unit,
		(rxh->PhyRxStatus_0 & PRXS0_OFDM) ? "OFDM" : "CCK"));

	f.fc = ltoh16(f.h->fc);
	f.type = FC_TYPE(f.fc);
	f.subtype = (f.fc & FC_SUBTYPE_MASK) >> FC_SUBTYPE_SHIFT;
	ASSERT(f.type == FC_TYPE_DATA);

	if (((f.fc & (FC_FROMDS | FC_TODS)) == FC_FROMDS))
		/* update rssi information */
		wlc->rssi = wlc_update_rssi_ma(wlc->band->pi, f.rxh);


	f.qos = (f.type == FC_TYPE_DATA && FC_SUBTYPE_ANY_QOS(f.subtype));

	/* WME: account for QoS Control Field */
	if (f.qos) 
		min_len += DOT11_QOS_LEN;

	if ((uint)PKTLEN(osh, f.p) < min_len) {
		WL_ERROR(("wl%d: wlc_recvdata: runt frame from %s\n", wlc->pub.unit,
			bcm_ether_ntoa(&(f.h->a2), (char*)eabuf)));
		WLCNTINCR(wlc->pub._cnt.rxrunt);
		goto toss;
	}

	/* check for a broadcast/multicast A1 */
	f.ismulti = ETHER_ISMULTI(&(f.h->a1));

	/*if we are
	 - monitoring
	 or
	 - a STA
	 or
	*/

	if (wlc->monitor || wlc->pub.promisc) {
		bool bssid_match;
		uint8* bssid;

		/* monitor mode */
		if (wlc->monitor)
			wlc_monitor(wlc, f.rxh, f.p);

		/* check if this frame is in our BSS */
		if (!wlc->pub.associated) {
			/* we are not participating in a BSS */
			bssid_match = FALSE;
		} else {
			if (f.fc & FC_TODS)
				bssid = f.h->a1.octet;
			else if (f.fc & FC_FROMDS)
				bssid = f.h->a2.octet;
			else
				bssid = f.h->a3.octet;

			bssid_match = (bcmp(bssid, wlc->pub.current_bss.BSSID.octet, ETHER_ADDR_LEN)
				== 0);
		}

		/* determine if frame is for us or received only due to promisc setting */
		promisc_frame = TRUE;

		/* A1 is the destination or receiver address for all FromDS/ToDS combination,
		 * so the unicast check is done on A1.
		 * Multicast/Broadcast is only allowed
		 *     1) when ToDS==0 and the STA is participating in a BSS or IBSS,
		 */


		if (bcmp(f.h->a1.octet, (char*)&wlc->pub.cur_etheraddr, ETHER_ADDR_LEN) == 0 ||
			/* unicast match */
			(!(f.fc & FC_TODS) && f.ismulti && bssid_match)) { 
			promisc_frame = FALSE;
		}
		else if (wlc->pub.promisc) {
			/* in promisc mode, only pass up frames from inside our BSS */
			if (!bssid_match)
				goto toss;
		}
		else {
			/* In monitor mode (sniffing while operating as a normal interface),
			 * don't continue processing packets that aren't for us
			 */
			ASSERT(wlc->monitor);
			goto toss;
		}
	} else {
		/* a race window exists while changing the RX MAC addr match registers where
		 * we may receive a fame that does not have our MAC address
		 */
		if (!f.ismulti && bcmp(f.h->a1.octet,
		                       (char*)&wlc->pub.cur_etheraddr, ETHER_ADDR_LEN))
			goto toss;
	}


	/* strip off PLCP header */
	PKTPULL(osh, f.p, D11_PHY_HDR_LEN);

	f.pbody = (uchar*)f.h + DOT11_A3_HDR_LEN;

	/* account for QoS Control Field */
	if (f.qos) {
		uint16 qoscontrol = ltoh16_ua(f.pbody);
		f.prio = (uint8)QOS_PRIO(qoscontrol);
		f.ac = WME_PRIO2AC(f.prio);
		f.apsd_eosp = QOS_EOSP(qoscontrol);
		f.pbody += DOT11_QOS_LEN;
		f.ividx = (uint8)PRIO2IVIDX(f.prio);
		ASSERT(f.ividx < WLC_NUMRXIVS);
	}

	PKTSETLEN(osh, f.p, PKTLEN(osh, f.p) - DOT11_FCS_LEN);

	/* assign f.len now */
	f.len = PKTLEN(osh, f.p);
	offset = (uint)(f.pbody - (uint8*)PKTDATA(osh, f.p));
	f.body_len = f.len - offset;

	f.totlen = pkttotlen(osh, p) - offset;

	/* fix up endianness */
	f.h->seq = ltoh16(f.h->seq);

	/* 802.11-specific rx processing... */

	/* For reference, DS bits on Data frames and addresses:
	 *
	 *         ToDS FromDS   a1    a2    a3    a4
	 *
	 * IBSS      0      0    DA    SA   BSSID  --
	 * To STA    0      1    DA   BSSID  SA    --
	 * To AP     1      0   BSSID  SA    DA    --
	 *
	 * For A-MSDU frame
	 *         ToDS FromDS   a1    a2    a3    a4
	 *
	 * IBSS      0      0    DA    SA   BSSID  --
	 * To STA    0      1    DA   BSSID BSSID  --
	 * To AP     1      0   BSSID  SA   BSSID  --
	 */

	/* state based filtering */
	if (!promisc_frame && wlc_recvfilter(wlc, f.h, f.rxh, &scb))
		goto toss;

	/* toss frames with invalid DS bits */
	if (!promisc_frame &&
		((f.fc & (FC_TODS | FC_FROMDS)) != (wlc->pub.BSS ? FC_FROMDS : 0))) {
		WL_ERROR(("wl%d: wlc_recvdata: bad DS from %s\n", wlc->pub.unit,
			bcm_ether_ntoa(&(f.h->a2), (char*)eabuf)));
		WLCNTINCR(wlc->pub._cnt.rxbadds);
		goto toss;
	}
	if (wlc->pub.BSS && !promisc_frame) {
		/* If we are a BSS STA, the packet should be from the AP, unless
		 * we are in promiscuous mode.
		 */
		ASSERT(scb && SCB_ASSOCIATED(scb));
	} else {
		/* If we are an IBSS STA, we need to create SCBs as needed.
		 * If we are a BSS STA in promisc mode, we may be getting packets
		 * from STAs in our BSS
		 */
		if (!scb && !(scb = wlc_scblookup(wlc, &(f.h->a2)))) {
			if (wlc->pub.BSS) {
				WL_INFORM(("wl%d: wlc_recvdata: out of scbs for promisc"
				           " data\n",
				           wlc->pub.unit));
			} else {
				WL_ERROR(("wl%d: wlc_recvdata: out of scbs for IBSS data\n",
				          wlc->pub.unit));
				WLCNTINCR(wlc->pub._cnt.rxnoscb);
			}
			goto toss;
		}
	}
	f.WPA_auth = scb->WPA_auth;

	ASSERT(scb);

	if (wlc->pub.BSS) {
		/* This pkt must have come from the AP so we know it is up */

		/* Adjust time_since_bcn only for directed traffic to us since
		 * only directed traffic indicates the AP considers us associated.
		 * We want to handle the case of an AP rebooting by "roaming" to
		 * the AP, but we also want to also detect the case of going out
		 * then in range of an AP.
		 * If our rebooted old AP sends multicast traffic to other newly
		 * associated STAs, we do not want to stop the roam mechanism.
		 * But if we get directed traffic from our old AP since we just
		 * moved back into range, we want to stop the roam mechanism.
		 */
		if (!f.ismulti && !promisc_frame)
			wlc->time_since_bcn = 0;
	}

	/* multicast address filtering */
	if (f.ismulti) {
		/* toss multicast frames we sent forwarded back to us by the AP */
		if (wlc->pub.BSS &&
		    bcmp((char*)&(f.h->a3),
		         (char*)&wlc->pub.cur_etheraddr, ETHER_ADDR_LEN) == 0) {
			goto toss;
		}

		/* filter with multicast list */
		if (!ETHER_ISBCAST(&(f.h->a1)) && !wlc->pub.allmulti && wlc_mcastfilter(wlc, f.h))
			goto toss;
	}

	wlc_recvdata_ordered(wlc, scb, &f);

	return;

toss:
	if (WME_ENAB(wlc) && !f.ismulti) {
		WLCNTINCR(wlc->pub._wme_cnt.rx_failed[WME_PRIO2AC(PKTPRIO(f.p))].packets);
		WLCNTADD(wlc->pub._wme_cnt.rx_failed[WME_PRIO2AC(PKTPRIO(f.p))].bytes,
		                                     pkttotlen(osh, f.p));
	}

	PKTFREE(osh, f.p, FALSE);
	return;
}

/* Performs reassembly and per-msdu operations. Expects the frames to be in order */
void
wlc_recvdata_ordered(wlc_info_t *wlc, struct scb *scb, struct wlc_frminfo *f)
{	
	struct dot11_llc_snap_header *lsh;
	struct ether_addr a1, a2, a3, a4;
	uint body_offset;
	uint16 prev_seqctl;
	wlc_bsscfg_t *bsscfg;
	osl_t *osh;
	uint8 prio = 0; /* Best-effort */
	uchar* pbody;

	osh = wlc->pub.osh;

	/* For WME, update the prio */
	if (f->qos) {
		pbody = (uchar *)(f->h) + DOT11_A3_HDR_LEN;
		prio = (uint8)QOS_PRIO(ltoh16_ua(pbody));
	}

	/* Detect and discard duplicates */
	if (!f->ismulti) {
		if ((f->fc & FC_RETRY) && (scb->seqctl[prio] == f->h->seq)) {
			WL_TRACE(("wl%d: wlc_recvdata_ordered: discarding duplicate MPDU %04x "
				  "received from %s prio: %d\n", wlc->pub.unit, f->h->seq,
				  bcm_ether_ntoa(&(f->h->a2), (char*)eabuf), prio));
			WLCNTINCR(wlc->pub._cnt.rxdup);
			goto toss;
		} else {
			WL_TRACE(("wl%d: wlc_recvdata: received new MPDU %04x from %s prio: %d\n",
				wlc->pub.unit, f->h->seq,
				bcm_ether_ntoa(&(f->h->a2), (char*)eabuf), prio));
		}
	}

	/* Check for a PS-Poll response */
	if (wlc->PSpoll &&			/* PS-Poll outstanding */
	    !f->ismulti &&			/* unicast */
	    (f->fc & FC_MOREFRAG) == 0 &&	/* last frag in burst, or non-frag frame */
	    !AC_BITMAP_TST(scb->apsd.ac_delv, f->ac)) {	/* not on a delivery-enabled AC */
		/* If there's more, send another PS-Poll */
		if (f->fc & FC_MOREDATA) {
			if ((wlc->PSpoll = wlc_sendpspoll(wlc)) == FALSE) {
				WL_ERROR(("wl%d: FC_MOREDATA: wlc_sendpspoll() failed\n",
				          wlc->pub.unit));
				/* allow core to sleep again */
				wlc_set_ps_ctrl(wlc);
			}
		} else {
			/* Done, let the core go back to sleep */
			wlc->PSpoll = FALSE;
			wlc_set_ps_ctrl(wlc);
		}
	}

	/* Toss null data frames */
	if (FC_SUBTYPE_ANY_NULL(f->subtype))
		goto toss;

	ASSERT(SCB_BSSCFG(scb));
	bsscfg = SCB_BSSCFG(scb);

	f->rx_wep = f->fc & FC_WEP;

	if (f->rx_wep || (WSEC_ENABLED(bsscfg->wsec) && bsscfg->wsec_restrict)) {
		/* the key, if any, will be provided by wsec_recvdata */
		f->key = NULL;
		if (!wlc_wsec_recvdata(wlc, osh, scb, f, prio))
			goto toss;

	}

	if (!f->ismulti) {
		/* Update cached seqctl */
		prev_seqctl = scb->seqctl[prio];
		scb->seqctl[prio] = f->h->seq;

		/* reassembly */
		if ((f->h->seq & FRAGNUM_MASK) == 0) {
			/* Discard partially-received frame */
			if (scb->fragbuf[prio]) {
				WL_ERROR(("wl%d: wlc_recvdata_ordered: discarding partial "
				          "MSDU %03x with prio %d received from %s\n",
				          wlc->pub.unit, prev_seqctl >> SEQNUM_SHIFT, prio,
				          bcm_ether_ntoa(&(f->h->a2), (char*)eabuf)));
				PKTFREE(osh, scb->fragbuf[prio], FALSE);
				scb->fragbuf[prio] = NULL;
				scb->fragresid[prio] = 0;
			}

			if ((f->fc & FC_MOREFRAG) != 0) {
				/* save packet for reassembly */
				scb->fragbuf[prio] = f->p;
				scb->fragresid[prio] = (RXBUFSZ - HWRXOFF - D11_PHY_HDR_LEN) -
					f->len;
				if (bcmp(wlc_802_1x_hdr, f->pbody, DOT11_LLC_SNAP_HDR_LEN)) {
					scb->flags &= ~SCB_8021XHDR;
				} else {
					scb->flags |= SCB_8021XHDR;
				}


				/* That's all for now... */
				return;
			}

			/* non-fragmented MSDUs fall-through here... */

		} else {
			/* subsequent frag */

			/*
			 * This isn't the first frag, but we don't have a partially-
			 * received MSDU.  We must have somehow missed the previous
			 * frags or timed-out the partially-received MSDU (not implemented yet).
			 */
			if (!scb->fragbuf[prio]) {
				WL_ERROR(("wl%d: wlc_recvdata_ordered: discarding MPDU %04x with "
				          "prio %d; previous MPDUs missed or partially-received "
				          "MSDU timed-out\n", wlc->pub.unit, f->h->seq, prio));
				WLCNTINCR(wlc->pub._cnt.rxfragerr);
				goto toss;
			}

			/* Make sure this MPDU:
			 * - matches the partially-received MSDU
			 * - is the one we expect (next in sequence)
			 */
			if (((f->h->seq & ~FRAGNUM_MASK) != (prev_seqctl & ~FRAGNUM_MASK)) ||
			    ((f->h->seq & FRAGNUM_MASK) != ((prev_seqctl & FRAGNUM_MASK) + 1))) {
				/* discard the partially-received MSDU */
				WL_ERROR(("wl%d: wlc_recvdata_ordered: discarding partial "
				          "MSDU %03x with prio:%d received from %s\n",
				          wlc->pub.unit, prev_seqctl >> SEQNUM_SHIFT, prio,
				          bcm_ether_ntoa(&(f->h->a2), (char*)eabuf)));
				PKTFREE(osh, scb->fragbuf[prio], FALSE);
				scb->fragbuf[prio] = NULL;
				scb->fragresid[prio] = 0;

				/* discard the MPDU */
				WL_INFORM(("wl%d: wlc_recvdata_ordered: discarding MPDU %04x with "
				           "prio %d; previous MPDUs missed\n", wlc->pub.unit,
				           f->h->seq, prio));
				WLCNTINCR(wlc->pub._cnt.rxfragerr);
				goto toss;

			}

			/* detect fragbuf overflow */
			if (f->body_len > scb->fragresid[prio]) {
				/* discard the partially-received MSDU */
				WL_ERROR(("wl%d: wlc_recvdata_ordered: discarding partial "
				          "MSDU %03x with prio %d received from %s\n",
				          wlc->pub.unit, prev_seqctl >> SEQNUM_SHIFT, prio,
				          bcm_ether_ntoa(&(f->h->a2), (char*)eabuf)));
				PKTFREE(osh, scb->fragbuf[prio], FALSE);
				scb->fragbuf[prio] = NULL;
				scb->fragresid[prio] = 0;

				/* discard the MPDU */
				WL_ERROR(("wl%d: wlc_recvdata: discarding MPDU %04x with prio %d; "
				          "resulting MSDU too big\n", wlc->pub.unit, f->h->seq,
				          prio));
				WLCNTINCR(wlc->pub._cnt.rxfragerr);
				goto toss;
			}

			/* copy frame into fragbuf */
			wlc_appendfrag(wlc, scb->fragbuf[prio], &scb->fragresid[prio], f->pbody,
			               f->body_len, osh);

			PKTFREE(osh, f->p, FALSE);

			if (f->fc & FC_MOREFRAG)
				return;

			/* last frag...fall through and sendup reassembled MSDU */
			f->p = scb->fragbuf[prio];
			scb->fragbuf[prio] = NULL;
			scb->fragresid[prio] = 0;

			/* reset packet pointers to beginning */
			f->h = (struct dot11_header *) PKTDATA(osh, f->p);
			f->len = PKTLEN(osh, f->p);
			f->pbody = (uchar*)(f->h) + DOT11_A3_HDR_LEN;
			f->body_len = f->len - DOT11_A3_HDR_LEN;
			/* WME: account for QoS Control Field */
			if (f->qos) {
				f->prio = prio;
				f->pbody += DOT11_QOS_LEN;
				f->body_len -= DOT11_QOS_LEN;
			}
			f->fc = ltoh16(f->h->fc);
			if (f->rx_wep) {
				/* strip WEP IV */
				f->pbody += f->key->iv_len;
				f->body_len -= f->key->iv_len;
			}
		}
	} /* unicast */

	/*
	 * 802.11 -> 802.3/Ethernet header conversion
	 * Part 1: eliminate possible overwrite problems, find right eh pointer
	 */
	bcopy((char *)&(f->h->a1), (char *)&a1, ETHER_ADDR_LEN);
	bcopy((char *)&(f->h->a2), (char *)&a2, ETHER_ADDR_LEN);
	bcopy((char *)&(f->h->a3), (char *)&a3, ETHER_ADDR_LEN);

	lsh = (struct dot11_llc_snap_header *)(f->pbody);
	/* calc the bytes included in the 802.11 header (A3 or A4 format + qos + iv) */
	body_offset = (uint)(f->pbody - (uchar*)PKTDATA(osh, f->p));

	/*
	 * 802.11 -> 802.3/Ethernet header conversion
	 * Part 2: find sa/da pointers
	 */
	if ((f->fc & FC_TODS) == 0) {
		f->da = &a1;
		if ((f->fc & FC_FROMDS) == 0)
			f->sa = &a2;
		else
			f->sa = &a3;
	} else {
		f->da = &a3;
		if ((f->fc & FC_FROMDS) == 0)
			f->sa = &a2;
		else
			f->sa = &a4;
	}

	if (f->key && f->key->algo != CRYPTO_ALGO_WEP1 && f->key->algo != CRYPTO_ALGO_WEP128)
		if (!wlc_wsec_miccheck(wlc, osh, scb, f))
			goto toss;

	/*
	 * 802.11 -> 802.3/Ethernet header conversion part 1:
	 * - if ([AA AA 03 00 00 00] and protocol is not in SST) or
	 *   if ([AA AA 03 00 00 F8] and protocol is in SST), convert to RFC894
	 * - otherwise,
	 *	 preserve 802.3 (including RFC1042 with protocol in SST)
	 *
	 * NOTE: Need to be careful when changing this because MIC calculation
	 * depends on 802.3 header still being present
	 */
	if (lsh->dsap == 0xaa && lsh->ssap == 0xaa && lsh->ctl == 0x03 &&
		lsh->oui[0] == 0 && lsh->oui[1] == 0 &&
		((lsh->oui[2] == 0x00 && !sstlookup(wlc, ntoh16(lsh->type))) ||
		(lsh->oui[2] == 0xf8 && sstlookup(wlc, ntoh16(lsh->type))))) {
		/* RFC894 */
		f->eh = (struct ether_header *)PKTPULL(osh, f->p,
			body_offset + DOT11_LLC_SNAP_HDR_LEN - ETHER_HDR_LEN);
	} else {
		/* RFC1042 */
		f->eh = (struct ether_header *)PKTPULL(osh, f->p,
			body_offset - ETHER_HDR_LEN);
		f->eh->ether_type = hton16((uint16)f->body_len);	/* length */
	}


	/*
	 * 802.11 -> 802.3/Ethernet header conversion
	 * Part 3: do the conversion
	 */
	bcopy((char*)(f->da), (char*)&(f->eh->ether_dhost), ETHER_ADDR_LEN);
	bcopy((char*)(f->sa), (char*)&(f->eh->ether_shost), ETHER_ADDR_LEN);

	/* WME: set packet priority based on first received MPDU */
	PKTSETPRIO(f->p, f->prio);


	/* apply more filters */
	if (ETHER_ISMULTI(f->da))
		WLCNTINCR(wlc->pub._cnt.rxmulti);

	if (WSEC_ENABLED(bsscfg->wsec)) {
		bool drop_frame = TRUE;
		/* unencrypted frame */
		if (!f->rx_wep) {
			if (!bsscfg->wsec_restrict ||
				((ntoh16(f->eh->ether_type) == ETHER_TYPE_802_1X) && (bsscfg->rx_unencrypted_eapol)))
				drop_frame = FALSE;
			if (drop_frame) {
				WL_ERROR(("Dropping frames...\n"));
				goto toss;
			}
		}
	}

	/* toss if station is not yet authorized to receive non-802.1X frames */
	if (bsscfg->eap_restrict && !SCB_AUTHORIZED(scb)) {
		if (ntoh16(f->eh->ether_type) != ETHER_TYPE_802_1X) {
			WL_ERROR(("wl%d: non-802.1X frame from unauthorized station %s\n",
				wlc->pub.unit, bcm_ether_ntoa(&scb->ea, eabuf)));
			goto toss;
		}
	}

	wlc->PMidlebcns = 0;


	wlc_recvdata_sendup(wlc, scb, 0, f->da, f->p);
	return;

toss:
	if (WME_ENAB(wlc)) {
		WLCNTINCR(wlc->pub._wme_cnt.rx_failed[WME_PRIO2AC(PKTPRIO(f->p))].packets);
		WLCNTADD(wlc->pub._wme_cnt.rx_failed[WME_PRIO2AC(PKTPRIO(f->p))].bytes,
		                                     pkttotlen(osh, f->p));
	}

	PKTFREE(osh, f->p, FALSE);
	return;
}

/* forward and/or sendup a received data frame */
void
wlc_recvdata_sendup(wlc_info_t *wlc, struct scb *scb, bool wds, struct ether_addr *da, void *p)
{
	struct ether_header *eh;
	osl_t *osh;


	ASSERT(scb);

	osh = wlc->pub.osh;
	eh = (struct ether_header*) PKTDATA(osh, p);

	/*
	 * If VLAN Mode is active, and an 802.1Q VLAN Priority Tag (VLAN ID 0) is present,
	 * extract the priority from it and strip the tag (satisfies requirements of WMM
	 * section A.6 and WHQL).
	 */
	if ((wlc->vlan_mode != OFF) && ntoh16(eh->ether_type) == ETHER_TYPE_8021Q) {
		struct ethervlan_header *vh = (struct ethervlan_header *)eh;

		if (!(ntoh16(vh->vlan_tag) & VLAN_VID_MASK)) {
			struct ether_header da_sa;

			PKTSETPRIO(p, (ntoh16(vh->vlan_tag) >> VLAN_PRI_SHIFT) & VLAN_PRI_MASK);

			bcopy(vh, &da_sa, VLAN_TAG_OFFSET);
			eh = (struct ether_header *)PKTPULL(osh, p, VLAN_TAG_LEN);
			bcopy(&da_sa, eh, VLAN_TAG_OFFSET);
		}
	}


	WL_PRPKT("rxpkt (Ethernet)\n", (uchar *)eh, PKTLEN(osh, p));
	WL_PRUSR("rx", (uchar*)eh, ETHER_HDR_LEN);
	WL_WSEC(("wlc_recvdata2: rx sendup, len %d\n", PKTLEN(osh, p)));

	/* update stat counters */
	WLCNTINCR(wlc->pub._cnt.rxframe);
	WLCNTADD(wlc->pub._cnt.rxbyte, PKTLEN(osh, p));

	if (WME_ENAB(wlc)) {
		WLCNTINCR(wlc->pub._wme_cnt.rx[WME_PRIO2AC(PKTPRIO(p))].packets);
		WLCNTADD(wlc->pub._wme_cnt.rx[WME_PRIO2AC(PKTPRIO(p))].bytes,
		         PKTLEN(osh, p));
	}

	scb->used = wlc->pub.now;


	/* Before sending the packet up the stack, set the packet prio correctly */
	if (WME_ENAB(wlc) && (PKTPRIO(p) == 0))
		pktsetprio(p, FALSE);

	wl_sendup(wlc->wl, SCB_INTERFACE(scb), p);
}

static void
wlc_appendfrag(wlc_info_t *wlc, void *fragbuf, uint *fragresid, uchar *body, uint body_len, void*
	osh)
{
	uchar *dst;
	uint fraglen;

	/* append frag payload to end of partial packet */
	fraglen = PKTLEN(osh, fragbuf);
	dst = PKTDATA(osh, fragbuf) + fraglen;
	bcopy(body, dst, body_len);
	PKTSETLEN(osh, fragbuf, (fraglen + body_len));
	*fragresid -= body_len;
}

void
wlc_intrson(wlc_info_t *wlc)
{
	ASSERT(wlc->defmacintmask);
	wlc->macintmask = wlc->defmacintmask;
	W_REG(&wlc->regs->macintmask, wlc->macintmask);
}


/* callback for sbutils.c, which has only wlc handler, no wl
 * they both check up, not only because there is no need to off/restore d11 interrupt
 *  but also because per-port code may require sync with valid interrupt.
 */

static uint32
wlc_wlintrsoff(wlc_info_t *wlc)
{
	if (!wlc->pub.up)
		return 0;

	return wl_intrsoff(wlc->wl);
}

static void
wlc_wlintrsrestore(wlc_info_t *wlc, uint32 macintmask)
{
	if (!wlc->pub.up)
		return;

	wl_intrsrestore(wlc->wl, macintmask);
}


/* intrs off called by isr, faster, no interrupt protection needed */
void
wlc_intrsoff_isr(wlc_info_t *wlc)
{
	if (!wlc->core->clk)
		return;

	W_REG(&wlc->regs->macintmask, 0);
	(void)R_REG(&wlc->regs->macintmask);	/* sync readback */
	wlc->macintmask = 0;
}

uint32
wlc_intrsoff(wlc_info_t *wlc)
{
	uint32 macintmask;

	if (!wlc->core->clk)
		return 0;

	macintmask = wlc->macintmask;	/* isr can still happen */

	W_REG(&wlc->regs->macintmask, 0);
	(void)R_REG(&wlc->regs->macintmask);	/* sync readback */
	OSL_DELAY(1); /* ensure int line is no longer driven */
	wlc->macintmask = 0;

	/* return previous macintmask; resolve race between us and our isr */
	return (wlc->macintstatus? 0 : macintmask);
}

void
wlc_intrsrestore(wlc_info_t *wlc, uint32 macintmask)
{
	if (!wlc->core->clk)
		return;

	wlc->macintmask = macintmask;
	W_REG(&wlc->regs->macintmask, wlc->macintmask);
}

void
wlc_mute(wlc_info_t *wlc, bool on)
{
	struct ether_addr null_ether_addr = {{0, 0, 0, 0, 0, 0}};

	if (on) {
		/* suspend tx fifos */
		wlc_tx_fifo_suspend(wlc, TX_DATA_FIFO);
		wlc_tx_fifo_suspend(wlc, TX_CTL_FIFO);

		/* zero the address match register so we do not send ACKs */
		wlc_set_addrmatch(wlc, RCM_MAC_OFFSET, &null_ether_addr);
	} else {
		/* resume tx fifos */
		if (!wlc->tx_suspended) {
			wlc_tx_fifo_resume(wlc, TX_DATA_FIFO);
		}
		wlc_tx_fifo_resume(wlc, TX_CTL_FIFO);

		/* Restore address */
		wlc_set_addrmatch(wlc, RCM_MAC_OFFSET, &wlc->pub.cur_etheraddr);
	}

	/* handle beacon and probe response */
	if (!wlc->pub.BSS) {
		wlc_mctrl(wlc, MCTL_INFRA, ((on == TRUE) ? MCTL_INFRA : 0));
	}
}

static void
wlc_tx_suspend(wlc_info_t *wlc)
{
	wlc->tx_suspended = TRUE;

	wlc_tx_fifo_suspend(wlc, TX_DATA_FIFO);
}

static bool
wlc_tx_suspended(wlc_info_t *wlc)
{
	return wlc_tx_fifo_suspended(wlc, TX_DATA_FIFO);
}

static void
wlc_tx_resume(wlc_info_t *wlc)
{
	bool silence = FALSE;

	wlc->tx_suspended = FALSE;
	if (QUIET_STATE(wlc) & SILENCE) {
		/* do not unsuspend when in quiet period */
		silence = TRUE;
	}

	if (!silence && !QUIET_CHANNEL(wlc, wlc->band->pi->radio_channel)) {
		wlc_tx_fifo_resume(wlc, TX_DATA_FIFO);
	}
}

/* wlc_tx_fifo_suspended:
 * Check the MAC's tx suspend status for a tx fifo.
 *
 * When the MAC acknowledges a tx suspend, it indicates that no more
 * packets will be transmitted out the radio. This is independent of
 * DMA channel suspension---the DMA may have finished suspending, or may still
 * be pulling data into a tx fifo, by the time the MAC acks the suspend
 * request.
 */
static bool
wlc_tx_fifo_suspended(wlc_info_t *wlc, uint tx_fifo)
{
	/* check that a suspend has been requested and is no longer pending */
	if (!PIO_ENAB(wlc)) {
		/*
		 * for DMA mode, the suspend request is set in xmtcontrol of the DMA engine,
		 * and the tx fifo suspend at the lower end of the MAC is acknowledged in the
		 * chnstatus register.
		 * The tx fifo suspend completion is independent of the DMA suspend completion and
		 *   may be acked before or after the DMA is suspended.
		 */
		if (dma_txsuspended(wlc->core->di[tx_fifo]) && (R_REG(&wlc->regs->chnstatus) &
			(1<<tx_fifo)) == 0)
			return TRUE;
	} else {
		if (wlc_pio_txsuspended(wlc->core->pio[tx_fifo]))
			return TRUE;
	}

	return FALSE;
}

static void
wlc_tx_fifo_suspend(wlc_info_t *wlc, uint tx_fifo)
{
	/* Two clients of this code, 11h Quiet period and scanning. */

	/* force the core awake */
	wlc->wakefortxfifo = TRUE;
	wlc_set_ps_ctrl(wlc);

	if (!PIO_ENAB(wlc)) {
		if (wlc->core->di[tx_fifo])
			dma_txsuspend(wlc->core->di[tx_fifo]);
	} else {
		wlc_pio_txsuspend(wlc->core->pio[tx_fifo]);
	}
}

static void
wlc_tx_fifo_resume(wlc_info_t *wlc, uint tx_fifo)
{
	/* Two clients of this code, 11h Quiet period and scanning. */
	if (!PIO_ENAB(wlc)) {
		if (wlc->core->di[tx_fifo])
			dma_txresume(wlc->core->di[tx_fifo]);
	} else {
		wlc_pio_txresume(wlc->core->pio[tx_fifo]);
		WLC_TX_FIFO_ENAB(wlc, tx_fifo);
	}

	wlc->wakefortxfifo = FALSE;
	/* allow core to sleep again */
	wlc_set_ps_ctrl(wlc);
}

/*
 * Read and clear macintstatus and intstatus registers.
 * Return:
 *   -1 if DEVICEREMOVED(wlc) evaluates to TRUE;
 *   0 if the interrupt is not for us, or we are in some special cases;
 *   device interrupt status bits otherwise.
 */
static INLINE uint32
wlc_intstatus(wlc_info_t *wlc, bool in_isr)
{
	d11regs_t *regs = wlc->regs;
	uint32 macintstatus;
	uint32 intstatus_rxfifo, intstatus_txsfifo;

	/* macintstatus includes a DMA interrupt summary bit */
	macintstatus = R_REG(&regs->macintstatus);

	WL_TRACE(("wl%d: macintstatus: 0x%x\n", wlc->pub.unit, macintstatus));

	/* detect cardbus removed, in power down(suspend) and in reset */
	if (DEVICEREMOVED(wlc))
		return -1;

	/* DEVICEREMOVED succeeds even when the core is still resetting,
	 * handle that case here.
	 */
	if (macintstatus == 0xffffffff)
		return 0;

	/* defer unsolicited interrupts */
	macintstatus &= (in_isr ? wlc->macintmask : wlc->defmacintmask);

	/* if not for us */
	if (macintstatus == 0)
		return 0;

	/* clear device interrupts */
	W_REG(&regs->macintstatus, macintstatus);

	/* MI_DMAINT is indication of non-zero intstatus */
	if (macintstatus & MI_DMAINT) {
		if (D11REV_LT(wlc->pub.corerev, 5)) {
			intstatus_rxfifo = R_REG(&regs->intctrlregs[RX_FIFO].intstatus);
			intstatus_txsfifo = R_REG(&regs->intctrlregs[RX_TXSTATUS_FIFO].intstatus);
			WL_TRACE(("wl%d: intstatus_rxfifo 0x%x, intstatus_txsfifo 0x%x\n",
				wlc->pub.unit, intstatus_rxfifo, intstatus_txsfifo));

			/* defer unsolicited interrupt hints */
			intstatus_rxfifo &= DEF_RXINTMASK;
			intstatus_txsfifo &= DEF_RXINTMASK;

			/* MI_DMAINT bit in macintstatus is indication of RX_FIFO interrupt */
			/* clear interrupt hints */
			if (intstatus_rxfifo)
				W_REG(&regs->intctrlregs[RX_FIFO].intstatus, intstatus_rxfifo);
			else
				macintstatus &= ~MI_DMAINT;

			/* MI_TFS bit in macintstatus is encoding of RX_TXSTATUS_FIFO interrupt */
			if (intstatus_txsfifo) {
				W_REG(&regs->intctrlregs[RX_TXSTATUS_FIFO].intstatus,
				      intstatus_txsfifo);
				macintstatus |= MI_TFS;
			}
		}
		else {
			/*
			 * For corerevs >= 5, only fifo interrupt enabled is I_RI in RX_FIFO.
			 * If MI_DMAINT is set, assume it is set and clear the interrupt.
			 */
			W_REG(&regs->intctrlregs[RX_FIFO].intstatus, DEF_RXINTMASK);
		}
	}

	return macintstatus;
}

/* Update wlc->macintstatus and wlc->intstatus[]. */
/* Return TRUE if they are updated successfully. FALSE otherwise */
bool
wlc_intrsupd(wlc_info_t *wlc)
{
	uint32 macintstatus;

	ASSERT(wlc->macintstatus != 0);

	/* read and clear macintstatus and intstatus registers */
	macintstatus = wlc_intstatus(wlc, FALSE);

	/* device is removed */
	if (macintstatus == 0xffffffff)
		return FALSE;

	/* update interrupt status in software */
	wlc->macintstatus |= macintstatus;

	return TRUE;
}

/*
 * First-level interrupt processing.
 * Return TRUE if this was our interrupt, FALSE otherwise.
 * *wantdpc will be set to TRUE if further wlc_dpc() processing is required,
 * FALSE otherwise.
 */
bool
wlc_isr(wlc_info_t *wlc, bool *wantdpc)
{
	uint32 macintstatus;

	*wantdpc = FALSE;

	if (!wlc->pub.up || !wlc->macintmask)
		return (FALSE);

	/* read and clear macintstatus and intstatus registers */
	macintstatus = wlc_intstatus(wlc, TRUE);

	/* device is removed */
	if (macintstatus == 0xffffffff)
		goto dead;

	/* it is not for us */
	if (macintstatus == 0)
		return (FALSE);

	*wantdpc = TRUE;

	/* save interrupt status bits */
	ASSERT(wlc->macintstatus == 0);
	wlc->macintstatus = macintstatus;

	return (TRUE);

dead:
	WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
	wl_down(wlc->wl);
	return (FALSE);
}


/* second-level interrupt processing */
/*
 * Return TRUE if another dpc needs to be re-scheduled. FALSE otherwise.
 * Param 'bounded' indicates if applicable loops should be bounded.
 */
bool
wlc_dpc(wlc_info_t *wlc, bool bounded)
{
	uint32 macintstatus;
	d11regs_t *regs = wlc->regs;

	/* grab and clear the saved software intstatus bits */
	macintstatus = wlc->macintstatus;
	wlc->macintstatus = 0;

	WL_TRACE(("wl%d: wlc_dpc: macintstatus 0x%x\n",
		wlc->pub.unit, macintstatus));

	/* PMQ entry addition */
	if (macintstatus & MI_PMQ)
		wlc_processpmq(wlc);

	/* tx status */
	if (macintstatus & MI_TFS) {
		if (D11REV_LT(wlc->pub.corerev, 5))
			wlc_txstatusrecv(wlc);
		else {
			/*
			 * restore relevant interrupt status for next round dpc
			 * if more tx status need to be processed.
			 */
			if (wlc_txstatus(wlc, bounded ? TXSBND : -1))
				wlc->macintstatus |= MI_TFS;
		}
	}

	/* TBTT indication */
	if (macintstatus & MI_TBTT) {
		WLCNTINCR(wlc->pub._cnt.tbtt);

		WL_TRACE(("wl%d: wlc_dpc: TBTT indication\n", wlc->pub.unit));

		/* Fast PS mode : check if we should return to PS mode */
		if (!wlc->PMenabled && !wlc->PM_override && !wlc->WME_PM_blocked &&
		    (wlc->PM == PM_FAST)) {
			/*
			 * We should decline to go into power save state
			 * while doing a scan.
			 * Attempts to do so will FAIL because we won't be able
			 * to send the NULL frame to our AP.
			 * Better to just wait till next time ...
			 *
			 * Finally, don't enter power save mode if we
			 * haven't seen a beacon. This is intended
			 * to fix a problem where we "lose" time in the TSF
			 * from band switches and thus wake up at the wrong
			 * time looking for beacons.
			 *
			 */
			if (wlc->PMidlebcns < FASTPM_IDLEBCNS)
			{
				wlc->PMidlebcns++;
			}
			else if (SCAN_IN_PROGRESS(wlc))
			{
				WL_PS(("PS-MODE: skipping wlc_set_pmstate: scan in"
					" progress\n"));
			}
			else
			{
			    WL_PS(("PS-MODE: beacons--> fast ps mode\n"));
			    wlc_set_pmstate(wlc, TRUE);
			}
		}

		/* stop waiting for PS-Poll response (may be too soon?) */
		if (wlc->PSpoll) {
			wlc->PSpoll = FALSE;
			WL_INFORM(("PS-MODE: PS-Poll timeout...go to sleep (%d)\n",
				STAY_AWAKE(wlc)));
			wlc_set_ps_ctrl(wlc);
		}

		if (!wlc->pub.BSS) {

			/* DirFrmQ is now valid...defer setting until end of ATIM window */
			wlc->qvalid |= MCMD_DIRFRMQVAL;
		}

		if (WL11H_ENAB(wlc)) {
			/* If we have previously detected radar and scheduled a channel switch in
			 * the future, then every TBTT interval we come here and decrement the
			 * 'TBTT time to chan switch' field in the outgoing beacons.
			 */
			if (wlc->spect_state & NEED_TO_UPDATE_BCN) {
				/* If all counts are 0, no further need to update outgoing beacons
				 */
				if ((wlc->quiet_cmd.count == 0) && (wlc->csa.count == 0)) {
					wlc->spect_state &= ~NEED_TO_UPDATE_BCN;
				}

				/* If time to switch channels... */
				if (wlc->csa.count == 0) {
					if (wlc->spect_state & NEED_TO_SWITCH_CHANNEL) {
						wlc_11h_do_chanswitch(wlc, wlc->csa.channel);
						wlc->spect_state &= ~NEED_TO_SWITCH_CHANNEL;
					}
					wlc->block_datafifo &= ~DATA_BLOCK_QUIET; /* Resume tx */
				}
				else {
					/* to updated channel switch count of csa ie. */
					wlc->csa.count--;
				}

				wlc_update_beacon(wlc);
			}
			if (wlc->pub.associated) {
				/* It's time to start a 802.11h quiet period. */
				if (QUIET_STATE(wlc) & WAITING_FOR_INTERVAL) {
					if (--wlc->quiet.countdown == 0) {
						wlc_start_quiet0(wlc);
					}
				}
			}
		} /* (WL11H_ENAB(wlc)) */
	}	/* MI_TBTT) */


	/* ATIM window end */
	if (macintstatus & MI_ATIMWINEND) {
		WL_TRACE(("wlc_isr: end of ATIM window\n"));

		OR_REG(&regs->maccommand, wlc->qvalid);
		wlc->qvalid = 0;
	}

	/* phy tx error */
	if (macintstatus & MI_PHYTXERR) {
		WL_ERROR(("wl%d: PHYTX error\n", wlc->pub.unit));
	}

	/* received data or control frame */
	/* MI_DMAINT is indication of RX_FIFO interrupt */
	if (macintstatus & MI_DMAINT) {
		/*
		 * restore relevant interrupt status for next round dpc
		 * if more frames need to be processed.
		 */
		if (wlc_recv(wlc, RX_FIFO, bounded ? RXBND : -1)) {
			wlc->macintstatus |= MI_DMAINT;
		}
	}

	/* TX FIFO suspend/flush completion */
	if (macintstatus & MI_TXSTOP) {
		if (wlc_tx_fifo_suspended(wlc, TX_DATA_FIFO)) {
			/* handle a scan request for tx suspend */
			if (SCAN_IN_PROGRESS(wlc) && (wlc->scan.state & SCAN_STATE_WSUSPEND)) {
				wlc->scan.state &= ~SCAN_STATE_WSUSPEND;
				wl_add_timer(wlc->wl, wlc->scan.timer, 0, 0);
			}
			/* move to next quiet state if both DATA and CTL fifos are suspended */
			if ((QUIET_STATE(wlc) & WAITING_FOR_FLUSH_COMPLETE) &&
			    wlc_tx_fifo_suspended(wlc, TX_CTL_FIFO)) {
				wlc->quiet._state &= ~WAITING_FOR_FLUSH_COMPLETE;
				wlc_start_quiet2(wlc);
			}
		}
	}

	if (macintstatus & MI_GP0) {
		WL_ERROR(("wl%d: The PSM microcode watchdog timer fired; reinitializing the"
			" device\n", wlc->pub.unit));
		ASSERT(0);
		wl_init(wlc->wl);	/* big hammer */
	}

	/* gptimer timeout */
	if (macintstatus & MI_TO) {
		W_REG(&wlc->regs->gptimer, 0);
	}

	if (macintstatus & MI_RFDISABLE) {
		WL_ERROR(("wl%d: MAC Detected a change on the RF Disable Input 0x%x\n",
			wlc->pub.unit, wlc_phy_radio_disabled((void *)wlc->regs)));
		if (BSSCFG_STA(&wlc->cfg) && wlc_phy_radio_disabled((void *)wlc->regs)) {
			/* delay the cleanup to wl_down in IBSS case */
			if (wlc->cfg.enable && wlc->pub.BSS) {
				wlc_bsscfg_disable(wlc, &wlc->cfg);
			}
		}
	}

	/* send any enq'd tx packets. Just makes sure to jump start tx */
	if (!pktq_empty(&wlc->txq))
		wlc_send_q(wlc, &wlc->txq);


	/* make sure the bound indication and the implementation are in sync */
	ASSERT(bounded == TRUE || wlc->macintstatus == 0);

	/* it isn't done and needs to be resched if macintstatus is non-zero */
	return wlc->macintstatus != 0;
}

/* process tx completion events for corerev >= 5 */
/*
 * Return TRUE if more tx status need to be processed. FALSE otherwise.
 * Param 'bound' indicates max. # tx status to process before break out.
 */
static bool
wlc_txstatus(wlc_info_t *wlc, uint bound)
{
	tx_status_t txstatus, *txs;
	uint32 s1, s2;
	uint n = 0;

	WL_TRACE(("wl%d: wlc_txstatus\n", wlc->pub.unit));

	txs = &txstatus;

	while ((s1 = R_REG(&wlc->regs->frmtxstatus)) & TXS_V) {
		s2 = R_REG(&wlc->regs->frmtxstatus2);

		txs->status = s1 & TXS_STATUS_MASK;
		txs->frameid = (s1 & TXS_FID_MASK) >> TXS_FID_SHIFT;
		txs->sequence = s2 & TXS_SEQ_MASK;
		txs->phyerr = (s2 & TXS_PTX_MASK) >> TXS_PTX_SHIFT;
		txs->lasttxtime = 0;

		wlc_dotxstatus(wlc, txs);

		/* !give others some time to run! */
		if (++n >= bound)
			break;
	}

	if (!pktq_empty(&wlc->txq))
		wlc_send_q(wlc, &wlc->txq);

	return n >= bound;
}

/* process tx completion events for corerev < 5 */
void
wlc_txstatusrecv(wlc_info_t *wlc)
{
	void *status_p;
	tx_status_t *txs;
	osl_t *osh;

	WL_TRACE(("wl%d: wlc_txstatusrecv\n", wlc->pub.unit));

	osh = wlc->pub.osh;

	while (PIO_ENAB(wlc) ?
		(status_p = wlc_pio_rx(wlc->core->pio[RX_TXSTATUS_FIFO])) :
		(status_p = dma_rx(wlc->core->di[RX_TXSTATUS_FIFO]))) {

		txs = (tx_status_t *)PKTDATA(osh, status_p);
		/* MAC uses little endian only */
		ltoh16_buf((void*)txs, sizeof(tx_status_t));

		/* shift low bits for tx_status_t status compatibility */
		txs->status = (txs->status & ~TXS_COMPAT_MASK)
			| (((txs->status & TXS_COMPAT_MASK) << TXS_COMPAT_SHIFT));

		wlc_dotxstatus(wlc, txs);

		PKTFREE(osh, status_p, FALSE);
	}

	/* post more rbufs */
	if (!PIO_ENAB(wlc))
		dma_rxfill(wlc->core->di[RX_TXSTATUS_FIFO]);

	if (!pktq_empty(&wlc->txq))
		wlc_send_q(wlc, &wlc->txq);
}

static void*
wlc_15420war(wlc_info_t *wlc, uint queue)
{
	hnddma_t *di;
	void *p;

	ASSERT(queue < NFIFO);

	if (PIO_ENAB(wlc))
		return (NULL);

	if ((D11REV_LT(wlc->pub.corerev, 5)) || (D11REV_GT(wlc->pub.corerev, 6)))
		return (NULL);

	di = wlc->core->di[queue];
	ASSERT(di);

	/* get next packet, ignoring XmtStatus.Curr */
	p = dma_getnexttxp(di, TRUE);

	/* sw block tx dma */
	dma_txblock(di);

	/* if tx ring is now empty, reset and re-init the tx dma channel */
	if (dma_txactive(wlc->core->di[queue]) == 0) {
		WLCNTINCR(wlc->pub._cnt.txdmawar);
		if (!dma_txreset(di))
			WL_ERROR(("wl%d: %s: dma_txreset[%d]: cannot stop dma\n",
			          wlc->pub.unit, __FUNCTION__, queue));
		dma_txinit(di);
	}

	return (p);
}

static void
wlc_war16165(wlc_info_t *wlc, bool tx)
{
	if (tx) {
		/* the post-increment is used in STAY_AWAKE macro */
		if (wlc->txpend16165war++ == 0)
			wlc_set_ps_ctrl(wlc);
	} else {
		wlc->txpend16165war--;
		if (wlc->txpend16165war == 0)
			wlc_set_ps_ctrl(wlc);
	}
}

/* process an individual tx_status_t */
static void
wlc_dotxstatus(wlc_info_t *wlc, tx_status_t *txs)
{
	void *p;
	uint queue;
	d11txh_t *txh;
	struct scb *scb;
	bool update_rate, free_pdu;
	osl_t *osh;
	int tx_rts, tx_frame_count, tx_rts_count;
	uint totlen, supr_status;
	bool lastframe;
	struct dot11_header *h;
	uint16 fc;

	osh = wlc->pub.osh;

	if (WL_PRHDRS_ON())
		wlc_dump_txstatus(txs);

	/* check head of xmt queues to see which packet to reclaim */
	queue = txs->frameid & TXFID_QUEUE_MASK;
	ASSERT(queue < NFIFO);

	/* just discard intermediate indications for now */
	if (txs->status & TX_STATUS_INTERMEDIATE) {
		WLCNTADD(wlc->pub._cnt.txnoack, ((txs->status & TX_STATUS_FRM_RTX_MASK) >>
			TX_STATUS_FRM_RTX_SHIFT));
		return;
	}
	supr_status = txs->status & TX_STATUS_SUPR_MASK;

	if (!PIO_ENAB(wlc)) {
		p = dma_getnexttxp(wlc->core->di[queue], FALSE);

		if (WLC_WAR16165(wlc))
			wlc_war16165(wlc, FALSE);
	} else {
		p = wlc_pio_getnexttxp(wlc->core->pio[queue]);
	}

	if (p == NULL)
		p = wlc_15420war(wlc, queue);

	if (p == NULL) {
		WL_ERROR(("%s: Weird, p NULL, returning status 0x%x\n", __FUNCTION__, txs->status));
		return;
	}
	ASSERT(p);

	txh = (d11txh_t *)PKTDATA(wlc->pub.osh, p);
	ASSERT(txs->frameid == htol16(txh->TxFrameID));

	/* scb pointer is cleared before enqueueing on
	 * DMA ring or PIO. So SCB should be NULL here
	 */
	ASSERT(WLPKTTAGSCB(p) == NULL);

	scb = wlc_scbfind(wlc, (struct ether_addr*)txh->TxFrameRA);

	if (scb)
		WLPKTTAGSCBSET(p, scb);

	tx_rts = htol16(txh->MacTxControlLow) & TXC_SENDRTS;

	tx_frame_count = (txs->status & TX_STATUS_FRM_RTX_MASK) >> TX_STATUS_FRM_RTX_SHIFT;
	tx_rts_count = (txs->status & TX_STATUS_RTS_RTX_MASK) >> TX_STATUS_RTS_RTX_SHIFT;

	/* count all retransmits */
	if (tx_frame_count)
		WLCNTADD(wlc->pub._cnt.txretrans, (tx_frame_count - 1));
	if (tx_rts_count)
		WLCNTADD(wlc->pub._cnt.txretrans, (tx_rts_count - 1));

	h = (struct dot11_header *)((uint8*)(txh+1) + D11_PHY_HDR_LEN);
	fc = ltoh16(h->fc);
	lastframe = (fc & FC_MOREFRAG) == 0;

	totlen = pkttotlen(wlc->pub.osh, p);

	/* plan for success */
	update_rate = TRUE;
	free_pdu = TRUE;

	/* process txstatus info */
	if (txs->status & TX_STATUS_ACK_RCV) {
		/* success... */
		/* if waiting for PM-indicated, let core go to sleep */
		if ( wlc->PMpending && (txs->status & TX_STATUS_PMINDCTD)) {
			wlc->PMpending = FALSE;
			WL_INFORM(("wl%d: PS-MODE: PM-indicated, %s\n", wlc->pub.unit,
				STAY_AWAKE(wlc)?"but remaining awake":"going to doze state"));
			wlc_set_ps_ctrl(wlc);
		}


		/* update counters */
		WLCNTINCR(wlc->pub._cnt.txfrag);
		WLCNTADD(wlc->pub._cnt.txnoack, (tx_frame_count - 1));
		if (tx_frame_count > 1)
			wlc->txretried += tx_frame_count - 1;
		if (lastframe) {
			WLCNTINCR(wlc->pub._cnt.txfrmsnt);
			if (wlc->txretried >= 1)
				WLCNTINCR(wlc->pub._cnt.txretry);
			if (wlc->txretried > 1)
				WLCNTINCR(wlc->pub._cnt.txretrie);
		}
		if (scb)
			scb->used = wlc->pub.now;

	} else if (((txs->status & TX_STATUS_MASK) == TX_STATUS_NO_ACK)) {

		update_rate = FALSE;

		/* update counters */
		WLCNTINCR(wlc->pub._cnt.txfrag);
		if (lastframe)
			WLCNTINCR(wlc->pub._cnt.txfrmsnt);

		if (scb)
			scb->used = wlc->pub.now;
	} else if (supr_status == TX_STATUS_SUPR_FRAG) {
		/* subsequent frag failure... */
		update_rate = FALSE;
	} else if (supr_status == TX_STATUS_SUPR_BADCH) {
		update_rate = FALSE;
		WLCNTINCR(wlc->pub._cnt.txchanrej);
	} else if (supr_status == TX_STATUS_SUPR_EXPTIME) {
			WLCNTINCR(wlc->pub._wme_cnt.tx_expired
			            [(WME_ENAB(wlc) ? WME_PRIO2AC(PKTPRIO(p)) : AC_BE)].packets);
	} else if (supr_status == TX_STATUS_SUPR_PMQ) {
		update_rate = FALSE;

	} else if (supr_status == TX_STATUS_SUPR_FLUSH) {
		update_rate = FALSE;
		free_pdu = FALSE;

		ASSERT(0);
	} else if (supr_status == TX_STATUS_SUPR_UF) {
		update_rate = FALSE;

	} else if (txs->phyerr) {
		update_rate = FALSE;
		WLCNTINCR(wlc->pub._cnt.txphyerr);
		WL_ERROR(("wl%d: wlc_dotxstatus: tx phy error (0x%x)\n", wlc->pub.unit,
			txs->phyerr));
	} else if (ETHER_ISMULTI(txh->TxFrameRA)) {
		/* mcast success
		 * since mcast doesn't get acked, and it'll be transmitted
		 * just once, there's nothing for rate selection to do
		 * with this tx status.
		 */
		update_rate = FALSE;

		/* update counters */
		if (tx_frame_count)
			WLCNTINCR(wlc->pub._cnt.txfrag);

		if (lastframe)
			if (!wlc->pub.BSS) {
				WLCNTINCR(wlc->pub._cnt.txfrmsnt);
				if (tx_frame_count)
					WLCNTINCR(wlc->pub._cnt.txmulti);
			}
	} else if (wlc->pub.BSS && ETHER_ISMULTI(&h->a3)) {
		if (lastframe) {
			WLCNTINCR(wlc->pub._cnt.txfrmsnt);
			if (tx_frame_count)
				WLCNTINCR(wlc->pub._cnt.txmulti);
		}
	} else if ((!tx_rts && tx_frame_count == wlc->SRL) ||
		(tx_rts && (tx_rts_count == wlc->SRL || tx_frame_count == wlc->LRL))) {
		WLCNTADD(wlc->pub._cnt.txnoack, tx_frame_count);
		/* ucast failure */
		if (lastframe) {
			WLCNTINCR(wlc->pub._cnt.txfail);
			wlc_mac_event(wlc, WLC_E_TXFAIL, (struct ether_addr *)txh->TxFrameRA,
				WLC_E_STATUS_TIMEOUT, 0, 0, 0, 0);
		}

		/* if PS-Poll failed, let core go back to sleep */
		if (wlc->PSpoll &&
		    (uint16)wlc->PSpoll == txs->frameid) {
			WL_INFORM(("PS-MODE: PS-Poll tx failed...go to sleep (%d)\n",
			           STAY_AWAKE(wlc)));
			wlc->PSpoll = FALSE;
			wlc_set_ps_ctrl(wlc);
		}
	} else {
		/* unexpected tx status */
		update_rate = FALSE;
		WLCNTINCR(wlc->pub._cnt.txserr);
		WL_ERROR(("wl%d: wlc_dotxstatus: unexpected tx status returned (0x%x)\n",
			wlc->pub.unit, txs->status));
	}

	/* Control/Mgmt frames do not use rate selection */
	if ((FC_TYPE(fc) == FC_TYPE_MNG) || (FC_TYPE(fc) == FC_TYPE_CTL))
		update_rate = FALSE;

	/* update rate state and scb used time */
	if (update_rate && scb) {

		ASSERT(!TX_STATUS_UNEXP(txs->status));

		wlc_rate_upd_tx(scb->ratestate, scb->select_rateset.rates,
			scb->select_rateset.count,
			txs, (tx_rts_count > 1 ? 1 : 0),
			wlc->SFBL, wlc->LFBL, 0);
	}

	wlc->core->txpktpend[queue]--;
	/* There is more room; mark precedences related to this FIFO sendable */
	WLC_TX_FIFO_ENAB(wlc, queue);
	ASSERT(wlc->core->txpktpend[queue] >= 0);




	/* update the bytes in/out counts */
	if (PIO_ENAB(wlc))
		wlc_pio_cntupd(wlc->core->pio[queue], totlen);

	/* call any matching pkt callbacks */
	if (WLPKTTAG(p)->callbackidx)
		wlc_pkt_callback(wlc, p, txs->status);

	/* free the mpdu */
	if (free_pdu)
		PKTFREE(osh, p, TRUE);

	if (lastframe)
		wlc->txretried = 0;
}

static void
wlc_tbtt_adopt(wlc_info_t *wlc, struct dot11_bcn_prb *bcn)
{
	wlc_tbtt_align(wlc, bcn, 1, 0, 0, 0);
}

static void
wlc_tbtt_align(wlc_info_t *wlc, struct dot11_bcn_prb *bcn, bool adopt, bool is_ofdm, uint phy_rate,
	uint short_preamble)
{
	d11regs_t *regs = wlc->regs;
	uint32 bo = ltoh16(bcn->beacon_interval);
	uint32 bp;
	uint32 tbtt, tbtt_leway;
	uint32 bcn_l, bcn_h, bcn_offset, bcn_tbtt;

	bcn_l = ltoh32(bcn->timestamp[0]);
	bcn_h = ltoh32(bcn->timestamp[1]);

	/* illed AP beacon frame, set beacon interval to WECA maximum */
	if (bo == 0) {
		WL_ERROR(("wlc_update_tbtt: !bad AP, change beacon period\n"));
		bp = 1024;
	} else
		bp = bo;

	if (!adopt && wlc->check_for_unaligned_tbtt) {
		uint tsf2preamble;

		/* use beacon start to determine if beacon is really unaligned
		 * [--------------------------[--------------------------[
		 *                                       =======TSF(1)
		 *                      =======TSF(2)
		 *                            <-- bcn_tbtt(based on tsf timestamp)
		 * <-- bcn_tbtt(based on beacon frame starting timestamp)
		 *
		 * case(1) is normal, tsf_offset can determine if beacon is unaligned
		 * case(2) is unusual, tsf_offset will result in aligned condition(false)
		 *   subtracting tsf2preamble out of bcn_l before calling wlc_calc_tbtt_offset
		 * will result in a large bcn_offset, which is out of WLC_MAX_TBTT_OFFSET and
		 * trigger adjustment
		 */
		tsf2preamble = wlc_compute_bcntsfoff(wlc, is_ofdm, phy_rate, short_preamble, FALSE);
		wlc_uint64_sub(&bcn_h, &bcn_l, 0, tsf2preamble);
		WL_ASSOC(("TBTT: wl%d: tsf2preamble %d is_ofdm %d phy_rate 0x%x, short_preamble"
			" %d\n",
			wlc->pub.unit, tsf2preamble, is_ofdm, phy_rate, short_preamble));
	}

	/* CFP start should be set to a future TBTT. Start by finding the most recent TBTT. */
	bcn_offset = wlc_calc_tbtt_offset(bp, bcn_h, bcn_l);

	/* TBTT = Beacon TSF Timestamp(uint64) - bcn_tsf_offset, but only interested in low 32 bits
	 */
	bcn_tbtt = bcn_l - bcn_offset;

	WL_ASSOC(("wl%d: bcn_h 0x%x bcn_l 0x%x bcn_offset 0x%x bcn_tbtt 0x%x\n", wlc->pub.unit,
		bcn_h, bcn_l, bcn_offset, bcn_tbtt));
	if (adopt || !wlc->check_for_unaligned_tbtt) {
		WL_ASSOC(("wl%d: adopting aligned TBTT\n", wlc->pub.unit));
	} else {
		if ((bcn_offset >> 10) > WLC_MAX_TBTT_OFFSET) {
			bcn_tbtt += bcn_offset;
			WL_ASSOC(("wl%d: adjust to unaligned (%d us) TBTT, new_tbtt 0x%x\n",
				wlc->pub.unit, bcn_offset, bcn_tbtt));
		} else {
			WL_ASSOC(("wl%d: adjust to aligned TBTT\n", wlc->pub.unit));
		}
	}

	/* Add at least 100 Kus of beacon intervals to make sure we still have enough time in the
	 * future.
	 */
	tbtt_leway = ROUNDUP((bcn_offset + (100 << 10)), (bp << 10));

	tbtt = bcn_tbtt + tbtt_leway;

	WL_ASSOC(("wl%d: old cfpstart = 0x%x, new = 0x%x\n", wlc->pub.unit,
	          R_REG(&regs->tsf_cfpstart), tbtt));

	W_REG(&regs->tsf_cfpstart, tbtt);
}

/* Given the beacon interval in kus, and a 64 bit TSF in us,
 * return the offset (in us) of the TSF from the last TBTT
 */
uint32
wlc_calc_tbtt_offset(uint32 bp, uint32 tsf_h, uint32 tsf_l)
{
	uint32 k, btklo, btkhi, offset;

	/* TBTT is always an even multiple of the beacon_interval,
	 * so the TBTT less than or equal to the beacon timestamp is
	 * the beacon timestamp minus the becon timestamp modulo
	 * the beacon interval.
	 *
	 * TBTT = BT - (BT % BIu)
	 *      = (BTk - (BTk % BP)) * 2^10
	 *
	 * BT = beacon timestamp (usec, 64bits)
	 * BTk = beacon timestamp (Kusec, 54bits)
	 * BP = beacon interval (Kusec, 16bits)
	 * BIu = BP * 2^10 = beacon interval (usec, 26bits)
	 *
	 * To keep the calculations in uint32s, the modulo operation
	 * on the high part of BT needs to be done in parts using the
	 * relations:
	 * X*Y mod Z = ((X mod Z) * (Y mod Z)) mod Z
	 * and
	 * (X + Y) mod Z = ((X mod Z) + (Y mod Z)) mod Z
	 *
	 * So, if BTk[n] = uint16 n [0,3] of BTk.
	 * BTk % BP = SUM((BTk[n] * 2^16n) % BP , 0<=n<4) % BP
	 * and the SUM term can be broken down:
	 * (BTk[n] *     2^16n)    % BP
	 * (BTk[n] * (2^16n % BP)) % BP
	 *
	 * Create a set of power of 2 mod BP constants:
	 * K[n] = 2^(16n) % BP
	 *      = (K[n-1] * 2^16) % BP
	 * K[2] = 2^32 % BP = ((2^16 % BP) * 2^16) % BP
	 *
	 * BTk % BP = BTk[0-1] % BP +
	 *            (BTk[2] * K[2]) % BP +
	 *            (BTk[3] * K[3]) % BP
	 *
	 * Since K[n] < 2^16 and BTk[n] is < 2^16, then BTk[n] * K[n] < 2^32
	 */

	/* BTk = BT >> 10, btklo = BTk[0-3], bkthi = BTk[4-6] */
	btklo = (tsf_h << 22) | (tsf_l >> 10);
	btkhi = tsf_h >> 10;

	/* offset = BTk % BP */
	offset = btklo % bp;

	/* K[2] = ((2^16 % BP) * 2^16) % BP */
	k = (uint32)(1<<16) % bp;
	k = (uint32)(k * 1<<16) % (uint32)bp;

	/* offset += (BTk[2] * K[2]) % BP */
	offset += ((btkhi & 0xffff) * k) % bp;

	/* BTk[3] */
	btkhi = btkhi >> 16;

	/* k[3] = (K[2] * 2^16) % BP */
	k = (k << 16) % bp;

	/* offset += (BTk[3] * K[3]) % BP */
	offset += ((btkhi & 0xffff) * k) % bp;

	offset = offset % bp;

	/* convert offset from kus to us by shifting up 10 bits and
	 * add in the low 10 bits of tsf that we ignored
	 */
	offset = (offset << 10) + (tsf_l & 0x3FF);


	return offset;
}

static int
wlc_bcn_tsf_later(wlc_info_t* wlc, d11rxhdr_t *rxh, void* bcn_plcp)
{
	struct dot11_bcn_prb *bcn;
	uint16 rxtime;
	uint32 tsf_l, tsf_h;
	uint32 bcn_tsf_l, bcn_tsf_h;
	uint32 offset;
	bool is_ofdm;
	uint phy_rate, phy_type;

	/* read the tsf from our chip */
	wlc_read_tsf(wlc, &tsf_l, &tsf_h);

	/* read the tsf from the beacon */
	bcn = (struct dot11_bcn_prb *)((int8*)bcn_plcp + D11_PHY_HDR_LEN + DOT11_MAC_HDR_LEN);
	bcn_tsf_l = ltoh32(((uint32*)&bcn->timestamp)[0]);
	bcn_tsf_h = ltoh32(((uint32*)&bcn->timestamp)[1]);

	/* if the bcn_tsf is later than the current tsf, then we know it is later than the
	 * tsf at the time of the beacon
	 */
	if (wlc_uint64_lt(tsf_h, tsf_l, bcn_tsf_h, bcn_tsf_l)) {
		WL_ASSOC(("wlc_bcn_tsf_later: current local time: %08x:%08x\n", tsf_h, tsf_l));
		WL_ASSOC(("wlc_bcn_tsf_later: beacon tsf time: %08x:%08x\n", bcn_tsf_h, bcn_tsf_l));
		return 1;
	}

	/* expand the rxtime to 64 bits */
	rxtime = rxh->RxTSFTime;
	if (rxtime < (tsf_l & 0xffff)) {
		tsf_l &= 0xffff0000;
	} else {
		/* tsf_l has rolled over 2^16 so roll the upper bits back */
		tsf_l &= 0xffff0000;
		if (tsf_l > 0) {
			tsf_l -= 1<<16;
		} else {
			tsf_l = 0xffff0000;
			tsf_h -= 1;
		}
	}
	tsf_l |= rxtime;

	/* calculate the rxtime offset to the transmission time of the first
	 * symbol of the beacon timestamp
	 */
	phy_type = ((rxh->RxChan & RXS_CHAN_PHYTYPE_MASK) >> RXS_CHAN_PHYTYPE_SHIFT);

	is_ofdm = (phy_type == PHY_TYPE_A) || (rxh->PhyRxStatus_0 & PRXS0_OFDM);
	phy_rate = PLCP_PHYRATE(bcn_plcp, is_ofdm);
	offset = wlc_compute_airtime(wlc, is_ofdm, phy_rate, DOT11_MAC_HDR_LEN);

	/* rxtime is slightly offset from the end of the plcp due to rf delay */
	offset -= M_BPHY_PLCPRX_DLY;

	/* do uint64 math to add offset to our tsf */
	wlc_uint64_add(&tsf_h, &tsf_l, 0, offset);

	WL_ASSOC(("wlc_bcn_tsf_later:  local tsf time: %08x:%08x\n", tsf_h, tsf_l));
	WL_ASSOC(("wlc_bcn_tsf_later: beacon tsf time: %08x:%08x\n", bcn_tsf_h, bcn_tsf_l));

	if (wlc_uint64_lt(tsf_h, tsf_l, bcn_tsf_h, bcn_tsf_l))
		return 1;

	return 0;
}

/*
 *	Set and ensure new PS mode is communicated
 */
void
wlc_set_pmstate(wlc_info_t *wlc, bool state)
{
	void *pkt = NULL;

	WL_INFORM(("wl%d: wlc_set_pmstate to %d\n", wlc->pub.unit, state));
	wlc->PMenabled = state;
	wlc->PMpending = PS_ALLOWED(wlc) ? TRUE : FALSE;
	wlc->PMidlebcns = 0;

	if (!wlc->pub.up)
		return;

	wlc_set_ps_ctrl(wlc);

	/* send NULL data frame to communicate PM state */
	if (wlc->pub.BSS && wlc->pub.associated) {
		/* don't bother sending a null data frame if we lost our AP connection */
		if (WLC_BSS_CONNECTED(wlc))
			if (!(pkt = wlc_sendnulldata(wlc, &wlc->BSSID, 0, 0, PRIO_8021D_BE)))
				WL_ERROR(("wl%d: wlc_set_pmstate: wlc_scbfind failed for BSSID"
					" %s\n",
					wlc->pub.unit, bcm_ether_ntoa(&wlc->BSSID, eabuf)));

		if (wlc->PMpending) {
			/* setup the packet callback or call directly in case of error */
			if (pkt == NULL ||
			    wlc_pkt_callback_register(wlc, wlc_ps_null_complete, NULL, pkt))
				wlc_ps_null_complete(wlc, TX_STATUS_NO_ACK, NULL);
		}
	}
}

static void
wlc_ps_null_complete(wlc_info_t *wlc, uint txstatus, void *arg)
{
	if (SCAN_IN_PROGRESS(wlc) && (wlc->scan.state & SCAN_STATE_PSPEND)) {
		wlc->scan.state &= ~SCAN_STATE_PSPEND;
		wl_add_timer(wlc->wl, wlc->scan.timer, 0, 0);
	}
}

static void
wlc_auth_complete(wlc_info_t *wlc, uint txstatus, void *arg)
{
	/* assoc aborted? */
	if (!(wlc->assoc_state == AS_SENT_AUTH_1 || wlc->assoc_state == AS_SENT_AUTH_3))
		return;

	/* no ack */
	if (!(txstatus & TX_STATUS_ACK_RCV)) {
		wlc_mac_event(wlc, WLC_E_AUTH, &wlc->target_bss.BSSID, WLC_E_STATUS_NO_ACK, 0, 0, 0,
			0);
		return;
	}
	wl_del_timer(wlc->wl, wlc->assoc_timer);
	wl_add_timer(wlc->wl, wlc->assoc_timer, WECA_ASSOC_TIMEOUT + 10, 0);
}

static void
wlc_assocreq_complete(wlc_info_t *wlc, uint txstatus, void *arg)
{
	/* assoc aborted? */
	if (wlc->assoc_state != AS_SENT_ASSOC)
		return;

	/* no ack */
	if (!(txstatus & TX_STATUS_ACK_RCV)) {
		wlc_mac_event(wlc, WLC_E_ASSOC, &wlc->target_bss.BSSID, WLC_E_STATUS_NO_ACK, 0, 0,
			0, 0);
		return;
	}
	wl_del_timer(wlc->wl, wlc->assoc_timer);
	wl_add_timer(wlc->wl, wlc->assoc_timer, WECA_ASSOC_TIMEOUT + 10, 0);
}

/*
 * Check if STA's bit is set in TIM
 * See Tality MS/futils.c: boFUTILS_InTIM()
 */
static bool
wlc_InTIM(wlc_info_t *wlc, void *body, uint plen)
{
	bcm_tlv_t *tim_ie;
	uint pvboff, AIDbyte;
	uint8 *pvb;
	int pvblen;

	tim_ie = bcm_parse_tlvs((uint8 *)body + DOT11_BCN_PRB_LEN, plen - DOT11_BCN_PRB_LEN,
		DOT11_MNG_TIM_ID);
	if (!tim_ie) {
		/* sick AP, prevent going to power-save mode */
		wlc->PM_override = TRUE;
		wlc_set_ps_ctrl(wlc);
		/* send NULL data frame to communicate PM state */
		(void)wlc_sendnulldata(wlc, &wlc->BSSID, 0, 0, PRIO_8021D_BE);
		return FALSE;
	}

#if defined(DEBUG_TIM)
	if (WL_INFORM_ON()) {
		prhex("BCN_INFO: TIM", tim_ie, TLV_HDR_LEN + tim_ie->len);
	}
#endif /* defined(DEBUG_TIM) */

	/* extract bitmap offset (floor(N1/2)) from bitmap control field */
	pvboff = (tim_ie->data[DOT11_MNG_TIM_BITMAP_CTL] >> 1) << 1;

	/* compute bitmap length (N2 - N1) from info element length */
	pvblen = tim_ie->len - 4;

	/* bail early if single-byte zero bitmap with zero offset */
	pvb = &tim_ie->data[DOT11_MNG_TIM_PVB];
	if (!pvblen && !*pvb && !pvboff)
		return FALSE;

	/* bail early if our AID precedes the TIM */
	AIDbyte = (wlc->AID & DOT11_AID_MASK) >> 3;
	if (AIDbyte < pvboff)
		return FALSE;

	/* search for our bit in bitmap */
	for (; pvblen >= 0 && AIDbyte >= pvboff; pvboff++, pvblen--, pvb++) {
		if (AIDbyte == pvboff) {
			return ((*pvb & (1 << (wlc->AID & 0x7))) ? TRUE : FALSE);
		}
	}

	/* returns here if the TIM finishes before we've reached our AID */
	return FALSE;
}

/*
 * Filters frames based on class and the current authentication and
 * association states of the remote station.
 * State 1 is unauthenticated and unassociated.
 * State 2 is authenticated and unassociated.
 * State 3 is authenticated and associated.
 * Class 1 frames are permitted from within states 1, 2, and 3.
 * Class 2 frames are permitted if and only if authenticated with the remote station
 *	(i.e., allowed from within States 2 and 3 only).
 * Class 3 frames are permitted if and only if associated with the remote station
 *	(i.e., allowed only from within State 3).
 *
 * Returns 0 on success (accept frame) and non-zero reason code on failure (toss frame).
 * Updates pointer to scb if provided.
 */
static uint16
wlc_recvfilter(wlc_info_t *wlc, struct dot11_header *h, d11rxhdr_t *rxh, struct scb **pscb)
{
	uint16 fc = ltoh16(h->fc), rc;
	struct scb *scb = NULL;
	int class;


	/* Default is class 1 */
	class = 1;
	rc = 0;

	switch (FC_TYPE(fc)) {

	case FC_TYPE_CTL:
	case FC_TYPE_MNG:
		/* Control and management frames should have no DS bits set */
		if (fc & (FC_TODS | FC_FROMDS))
			goto bad;

		switch (fc & FC_KIND_MASK) {

		/* Class 2 control and management frames */
		case FC_ASSOC_REQ:
		case FC_ASSOC_RESP:
		case FC_REASSOC_REQ:
		case FC_REASSOC_RESP:
		case FC_DISASSOC:
			class = 2;
			rc = DOT11_RC_INVAL_CLASS_2;
			break;

		/* Class 3 control and management frames */
		case FC_ACTION:
		case FC_PS_POLL:
		case FC_BLOCKACK:
		case FC_BLOCKACK_REQ:
			class = 3;
			rc = DOT11_RC_INVAL_CLASS_3;
			break;

		/* All other frame types are class 1 */
		default:
			goto done;
		}
		break;

	case FC_TYPE_DATA:
		/* Data frames with no DS bits set are class 1 */
		if (!(fc & (FC_TODS | FC_FROMDS)))
			goto done;

		/* wds frames not supported */
		if ((fc & (FC_TODS | FC_FROMDS)) == (FC_TODS | FC_FROMDS))
			goto done;

		else {
			/* Data frames with DS bits set are class 3 */
			class = 3;
			rc = DOT11_RC_INVAL_CLASS_3;
		}

		break;

	bad:
	default:
		WL_ERROR(("wl%d: %s: bad frame control 0x%.4x\n", wlc->pub.unit, __FUNCTION__, fc));
		rc = DOT11_RC_UNSPECIFIED;
		WLCNTINCR(wlc->pub._cnt.rxbadcm);
		goto done;
	}

	/* by this point, incoming packets are either class 2, 3, or 4 (WDS) */

	scb = wlc_scbfindband(wlc, &h->a2, CHANNEL_BANDUNIT(wlc, wlc_rx_channel(rxh)));

	/* early bail-out, all frames accepted at associated or wds state */
	/* Need to restrict to some subset */
	if (scb && (SCB_ASSOCIATED(scb))) {
		rc = 0;		/* Accept frame */
	}
	/* Toss if nonauthenticated */
	else if (!scb || !SCB_AUTHENTICATED(scb)) {
		WL_INFORM(("wl%d: %s: invalid class %d frame from non-authenticated station %s\n",
			wlc->pub.unit, __FUNCTION__, class, sa));
		/* do not send a deauth if we are on the way to authenticating since
		 * our deauth and an incoming auth response may cross paths
		 * also, don't send if we're not currently on-channel
		 */
		if (!ETHER_ISMULTI(&h->a1) && !(scb && (scb->state & PENDING_AUTH)) &&
		    (CHANNEL_BANDUNIT(wlc, wlc_rx_channel(rxh)) == (int) wlc->band->bandunit))
		{
			struct ether_addr *bssid = &wlc->BSSID;

			(void)wlc_senddeauth(wlc, &h->a2, bssid, scb, rc);
			wlc_mac_event(wlc, WLC_E_DEAUTH, &h->a2, WLC_E_STATUS_SUCCESS, rc,
			              0, 0, 0);
		}
		WLCNTINCR(wlc->pub._cnt.rxbadproto);
	}
	/* Toss if nonassociated */
	else if (class == 3 && !SCB_ASSOCIATED(scb)) {
		WL_INFORM(("wl%d: %s: invalid class %d frame from non-associated station %s\n",
			wlc->pub.unit, __FUNCTION__, class, sa));
		/* do not send a disassoc if we are on the way to associating since
		 * our disassoc and an incoming assoc response may cross paths
		 */
		if (!ETHER_ISMULTI(&h->a1) && !(scb->state & PENDING_ASSOC))
			if (CHANNEL_BANDUNIT(wlc, wlc_rx_channel(rxh)) == (int) wlc->band->bandunit)
			{
				struct ether_addr *bssid;
				bssid = (scb->flags & SCB_MYAP) ? &h->a2 : &wlc->announce_BSSID;
				wlc_senddisassoc(wlc, &h->a2, bssid, scb, rc);
			}
		WLCNTINCR(wlc->pub._cnt.rxbadproto);
	}
	/* Accept frame */
	else
		rc = 0;

done:
	if (pscb)
		*pscb = scb;
	return rc;
}

/* Can be called in either STA or AP context.
 * In STA mode, we received a Channel Switch announcement.
 * In AP, we detected radar and need to move off channel.
 */
static void
wlc_11h_do_chanswitch(wlc_info_t *wlc, uint8 newchan)
{
	WL_INFORM(("wl%d: %s: new channel %d\n", wlc->pub.unit, __FUNCTION__, newchan));

	if (VALID_CHANNEL(wlc, newchan)) {
		/* restore the current channel's quiet bit */
		if (wlc_radar_channel(wlc, (uint)wlc->pub.current_bss.channel) == TRUE) {
			SETQUIET(wlc, wlc->pub.current_bss.channel);
		}

		wlc->pub.current_bss.channel = newchan;

		wlc_suspend_mac_and_wait(wlc);
		wlc_set_channel(wlc, newchan);
		wlc_enable_mac(wlc);
		WL_REGULATORY(("%s: switching to channel %d\n", __FUNCTION__, newchan));
	} else {
		WL_REGULATORY(("%s: Received invalid channel - ignoring\n", __FUNCTION__));
		/* something needs to be done here ?? */
	}
}


/* STA: Handle incoming Channel Switch Announcement */
static void
wlc_11h_channel_switch(wlc_info_t *wlc, dot11_channel_switch_t *csa)
{
	uint32 bi_us = (uint32)wlc->pub.current_bss.beacon_period * DOT11_TU_TO_US;
	uint32 secs;


	if (!VALID_CHANNEL(wlc, csa->channel)) {
		WL_ERROR(("wl%d: %s: Received invalid channel: %d\n",
			wlc->pub.unit, __FUNCTION__, csa->channel));
		/* Received bogus, malformed or otherwise screwed CSA.
		 * Err on the side of caution and goto an active A band channel now
		 */
		csa->channel = wlc_next_channel(wlc, WLC_MAX_2G_CHANNEL+1, CHAN_TYPE_CHATTY, 0);
		if (csa->channel == wlc->pub.current_bss.channel)
			csa->channel = wlc_next_channel(wlc, csa->channel, CHAN_TYPE_CHATTY, 0);

		csa->count = 1;
	}

	secs = (bi_us * csa->count)/1000000;

	if (wlc->channel_sw.secs == 0) {
		WL_REGULATORY(("%s: Recved CSA: mode %d, channel %d, count %d, secs %d\n",
			__FUNCTION__,
			csa->mode, csa->channel, csa->count, secs));

		/* If target < than 2 second and we need to be quiet until switch time
		 * then might as well switch now.
		 */
		if (secs < 2 && (csa->mode || secs == 0)) {
			WL_REGULATORY(("%s: switch is only %d secs, switching now\n", __FUNCTION__,
				secs));
			wlc_11h_do_chanswitch(wlc, csa->channel);
			return;
		}

		WL_REGULATORY(("%s: Scheduling channel switch in %d secs\n", __FUNCTION__, secs));
		/* Otherwise wait for wlc->channel_sw.secs */
		wlc->channel_sw.secs = secs;
		wlc->channel_sw.channel = csa->channel;
	} else {
		WL_REGULATORY(("%s: ignoring csa: mode %d, channel %d, count %d, secs %d\n",
			__FUNCTION__,
			csa->mode, csa->channel, csa->count, secs));
	}
}

#define FIFO_DRAIN_TIME	33	/* Use worst case: 33 millisecs */
/* ************
    Theory of quiet mode:
	AP sends quiet mode info element.
	wlc_11h_quiet()-parses IE. Take into account the overhead in
	to shutting down the FIFOs (FIFO_DRAIN_TIME) and determine
	proper interval and offset. Register quiet0 to be called when
	we have reached the target beacon interval.

	step0() -	Now in correct beacon interval but need to wait
			for the right offset within beacon.  Use OS timer
			then proceed to step1().

	step1()-	Timer dinged and we are now at the right place to start
			shutting down FIFO's. Wait for FIFO drain interrupts.

	step2()-	Interrupts came in. We are now mute. Set timer
			when its time to come out of quiet mode.

	step3()-	Timer dinged. Resume normal operation.
**********
*/

/* Came from: wlc_parse_11h()
 * Will go to: wlc_start_quiet0()
 */
static void
wlc_11h_quiet(wlc_info_t *wlc, dot11_quiet_t *quiet, struct dot11_bcn_prb *bcn)
{
	uint32 bi_us = (uint32)wlc->pub.current_bss.beacon_period * DOT11_TU_TO_US;
	uint32 tsf_h, tsf_l;
	uint32 offset, beacon_start;
	int32 countdown;

	/* Ignore further requests for quiet if we are already handling one. */
	if (QUIET_STATE(wlc) != 0) {
		WL_REGULATORY(("quiet: rcvd but ignoring:count %d, period %d, duration %d, offset"
			" %d\n",
			quiet->count, quiet->period, ltoh16(quiet->duration),
			ltoh16(quiet->offset)));
		return;
	}

	/* Parse info and setup to wait for the right beacon interval */
	WL_REGULATORY(("quiet: count %d, period %d, duration %d, offset %d\n",
		quiet->count, quiet->period, ltoh16(quiet->duration), ltoh16(quiet->offset)));

	wlc->quiet.duration = ltoh16(quiet->duration);	/* TU's */
	wlc->quiet.offset = ltoh16(quiet->offset);	/* TU's offset from TBTT in Count field */
	wlc->quiet.period = quiet->period;
	countdown = quiet->count;			/* num beacons until start */

	if (!countdown)
		return;
	if (countdown == 1 && wlc->quiet.offset < FIFO_DRAIN_TIME) {
		WL_REGULATORY(("wl%d: %s: Not enough lead time for quiet period\n",
			wlc->pub.unit, __FUNCTION__));
		return;
	}

	/* The FIFO's will take a while to drain.  If the offset is less than that
	 * time, back up into the previous beacon interval.
	 */
	if (wlc->quiet.offset < FIFO_DRAIN_TIME) {
		countdown--;
	}

	/* Calculate absolute TSF of when to start preparing to be quiet */
	tsf_l = ltoh32(bcn->timestamp[0]);
	tsf_h = ltoh32(bcn->timestamp[1]);
	offset = wlc_calc_tbtt_offset(ltoh16(bcn->beacon_interval), tsf_h, tsf_l);
	beacon_start = tsf_l - offset;

	/* usecs for starting quiet period */
	wlc->quiet.start_tsf = beacon_start + (bi_us * quiet->count) + (wlc->quiet.offset *
		DOT11_TU_TO_US);
	wlc->quiet.start_prep_tsf = wlc->quiet.start_tsf - FIFO_DRAIN_TIME * 1000;
	wlc->quiet.end_tsf = wlc->quiet.start_tsf + wlc->quiet.duration * DOT11_TU_TO_US;

	/* Save this for last cuz this is what the dpc looks at */
	wlc->quiet._state = WAITING_FOR_INTERVAL;
	wlc->quiet.countdown = countdown;

	/* The TBTT isr handler will wait until wlc->quiet.countdown reaches zero, and
	 * it will call wlc_start_quiet0().
	 */
}

/* Now in correct beacon interval but need to wait for offset.
 * Came from: TBTT handler in DPC.
 * Will go to: wlc_start_quiet1() via wlc_quiet_timer()
 */
static void
wlc_start_quiet0(wlc_info_t *wlc)
{
	uint32 tsf_hi, tsf_lo;
	uint32 start_prep_ms;

	ASSERT(wlc->quiet._state & WAITING_FOR_INTERVAL);
	WL_REGULATORY(("Entering %s\n", __FUNCTION__));

	wlc->quiet._state = WAITING_FOR_OFFSET;

	wlc_read_tsf(wlc, &tsf_lo, &tsf_hi);
	if (tsf_lo >= wlc->quiet.start_prep_tsf) {
		/* Already late so call routine directly instead of through timer. */
		wlc_start_quiet1(wlc);
	} else {
		start_prep_ms = (wlc->quiet.start_prep_tsf - tsf_lo) / 1000;
		/* Set timer for difference (in millisecs) */
		WL_REGULATORY(("%s: Setting timer for %d millisecs\n",
			__FUNCTION__, start_prep_ms));
		wl_add_timer(wlc->wl, wlc->quiet.timer, start_prep_ms, 0);
	}
}

/* Dual use timer handler: Either
	- Waiting to advance to 'offset' usecs into bcn interval to start quiet period.
	- Or waiting for end of quiet period.
	Came from:  wlc_start_quiet0() via timer interrupt, or
		wlc_start_quiet2() via timer interrupt.
	Will go to: wlc_start_quiet1() or wlc_start_quiet3()
*/
static void
wlc_quiet_timer(void *arg)
{
	wlc_info_t *wlc = (wlc_info_t*)arg;

	if (!wlc->pub.up)
		return;

	if (DEVICEREMOVED(wlc)) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}

	WL_REGULATORY(("Entering %s\n", __FUNCTION__));
	if (QUIET_STATE(wlc) & WAITING_FOR_OFFSET) {
		wlc_start_quiet1(wlc);
	}
	if (QUIET_STATE(wlc) & WAITING_FOR_DURATION) {
		wlc_start_quiet3(wlc);
	}
}

/* Called from wlc_quiet_timer() when we are at proper offset within
 * proper beacon.  Need to initiate quiet period now.
 * Came from: wlc_start_quiet0() via wlc->quiet_timer.
 * Will go to: wlc_start_quiet2().
 */
static void
wlc_start_quiet1(wlc_info_t *wlc)
{

	WL_REGULATORY(("Entering %s\n", __FUNCTION__));

	ASSERT(wlc->quiet._state & WAITING_FOR_OFFSET);
	wlc->quiet._state &= ~WAITING_FOR_OFFSET;

	if (!wlc_mac_request_entry(wlc, WLC_ACTION_QUIET)) {
		/* blocked from entry, cancel quiet period */
		wlc->quiet._state = 0;	/* All done */
		return;
	}

	/* reject future scans until done with quiet */
	wlc->quiet._state |= SILENCE;

	/* Send feedback upstream */
	wlc_mac_event(wlc, WLC_E_QUIET_START, NULL, WLC_E_STATUS_SUCCESS, 0, 0, 0, 0);

	wlc_tx_fifo_suspend(wlc, TX_DATA_FIFO);
	wlc_tx_fifo_suspend(wlc, TX_CTL_FIFO);

	wlc->quiet._state |= WAITING_FOR_FLUSH_COMPLETE;

	/* Now wait for FIFO drain interrupt and proceed to wlc_start_quiet2() */
}

/* Fifo's have drained, now be quiet.
 * Came from: wlc_start_quiet1() via dpc FIFO drain handler.
 * Will go to: wlc_start_quiet3() via wlc_quiet_timer.
 */
static void
wlc_start_quiet2(wlc_info_t *wlc)
{
	uint32 tsf_hi, tsf_lo;
	struct ether_addr null_ether_addr = {{0, 0, 0, 0, 0, 0}};

	WL_REGULATORY(("Entering %s\n", __FUNCTION__));
	ASSERT(!(wlc->quiet._state & WAITING_FOR_FLUSH_COMPLETE));

	/* zero the address match register so we do not send ACKs */
	wlc_set_addrmatch(wlc, RCM_MAC_OFFSET, &null_ether_addr);

	wlc_read_tsf(wlc, &tsf_lo, &tsf_hi);

	wlc->quiet._state |= WAITING_FOR_DURATION;
	/* Set timer for difference (in millisecs) */
	wl_add_timer(wlc->wl, wlc->quiet.timer, (wlc->quiet.end_tsf - tsf_lo) / 1000, 0);
}

/* Done with quiet period.
 * Called from: timer via wlc_start_quiet2().
 * Will go to: wlc_quiet_timer()->dpc handler.
 */
static void
wlc_start_quiet3(wlc_info_t *wlc)
{
	WL_REGULATORY(("Entering %s\n", __FUNCTION__));
	ASSERT(wlc->quiet._state & WAITING_FOR_DURATION);
	wlc->quiet._state &= ~WAITING_FOR_DURATION;

	wlc_mute(wlc, OFF);

	wlc_mac_event(wlc, WLC_E_QUIET_END, NULL, WLC_E_STATUS_SUCCESS, 0, 0, 0, 0);

	WL_REGULATORY(("%s:Quiet Complete\n", __FUNCTION__));
	wlc->quiet._state = 0;	/* All done */
}



static void
wlc_parse_11h(wlc_info_t *wlc, uint8 *params, int len, struct dot11_bcn_prb *bcn)
{
	bcm_tlv_t *tag = (bcm_tlv_t*)params;
	uint8 *end = params + len;


	if ((params < end) && (tag = bcm_parse_ordered_tlvs(params, (int)(end - params),
		DOT11_MNG_PWR_CONSTRAINT_ID)) != NULL) {
		wlc_phy_11h_pwr(wlc->band->pi, (dot11_power_cnst_t *)tag);
		params = (uchar *)tag + TLV_HDR_LEN + tag->len;
	}

	if ((params < end) && (tag = bcm_parse_ordered_tlvs(params, (int)(end - params),
		DOT11_MNG_CHANNEL_SWITCH_ID)) != NULL) {
		wlc_11h_channel_switch(wlc, (dot11_channel_switch_t *)tag);
		params = (uchar *)tag + TLV_HDR_LEN + tag->len;
	}

	if ((params < end) && (tag = bcm_parse_ordered_tlvs(params, (int)(end - params),
		DOT11_MNG_QUIET_ID)) != NULL) {
		wlc_11h_quiet(wlc, (dot11_quiet_t *)tag, bcn);
		params = (uchar *)tag + TLV_HDR_LEN + tag->len;
	}
}

/* Decode the WMM 2.1.1 QoS Info field, as sent by WMM STA, and save in scb */
static void
wlc_qosinfo_update(struct scb *scb, uint8 qosinfo)
{
	/* Nybble swap; QoS Info bits are in backwards order from AC bitmap */
	static ac_bitmap_t qi_bitmap_to_ac_bitmap[16] = {
		0, 8, 4, 12, 2, 10, 6, 14, 1, 9, 5, 13, 3, 11, 7, 15
	};
	/* Max USP Length field decoding (WMM 2.2.1) */
	static uint16 maxsp[4] = {
		0xfff, 2, 4, 6
	};

	scb->apsd.maxsplen =
	        maxsp[(qosinfo & WME_QI_STA_MAXSPLEN_MASK) >> WME_QI_STA_MAXSPLEN_SHIFT];

	/* ACs negotiated to be APSD-enabled during association. */
	scb->apsd.ac_defl = qi_bitmap_to_ac_bitmap[qosinfo & WME_QI_STA_APSD_ALL_MASK];

	/* Initial config; can be updated by TSPECs, and revert back to ac_defl on DELTS. */
	scb->apsd.ac_trig = scb->apsd.ac_defl;
	scb->apsd.ac_delv = scb->apsd.ac_defl;
}

static void
wlc_recvctl(wlc_info_t *wlc, osl_t *osh, d11rxhdr_t *rxh, void *p)
{
	struct dot11_management_header *hdr;
	void *wl;
	int len_mpdu, body_len;
	uint8 *plcp, *body;
	struct scb *scb;
	uint16 fc, ft, fk;
	uint16 reason;
	bool sta_mode;


	WL_TRACE(("wl%d: wlc_recvctl\n", wlc->pub.unit));

	wl = wlc->wl;


	plcp = PKTDATA(osh, p);
	hdr = (struct dot11_management_header*)(plcp + D11_PHY_HDR_LEN);
	ASSERT(ISALIGNED((uintptr)hdr, sizeof(uint16)));

	fc = ltoh16(hdr->fc);
	ft = FC_TYPE(fc);
	fk = (fc & FC_KIND_MASK);
	ASSERT((ft == FC_TYPE_CTL) || (ft == FC_TYPE_MNG));

	/* mac header+body length, exclude CRC and plcp header */
	len_mpdu = PKTLEN(osh, p) - D11_PHY_HDR_LEN - DOT11_FCS_LEN;

	/* check for runts */
	if (((ft == FC_TYPE_MNG) && (len_mpdu < DOT11_MGMT_HDR_LEN)) ||
		((fk == FC_PS_POLL) && (len_mpdu < DOT11_PS_POLL_LEN))) {
		WLCNTINCR(wlc->pub._cnt.rxrunt);
		goto toss;
	}

	if (wlc->monitor || wlc->pub.promisc) {
		/* monitor mode */
		if (wlc->monitor)
			wlc_monitor(wlc, rxh, p);

		/* In monitor or promiscuous mode
		 * don't continue processing packets that aren't for us.
		 */

		/* ctl pkts: only handle PS-Poll/BAR/BA, must be unicast */
		if (((fk == FC_PS_POLL) || (fk == FC_BLOCKACK_REQ) || (fk == FC_BLOCKACK)) &&
			!bcmp(hdr->da.octet, (char*)&wlc->pub.cur_etheraddr, ETHER_ADDR_LEN)) {
			/* accept */
		}
		/* continue to process Bcns or Probe Resp, since we are normally promisc to these */
		else if (fk == FC_BEACON || fk == FC_PROBE_RESP) {
			/* accept */
		}
		/* Management frames can match unicast DA or
		 * broadcast DA with matching BSSID or broadcast BSSID
		 */
		else if ((ft == FC_TYPE_MNG) &&
			(!bcmp(hdr->da.octet, (char*)&wlc->pub.cur_etheraddr, ETHER_ADDR_LEN) ||
			(ETHER_ISBCAST(&hdr->da) &&
		           (WLC_IS_TARGET_BSSID(wlc, &hdr->bssid) ||
		            WLC_IS_CURRENT_BSSID(wlc, &hdr->bssid) ||
		            ETHER_ISBCAST(&hdr->bssid))))) {
			/* accept */
		}
		/* Toss otherwise */
		else
			goto toss;
	} else {
		/* a race window exists while changing the RX MAC addr match registers where
		 * we may receive a fame that does not have our MAC address
		 */
		if (!ETHER_ISMULTI(&hdr->da) &&
		    bcmp(hdr->da.octet, (char*)&wlc->pub.cur_etheraddr, ETHER_ADDR_LEN)) {
			WLCNTINCR(wlc->pub._cnt.rxbadda);
			goto toss;
		}
	}

	/* strip off CRC and plcp */
	PKTPULL(osh, p, D11_PHY_HDR_LEN);
	PKTSETLEN(osh, p, len_mpdu);

	WLCNTINCR(wlc->pub._cnt.rxctl);
	if (ETHER_ISMULTI(&hdr->da))
		WLCNTINCR(wlc->pub._cnt.rxmulti);

	/* MAC is promisc wrt beacons and probe responses, trace other packets */
	if ((fk != FC_PROBE_RESP) && (fk != FC_PROBE_REQ) && (fk != FC_BEACON)) {
		WL_PRHDRS(wlc, "rxpkt hdr (ctrl/mgmt)", plcp, NULL, rxh, len_mpdu + D11_PHY_HDR_LEN
			+ DOT11_FCS_LEN);
		WL_PRPKT("rxpkt (ctrl/mgmt)", (uchar*)plcp, len_mpdu + D11_PHY_HDR_LEN
			+ DOT11_FCS_LEN);
	}

	if (wlc_recvfilter(wlc, (struct dot11_header *) hdr, rxh, &scb)) {
		WLCNTINCR(wlc->pub._cnt.rxfilter);
		goto toss;
	}

	/* Update scb used as we just received a control frame from the STA */
	if (scb)
	    scb->used = wlc->pub.now;

	/* a. ctl pkts, we only process PS_POLL/BAR/BA, others are handled in ucode */
	if (ft == FC_TYPE_CTL) {
		if (fk == FC_PS_POLL) {
		}
		else {
			WL_ERROR(("wl%d: wlc_recvctl: unhandled frametype (0x%x)\n", wlc->pub.unit,
				fc));
		}
		goto toss;
	}

	/* b. mgmt pkts, ATIM, probe_request and beacon are handled in ucode first */

	/* set pointer to frame body and set frame body length */
	body = (uchar*)PKTPULL(osh, p, sizeof(struct dot11_management_header));
	body_len = PKTLEN(osh, p);	/* w/o 4 bytes CRC */

	/* deal with encrypted frame */
	if (fc & FC_WEP) {
		if (body_len < WLC_WEP_IV_SIZE + WLC_WEP_ICV_SIZE) {
			WLCNTINCR(wlc->pub._cnt.rxrunt);
			goto toss;
		}
		/* only auth  mgmt frame seq 3  can be encrypted(WEP) */

		if (fk != FC_AUTH)
			goto toss;
		/*
		   decryption error is now checked in authresp_ap so we can
		   check if it was supposed to use a different key for another ssid.
		*/

	}

	sta_mode = 1;

	switch (fk) {

	case FC_AUTH:
		if (body_len < DOT11_AUTH_FIXED_LEN)
			goto rxbadproto;

		wlc_authresp_client(wlc, rxh, hdr, (struct dot11_auth *) body, body_len);
		break;


	case FC_ASSOC_RESP:
	case FC_REASSOC_RESP: {
		struct dot11_assoc_resp *assoc = (struct dot11_assoc_resp *) body;
		uchar *tlvs = NULL;
		uint tlvs_len, parse_len;
		uint8 *parse = NULL;

		if (body_len < 6)
			goto rxbadproto;

		if (wlc->assoc_state != AS_SENT_ASSOC ||
			bcmp(hdr->bssid.octet, wlc->target_bss.BSSID.octet, ETHER_ADDR_LEN)) {
			WL_ERROR(("wl%d: unsolicited association response from %s\n",
				wlc->pub.unit, eabuf));
			wlc_mac_event(wlc, (fk == FC_ASSOC_RESP) ? WLC_E_ASSOC : WLC_E_REASSOC,
				&hdr->sa, WLC_E_STATUS_UNSOLICITED, 0, 0, 0, 0);
			break;
		}

		/* capability */

		/* save last (re)association response */
		if (wlc->assoc_resp)
			MFREE(wlc->pub.osh, wlc->assoc_resp, wlc->assoc_resp_len);
		if (!(wlc->assoc_resp = MALLOC(wlc->pub.osh, body_len)))
			WL_ERROR(("wl%d: wlc_recvctl, out of memory, malloced %d bytes\n",
				wlc->pub.unit, MALLOCED(wlc->pub.osh)));
		else {
			wlc->assoc_resp_len = body_len;
			bcopy((char*)assoc, (char*)wlc->assoc_resp, body_len);
		}


		/* association denied */
		if (ltoh16(assoc->status) != DOT11_STATUS_SUCCESS) {
			wlc_mac_event(wlc, (fk == FC_ASSOC_RESP) ? WLC_E_ASSOC : WLC_E_REASSOC,
				&wlc->target_bss.BSSID, WLC_E_STATUS_FAIL, ltoh16(assoc->status), 0,
				0, 0);
			break;
		}

		ASSERT(scb);

		/* Is this a BRCM AP */
		tlvs = (uint8 *) &assoc[1];
		tlvs_len = body_len - sizeof(struct dot11_assoc_resp);
		wlc_process_brcm_ie(wlc, scb, (brcm_ie_t*)wlc_find_vendor_ie(tlvs, tlvs_len,
			(uchar*)BRCM_OUI, NULL, 0));

		/* If WME is enabled, check if response indicates WME association */
		scb->flags &= ~SCB_WME_CAPABLE;
		if (WME_ENAB(wlc)) {
			bcm_tlv_t *wme_ie;
			parse = (uint8 *)&assoc[1];
			parse_len = body_len - sizeof(struct dot11_assoc_resp);
			while ((wme_ie = bcm_parse_tlvs(parse, parse_len,
			                                DOT11_MNG_PROPR_ID)) != NULL)
				if (wlc_is_wme_ie(wlc, (uint8 *)wme_ie, &parse, &parse_len))
					break;
			wlc_qosinfo_update(scb, 0);	/* Clear Qos Info by default */
			if (wme_ie) {
				wme_param_ie_t *wme_ie_body = (wme_param_ie_t *)&wme_ie->data;
				scb->flags |= SCB_WME_CAPABLE;
				/* save the new IE, or params IE which is superset of IE */
				bcopy(wme_ie_body, &wlc->wme_param_ie, wme_ie->len);
				/* Use locally-requested APSD config if AP advertised APSD */
				/* STA is in AUTO WME mode,
				 *     AP has UAPSD enabled, then allow STA to use wlc->PM
				 *            else, don't allow STA to sleep based on wlc->PM,
				 *                  and leave PM mode if already set
				 */
				if (wlc->wme_param_ie.qosinfo & WME_QI_AP_APSD_MASK) {
					wlc_qosinfo_update(scb, wlc->apsd_sta_qosinfo);
					wlc->WME_PM_blocked = FALSE;
					if (wlc->PM == PM_MAX)
						wlc_set_pmstate(wlc, TRUE);
				} else if (WME_AUTO(wlc)) {
					wlc->WME_PM_blocked = TRUE;
					wlc_set_pmstate(wlc, FALSE);
				}

				/* Apply the STA AC params sent by AP */
				wlc_wme_setparams(wlc, TRUE);

			}
		}


		/* Association success */
		wlc->AID = ltoh16(assoc->aid);
		wlc_mac_event(wlc, (fk == FC_ASSOC_RESP) ? WLC_E_ASSOC : WLC_E_REASSOC,
			&wlc->target_bss.BSSID, WLC_E_STATUS_SUCCESS, ltoh16(assoc->status), 0, 0,
			0);

		WL_ASSOC(("Checking if key needs to be inserted\n"));
		/* If Multi-SSID is enabled, and Legacy WEP is in use for this bsscfg,
		 * a "pairwise" key must be created by copying the default key from the bsscfg.
		 */
		if ((wlc->cfg.WPA_auth == WPA_AUTH_DISABLED))
			WL_ASSOC(("WPA disabled \n "));

		if (WSEC_WEP_ENABLED(wlc->cfg.wsec))
			WL_ASSOC(("WEP enabled \n "));
		break;
	}


	case FC_PROBE_REQ:
		break;

	case FC_PROBE_RESP: {
		if (body_len < DOT11_BCN_PRB_LEN)
			goto rxbadproto;

		/* check for our probe responses:
		 * - must be a packet with our MAC addr
		 * (but if we're no longer scanning, ignore it)
		 */
		if (SCAN_IN_PROGRESS(wlc) &&
		    !bcmp((char*)&wlc->pub.cur_etheraddr, (uchar *)&hdr->da, ETHER_ADDR_LEN)) {
			wlc_scan_parse(wlc, rxh, plcp, hdr, body, body_len);
		}
		break;
	}

	case FC_BEACON: {
		struct dot11_bcn_prb *bcn = (struct dot11_bcn_prb *)body;
		ASSERT(ISALIGNED((uintptr)body, sizeof(uint32)));

		if (body_len < DOT11_BCN_PRB_LEN)
			goto rxbadproto;


		wlc_process_beacon(wlc, rxh, plcp, hdr, len_mpdu);

		if (SCAN_IN_PROGRESS(wlc)) {
			if ((wlc->scan.state & SCAN_STATE_PASSIVE) ||
			    QUIET_CHANNEL(wlc, wlc->band->pi->radio_channel)) {
				wlc_scan_parse(wlc, rxh, plcp, hdr, body, body_len);
			}

			/* if we get an AP beacon on a radar quiet channel, signal
			 * that the channel is temporarily clear for transmissions.
			 */
			if ((ltoh16(bcn->capability) & DOT11_CAP_ESS) &&
			    QUIET_CHANNEL(wlc, wlc->band->pi->radio_channel) &&
			    wlc_rx_channel(rxh) == wlc->band->pi->radio_channel)
				wlc_scan_radar_clear(wlc);
		}

		/* See if we are waiting for this beacon to join a BSS/IBSS */
		if (wlc->assoc_state == AS_WAIT_RCV_BCN && WLC_IS_TARGET_BSSID(wlc, &hdr->bssid)) {
			WL_ASSOC(("JOIN: got the desired beacon...update TBTT...\n"));
			wlc_join_complete(wlc, bcn, body_len);
		}
		break;
	}

	case FC_DISASSOC:
		if (body_len < 2)
			goto rxbadproto;

		reason = ltoh16(*(uint16*)body);

		/* wlc_recvfilter(): frame tossed if node not authenticated with us */
		ASSERT(scb && SCB_AUTHENTICATED(scb));

		/* delete pairwise key on disassociate */
		if (scb->key) {
			WL_WSEC(("wl%d: %s: received disassoc, deleting pairwise key for %s\n",
				wlc->pub.unit, __FUNCTION__, eabuf));
			wlc_scb_key_delete(wlc, scb);
		}

		if (!(scb->flags & SCB_MYAP)) {
			WL_ERROR(("wl%d: wlc_recvctl: STA DISASSOC from non-AP SCB %s\n",
				wlc->pub.unit, eabuf));
		}
		/* STA */
		WL_ASSOC(("wl%d: %s%s from %s (reason %d)\n", wlc->pub.unit,
			ETHER_ISBCAST(&hdr->da) ? "broadcast " : "",
			  "disassociation", eabuf, reason));
		if (SCB_ASSOCIATED(scb)) {
			wlc_scb_clearstatebit(scb, ASSOCIATED);
			wlc_mac_event(wlc, WLC_E_DISASSOC_IND, &hdr->sa,
				WLC_E_STATUS_SUCCESS, reason, 0, 0, 0);
		}
		/* allow STA to roam back to disassociating AP */
		if (WLC_IS_CURRENT_BSSID(wlc, &hdr->bssid))
			wlc_roam_start(wlc, WLC_E_DISASSOC_IND);
		break;

	case FC_DEAUTH:
		if (body_len < 2)
			goto rxbadproto;

		reason = ltoh16(*(uint16*)body);

		if (!scb)
			scb = wlc_scbfindband(wlc, (struct ether_addr*)&hdr->sa,
				CHANNEL_BANDUNIT(wlc, wlc_rx_channel(rxh)));

		/* delete pairwise key on deauth */
		if (scb && scb->key) {
			WL_WSEC(("wl%d: %s: received deauth, deleting pairwise key for %s\n",
				wlc->pub.unit, __FUNCTION__, eabuf));
			wlc_scb_key_delete(wlc, scb);
		}

		if (scb && !(scb->flags & SCB_MYAP)) {
			WL_ERROR(("wl%d: wlc_recvctl: STA DEAUTH from non-AP SCB %s\n",
				wlc->pub.unit, eabuf));
		}
		/* STA */
		WL_ASSOC(("wl%d: %s%s from %s (reason %d)\n", wlc->pub.unit,
			ETHER_ISBCAST(&hdr->da) ? "broadcast " : "",
			  "deauthentication", eabuf, reason));
		if (scb) {
			wlc_scb_clearstatebit(scb, ASSOCIATED | AUTHORIZED);
			if (SCB_AUTHENTICATED(scb)) {
				wlc_scb_clearstatebit(scb, AUTHENTICATED);
				/* DEAUTH w/reason 13-22 is considered a 4-way handshake
				 * failure
				 */
				if (reason > 12 && reason < 23)
					WLCNTINCR(wlc->pub._cnt.fourwayfail);
				wlc_mac_event(wlc, WLC_E_DEAUTH_IND, &hdr->sa,
					WLC_E_STATUS_SUCCESS, reason, 0, 0, 0);
			}
		}

		/* allow STA to roam back to deauthenticating AP */
		if (WLC_IS_CURRENT_BSSID(wlc, &hdr->bssid))
			wlc_roam_start(wlc, WLC_E_DEAUTH_IND);
		break;


	case FC_ACTION: {
		uint action_category;
		uint action_id = 0;

		if (body_len < 1)
			goto rxbadproto;

		/* Action Frame:
		 * byte 0: Category
		 * byte 1: Action field
		 */
		action_category = (uint)body[WME_CATEGORY_CODE_OFFSET];
		if (body_len >= 2)
			action_id = (uint)body[WME_ACTION_CODE_OFFSET];

		/* Weed out error report frames */
		if (action_category & DOT11_ACTION_CAT_ERR_MASK) {
			break;
		}
		/* Spectrum Management */
		else if (WL11H_ENAB(wlc) &&
			action_category == DOT11_ACTION_CAT_SPECT_MNG) {
			if (body_len < 2) {
				wlc_send_action_err(wlc, hdr, body, body_len);
				goto rxbadproto;
			}
			/* Process spectrum management action_id's */
			wlc_frameaction_specmgmt(action_id, wlc, hdr,
				rxh, plcp, body, body_len);
		}
		/* WME Management Notification */
		else if (WME_ENAB(wlc) &&
			(action_category == DOT11_ACTION_NOTIFICATION)) {
			if (body_len < (int)(DOT11_MGMT_NOTIFICATION_LEN +
				WME_TSPEC_HDR_LEN + WME_TSPEC_LEN)) {
				wlc_send_action_err(wlc, hdr, body, body_len);
				goto rxbadproto;
			}
			/* Process WME Management action_id's */
			wlc_frameaction_wme(action_id, wlc, hdr, body, body_len);
		}
		/* Unrecognized (or disabled) action category */
		else wlc_send_action_err(wlc, hdr, body, body_len);
	}
	break;

rxbadproto:
	WLCNTINCR(wlc->pub._cnt.rxbadproto);

	default:
		break;
	}

toss:
	PKTFREE(osh, p, FALSE);
}

static void
wlc_scan_parse(wlc_info_t *wlc, d11rxhdr_t *rxh, uint8 *plcp,
	struct dot11_management_header *hdr, uint8 *body, int body_len)
{
	struct dot11_bcn_prb *bcn = (struct dot11_bcn_prb *)body;
	uint16 cap = ltoh16(bcn->capability);
	wlc_bss_info_t bi;
	wlc_bss_info_t *BSS;

	/*
	 * check for probe responses/beacons:
	 * - must have the correct Infra mode
	 * - must have the SSID we were looking for,
	 *   unless we were doing a broadcast SSID request (ssid len == 0)
	 * - must have matching BSSID (if unicast was specified)
	 */
	if ((wlc->scan.bss_type == DOT11_BSSTYPE_ANY ||
	     (wlc->scan.bss_type == DOT11_BSSTYPE_INFRASTRUCTURE && (cap & DOT11_CAP_ESS)) ||
	     (wlc->scan.bss_type == DOT11_BSSTYPE_INDEPENDENT && (cap & DOT11_CAP_IBSS))) &&
	    (wlc->scan.ssid_len == 0 ||
	     (wlc->scan.ssid_len == BCN_PRB_SSID(hdr)[1] &&
	      !bcmp(wlc->scan.ssid, BCN_PRB_SSID(hdr)+2, wlc->scan.ssid_len))) &&
	    (ETHER_ISMULTI(&wlc->scan.bssid) ||
	     !bcmp(&wlc->scan.bssid, &hdr->sa, ETHER_ADDR_LEN))) {
		if (wlc_parse_bcn_prb(wlc, rxh, hdr, body, body_len, &bi)) {
			/* parse fail, discard this new bi */
			return;
		}
		/* check bssid AND band matching */
		BSS = wlc_BSSlookup(wlc, (uchar *)&hdr->bssid, bi.channel, bi.SSID, bi.SSID_len);
		if (BSS) {
			if ((BSS->flags & WLC_BSS_RSSI_ON_CHANNEL) ==
			    (bi.flags & WLC_BSS_RSSI_ON_CHANNEL)) {
				/* preserve max RSSI if the measurements are
				 * both on-channel or both off-channel
				 */
				bi.RSSI = MAX(BSS->RSSI, bi.RSSI);
			} else if ((BSS->flags & WLC_BSS_RSSI_ON_CHANNEL) &&
			           (bi.flags & WLC_BSS_RSSI_ON_CHANNEL) == 0) {
				/* preserve the on-channel rssi measurement
				 * if the new measurement is off channel
				 */
				bi.RSSI = BSS->RSSI;
				bi.flags |= WLC_BSS_RSSI_ON_CHANNEL;
			}
			/* free the prb pointer, prevent memory leakage */
			if (BSS->bcn_prb) {
				MFREE(wlc->pub.osh, BSS->bcn_prb, BSS->bcn_prb_len);
				BSS->bcn_prb = NULL;
				BSS->bcn_prb_len = 0;
			}
			/* update the entry regardless existing or new */
			bcopy((char*)&bi, (char*)BSS, sizeof(wlc_bss_info_t));

			if (wlc->scan.state & SCAN_STATE_SAVE_PRB) {
				/* scan completion responsible for freeing frame */
				ASSERT(!BSS->bcn_prb);
				/* save raw probe response frame */
				if ((BSS->bcn_prb = (struct dot11_bcn_prb *) MALLOC(wlc->pub.osh,
					body_len))) {
					bcopy((char*)body, (char*)BSS->bcn_prb, body_len);
					BSS->bcn_prb_len = (uint16) body_len;
				} else {
					WL_ERROR(("wl%d: wlc_scan_parse: out of memory, malloced %d"
						" bytes\n",
						wlc->pub.unit, MALLOCED(wlc->pub.osh)));
					BSS->bcn_prb_len = 0;
				}
			}
		}
	}
}

static void
wlc_process_beacon(wlc_info_t *wlc, d11rxhdr_t *rxh, uint8 *plcp, struct dot11_management_header
	*hdr, int frame_len)
{
	struct scb *scb;
	struct dot11_bcn_prb *bcn;
	int bcn_len;
	uint8 *tag_params;
	int tag_params_len;
	bcm_tlv_t *ds_params;
	bcm_tlv_t *rate_ie;
	bcm_tlv_t *wme_ie;
	wme_param_ie_t *wme_ie_body;
	uint8 *parse;
	uint parse_len;
	uint32 bcn_l, bcn_h;
	uint8 *erp;
	int erp_len;
	bool is_erp;
	bool on_home_channel;
	bool current_bss_bcn;
	uint8 rx_channel;

	ASSERT(ISALIGNED((uintptr)((int8*)hdr + DOT11_MGMT_HDR_LEN), sizeof(uint32)));
	bcn = (struct dot11_bcn_prb*)((int8*)hdr + DOT11_MGMT_HDR_LEN);
	bcn_len = frame_len - DOT11_MGMT_HDR_LEN;
	tag_params = ((uint8*)bcn + DOT11_BCN_PRB_LEN);
	tag_params_len = bcn_len - DOT11_BCN_PRB_LEN;

	rx_channel = (uint8)wlc_rx_channel(rxh);

	/* lookup the scb in case we already have info on it */
	scb = wlc_scbfind(wlc, (struct ether_addr*)&hdr->sa);

	/* check for an ERP info elt */
	erp_len = 0;
	erp = NULL;
	if (!wlc_erp_find(wlc, bcn, bcn_len, &erp, &erp_len))
		wlc_nonerp_find(wlc, bcn, bcn_len, &erp, &erp_len);

	/* check rates for ERP capability */
	is_erp = FALSE;
	if (scb && (scb->flags & SCB_NONERP)) {
		/* This STA was already marked as NonERP */
	} else {
		rate_ie = bcm_parse_tlvs(tag_params, tag_params_len, DOT11_MNG_EXT_RATES_ID);
		if (rate_ie && wlc_rateset_isofdm(rate_ie->len, rate_ie->data))
			is_erp = TRUE;
		if (!is_erp) {
			rate_ie = bcm_parse_tlvs(tag_params, tag_params_len, DOT11_MNG_RATES_ID);
			if (rate_ie && wlc_rateset_isofdm(rate_ie->len, rate_ie->data))
				is_erp = TRUE;
		}
		if (scb && !is_erp)
			scb->flags |= SCB_NONERP;
	}

	/* check for a beacon on our channel */
	ds_params = bcm_parse_tlvs(tag_params, tag_params_len, DOT11_MNG_DS_PARMS_ID);
	if (!ds_params || ds_params->len < 1) {
		/* if no channel info, use the channel on which we received the beacon */
		on_home_channel = (wlc->pub.current_bss.channel == rx_channel);
	}
	else {
		on_home_channel = (wlc->pub.current_bss.channel == ds_params->data[0]);
	}

	/* check for beacon in our BSS (STA's AP or IBSS member) */
	current_bss_bcn = on_home_channel && WLC_IS_CURRENT_BSSID(wlc, &hdr->bssid);

	/* For G APs or IBSSs, check overlapping beacon information */
	if (wlc->band->gmode && on_home_channel &&
	    (!wlc->pub.BSS && wlc->pub.associated && !current_bss_bcn)) {
		/* Check for overlapping Barker long-preamble-only signaled */
		if (wlc->barker_overlap_control &&
		    erp_len > 0 && (erp[0] & DOT11_MNG_BARKER_PREAMBLE))
			wlc->barker_detect_timeout = WLC_IBSS_BCN_TIMEOUT;

		/* Check for overlapping NonERP long-preamble-only beacons */
		if (wlc->barker_overlap_control &&
		    !is_erp && (ltoh16(bcn->capability) & DOT11_CAP_SHORT) == 0)
			wlc->longpre_detect_timeout = WLC_IBSS_BCN_TIMEOUT;

		/* Check for overlapping NonERP beacon
		 * Consider as NonERP beacon if there are no OFDM rates
		 * or the ERP Info indicates that NonERP STAs are associated
		 */
		if (!is_erp || (erp_len > 0 && (erp[0] & DOT11_MNG_NONERP_PRESENT)))
		    wlc->nonerp_ovlp_timeout = WLC_IBSS_BCN_TIMEOUT;

		wlc_update_gbss_modes(wlc);
	}

	if (!wlc->pub.associated)
		return;

	if (current_bss_bcn) {
		wlc->rssi = wlc_update_rssi_ma(wlc->band->pi, rxh);
		wlc->time_since_bcn = 0;
		/* If we are in an IBSS, indicate link up if we are currently down.
		 * bcns_lost indicates whether the link is marked 'down' or not
		 */
		if (!wlc->pub.BSS && wlc->bcns_lost == TRUE) {
			wlc->bcns_lost = FALSE;
			wlc_mac_event(wlc, WLC_E_GOT_BEACONS, NULL, WLC_E_STATUS_SUCCESS, 0, 0, 0,
				0);
		}
	}

	/* Check for a BSS beacon from our associated AP */
	if (wlc->pub.BSS && current_bss_bcn) {
		/* Check the TIM to see if data is buffered for us... */
		if (wlc_InTIM(wlc, bcn, bcn_len) && !wlc->PMblocked) {
			/* Fast PS mode : leave PS mode */
			if (wlc->PM == PM_FAST) {
				wlc_set_pmstate(wlc, FALSE);
			}
			/* WMM/APSD 3.6.2.3: Don't send PS poll if all ACs are delivery-enabled. */
			else if (!scb || (scb->apsd.ac_delv != AC_BITMAP_ALL)) {
				if (wlc->PMenabled &&
				    ((wlc->PSpoll = wlc_sendpspoll(wlc)) == FALSE)) {
					WL_ERROR(("wl%d: wlc_sendpspoll() failed\n",
					          wlc->pub.unit));
				}
				/* force core awake if above wlc_sendpspoll() succeeded; otherwise,
				 * allow core to sleep
				 */
				wlc_set_ps_ctrl(wlc);
			}
		}

		/* If we were waiting for a beacon before allowing doze,
		 * clear the flag and update ps_ctrl
		 */
		if (wlc->PMawakebcn) {
			if (wlc->PMenabled)
				WL_PS(("wl%d: got beacon, clearing PMawakebcn\n", wlc->pub.unit));
			wlc->PMawakebcn = FALSE;
			wlc_set_ps_ctrl(wlc);
		}

		if (!wlc->aps_associated) {

			/* check for retrograde TSF; wraparound not handled */
			bcn_l = ltoh32(bcn->timestamp[0]);
			bcn_h = ltoh32(bcn->timestamp[1]);
			if ((wlc->tsf_h || wlc->tsf_l) &&
			    ((bcn_h < wlc->tsf_h) || ((bcn_h == wlc->tsf_h) && (bcn_l <
			    wlc->tsf_l))))
				wlc_mac_event(wlc, WLC_E_RETROGRADE_TSF, &hdr->bssid, 0, 0, 0, 0,
					0);
			wlc->tsf_l = bcn_l;
			wlc->tsf_h = bcn_h;

			if (wlc->check_for_unaligned_tbtt) {
				bool is_ofdm;
				uint phy_type;
				uint short_preamble, phy_rate;

				phy_type = ((rxh->RxChan & RXS_CHAN_PHYTYPE_MASK) >>
					RXS_CHAN_PHYTYPE_SHIFT);
				is_ofdm = (phy_type == PHY_TYPE_A) || (rxh->PhyRxStatus_0 &
					PRXS0_OFDM);
				short_preamble = (rxh->PhyRxStatus_0 & PRXS0_SHORTH);
				phy_rate = PLCP_PHYRATE(plcp, is_ofdm);
				wlc_tbtt_align(wlc, bcn, 0, is_ofdm, phy_rate, short_preamble);

				/* allow core to sleep again */
				wlc->check_for_unaligned_tbtt = FALSE;
				wlc_set_ps_ctrl(wlc);
			}
		}

		/* Track 11g modes */
		if (wlc->band->gmode) {
			/* Track network ShortSlotTime */
			wlc_switch_shortslot(wlc, (ltoh16(bcn->capability) & DOT11_CAP_SHORTSLOT) !=
				0);

			if (erp && erp_len > 0) {

				/* Track protection mode */
				if (wlc->g_protection_override == WLC_PROTECTION_AUTO)
					wlc->g_protection = (erp[0] & DOT11_MNG_USE_PROTECTION) !=
						0;

				/* Track Barker Preamble Mode */
				if (erp[0] & DOT11_MNG_BARKER_PREAMBLE)
					wlc->barker_preamble = WLC_BARKER_LONG_ONLY;
				else
					wlc->barker_preamble = WLC_BARKER_SHORT_ALLOWED;

				/* update our shortpreamble use */
				if (wlc->barker_preamble == WLC_BARKER_LONG_ONLY ||
					wlc->barker_detect_timeout > 0)
					wlc->shortpreamble = FALSE;
				else
					wlc->shortpreamble = TRUE;

				/* update the AP's scb */
				if (scb) {
					scb->flags &= ~SCB_SHORTPREAMBLE;
					if (wlc->barker_preamble == WLC_BARKER_SHORT_ALLOWED)
						scb->flags |= SCB_SHORTPREAMBLE;
				}
			}
		}

#if ACONF     /* RADAR */
		/* . Make sure this part is done before wlc_parse_11h, which will
		 * process Channel Switch Announcement (CSA) information.
		 * . if this channel is marked as quiet, we can clear the bit and un-mute
		 * since we reestablished our AP connection
		 */
		if (QUIET_CHANNEL(wlc, wlc->pub.current_bss.channel)) {
			CLRQUIET(wlc, wlc->pub.current_bss.channel);
			if (wlc->band->pi->radio_channel == wlc->pub.current_bss.channel) {
				wlc_mute(wlc, OFF);
			}
		}

#endif /* ACONF */

		/* WME: check if the AP has supplied new acparams */
		if (scb && SCB_WME(scb)) {
			/* find the params ie */
			parse = tag_params;
			parse_len = tag_params_len;
			while ((wme_ie = bcm_parse_tlvs(parse, parse_len, DOT11_MNG_PROPR_ID)))
				if (wlc_is_wme_ie(wlc, (uint8 *)wme_ie, &parse, &parse_len))
					break;
			if (wme_ie) {
				wme_ie_body = (wme_param_ie_t *)&wme_ie->data;
				/* check if it's a Params IE where the count field has changed */
				if (wme_ie_body->subtype == WME_SUBTYPE_PARAM_IE &&
				    ((wme_ie_body->qosinfo & WME_QI_AP_COUNT_MASK) !=
				     (wlc->wme_param_ie.qosinfo & WME_QI_AP_COUNT_MASK))) {
					/* save and apply new params ie */
					bcopy(wme_ie_body, &wlc->wme_param_ie,
					      sizeof(wme_param_ie_t));
					wlc_wme_setparams(wlc, TRUE);
				}
			} else
				WL_ERROR(("wl%d: wlc_process_beacon: wme params ie missing\n",
					wlc->pub.unit));
		}

		/* process any BRCM info element */
		wlc_process_brcm_ie(wlc, scb, (brcm_ie_t*)wlc_find_vendor_ie(tag_params,
			tag_params_len, (uchar*)BRCM_OUI, NULL, 0));

		if (WL11H_ENAB(wlc) &&
			wlc->pub.BSS && ltoh16(bcn->capability) & DOT11_CAP_SPECTRUM) {
			wlc_parse_11h(wlc, tag_params, tag_params_len, bcn);
		}
	}
	/* Check for an IBSS beacon with the same BSSID */
	else if (!wlc->pub.BSS && current_bss_bcn) {
		bool shortpreamble;

		shortpreamble = (is_erp || (ltoh16(bcn->capability) & DOT11_CAP_SHORT));

		/* Record short preamble capability */
		if (scb) {
			if (shortpreamble)
				scb->flags |= SCB_SHORTPREAMBLE;
			else
				scb->flags &= ~SCB_SHORTPREAMBLE;
		}

		if (wlc->band->gmode) {
			if (erp_len > 0) {
				/* Check for Barker Long preamble only signaled in our IBSS */
				if (erp[0] & DOT11_MNG_BARKER_PREAMBLE)
					wlc->barker_detect_timeout = WLC_IBSS_BCN_TIMEOUT;
				/* Check for Use_Protection signaled in our IBSS */
				if (erp[0] & DOT11_MNG_USE_PROTECTION)
					wlc->protection_detect_timeout = WLC_IBSS_BCN_TIMEOUT;
			}

			/* Check for NonERP beacons */
			if (!is_erp)
				wlc->nonerp_ibss_timeout = WLC_IBSS_BCN_TIMEOUT;

			/* Check for long-preamble-only STAs in our IBSS */
			if (!shortpreamble)
				wlc->longpre_detect_timeout = WLC_IBSS_BCN_TIMEOUT;

			wlc_update_gbss_modes(wlc);
		}
	}
	/* Check for an IBSS Coalesce, when we get a beacon with a different
	 * BSSID, but the same SSID, and a later TSF.
	 */
	else if (!wlc->pub.BSS && (ltoh16(bcn->capability) & DOT11_CAP_IBSS) &&
	         WLC_IS_CURRENT_SSID(wlc, BCN_PRB_SSID(hdr)+2, BCN_PRB_SSID(hdr)[1]) &&
	         wlc_bcn_tsf_later(wlc, rxh, plcp)) {
		/* The received beacon has a greater TSF time, so adopt its parameters */
		if (wlc_parse_bcn_prb(wlc, rxh, hdr, bcn, bcn_len, &wlc->target_bss) == 0) {
			wlc_sort_rates(&wlc->target_bss.rateset, &wlc->band->hw_rateset);
			wlc_join_complete(wlc, bcn, bcn_len);
		}
	}

}

/*
 * Frame received, frame type FC_ACTION,
 *  action_category DOT11_ACTION_CAT_SPECT_MNG
 */
static void
wlc_frameaction_specmgmt(uint action_id, wlc_info_t *wlc,
	struct dot11_management_header *hdr, d11rxhdr_t *rxh, uint8 *plcp,
	uint8 *body, int body_len)
{
	/* Spectrum Management action_id's */
	if (action_id == DOT11_ACTION_ID_M_REQ) {
		wlc_recv_measure_request(wlc, hdr, body, body_len);
	} else if (action_id == DOT11_ACTION_ID_M_REP) {
		wlc_recv_measure_report(wlc, hdr, body, body_len);
	} else if (action_id == DOT11_ACTION_ID_TPC_REQ) {
		wlc_recv_tpc_request(wlc, rxh, plcp, hdr, body, body_len);
	} else if (action_id == DOT11_ACTION_ID_CHANNEL_SWITCH) {
		wlc_recv_csa_action(wlc, hdr, body, body_len);
	} else if ((action_id != DOT11_ACTION_ID_TPC_REP)) {
		wlc_send_action_err(wlc, hdr, body, body_len);
	}
}

static void
wlc_recv_tpc_request(wlc_info_t *wlc, d11rxhdr_t *rxh, void *plcp,
	struct dot11_management_header *hdr, uint8 *body, int body_len)
{
	int i;
	int txpwr;
	int link_margin;
	int rate;
	int nominal_req_pwr;
	static struct {int rate; int pwr;} nominal_req_pwr_table[] = {
		{WLC_RATE_1M, -80},
		{WLC_RATE_2M, -80},
		{WLC_RATE_5M5, -76},
		{WLC_RATE_11M, -76},
		{WLC_RATE_6M, -82},
		{WLC_RATE_9M, -81},
		{WLC_RATE_12M, -79},
		{WLC_RATE_18M, -77},
		{WLC_RATE_24M, -74},
		{WLC_RATE_36M, -70},
		{WLC_RATE_48M, -66},
		{WLC_RATE_54M, -65},
		{0, -100}
	};
	struct dot11_action_measure * action_hdr;
	struct ether_addr *ea = &hdr->sa;
	struct ether_addr *bssid = &hdr->bssid;

	if (body_len < 3) {
		WL_ERROR(("wl%d: wlc_recv_tpc_request: got TPC Request from %s, but frame body len"
			" was %d, expected 3\n",
			wlc->pub.unit, bcm_ether_ntoa(ea, eabuf), body_len));
		return;
	}

	action_hdr = (struct dot11_action_measure *)body;

	WL_INFORM(("wl%d: wlc_recv_tpc_request: got TPC Request (token %d) from %s\n",
		wlc->pub.unit, action_hdr->token, bcm_ether_ntoa(ea, eabuf)));

	/* tx power for the outgoing frame will be our current txpwr setting
	 * include the antenna gain value to get radiated power, EIRP.
	 * Adjust from internal units to dbm.
	 */
	txpwr = (wlc->band->pi->tx_power_max + wlc->band->antgain) / WLC_TXPWR_DB_FACTOR;

	rate = wlc_compute_rate(rxh, plcp);

	/* Find the nominal required power for the request packet's rate */
	i = 0;
	nominal_req_pwr = 0;
	while (nominal_req_pwr_table[i].rate != 0) {
		if (nominal_req_pwr_table[i].rate == rate) {
			nominal_req_pwr = nominal_req_pwr_table[i].pwr;
			break;
		}
		i++;
	}

	/* extract RSSI from rxh */
	link_margin = wlc_compute_rssi(wlc->band->pi, rxh);
	link_margin -= nominal_req_pwr;
	link_margin -= 3; /* TPC Report Safety Margin, 3 dB */
	/* clamp link_margin value if we overflow an int8 */
	link_margin = MIN(link_margin, 127);
	link_margin = MAX(link_margin, -128);

	wlc_send_tpc_report(wlc, ea, bssid, action_hdr->token, (int8)txpwr, (int8)link_margin);
}

static void
wlc_recv_measure_request(wlc_info_t *wlc, struct dot11_management_header *hdr, uint8 *body,
	int body_len)
{
	struct dot11_action_measure * action_hdr;
	int len;
	int ie_tot_len;
	int report_len;
	uint8 *report;
	dot11_meas_req_t* ie;
	dot11_meas_rep_t* report_ie;


	if (body_len < 3) {
		WL_ERROR(("wl%d: wlc_recv_measure_request: got Measure Request from %s, "
			  "but frame body len was %d, expected > 3\n",
			  wlc->pub.unit, bcm_ether_ntoa(&hdr->sa, eabuf), body_len));
		return;
	}

	action_hdr = (struct dot11_action_measure *)body;

	/* is this a request from someone we should care about? */
	if (!wlc->pub.associated) {
		WL_INFORM(("wl%d: wlc_recv_measure_request: ignoring request from %s since we are"
			" not in a BSS or IBSS\n",
			wlc->pub.unit, bcm_ether_ntoa(&hdr->sa, eabuf)));
		return;
	} else if (wlc->pub.BSS) {
		if (bcmp(hdr->sa.octet, wlc->BSSID.octet, ETHER_ADDR_LEN)) {
			/* STAs should only get requests from the AP */
			WL_INFORM(("wl%d: wlc_recv_measure_request: ignoring request from %s since"
				" it is not our AP\n",
				wlc->pub.unit, bcm_ether_ntoa(&hdr->sa, eabuf)));
			return;
		}
	} else if (bcmp(hdr->bssid.octet, wlc->BSSID.octet, ETHER_ADDR_LEN)) {
		/* IBSS STAs should only get requests from other IBSS members */
		WL_INFORM(("wl%d: wlc_recv_measure_request: ignoring request from %s since it is in"
			" a foreign IBSS %s\n",
			wlc->pub.unit, bcm_ether_ntoa(&hdr->sa, eabuf),
			bcm_ether_ntoa(&hdr->bssid, eabuf)));
			return;
	}

	/* calculate the length of the report */
	report_len = 0;
	ie = (dot11_meas_req_t*)action_hdr->data;
	len = body_len - DOT11_ACTION_MEASURE_LEN;

	/* for each measurement request, calc the length of the report in the response */
	while (len > 2) {
		ie_tot_len = TLV_HDR_LEN + ie->len;

		if (ie->id != DOT11_MNG_MEASURE_REQUEST_ID ||
		    ie->len < DOT11_MNG_IE_MREQ_LEN ||
		    (ie->len >= 3 && (ie->mode & DOT11_MEASURE_MODE_ENABLE))) {
			/* ignore non-measure ie, short, or Mode == ENABLED requests */
		} else if (ie->type == DOT11_MEASURE_TYPE_BASIC) {
			/* Basic report with Unmeasured set */
			report_len += TLV_HDR_LEN + DOT11_MNG_IE_MREP_FIXED_LEN +
				DOT11_MEASURE_BASIC_REP_LEN;
		} else {
			/* CCA, RPI, or other req: Measure report with Incapable */
			report_len += TLV_HDR_LEN + DOT11_MNG_IE_MREP_FIXED_LEN;
		}

		ie = (dot11_meas_req_t*)((int8*)ie + ie_tot_len);
		len -= ie_tot_len;
	}

	/* allocate space and create the report */
	report = (uint8*)MALLOC(wlc->pub.osh, report_len);
	if (report == NULL) {
		WL_ERROR(("wl%d: wlc_recv_measure_request: out of memory, malloced %d bytes\n",
			wlc->pub.unit, MALLOCED(wlc->pub.osh)));
		return;
	}

	report_ie = (dot11_meas_rep_t*)report;
	ie = (dot11_meas_req_t*)action_hdr->data;
	len = body_len - DOT11_ACTION_MEASURE_LEN;

	/* for each measurement request, generate the report for the response */
	while (len > 2) {
		ie_tot_len = TLV_HDR_LEN + ie->len;
		if (ie->id != DOT11_MNG_MEASURE_REQUEST_ID) {
			WL_INFORM(("wl%d: wlc_recv_measure_request: got unexpected IE (id %d len"
				" %d), ignoring\n",
				wlc->pub.unit, ie->id, ie->len));
		} else if (ie->len >= 3 && (ie->mode & DOT11_MEASURE_MODE_ENABLE)) {
			WL_INFORM(("wl%d: wlc_recv_measure_request: got Measure Request mode Enable"
				" bit, ignoring\n",
				wlc->pub.unit));
		} else if (ie->len < DOT11_MNG_IE_MREQ_LEN) {
			WL_ERROR(("wl%d: wlc_recv_measure_request: got short Measure Request IE len"
				" %d, ignoring\n",
				wlc->pub.unit, ie->len));
		} else {
			if (ie->type == DOT11_MEASURE_TYPE_BASIC) {
				/* Basic report with Unmeasured set */
				report_ie->id = DOT11_MNG_MEASURE_REPORT_ID;
				report_ie->len = DOT11_MNG_IE_MREP_FIXED_LEN +
					DOT11_MEASURE_BASIC_REP_LEN;
				bzero((uint8*)&report_ie->token, report_ie->len);
				report_ie->token = ie->token;
				report_ie->mode = 0;
				report_ie->type = ie->type;
				report_ie->rep.basic.channel = ie->channel;
				bcopy(ie->start_time, report_ie->rep.basic.start_time, 8);
				bcopy(&ie->duration, &report_ie->rep.basic.duration, 2);
				report_ie->rep.basic.map = DOT11_MEASURE_BASIC_MAP_UNMEAS;
			} else {
				/* CCA, RPI, or other req: Measure report with Incapable */
				report_ie->id = DOT11_MNG_MEASURE_REPORT_ID;
				report_ie->len = DOT11_MNG_IE_MREP_FIXED_LEN;
				bzero((uint8*)&report_ie->token, report_ie->len);
				report_ie->token = ie->token;
				report_ie->mode = DOT11_MEASURE_MODE_INCAPABLE;
				report_ie->type = ie->type;
			}
			report_ie = (dot11_meas_rep_t*)((int8*)report_ie + TLV_HDR_LEN +
				report_ie->len);
		}

		ie = (dot11_meas_req_t*)((int8*)ie + ie_tot_len);
		len -= ie_tot_len;
	}

	ASSERT(((uint8*)report_ie - report) == (int)report_len);

	wlc_send_measure_report(wlc, &hdr->sa, &hdr->bssid, action_hdr->token, report, report_len);
	MFREE(wlc->pub.osh, report, report_len);
}

static void
wlc_recv_csa_action(wlc_info_t *wlc, struct dot11_management_header *hdr, uint8 *body, int body_len)
{
	struct dot11_action_switch_channel *action_hdr;
	dot11_channel_switch_t *req_ie;

	action_hdr = (struct dot11_action_switch_channel *)body;
	req_ie = (dot11_channel_switch_t *)action_hdr->data;
	WL_REGULATORY(("%s: mode %d, channel %d, count %d\n",
		__FUNCTION__, req_ie->mode, req_ie->channel, req_ie->count));

	return;
}

static void
wlc_recv_measure_report(wlc_info_t *wlc, struct dot11_management_header *hdr, uint8 *body,
	int body_len)
{
}


static void
wlc_send_tpc_report(wlc_info_t *wlc, struct ether_addr *da, struct ether_addr *bssid, uint8 token,
	int8 txpwr, int8 link_margin)
{
	void *p;
	uint8* pbody;
	uint8* end;
	uint body_len;
	struct dot11_action_measure * action_hdr;
	int8 tpc_report[DOT11_MNG_IE_TPC_REPORT_LEN];
	WL_INFORM(("wl%d: wlc_send_tpc_report: sending TPC Report to %s, txpwr %d, link margin"
		" %d\n",
		wlc->pub.unit, bcm_ether_ntoa(da, eabuf), txpwr, link_margin));

	/* TPC Report frame is
	 * 3 bytes Action Measure Req frame
	 * 4 bytes TPC Report IE
	 */
	body_len = DOT11_ACTION_MEASURE_LEN + TLV_HDR_LEN + DOT11_MNG_IE_TPC_REPORT_LEN;

	if ((p = wlc_frame_get_mgmt(wlc, FC_ACTION, da, bssid, body_len, &pbody)) == NULL)
		return;

	action_hdr = (struct dot11_action_measure *)pbody;
	action_hdr->category = DOT11_ACTION_CAT_SPECT_MNG;
	action_hdr->action = DOT11_ACTION_ID_TPC_REP;
	action_hdr->token = token;

	/* TPC Report IE */
	tpc_report[0] = txpwr;
	tpc_report[1] = link_margin;

	end = wlc_write_info_elt(action_hdr->data, DOT11_MNG_TPC_REPORT_ID,
		DOT11_MNG_IE_TPC_REPORT_LEN, tpc_report);

	ASSERT((end - pbody) == (int)body_len);

	wlc_sendmgmt(wlc, p, NULL, 0, NULL, 0);
}

static void
wlc_send_measure_report(wlc_info_t *wlc, struct ether_addr *da, struct ether_addr *bssid,
	uint8 token, uint8 *report, uint report_len)
{
	void *p;
	uint8* pbody;
	uint body_len;
	struct dot11_action_measure * action_hdr;
	WL_INFORM(("wl%d: wlc_send_measure_report: sending Measure Report (token %d) to %s\n",
		wlc->pub.unit, token, bcm_ether_ntoa(da, eabuf)));

	/* Measure Report frame is
	 * 3 bytes Action Measure Req frame
	 * variable len report
	 */
	body_len = DOT11_ACTION_MEASURE_LEN + report_len;

	if ((p = wlc_frame_get_mgmt(wlc, FC_ACTION, da, bssid, body_len, &pbody)) == NULL)
		return;

	action_hdr = (struct dot11_action_measure *)pbody;
	action_hdr->category = DOT11_ACTION_CAT_SPECT_MNG;
	action_hdr->action = DOT11_ACTION_ID_M_REP;
	action_hdr->token = token;

	bcopy(report, action_hdr->data, report_len);

	wlc_sendmgmt(wlc, p, NULL, 0, NULL, 0);
}

/*
 * Frame received, frame type FC_ACTION,
 * action_category DOT11_ACTION_NOTIFICATION
 */
static void
wlc_frameaction_wme(uint action_id, wlc_info_t *wlc,
	struct dot11_management_header *hdr, uint8 *body, int body_len)
{
	/* action_category == DOT11_ACTION_NOTIFICATION */
	/* WME Management action_id's */
	switch (action_id) {
	case WME_ADDTS_REQUEST:		/* AP only */
		break;
	case WME_ADDTS_RESPONSE:	/* STA only */
		/* Process WME Management action_id's */
		wlc_wme_setup_resp(wlc, hdr, body, body_len);
		break;
	case WME_DELTS_REQUEST:		/* STA only */
		break;
	default:
		WL_ERROR(("FC_ACTION, Invalid WME action code 0x%x\n", action_id));
		break;
	}
}


/* STA processes WME setup response */
static void
wlc_wme_setup_resp(wlc_info_t *wlc, struct dot11_management_header *hdr,
	uint8 *body, int body_len)
{
	struct dot11_management_notification* mgmt_hdr;

	mgmt_hdr = (struct dot11_management_notification *)body;
	/*
	 * Placeholder - deal with admission accepted or rejected.
	 * Algorithm for admission control still under discussion
	 * by WME committee.
	 * For now, all admission requests are refused.
	 */
	if (mgmt_hdr->status == WME_ADMISSION_REFUSED)
		WL_INFORM(("SETUP_RESPONSE: admission refused\n"));
}

void
wlc_send_action_err(wlc_info_t *wlc, struct dot11_management_header *hdr, uint8 *body, int body_len)
{
	void *p;
	uint8* pbody;
	uint action_category;
	struct ether_addr *bssid = &wlc->BSSID;

	if (ETHER_ISMULTI(&hdr->da)) {
		WL_INFORM(("wl%d: wlc_send_action_err: suppressing error reply to %s due to"
			" multicast address %s\n",
			wlc->pub.unit, bcm_ether_ntoa(&hdr->sa, eabuf),
			bcm_ether_ntoa(&hdr->da, eabuf)));
		return;
	}

	action_category = (uint)body[0];

	if (body_len < 2) {
		WL_INFORM(("wl%d: wlc_send_action_err: sending error reply to %s for Category %d"
			" with missing Action field\n",
			wlc->pub.unit, bcm_ether_ntoa(&hdr->sa, eabuf), action_category));
	} else {
		WL_INFORM(("wl%d: wlc_send_action_err: sending error reply to %s for"
			" Category/Action %d/%d\n",
			wlc->pub.unit, bcm_ether_ntoa(&hdr->sa, eabuf), action_category,
			(int)body[1]));
	}

	action_category |= DOT11_ACTION_CAT_ERR_MASK;

	if ((p = wlc_frame_get_mgmt(wlc, FC_ACTION, &hdr->sa, bssid, body_len, &pbody)) == NULL)
		return;

	bcopy(body, pbody, body_len);
	pbody[0] = (action_category | DOT11_ACTION_CAT_ERR_MASK);

	wlc_sendmgmt(wlc, p, NULL, 0, NULL, 0);
}

static wlc_bss_info_t *
wlc_BSSlookup(wlc_info_t *wlc, uchar *bssid, uint channel, uchar ssid[], uint ssid_len)
{
	wlc_bss_info_t	*BSS;
	uint index;

	BSS = NULL;

	/* search for the BSS descriptor which matches
	 * the bssid AND band(2G/5G) AND SSID
	 */
	for (index = 0; index < wlc->scan_results.count; index++) {
		BSS = wlc->scan_results.ptrs[index];
		if (!bcmp(bssid, (char*)&BSS->BSSID, ETHER_ADDR_LEN) &&
		    BAND_OF_CHANNEL(wlc, channel) == BAND_OF_CHANNEL(wlc, BSS->channel) &&
		    ssid_len == BSS->SSID_len &&
		    !bcmp(ssid, BSS->SSID, ssid_len))
			break;
	}

	if (index != wlc->scan_results.count)
		return (BSS);

	if (wlc->scan_results.count == MAXBSS)
		return (NULL);

	/* allocate a new entry */
	BSS = (wlc_bss_info_t *)MALLOC(wlc->pub.osh, sizeof(wlc_bss_info_t));
	if (BSS) {
		bzero((char*)BSS, sizeof(wlc_bss_info_t));
		BSS->RSSI = WLC_RSSI_MINVAL;
		wlc->scan_results.ptrs[wlc->scan_results.count++] = BSS;
	}
	return BSS;
}

void
wlc_bss_list_free(wlc_info_t *wlc, wlc_bss_list_t *bss_list)
{
	uint index;
	wlc_bss_info_t *bi;

	/* inspect all BSS descriptor */
	for (index = 0; index < bss_list->count; index++) {
		bi = bss_list->ptrs[index];
		if (bi) {
			if (bi->bcn_prb) {
				MFREE(wlc->pub.osh, bi->bcn_prb, bi->bcn_prb_len);
			}
			MFREE(wlc->pub.osh, bi, sizeof(wlc_bss_info_t));
			bss_list->ptrs[index] = NULL;
		}
	}
	bss_list->count = 0;
}

void
wlc_bss_list_xfer(wlc_bss_list_t *from, wlc_bss_list_t *to)
{
	uint size;

	ASSERT(to->count == 0);
	size = from->count * sizeof(wlc_bss_info_t*);
	bcopy((char*)from->ptrs, (char*)to->ptrs, size);
	bzero((char*)from->ptrs, size);

	to->count = from->count;
	from->count = 0;
}

/* Process received frames */
/*
 * Return TRUE if more frames need to be processed. FALSE otherwise.
 * Param 'bound' indicates max. # frames to process before break out.
 */
static bool
wlc_recv(wlc_info_t *wlc, uint fifo, uint bound)
{
	d11rxhdr_t *rxh;
	struct dot11_header *h;
	osl_t *osh;
	void *p;
	uint16 fc;
	uint len;
	uint n = 0;

	WL_TRACE(("wl%d: wlc_recv\n", wlc->pub.unit));

	osh = wlc->pub.osh;

	/* process next received frame */
	while (PIO_ENAB(wlc) ?
		(p = wlc_pio_rx(wlc->core->pio[fifo])) :
		(p = dma_rx(wlc->core->di[fifo]))) {

		/* frame starts with rxhdr */
		rxh = (d11rxhdr_t *)PKTDATA(osh, p);

		/* strip off rxhdr */
		PKTPULL(osh, p, HWRXOFF);


		/* fixup rx header endianness */
		ltoh16_buf((void*)rxh, sizeof(d11rxhdr_t));

		/* MAC inserts 2 pad bytes for a4 headers or QoS or A-MSDU subframes */
		if (rxh->RxStatus1 & RXS_PBPRES)
			PKTPULL(osh, p, 2);

		h = (struct dot11_header *)(PKTDATA(osh, p) + D11_PHY_HDR_LEN);
		len = PKTLEN(osh, p);

		if (rxh->RxStatus1 & RXS_FCSERR) {
			ASSERT(R_REG(&wlc->regs->maccontrol) & MCTL_KEEPBADFRAMES);
			WL_ERROR(("wl%d: wlc_recv: bad crc from %s\n", wlc->pub.unit,
				bcm_ether_ntoa(&h->a2, (char*)eabuf)));
			goto toss;
		}

		/* check received pkt has at least frame control field */
		if (len >= D11_PHY_HDR_LEN + sizeof(h->fc)) {
			fc = ltoh16(h->fc);
		} else {
			WLCNTINCR(wlc->pub._cnt.rxrunt);
			goto toss;
		}
		/* explicitly test bad src address to avoid sending bad deauth */
		if (FC_TYPE(fc) == FC_TYPE_DATA || FC_TYPE(fc) == FC_TYPE_MNG) {
			if ((ETHER_ISNULLADDR(&h->a2) || ETHER_ISMULTI(&h->a2))) {
				WL_ERROR(("wl%d: wlc_recv: dropping a frame with invalid"
					  " src mac address %s\n", wlc->pub.unit,
					  bcm_ether_ntoa(&h->a2, (char*)eabuf)));
				WLCNTINCR(wlc->pub._cnt.rxbadsrcmac);
				goto toss;
			}
			WLCNTINCR(wlc->pub._cnt.rxfrag);
		}

		if (FC_TYPE(fc) == FC_TYPE_DATA) {
			/*
			 * Call common receive, dispatch, and sendup code.
			 * The latter drops the perimeter lock and anything can happen!
			 */

			wlc_recvdata(wlc, osh, rxh, p);

		} else if ((FC_TYPE(fc) == FC_TYPE_MNG) || (FC_TYPE(fc) == FC_TYPE_CTL)) {

			wlc_recvctl(wlc, osh, rxh, p);

		} else {
			WLCNTINCR(wlc->pub._cnt.rxbadproto);
toss:
			PKTFREE(osh, p, FALSE);
			continue;
		}

		/* !give others some time to run! */
		if (++n >= bound)
			break;
	}

	/* post more rbufs */
	if (!PIO_ENAB(wlc))
		dma_rxfill(wlc->core->di[fifo]);

	return n >= bound;
}





uint16
wlc_read_shm(wlc_info_t *wlc, uint offset)
{
	d11regs_t *regs = wlc->regs;
	volatile uint16* objdata_lo = (uint16*)&regs->objdata;
	volatile uint16* objdata_hi = objdata_lo + 1;
	uint16 v;

	ASSERT((offset & 1) == 0);

	W_REG(&regs->objaddr, OBJADDR_SHM_SEL | (offset >> 2));
	if (offset & 2) {
		v = R_REG(objdata_hi);
	} else {
		v = R_REG(objdata_lo);
	}

	return v;
}

void
wlc_write_shm(wlc_info_t *wlc, uint offset, uint16 v)
{
	d11regs_t *regs = wlc->regs;
	volatile uint16* objdata_lo = (uint16*)&regs->objdata;
	volatile uint16* objdata_hi = objdata_lo + 1;

	ASSERT((offset & 1) == 0);

	W_REG(&regs->objaddr, OBJADDR_SHM_SEL | (offset >> 2));
	if (offset & 2) {
		W_REG(objdata_hi, v);
	} else {
		W_REG(objdata_lo, v);
	}
}

/* Copy a buffer to shared memory.
 * SHM 'offset' needs to be an even address and
 * Buffer length 'len' must be an even number of bytes
 */
void
wlc_copyto_shm(wlc_info_t *wlc, uint offset, const void* buf, int len)
{
	uint16 v;
	const uint8* p = (const uint8*)buf;
	int i;

	/* offset and len need to be even */
	ASSERT((offset & 1) == 0);
	ASSERT((len & 1) == 0);

	if (len <= 0)
		return;

	for (i = 0; i < len; i += 2) {
		v = p[i] | (p[i+1] << 8);
		wlc_write_shm(wlc, offset + i, v);
	}
}

/* Copy from shared memory to a buffer.
 * SHM 'offset' needs to be an even address and
 * Buffer length 'len' must be an even number of bytes
 */
void
wlc_copyfrom_shm(wlc_info_t *wlc, uint offset, void* buf, int len)
{
	uint16 v;
	uint8* p = (uint8*)buf;
	int i;

	/* offset and len need to be even */
	ASSERT((offset & 1) == 0);
	ASSERT((len & 1) == 0);

	if (len <= 0)
		return;

	for (i = 0; i < len; i += 2) {
		v = wlc_read_shm(wlc, offset + i);
		p[i] = v & 0xFF;
		p[i+1] = (v >> 8) & 0xFF;
	}
}

/* Set a range of shared memory to a value.
 * SHM 'offset' needs to be an even address and
 * Buffer length 'len' must be an even number of bytes
 */
void
wlc_set_shm(wlc_info_t *wlc, uint offset, uint16 v, int len)
{
	int i;

	/* offset and len need to be even */
	ASSERT((offset & 1) == 0);
	ASSERT((len & 1) == 0);

	if (len <= 0)
		return;

	for (i = 0; i < len; i += 2) {
		wlc_write_shm(wlc, offset + i, v);
	}
}

uint16
wlc_read_ihr(wlc_info_t *wlc, uint offset)
{
	d11regs_t *regs = wlc->regs;
	volatile uint16* objdata_lo = (uint16*)&regs->objdata;
	uint16 v;

	W_REG(&regs->objaddr, OBJADDR_IHR_SEL | offset);
	v = (uint16)R_REG(objdata_lo);

	return v;
}

void
wlc_write_ihr(wlc_info_t *wlc, uint offset, uint16 v)
{
	d11regs_t *regs = wlc->regs;
	volatile uint16* objdata_lo = (uint16*)&regs->objdata;

	W_REG(&regs->objaddr, OBJADDR_IHR_SEL | offset);
	W_REG(objdata_lo, v);
}

void
wlc_suspend_mac_and_wait(wlc_info_t *wlc)
{
	d11regs_t *regs = wlc->regs;
	uint i = 0;
	uint32 mc, mi;

	WL_TRACE(("wl%d: wlc_suspend_mac_and_wait: bandunit %d\n", wlc->pub.unit,
		wlc->band->bandunit));

	/* force the core awake */
	wlc->wakeformacsuspend = TRUE;
	wlc_set_ps_ctrl(wlc);

	mc = R_REG(&regs->maccontrol);
	if (mc == 0xffffffff) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}
	ASSERT(!(mc & MCTL_PSM_JMP_0));
	ASSERT(mc & MCTL_PSM_RUN);
	ASSERT(mc & MCTL_EN_MAC);

	mi = R_REG(&regs->macintstatus);
	if (mi == 0xffffffff) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}
	ASSERT(!(mi & MI_MACSSPNDD));

	wlc_mctrl(wlc, ~0, (mc & ~MCTL_EN_MAC));
	while ((!(R_REG(&regs->macintstatus) & MI_MACSSPNDD)) &&
	       (i < (WLC_MAX_MAC_SUSPEND / 10))) {
		OSL_DELAY(10);
		i++;
	}

	if (i >= (WLC_MAX_MAC_SUSPEND / 10)) {
		WL_ERROR(("wl%d: wlc_suspend_mac_and_wait: waited %d uS and MI_MACSSPNDD is still"
			" not on.\n",
			wlc->pub.unit, i * 10));
		WL_ERROR(("wl%d: psmdebug 0x%08x, phydebug 0x%08x, psm_brc 0x%04x\n",
		          wlc->pub.unit, R_REG(&regs->psmdebug),
		          R_REG(&regs->phydebug), R_REG(&regs->psm_brc)));
	} else if (i > 500)
		WL_INFORM(("wl%d: wlc_suspend_mac_and_wait: MI_MACSSPNDD took %d uS to come on.\n",
			wlc->pub.unit, i * 10));

	mc = R_REG(&regs->maccontrol);
	if (mc == 0xffffffff) {
		WL_ERROR(("wl%d: %s: dead chip\n", wlc->pub.unit, __FUNCTION__));
		wl_down(wlc->wl);
		return;
	}
	ASSERT(!(mc & MCTL_PSM_JMP_0));
	ASSERT(mc & MCTL_PSM_RUN);
	ASSERT(!(mc & MCTL_EN_MAC));
}

void wlc_enable_mac(wlc_info_t *wlc)
{
	d11regs_t *regs = wlc->regs;
	uint32 mc, mi;

	WL_TRACE(("wl%d: wlc_enable_mac: bandunit %d\n", wlc->pub.unit, wlc->band->bandunit));

	mc = R_REG(&regs->maccontrol);
	ASSERT(!(mc & MCTL_PSM_JMP_0));
	ASSERT(!(mc & MCTL_EN_MAC));
	ASSERT(mc & MCTL_PSM_RUN);


	wlc_mctrl(wlc, ~0, (mc | MCTL_EN_MAC));
	W_REG(&regs->macintstatus, MI_MACSSPNDD);

	mc = R_REG(&regs->maccontrol);
	ASSERT(!(mc & MCTL_PSM_JMP_0));
	ASSERT(mc & MCTL_EN_MAC);
	ASSERT(mc & MCTL_PSM_RUN);

	mi = R_REG(&regs->macintstatus);
	ASSERT(!(mi & MI_MACSSPNDD));

	/* allow core to sleep again */
	wlc->wakeformacsuspend = FALSE;
	wlc_set_ps_ctrl(wlc);
}

int
wlc_wsec(wlc_info_t *wlc, wlc_bsscfg_t *bsscfg, uint32 val)
{
	uint32 wsec_prev;
	int i;

	WL_WSEC(("wl%d: wlc_wsec: setting wsec 0x%x\n", wlc->pub.unit, val));

	wsec_prev = bsscfg->wsec;

	bsscfg->wsec = val & (WEP_ENABLED|TKIP_ENABLED|AES_ENABLED|WSEC_SWFLAG);

	if (!wlc->pub.up)
		return (0);

	/* update the hardware keys if we are changing the WSEC_SW flag */
	if ((wsec_prev & WSEC_SWFLAG) != (bsscfg->wsec & WSEC_SWFLAG)) {
		for (i = 0; i < WLC_MAX_WSEC_HW_KEYS(wlc); i++) {
			if (!WSEC_KEY(wlc, i))
				continue;
			wlc_key_hw_init(wlc, i);
			if (wlc->ncores > 1)
				setbit(wlc->wsec_bsupdate.vec, i);
		}
	}

	return (0);
}

/* return the multicast cipher defined as the weakest
 * enabled encryption in the order: WEP, TKIP, AES
 */
static uint8
wlc_wpa_mcast_cipher(wlc_info_t *wlc, wlc_bsscfg_t *cfg)
{
	wsec_key_t *key;
	uint32 wsec = cfg->wsec;
	uint8 cipher = 0;

	if (WSEC_WEP_ENABLED(wsec)) {
		/* wsec does not specify WEP key size, so use the default key algo */

		key = WSEC_BSS_DEFAULT_KEY(cfg);

		if (key && key->algo == CRYPTO_ALGO_WEP128) {
			cipher = WPA_CIPHER_WEP_104;
		} else if (key && key->algo == CRYPTO_ALGO_WEP1) {
			cipher = WPA_CIPHER_WEP_40;
		} else {
			if (!key)
				WL_ERROR(("wl%d: wlc_wpa_mcast_cipher: WEP encryption enabled but"
					" key is not plumbed\n",
					wlc->pub.unit));
			else
				WL_ERROR(("wl%d: wlc_wpa_mcast_cipher: WEP encryption enabled but"
					" default key algo (%d) is not WEP1 or WEP128\n",
					wlc->pub.unit, key->algo));
			cipher = WPA_CIPHER_WEP_40;
		}
	} else if (WSEC_TKIP_ENABLED(wsec)) {
		cipher = WPA_CIPHER_TKIP;
	} else if (WSEC_AES_ENABLED(wsec)) {
		cipher = WPA_CIPHER_AES_CCM;
	}

	return cipher;
}

uint
wlc_calc_frame_time(wlc_info_t *wlc, ratespec_t ratespec, int short_preamble, uint mac_len)
{
	uint nsyms, dur = 0, Ndps;
	uint rate = RSPEC2RATE(ratespec);

	WL_TRACE(("wl%d: wlc_calc_frame_time: rate %d, short %d, len%d\n",
		wlc->pub.unit, rate, short_preamble, mac_len));

	if (IS_OFDM(rate)) {
		dur = APHY_PREAMBLE_TIME;
		dur += APHY_SIGNAL_TIME;
		/* Ndbps = Mbps * 4 = rate(500Kbps) * 2 */
		Ndps = rate*2;
		/* NSyms = CEILING((SERVICE + 8*NBytes + TAIL) / Ndbps) */
		nsyms = CEIL((APHY_SERVICE_NBITS + 8 * mac_len + APHY_TAIL_NBITS), Ndps);
		dur += APHY_SYMBOL_TIME * nsyms;
		if (BAND_2G(wlc->band->bandtype))
			dur += DOT11_OFDM_SIGNAL_EXTENSION;
	} else {
		/* calc # bits * 2 so factor of 2 in rate (1/2 mbps) will divide out */
		mac_len = mac_len * 8 * 2;
		/* calc cieling of bits/rate = microseconds of air time */
		dur = (mac_len + rate-1) / rate;
		if (short_preamble)
			dur += BPHY_PLCP_SHORT_TIME;
		else
			dur += BPHY_PLCP_TIME;
	}
	return dur;
}

/* The opposite of wlc_calc_frame_time */
static uint
wlc_calc_frame_len(wlc_info_t *wlc, ratespec_t ratespec, int short_preamble, uint dur)
{
	uint nsyms, mac_len, Ndps;
	uint rate = RSPEC2RATE(ratespec);

	WL_TRACE(("wl%d: wlc_calc_frame_len: rate %d, short %d, dur %d\n",
		wlc->pub.unit, rate, short_preamble, dur));

	if (IS_OFDM(rate)) {
		dur -= APHY_PREAMBLE_TIME;
		dur -= APHY_SIGNAL_TIME;
		if (ISGPHY(wlc->band->pi))
			dur -= DOT11_OFDM_SIGNAL_EXTENSION;
		/* Ndbps = Mbps * 4 = rate(500Kbps) * 2 */
		Ndps = rate*2;
		nsyms = dur / APHY_SYMBOL_TIME;
		mac_len = ((nsyms * Ndps) - (APHY_SERVICE_NBITS + APHY_TAIL_NBITS)) / 8;
	} else {
		if (short_preamble)
			dur -= BPHY_PLCP_SHORT_TIME;
		else
			dur -= BPHY_PLCP_TIME;
		mac_len = dur * rate;
		/* divide out factor of 2 in rate (1/2 mbps) */
		mac_len = mac_len / 8 / 2;
	}

	return mac_len;
}

static uint
wlc_calc_ack_time(wlc_info_t *wlc, ratespec_t rspec, int short_preamble)
{
	uint dur = 0;

	WL_TRACE(("wl%d: wlc_calc_ack_time: rate %d, short %d\n", wlc->pub.unit, RSPEC2RATE(rspec),
		short_preamble));
	/* Spec 9.6: ack rate is the highest rate in BSSBasicRateSet that is less than
	 * or equal to the rate of the immediately previous frame in the FES
	 */
	rspec = WLC_BASIC_RATE(wlc, rspec);
	ASSERT(VALID_RATE_DBG(wlc, rspec));

	/* ACK frame len == 14 == 2(fc) + 2(dur) + 6(ra) + 4(fcs) */
	dur = wlc_calc_frame_time(wlc, rspec, short_preamble, (DOT11_ACK_LEN + DOT11_FCS_LEN));
	return dur;
}

static uint
wlc_calc_cts_time(wlc_info_t *wlc, ratespec_t rspec, int short_preamble)
{
	WL_TRACE(("wl%d: wlc_calc_cts_time: ratespec 0x%x, short %d\n", wlc->pub.unit, rspec,
		short_preamble));
	return wlc_calc_ack_time(wlc, rspec, short_preamble);
}


/* derive wlc->band->basic_rate[] table from 'rateset' */
void
wlc_rate_lookup_init(wlc_info_t *wlc, struct rateset *rateset)
{
	uint8 rate;
	uint8 mandatory;
	uint8 cck_basic = 0;
	uint8 ofdm_basic = 0;
	uint8 *br = wlc->band->basic_rate;
	uint i;

	/* incoming rates are in 500kbps units as in 802.11 Supported Rates */
	bzero(br, WLC_MAXRATE + 1);

	/* For each basic rate in the rates list, make an entry in the
	 * best basic lookup.
	 */
	for (i = 0; i < rateset->count; i++) {
		/* only make an entry for a basic rate */
		if (!(rateset->rates[i] & WLC_RATE_FLAG))
			continue;

		/* mask off basic bit */
		rate = (rateset->rates[i] & RATE_MASK);

		if (rate > WLC_MAXRATE) {
			WL_ERROR(("wlc_rate_lookup_init: invalid rate 0x%X in rate set\n",
				rateset->rates[i]));
			continue;
		}

		br[rate] = rate;
	}

	/* The rate lookup table now has non-zero entries for each
	 * basic rate, equal to the basic rate: br[basicN] = basicN
	 *
	 * To look up the best basic rate corresponding to any
	 * particular rate, code can use the basic_rate table
	 * like this
	 *
	 * basic_rate = wlc->band->basic_rate[tx_rate]
	 *
	 * Make sure there is a best basic rate entry for
	 * every rate by walking up the table from low rates
	 * to high, filling in holes in the lookup table
	 */

	for (i = 0; i < wlc->band->hw_rateset.count; i++) {
		rate = wlc->band->hw_rateset.rates[i];
		ASSERT(rate <= WLC_MAXRATE);

		if (br[rate] != 0) {
			/* This rate is a basic rate.
			 * Keep track of the best basic rate so far by
			 * modulation type.
			 */
			if (IS_OFDM(rate))
				ofdm_basic = rate;
			else
				cck_basic = rate;

			continue;
		}

		/* This rate is not a basic rate so figure out the
		 * best basic rate less than this rate and fill in
		 * the hole in the table
		 */

		br[rate] = IS_OFDM(rate) ? ofdm_basic : cck_basic;

		if (br[rate] != 0)
			continue;

		/* This is a weird setup where the lowest basic rate
		 * of a modulation type is non-existent or higher than
		 * some supported rates in the same modulation
		 */
		WL_RATE(("wl%d: no basic rate with the same modulation less than or equal to rate"
			" %d%s\n",
			 wlc->pub.unit, rate/2, (rate % 2)?".5":""));

		if (IS_OFDM(rate)) {
			/* In 11g and 11a, the OFDM mandatory rates are 6, 12, and 24 Mbps */
			if (rate >= WLC_RATE_24M)
				mandatory = WLC_RATE_24M;
			else if (rate >= WLC_RATE_12M)
				mandatory = WLC_RATE_12M;
			else
				mandatory = WLC_RATE_6M;
		} else {
			/* In 11b, all the CCK rates are mandatory 1 - 11 Mbps */
			mandatory = rate;
		}

		br[rate] = mandatory;
	}
}

static void
wlc_write_rate_shm(wlc_info_t *wlc, uint8 rate, uint8 basic_rate)
{
	uint8 phy_rate, index;
	uint8 basic_phy_rate, basic_index;
	uint16 dir_table, basic_table;
	uint16 basic_ptr;

	/* Shared memory address for the table we are reading */
	dir_table = IS_OFDM(basic_rate) ? M_RT_DIRMAP_A : M_RT_DIRMAP_B;

	/* Shared memory address for the table we are writing */
	basic_table = IS_OFDM(rate) ? M_RT_BBRSMAP_A : M_RT_BBRSMAP_B;

	/*
	 * for a given rate, the LS-nibble of the PLCP SIGNAL field is
	 * the index into the rate table.
	 */
	phy_rate = rate_info[rate] & RATE_MASK;
	basic_phy_rate = rate_info[rate] & RATE_MASK;
	index = phy_rate & 0xf;
	basic_index = basic_phy_rate & 0xf;

	/* Find the SHM pointer to the ACK rate entry by looking in the
	 * Direct-map Table
	 */
	basic_ptr = wlc_read_shm(wlc, (dir_table + basic_index * 2));

	/* Update the SHM BSS-basic-rate-set mapping table with the pointer
	 * to the correct basic rate for the given incoming rate
	 */
	wlc_write_shm(wlc, (basic_table + index * 2), basic_ptr);
}

void
wlc_set_ratetable(wlc_info_t *wlc)
{
	const rateset_t *rs;
	uint8 rate, basic_rate;
	uint i;


	if (ISAPHY(wlc->band->pi))
		rs = ofdm_rates;
	else if (wlc->band->gmode)
		rs = cck_ofdm_rates;
	else
		rs = cck_rates;

	ASSERT(rs);

	/* walk the phy rate table and update SHM basic rate lookup table */
	for (i = 0; i < rs->count; i++) {
		rate = rs->rates[i] & RATE_MASK;

		/* for a given rate WLC_BASIC_RATE returns the rate at
		 * which a response ACK/CTS should be sent.
		 */
		basic_rate = WLC_BASIC_RATE(wlc, rate);
		if (basic_rate == 0) {
			/* This should only happen if we are using a
			 * restricted rateset.
			 */
			WL_RATE(("wl%d: set_ratetable: Adding rate %d to ratetable\n",
				wlc->pub.unit, rate));
			basic_rate = rs->rates[0] & RATE_MASK;
		}

		wlc_write_rate_shm(wlc, rate, basic_rate);
	}
}

/*
 * Return true if the specified rate is supported by the specified band.
 * WLC_BAND_AUTO indicates the current band.
 */
static bool
wlc_valid_rate(wlc_info_t *wlc, ratespec_t rspec, int band, bool verbose)
{
	struct rateset *hw_rateset;
	uint i;

	if ((band == WLC_BAND_AUTO) || (band == wlc->band->bandtype)) {
		hw_rateset = &wlc->band->hw_rateset;
	} else if (NBANDS(wlc) > 1) {
		hw_rateset = &wlc->bandstate[OTHERBANDUNIT(wlc)].hw_rateset;
	} else {
		/* other band specified and we are a single band device */
		return (FALSE);
	}

	for (i = 0; i < hw_rateset->count; i++)
		if (hw_rateset->rates[i] == RSPEC2RATE(rspec))
			return (TRUE);
	if (verbose)
		WL_ERROR(("wl%d: wlc_valid_rate: rate spec 0x%x not in hw_rateset\n",
		          wlc->pub.unit, rspec));

	return (FALSE);
}


static void
wlc_mod_prb_rsp_rate_table(wlc_info_t *wlc, uint frame_len)
{
	const rateset_t *rs;
	uint8 rate;
	uint16 entry_ptr;
	uint8 plcp[D11_PHY_HDR_LEN];
	uint16 dur, sifs;
	uint i;


	if (ISAPHY(wlc->band->pi))
		rs = ofdm_rates;
	else if (wlc->band->gmode)
		rs = cck_ofdm_rates;
	else
		rs = cck_rates;


	ASSERT(rs);

	sifs = SIFS(wlc->band);

	/* walk the phy rate table and update MAC core SHM basic rate table entries */
	for (i = 0; i < rs->count; i++) {
		rate = rs->rates[i] & RATE_MASK;

		entry_ptr = wlc_rate_shm_offset(wlc, rate);

		/* Calculate the Probe Response PLCP for the given rate */
		wlc_compute_plcp(wlc, rate, frame_len, plcp);

		/* Calculate the duration of the Probe Response frame plus SIFS for the MAC */
		dur = (uint16)wlc_calc_frame_time(wlc, rate, 0, frame_len);
		dur += sifs;

		/* Update the SHM Rate Table entry Probe Response values */
		wlc_write_shm(wlc, entry_ptr + M_RT_PRS_PLCP_POS, (uint16)(plcp[0] + (plcp[1]
			<< 8)));
		wlc_write_shm(wlc, entry_ptr + M_RT_PRS_PLCP_POS + 2, (uint16)(plcp[2] +
			(plcp[3] << 8)));
		wlc_write_shm(wlc, entry_ptr + M_RT_PRS_DUR_POS, dur);
	}
}

static uint16
wlc_compute_bcntsfoff(wlc_info_t *wlc, bool is_ofdm, uint phy_rate, uint short_preamble,
	bool phydelay)
{
	uint bcntsfoff = 0;

	if (is_ofdm) {
		/* tx delay from MAC through phy to air (2.1 usec) +
		 * phy header time (preamble + PLCP SIGNAL == 20 usec) +
		 * PLCP SERVICE + MAC header time (SERVICE + FC + DUR + A1 + A2 + A3 + SEQ == 26
		 * bytes at beacon rate)
		 */
		bcntsfoff += phydelay ? D11A_PHY_TX_DELAY : 0;
		bcntsfoff += APHY_PREAMBLE_TIME + APHY_SIGNAL_TIME;
		bcntsfoff += wlc_compute_airtime(wlc, is_ofdm, phy_rate, APHY_SERVICE_NBITS/8 +
			DOT11_MAC_HDR_LEN);
	} else {
		/* tx delay from MAC through phy to air (3.4 usec) +
		 * phy header time (long preamble + PLCP == 192 usec) +
		 * MAC header time (FC + DUR + A1 + A2 + A3 + SEQ == 24 bytes at beacon rate)
		 */
		bcntsfoff += phydelay ? D11B_PHY_TX_DELAY : 0;
		bcntsfoff += (short_preamble != 0) ? D11B_PHY_SPREHDR_TIME: D11B_PHY_LPREHDR_TIME;
		bcntsfoff += wlc_compute_airtime(wlc, is_ofdm, phy_rate, DOT11_MAC_HDR_LEN);
	}
	return (uint16)(bcntsfoff);
}

/* decode wpa IE to retrieve mcast/unicast ciphers and auth modes */
static int
wlc_parse_wpa_ie(wlc_info_t *wlc, wpa_ie_fixed_t *wpaie, wlc_bss_info_t *bi)
{
	int len = wpaie->length;	/* value length */
	wpa_suite_mcast_t *mcast;
	wpa_suite_ucast_t *ucast;
	wpa_suite_auth_key_mgmt_t *mgmt;
	uint16 count;
	uint i, j;

	/*
	* The TLV tag is generic for all vendor-specific IEs. We need
	* to check OUI as well to make sure this TLV is indeed WPA,
	* and quietly discard the IE if it is not WPA IE.
	*/
	if ((len < WPA_IE_OUITYPE_LEN) ||
	    bcmp(wpaie->oui, WPA_OUI"\01", WPA_IE_OUITYPE_LEN))
		return 0;

	if ((len < WPA_IE_TAG_FIXED_LEN) ||
	    (ltoh16_ua(&wpaie->version) != WPA_VERSION)) {
		WL_ERROR(("wl%d: unsupported WPA version %d\n", wlc->pub.unit,
			ltoh16_ua(&wpaie->version)));
		return BCME_UNSUPPORTED;
	}
	len -= WPA_IE_TAG_FIXED_LEN;

	/* Default WPA parameters */
	bi->wpa.flags = RSN_FLAGS_SUPPORTED;
	bi->wpa.multicast = WPA_CIPHER_TKIP;
	bi->wpa.ucount = 1;
	bi->wpa.unicast[0] = WPA_CIPHER_TKIP;
	bi->wpa.acount = 1;
	bi->wpa.auth[0] = RSN_AKM_UNSPECIFIED;


	/* Check for multicast cipher suite */
	if (len < WPA_SUITE_LEN) {
		WL_INFORM(("wl%d: no multicast cipher suite\n", wlc->pub.unit));
		/* it is ok to not have multicast cipher */
		return 0;
	}
	/* pick up multicast cipher if we know what it is */
	mcast = (wpa_suite_mcast_t *)&wpaie[1];
	len -= WPA_SUITE_LEN;
	if (!bcmp(mcast->oui, WPA_OUI, WPA_OUI_LEN)) {
		if (IS_WPA_CIPHER(mcast->type))
			bi->wpa.multicast = mcast->type;
		else
			WL_INFORM(("wl%d: unsupported WPA multicast cipher %d\n",
				wlc->pub.unit, mcast->type));
	} else
		WL_INFORM(("wl%d: unsupported proprietary multicast cipher OUI "
			   "%02X:%02X:%02X\n", wlc->pub.unit,
			   mcast->oui[0], mcast->oui[1], mcast->oui[2]));

	/* Check for unicast suite(s) */
	if (len < WPA_IE_SUITE_COUNT_LEN) {
		WL_INFORM(("wl%d: no unicast suite\n", wlc->pub.unit));
		/* it is ok to not have unicast cipher(s) */
		return 0;
	}
	/* walk thru unicast cipher list and pick up what we recognize */
	ucast = (wpa_suite_ucast_t *)&mcast[1];
	count = ltoh16_ua(&ucast->count);
	len -= WPA_IE_SUITE_COUNT_LEN;
	for (i = 0, j = 0;
	     i < count && j < ARRAYSIZE(bi->wpa.unicast) && len >= WPA_SUITE_LEN;
	     i ++, len -= WPA_SUITE_LEN) {
		if (!bcmp(ucast->list[i].oui, WPA_OUI, WPA_OUI_LEN)) {
			if (IS_WPA_CIPHER(ucast->list[i].type))
				bi->wpa.unicast[j++] = ucast->list[i].type;
			else
				WL_INFORM(("wl%d: unsupported WPA unicast cipher %d\n",
					wlc->pub.unit, ucast->list[i].type));
		} else
			WL_INFORM(("wl%d: unsupported proprietary unicast cipher OUI "
				   "%02X:%02X:%02X\n", wlc->pub.unit,
				   ucast->list[i].oui[0], ucast->list[i].oui[1],
				   ucast->list[i].oui[2]));
	}
	bi->wpa.ucount = (uint8)j;

	/* jump to auth key mgmt suites */
	len -= (count - i) * WPA_SUITE_LEN;

	/* Check for auth key management suite(s) */
	if (len < WPA_IE_SUITE_COUNT_LEN) {
		WL_INFORM(("wl%d: auth key mgmt suite\n", wlc->pub.unit));
		/* it is ok to not have auth key mgmt suites */
		return 0;
	}
	/* walk thru auth management suite list and pick up what we recognize */
	mgmt = (wpa_suite_auth_key_mgmt_t *)&ucast->list[count];
	count = ltoh16_ua(&mgmt->count);
	len -= WPA_IE_SUITE_COUNT_LEN;
	for (i = 0, j = 0;
	     i < count && j < ARRAYSIZE(bi->wpa.auth) && len >= WPA_SUITE_LEN;
	     i ++, len -= WPA_SUITE_LEN) {
		if (!bcmp(mgmt->list[i].oui, WPA_OUI, WPA_OUI_LEN)) {
			if (IS_WPA_AKM(mgmt->list[i].type))
				bi->wpa.auth[j++] = mgmt->list[i].type;
			else
				WL_INFORM(("wl%d: unsupported WPA auth %d\n",
					wlc->pub.unit, mgmt->list[i].type));
		} else
			WL_INFORM(("wl%d: unsupported proprietary auth OUI "
				   "%02X:%02X:%02X\n", wlc->pub.unit,
				   mgmt->list[i].oui[0], mgmt->list[i].oui[1],
				   mgmt->list[i].oui[2]));
	}
	bi->wpa.acount = (uint8)j;

	bi->flags |= WLC_BSS_WPA;


	return 0;
}

#ifdef BCMWPA2
/* decode wpa2 IE to retrieve mcast/unicast ciphers and auth modes */
static int
wlc_parse_wpa2_ie(wlc_info_t *wlc, bcm_tlv_t *wpa2ie, wlc_bss_info_t *bi)
{
	int len = wpa2ie->len;		/* value length */
	wpa_suite_mcast_t *mcast;
	wpa_suite_ucast_t *ucast;
	wpa_suite_auth_key_mgmt_t *mgmt;
	uint8 *cap;
	uint16 count;
	uint i, j;

	/* Check min length and version */
	if ((len < WPA2_VERSION_LEN) ||
	    (ltoh16_ua(wpa2ie->data) != WPA2_VERSION)) {
		WL_ERROR(("wl%d: unsupported WPA2 version %d\n", wlc->pub.unit,
			ltoh16_ua(wpa2ie->data)));
		return BCME_UNSUPPORTED;
	}
	len -= WPA2_VERSION_LEN;

	/* Default WPA2 parameters */
	bi->wpa2.flags = RSN_FLAGS_SUPPORTED;
	bi->wpa2.multicast = WPA_CIPHER_AES_CCM;
	bi->wpa2.ucount = 1;
	bi->wpa2.unicast[0] = WPA_CIPHER_AES_CCM;
	bi->wpa2.acount = 1;
	bi->wpa2.auth[0] = RSN_AKM_UNSPECIFIED;


	/* Check for multicast cipher suite */
	if (len < WPA_SUITE_LEN) {
		WL_INFORM(("wl%d: no multicast cipher suite\n", wlc->pub.unit));
		/* it is ok to not have multicast cipher */
		return 0;
	}
	/* pick up multicast cipher if we know what it is */
	mcast = (wpa_suite_mcast_t *)&wpa2ie->data[WPA2_VERSION_LEN];
	len -= WPA_SUITE_LEN;
	if (!bcmp(mcast->oui, WPA2_OUI, DOT11_OUI_LEN)) {
		if (IS_WPA_CIPHER(mcast->type))
			bi->wpa2.multicast = mcast->type;
		else
			WL_INFORM(("wl%d: unsupported WPA2 multicast cipher %d\n",
				wlc->pub.unit, mcast->type));
	} else
		WL_INFORM(("wl%d: unsupported proprietary multicast cipher OUI "
			   "%02X:%02X:%02X\n", wlc->pub.unit,
			   mcast->oui[0], mcast->oui[1], mcast->oui[2]));

	/* Check for unicast suite(s) */
	if (len < WPA_IE_SUITE_COUNT_LEN) {
		WL_INFORM(("wl%d: no unicast suite\n", wlc->pub.unit));
		/* it is ok to not have unicast cipher(s) */
		return 0;
	}
	/* walk thru unicast cipher list and pick up what we recognize */
	ucast = (wpa_suite_ucast_t *)&mcast[1];
	count = ltoh16_ua(&ucast->count);
	len -= WPA_IE_SUITE_COUNT_LEN;
	for (i = 0, j = 0;
	     i < count && j < ARRAYSIZE(bi->wpa2.unicast) && len >= WPA_SUITE_LEN;
	     i ++, len -= WPA_SUITE_LEN) {
		if (!bcmp(ucast->list[i].oui, WPA2_OUI, DOT11_OUI_LEN)) {
			if (IS_WPA_CIPHER(ucast->list[i].type))
				bi->wpa2.unicast[j++] = ucast->list[i].type;
			else
				WL_INFORM(("wl%d: unsupported WPA2 unicast cipher %d\n",
					wlc->pub.unit, ucast->list[i].type));
		} else
			WL_INFORM(("wl%d: unsupported proprietary unicast cipher OUI "
				   "%02X:%02X:%02X\n", wlc->pub.unit,
				   ucast->list[i].oui[0], ucast->list[i].oui[1],
				   ucast->list[i].oui[2]));
	}
	bi->wpa2.ucount = (uint8)j;

	/* jump to auth key mgmt suites */
	len -= (count - i) * WPA_SUITE_LEN;

	/* Check for auth key management suite(s) */
	if (len < WPA_IE_SUITE_COUNT_LEN) {
		WL_INFORM(("wl%d: auth key mgmt suite\n", wlc->pub.unit));
		/* it is ok to not have auth key mgmt suites */
		return 0;
	}
	/* walk thru auth management suite list and pick up what we recognize */
	mgmt = (wpa_suite_auth_key_mgmt_t *)&ucast->list[count];
	count = ltoh16_ua(&mgmt->count);
	len -= WPA_IE_SUITE_COUNT_LEN;
	for (i = 0, j = 0;
	     i < count && j < ARRAYSIZE(bi->wpa2.auth) && len >= WPA_SUITE_LEN;
	     i ++, len -= WPA_SUITE_LEN) {
		if (!bcmp(mgmt->list[i].oui, WPA2_OUI, DOT11_OUI_LEN)) {
			if (IS_WPA2_AKM(mgmt->list[i].type))
				bi->wpa2.auth[j++] = mgmt->list[i].type;
			else
				WL_INFORM(("wl%d: unsupported WPA2 auth %d\n",
					wlc->pub.unit, mgmt->list[i].type));
		} else
			WL_INFORM(("wl%d: unsupported proprietary auth OUI "
				   "%02X:%02X:%02X\n", wlc->pub.unit,
				   mgmt->list[i].oui[0], mgmt->list[i].oui[1],
				   mgmt->list[i].oui[2]));
	}
	bi->wpa2.acount = (uint8)j;

	bi->flags |= WLC_BSS_WPA;

	/* parse RSN capabilities */
	cap = (uint8 *)&mgmt->list[count];
	if (cap[0] & WPA_CAP_WPA2_PREAUTH)
		bi->wpa2.flags |= RSN_FLAGS_PREAUTH;

	return 0;
}
#endif /* BCMWPA2 */

/* grab the channel info from the rxheader, and convert to a channel number */
uint
wlc_rx_channel(d11rxhdr_t *rxh)
{
	uint rx_chan_id;
	uint rx_phy;

	rx_chan_id = WLC_CHAN_ID(rxh->RxChan);
	rx_phy = WLC_CHAN_PHYTYPE(rxh->RxChan);

	return wlc_radiocode2channel(rx_chan_id, rx_phy);
}

static int
wlc_parse_bcn_prb(wlc_info_t *wlc, d11rxhdr_t *rxh, struct dot11_management_header *hdr, void *body,
	uint body_len, wlc_bss_info_t *bi)
{
	struct dot11_bcn_prb *fixed = (struct dot11_bcn_prb *) body;
	uint16 cap;
	uint8 chan = 0;
	uint rx_chan;
	uint8 *tlvs;
	uint tlvs_len;
	uint8 *parse;
	uint parse_len;
	bcm_tlv_t *ie;
	wpa_ie_fixed_t *wpaie;

	if (body_len < sizeof(struct dot11_bcn_prb))
		return (-1);

	/* check validity of beacon before update */
	tlvs_len = body_len - DOT11_BCN_PRB_LEN;
	tlvs = (uint8 *)body + DOT11_BCN_PRB_LEN;

	/* grab the channel from the rxheader, need conversion for b/g phy */
	rx_chan = wlc_rx_channel(rxh);

	/* check the freq. */
	if ((ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_DS_PARMS_ID)) != NULL) {
		chan = ie->data[0];
		if (!VALID_CHANNEL(wlc, chan)) {
			WL_INFORM(("Bad channel in beacon: %d\n", chan));
			return (-1);
		}
	} else {
		/* 802.11a beacons don't have a DS tlv in them.
		 * Figure it out from the current mac channel.
		 */
		chan = (uint8) rx_chan;
	}

	/* Fill in bss with new info */
	bzero((char*)bi, sizeof(wlc_bss_info_t));

	bcopy((char *)&hdr->bssid, (char *)&bi->BSSID, ETHER_ADDR_LEN);

	cap = ltoh16_ua(&fixed->capability);
	if ((cap & DOT11_CAP_ESS) && !(cap & DOT11_CAP_IBSS))
		bi->infra = 1;
	bi->capability = cap;

	bi->RSSI = (uint16)wlc_compute_rssi(wlc->band->pi, rxh);
	/* flag if this is an RSSI reading on same channel as the bcn/prb was transmitted */
	if (rx_chan == chan)
		bi->flags |= WLC_BSS_RSSI_ON_CHANNEL;

	/* extract phy_noise from the channel indexed array */
	bi->phy_noise = wlc->phy_noise_list[chan];

	bi->beacon_period = ltoh16_ua(&fixed->beacon_interval);

	/* process tagged fields */

	/* SSID */
	if ((ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_SSID_ID)) != NULL) {
		bi->SSID_len = ie->len;
		bcopy(ie->data, bi->SSID, bi->SSID_len);
	} else {
		WL_INFORM(("Missing SSID info in beacon\n"));
		return (-1);
	}

	/* parse rateset and pull out raw rateset */
	if (wlc_parse_rates(wlc, tlvs, tlvs_len, bi, NULL) != 0) {
		WL_INFORM(("Missing or Invalid rate info in beacon\n"));
		return (-1);
	}

	/* Check for a legacy 54G bcn/proberesp by looking for more than 8 rates
	 * in the Supported Rates elt
	 */
	ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_RATES_ID);
	if (ie && ie->len > 8) {
		bi->flags |= WLC_BSS_54G;
	}

	/* DS parameters */
	bi->channel = chan;

	/* IBSS parameters */
	if (cap & DOT11_CAP_IBSS) {
		if ((ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_IBSS_PARMS_ID)) != NULL) {
			bi->atim_window = ltoh16_ua(ie->data);
		}
	}

#ifdef BCMWPA2
        /* WPA2 parameters */
        if ((ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_RSN_ID)) != NULL)
                wlc_parse_wpa2_ie(wlc, ie, bi);
#endif /* BCMWPA2 */

	/* WPA parameters */
	parse = tlvs;
	parse_len = tlvs_len;
	if ((wpaie = bcm_find_wpaie(parse, parse_len)) != NULL)
		wlc_parse_wpa_ie(wlc, wpaie, bi);

	/* WME parameters */
	if (WME_ENAB(wlc)) {
		parse = tlvs;
		parse_len = tlvs_len;
		while ((ie = bcm_parse_tlvs(parse, parse_len, DOT11_MNG_PROPR_ID)))
			if (wlc_is_wme_ie(wlc, (uint8 *)ie, &parse, &parse_len))
				break;
		if (ie) {
			wme_param_ie_t *wmeie_body = (wme_param_ie_t *)&ie->data;
			bi->flags |= WLC_BSS_WME;
			bi->wme_qosinfo = wmeie_body->qosinfo;
		}
	}

	/* Is the AP from Broadcom */
	if (wlc_find_vendor_ie(tlvs, tlvs_len, (uchar*)BRCM_OUI, NULL, 0) != NULL)
		bi->flags |= WLC_BSS_BRCM;

	return 0;
}

static bool
wlc_erp_find(wlc_info_t *wlc, void *body, uint body_len, uint8 **erp, int *len)
{
	bcm_tlv_t *ie;
	uint8 *tlvs = (uint8 *)body + DOT11_BCN_PRB_LEN;
	int tlvs_len = body_len - DOT11_BCN_PRB_LEN;

	ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_ERP_ID);

	if (ie == NULL)
		return FALSE;

	*len = ie->len;
	*erp = ie->data;

	return TRUE;
}

/* Look for the 802.11g 4.0 draft NonERP information elt
 * for legacy drivers that implemented it.
 */
static bool
wlc_nonerp_find(wlc_info_t *wlc, void *body, uint body_len, uint8 **erp, int *len)
{
	bcm_tlv_t *ie;
	uint nonerp_len;
	uint8 *tlvs = (uint8 *)body + DOT11_BCN_PRB_LEN;
	int tlvs_len = body_len - DOT11_BCN_PRB_LEN;

	ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_NONERP_ID);

	if (ie == NULL)
		return FALSE;

	/* Use specific details to validate the element as the legacy
	 * NonERP. Since the NonERP ID is not reserved, it may be used
	 * in the future for some other Info Element. The validation
	 * is an effort to avoid incorrectly identifying a future Info
	 * Element as a legacy NonERP.  This validation can be dropped
	 * if ID 47 is reserved for legacy NonERP.
	 */

	nonerp_len = ie->len;

	/* validate the legacy NonERP info elt as 1 or 2 bytes long
	 * and that only bits 0 and 1 may be set, all others are zero
	 */
	if ((nonerp_len == 1 && (ie->data[0] & 0xFC) == 0) ||
	    (nonerp_len == 2 && (ie->data[0] & 0xFC) == 0 && ie->data[1] == 0)) {
		*len = nonerp_len;
		*erp = ie->data;

		return TRUE;
	}

	return FALSE;
}

/*	Max buffering needed for beacon template/prb resp template is 142 bytes.

	PLCP header is 6 bytes.
	802.11 A3 header is 24 bytes.
	Max beacon frame body template length is 112 bytes.
	Max probe resp frame body template length is 110 bytes.

	The *len value is set to the number of bytes in buf used, and starts with the PLCP
	and included up to, but not including, the 4 byte FCS.
*/
static void
wlc_bcn_prb_template(wlc_info_t *wlc, uint type, ratespec_t bcn_rate, wlc_bsscfg_t *cfg,
                     uint16 *buf, int *len)
{
	cck_phy_hdr_t *plcp;
	struct dot11_management_header *h;
	int hdr_len, body_len;

	ASSERT(*len >= 142);
	ASSERT(type == FC_BEACON || type == FC_PROBE_RESP);

	hdr_len = D11_PHY_HDR_LEN + DOT11_MAC_HDR_LEN;
	body_len = *len - hdr_len; /* calc buffer size provided for frame body */

	wlc_bcn_prb_body(wlc, type, cfg, (uint8 *)buf + hdr_len, &body_len);

	*len = hdr_len + body_len; /* return actual size */

	/* format PHY and MAC headers */
	bzero((char*)buf, hdr_len);

	plcp = (cck_phy_hdr_t *)buf;

	/* PLCP for Probe Response frames are filled in from core's rate table */

	if (type == FC_BEACON) {
		/* fill in PLCP */
		wlc_compute_plcp(wlc, bcn_rate, (DOT11_MAC_HDR_LEN + body_len + DOT11_FCS_LEN),
			(uint8*)plcp);
	}

	h = (struct dot11_management_header *)&plcp[1];

	/* fill in 802.11 header */
	h->fc = htol16((uint16)type);

	/* DUR is 0 for multicast bcn, or filled in by MAC for prb resp */
	/* A1 filled in by MAC for prb resp, broadcast for bcn */
	if (type == FC_BEACON)
		bcopy((char*)&ether_bcast, (char*)&h->da, ETHER_ADDR_LEN);
	bcopy((char*)&wlc->pub.cur_etheraddr, (char*)&h->sa, ETHER_ADDR_LEN);
	bcopy((char*)&wlc->announce_BSSID, (char*)&h->bssid, ETHER_ADDR_LEN);
	/* SEQ filled in by MAC */

	return;
}

/* This routine is used to update the PVB portion of the TIM for PS Mode */
void
wlc_update_beacon(wlc_info_t *wlc)
{
	uint16	bcn[BCN_TMPL_LEN / 2];
	int	len = BCN_TMPL_LEN;
	uint32	both_valid = MCMD_BCN0VLD | MCMD_BCN1VLD;
	d11regs_t *regs = wlc->regs;
	wlc_bsscfg_t* cfg = wlc->bsscfg[wlc->bsscfg_bcn_idx];

	if (!cfg || wlc->pub.BSS || !cfg->up)
		return;

	/* Clear the soft intmask */
	wlc->defmacintmask &= ~MI_BCNTPL;

	/* Check if both templates are in use, if so sched. an interrupt
	 *	that will call back into this routine
	 */
	if ((R_REG(&regs->maccommand) & both_valid) == both_valid) {
		/* clear any previous status */
		W_REG(&regs->macintstatus, MI_BCNTPL);
	}
	/* Check that after scheduling the interrupt both of the
	 *	templates are still busy. if not clear the int. & remask
	 */
	if ((R_REG(&regs->maccommand) & both_valid) == both_valid) {
		wlc->defmacintmask |= MI_BCNTPL;
		return;
	}

	/* update the template */
	wlc_bcn_prb_template(wlc, FC_BEACON, wlc_bcn_rate(wlc), cfg, bcn, &len);

	/* use bcn 0 template ? */
	if (!(R_REG(&regs->maccommand) & MCMD_BCN0VLD)) {
		wlc_write_template_ram(wlc, T_BCN0_TPL_BASE, (len + 3) & ~3, bcn);
		ASSERT(len < 65536);
		wlc_write_shm(wlc, M_BCN0_FRM_BYTESZ, (uint16)len);
		/* mark beacon 0 valid */
		OR_REG(&regs->maccommand, MCMD_BCN0VLD);
	}
	/* use bcn 1 template ? */
	else if (!(R_REG(&regs->maccommand) & MCMD_BCN1VLD)) {
		wlc_write_template_ram(wlc, T_BCN1_TPL_BASE, (len + 3) & ~3, bcn);
		ASSERT(len < 65536);
		wlc_write_shm(wlc, M_BCN1_FRM_BYTESZ, (uint16)len);
		/* mark beacon 1 valid */
		OR_REG(&regs->maccommand, MCMD_BCN1VLD);
	}
	else
		ASSERT(0);	/* one template should always have been available */

}


static void
wlc_update_all_probe_resp(wlc_info_t *wlc, bool suspend)
{
}

static void
wlc_update_probe_resp(wlc_info_t *wlc, wlc_bsscfg_t *cfg, bool suspend)
{
	uint16 prb_resp[BCN_TMPL_LEN/2];
	int len, i;

	/* create the probe response template */
	len = BCN_TMPL_LEN;
	wlc_bcn_prb_template(wlc, FC_PROBE_RESP, 0, cfg, prb_resp, &len);

	/* write the probe response to hardware, or save in the config structure */
	if (cfg == wlc->bsscfg[wlc->bsscfg_prb_idx]) {
		if (suspend)
			wlc_suspend_mac_and_wait(wlc);

		/* write the probe response into the template region */
		wlc_write_template_ram(wlc, T_PRS_TPL_BASE, (len + 3) & ~3, prb_resp);

		/* write the length of the probe response frame (+PLCP/-FCS) */
		wlc_write_shm(wlc, M_PRB_RESP_FRM_LEN, (uint16)len);

		/* write the SSID and SSID length */
		for (i = 0; i < cfg->SSID_len; i += 2) {
			uint16 tmp = (uint16)(cfg->SSID[i] + (cfg->SSID[i+1] << 8));
			wlc_write_shm(wlc, M_SSID + i, tmp);
		}
		wlc_write_shm(wlc, M_SSIDLEN, (uint16)cfg->SSID_len);

		/*
		 * Write PLCP headers and durations for probe response frames at all rates.
		 * Use the actual frame length covered by the PLCP header for the call to
		 * wlc_mod_prb_rsp_rate_table() by subtracting the PLCP len and adding the FCS.
		 */
		len += (-D11_PHY_HDR_LEN + DOT11_FCS_LEN);
		wlc_mod_prb_rsp_rate_table(wlc, (uint16)len);

		if (suspend)
			wlc_enable_mac(wlc);
	} else {
		if (cfg->prb_resp == NULL) {
			cfg->prb_resp = MALLOC(wlc->pub.osh, BCN_TMPL_LEN);
			if (cfg->prb_resp == NULL)
				return;
		}

		/* Save the probe response starting at the 802.11 header, skipping the PLCP */
		cfg->prb_resp_len = len - D11_PHY_HDR_LEN;

		ASSERT(cfg->prb_resp_len < BCN_TMPL_LEN);
		bcopy((int8*)prb_resp + D11_PHY_HDR_LEN, cfg->prb_resp, cfg->prb_resp_len);
	}
}

uint8 *
wlc_write_info_elt(uint8 *cp, int id, int len, const void* data)
{
	ASSERT(len < 256);
	ASSERT(len >= 0);

	cp[0] = (uint8)id;
	cp[1] = (uint8)len;
	if (len > 0)
		bcopy(data, (char*)cp+2, len);

	return (cp + 2 + len);
}

static uint8 *
wlc_copy_info_elt(uint8 *cp, const void* data)
{
	uint totlen = 2 + ((const uint8*)data)[1];

	bcopy(data, (char*)cp, totlen);

	return (cp + totlen);
}

/* Beacon and Probe Response frame body
 *  max sz  order
 *  8       1 Timestamp
 *  2       2 Beacon interval
 *  2       3 Capability information
 *  34      4 SSID
 *  10      5 Supported rates
 *  (0)     6 FH Params             (not supported)
 *  3       7 DS Parameter Set
 *  (0)     8 CF Params             (not supported)
 *  4       9 IBSS Parameter Set    (IBSS only)
 *  6(256)  10 TIM                  (BSS-Beacon only)
 *  8(113)  11 Country Information  (11d)
 *  3       15 Channel Switch Announcement (11h)
 *  6       16 Quiet period	    (11h)
 *  3       19 ERP Information      (11g)
 *  10(257) 20 Ext. Sup. Rates      (11g)
 *  7          BRCM proprietary
 *  27         WME params
 *
 *  The TIM can be 256 bytes long in general. For a beacon template,
 *  a NULL TIM is used, which is 6 bytes long.
 *
 *  Country element is minimum of 8 which includes a base of 5 plus
 *  possibly 3 bytes per supported channel (less if the channel numbers
 *  are monotonic and can be compressed.
 *
 *  Max template beacon frame body length is 112 bytes. (items 1-7 and 10, 19, 20, 221)
 *  Max template probe response frame body length is 110 bytes. (items 1-7 and 9, 19, 20, 221)
 */

static void
wlc_bcn_prb_body(wlc_info_t *wlc, uint type, wlc_bsscfg_t *bsscfg, uint8 *bcn, int *len)
{
	struct dot11_bcn_prb *fixed;
	struct rateset sup_rates, ext_rates;
	uint8 *cp;
	int bcn_len;
	uint32 wsec;
	int8 WPA_auth;

	ASSERT(bsscfg != NULL);
	ASSERT(len != NULL);

	wsec = bsscfg->wsec;
	WPA_auth = bsscfg->WPA_auth;

	ASSERT(*len >= 112);
	ASSERT(ISALIGNED((uintptr)bcn, sizeof(uint16)));
	ASSERT(type == FC_BEACON || type == FC_PROBE_RESP);

	bzero(&sup_rates, sizeof(struct rateset));
	bzero(&ext_rates, sizeof(struct rateset));

	/* check for a supported rates override, add back iBSS Gmode setting */
	if (wlc->sup_rates_override.count > 0)
		wlc_rateset_copy(&wlc->sup_rates_override, &sup_rates);
	else if (wlc->band->gmode == GMODE_LRS)
		wlc_rateset_copy(cck_rates, &sup_rates);
	else if (wlc->band->gmode == GMODE_PERFORMANCE)
		wlc_default_rateset(wlc, &sup_rates);

	wlc_rateset_elts(wlc, wlc->pub.current_bss.rateset.count,
		wlc->pub.current_bss.rateset.rates, &sup_rates, &ext_rates);

	fixed = (struct dot11_bcn_prb *)bcn;

	/* fill in fixed info, Timestamp, Beacon interval, and Capability */
	bzero((char*)fixed, sizeof(*fixed));
	/* Timestamp is zero in bcn template, filled in by MAC */
	/* [N.B.: fixed->timestamp is misaligned but unreferenced, so "safe".] */

	fixed->beacon_interval = htol16((uint16)wlc->pub.current_bss.beacon_period);
	if (wlc->pub.current_bss.infra)
		fixed->capability = DOT11_CAP_ESS;
	else
		fixed->capability = DOT11_CAP_IBSS;

	if (WSEC_ENABLED(bsscfg->wsec) && bsscfg->wsec_restrict)
		fixed->capability |= DOT11_CAP_PRIVACY;

	/* Advertise short preamble capability if we have not been forced long */
	if (ISGPHY(wlc->band->pi) &&
	    (wlc->pub.current_bss.capability & DOT11_CAP_SHORT))
		fixed->capability |= DOT11_CAP_SHORT;
	if (wlc->band->gmode && wlc->shortslot)
		fixed->capability |= DOT11_CAP_SHORTSLOT;

	fixed->capability = htol16(fixed->capability);

	/* fill in Info elts, SSID, Sup Rates, DS params, and IBSS params */
	cp = (uint8 *)&fixed[1];

	cp = wlc_write_info_elt(cp, DOT11_MNG_SSID_ID, bsscfg->SSID_len, bsscfg->SSID);

	/* Supported Rates */
	cp = wlc_write_info_elt(cp, DOT11_MNG_RATES_ID, sup_rates.count, sup_rates.rates);

	/* The *other* 11a products do not send channel info elements */
	if (!ISAPHY(wlc->band->pi)) {
		/* DS Parameters */
		cp[0] = DOT11_MNG_DS_PARMS_ID;
		cp[1] = 1;
		cp[2] = (uint8) wlc->pub.current_bss.channel;
		cp += 3;
	}

	/* IBSS Params */
	if (!wlc->pub.current_bss.infra) {
		cp[0] = DOT11_MNG_IBSS_PARMS_ID;
		cp[1] = 2;
		cp[2] = (uint8)(wlc->pub.current_bss.atim_window & 0xff);
		cp[3] = (uint8)((wlc->pub.current_bss.atim_window >> 8) & 0xff);
		cp += 2 + DOT11_MNG_IBSS_PARAM_LEN;
	}

	if (wlc->band->gmode) {
		uint8 erp_flags[DOT11_MNG_ERP_LEN] = { 0 };

		if (wlc->erp_ie_nonerp)
			erp_flags[0] |= DOT11_MNG_NONERP_PRESENT;
		if (wlc->erp_ie_use_protection)
			erp_flags[0] |= DOT11_MNG_USE_PROTECTION;
		if (wlc->barker_preamble)
			erp_flags[0] |= DOT11_MNG_BARKER_PREAMBLE;

		/* ERP Information Element */
		cp = wlc_write_info_elt(cp, DOT11_MNG_ERP_ID, DOT11_MNG_ERP_LEN, erp_flags);

		/* Legacy NonERP Information Element if required */
		if (wlc->include_legacy_erp)
			cp = wlc_write_info_elt(cp, DOT11_MNG_NONERP_ID, 1, erp_flags);
	}

	if (wlc->band->gmode) {
		/* Extended Supported Rates Information Element if any */
		if (ext_rates.count > 0) 
			cp = wlc_write_info_elt(cp, DOT11_MNG_EXT_RATES_ID, ext_rates.count,
				ext_rates.rates);
	}

	/* BRCM proprietary Information Element */
	if (wlc->brcm_ie[TLV_LEN_OFF] > 0)
		cp = wlc_copy_info_elt(cp, wlc->brcm_ie);
#ifdef BCMWPA2
	/* RSN IE */
	if (INCLUDES_WPA2_AUTH(WPA_auth) && WSEC_ENABLED(wsec)) {
		/* Infrastructure WPA info element */
		uint WPA_len = 0;	/* tag length */
		bcm_tlv_t *wpa2ie = (bcm_tlv_t *)cp;
		wpa_suite_mcast_t *mcast;
		wpa_suite_ucast_t *ucast;
		wpa_suite_auth_key_mgmt_t *auth;
		uint16 count;
		uint8 *cap;

		WL_WSEC(("wl%d: adding RSN IE, wsec = 0x%x\n", wlc->pub.unit, wsec));

		/* fixed portion */
		wpa2ie->id = DOT11_MNG_RSN_ID;
		wpa2ie->data[0] = (uint8)WPA2_VERSION;
		wpa2ie->data[1] = (uint8)(WPA2_VERSION>>8);
		WPA_len = WPA2_VERSION_LEN;

		/* multicast suite */
		mcast = (wpa_suite_mcast_t *)&wpa2ie->data[WPA2_VERSION_LEN];
		bcopy(WPA2_OUI, mcast->oui, DOT11_OUI_LEN);
		mcast->type = wlc_wpa_mcast_cipher(wlc, bsscfg);
		WPA_len += WPA_SUITE_LEN;

		/* unicast suite list */
		ucast = (wpa_suite_ucast_t *)&mcast[1];
		count = 0;
		WPA_len += WPA_IE_SUITE_COUNT_LEN;
		if (WSEC_AES_ENABLED(wsec)) {
			bcopy(WPA2_OUI, ucast->list[count].oui, DOT11_OUI_LEN);
			ucast->list[count++].type = WPA_CIPHER_AES_CCM;
			WPA_len += WPA_SUITE_LEN;
		}
		if (WSEC_TKIP_ENABLED(wsec)) {
			bcopy(WPA2_OUI, ucast->list[count].oui, DOT11_OUI_LEN);
			ucast->list[count++].type = WPA_CIPHER_TKIP;
			WPA_len += WPA_SUITE_LEN;
		}
		ASSERT(count);
		ucast->count.low = (uint8)count;
		ucast->count.high = (uint8)(count>>8);

		/* authenticated key management suite list */
		auth = (wpa_suite_auth_key_mgmt_t *)&ucast->list[count];
		count = 0;
		WPA_len += WPA_IE_SUITE_COUNT_LEN;
		if (WPA_auth & WPA2_AUTH_UNSPECIFIED) {
			bcopy(WPA2_OUI, auth->list[count].oui, DOT11_OUI_LEN);
			auth->list[count++].type = RSN_AKM_UNSPECIFIED;
			WPA_len += WPA_SUITE_LEN;
		}
		if (WPA_auth & WPA2_AUTH_PSK) {
			bcopy(WPA2_OUI, auth->list[count].oui, DOT11_OUI_LEN);
			auth->list[count++].type = RSN_AKM_PSK;
			WPA_len += WPA_SUITE_LEN;
		}
		ASSERT(count);
		auth->count.low = (uint8)count;
		auth->count.high = (uint8)(count>>8);

		/* WPA capabilities */
		cap = (uint8 *)&auth->list[count];
		wlc_wpa_cap(wlc, bsscfg, cap, WPA_CAP_LEN);
		WPA_len += WPA_CAP_LEN;

		/* update tag length */
		wpa2ie->len = (uint8)WPA_len;

		if (WPA_len)
			cp += TLV_HDR_LEN + WPA_len;
	}
#endif /* BCMWPA2 */

	/* WPA Information Element */
	if (INCLUDES_WPA_AUTH(WPA_auth) && WSEC_ENABLED(wsec)) {
		uint WPA_len = 0;	/* tag length */

		WL_WSEC(("wl%d: adding WPA IE, wsec = 0x%x\n", wlc->pub.unit, wsec));

		/* IBSS WPA info element */
		if (!wlc->pub.current_bss.infra) {
			/* copy WPA info element template */
			WPA_len = WPA_info_element[1];
			bcopy(WPA_info_element, cp, TLV_HDR_LEN + WPA_len);

			/* select multicast cipher */
			/* - use highest encryption status enabled */
			if (WSEC_AES_ENABLED(wsec))
				cp[11] = WPA_CIPHER_AES_CCM;
			else if (WSEC_TKIP_ENABLED(wsec))
				cp[11] = WPA_CIPHER_TKIP;
			else {
				/* - w/ v.22 client, IE must specify WEP key size
				 *   which isn't available from config, so use the key...
				 */
				wsec_key_t *key = WSEC_BSS_DEFAULT_KEY(bsscfg);

				if (key != NULL) {
					if (key->algo == CRYPTO_ALGO_WEP1) {
						cp[11] = WPA_CIPHER_WEP_40;
					} else if (key->algo == CRYPTO_ALGO_WEP128) {
						cp[11] = WPA_CIPHER_WEP_104;
					} else {
						WL_ERROR(("wl%d: wlc_bcn_prb_body: key->algo %d"
							" doesn't match wsec 0x%x\n", wlc->pub.unit,
							key->algo, wsec));
						cp[11] = WPA_CIPHER_WEP_40;
					}
				} else {
					cp[11] = WPA_CIPHER_WEP_40;
				}
			}

			/* set unicast cipher and authenticated key management */
			cp[17] = WPA_CIPHER_NONE;
			cp[23] = RSN_AKM_NONE;

			/* WPA capabilities */
			wlc_wpa_cap(wlc, bsscfg, &cp[24], WPA_CAP_LEN);
		} else {
			wpa_ie_fixed_t *wpaie = (wpa_ie_fixed_t *)cp;
			wpa_suite_mcast_t *mcast;
			wpa_suite_ucast_t *ucast;
			wpa_suite_auth_key_mgmt_t *auth;
			uint16 count;
			uint8 *cap;

			/* fixed portion */
			wpaie->tag = DOT11_MNG_WPA_ID;
			bcopy(WPA_OUI, wpaie->oui, sizeof(wpaie->oui));
			wpaie->oui_type = 1;
			wpaie->version.low = (uint8)WPA_VERSION;
			wpaie->version.high = (uint8)(WPA_VERSION>>8);
			WPA_len = WPA_IE_TAG_FIXED_LEN;

			/* multicast suite */
			mcast = (wpa_suite_mcast_t *)&wpaie[1];
			bcopy(WPA_OUI, mcast->oui, DOT11_OUI_LEN);
			mcast->type = wlc_wpa_mcast_cipher(wlc, bsscfg);
			WPA_len += WPA_SUITE_LEN;

			/* unicast suite list */
			ucast = (wpa_suite_ucast_t *)&mcast[1];
			count = 0;
			WPA_len += WPA_IE_SUITE_COUNT_LEN;
			if (WSEC_AES_ENABLED(wsec)) {
				bcopy(WPA_OUI, ucast->list[count].oui, DOT11_OUI_LEN);
				ucast->list[count++].type = WPA_CIPHER_AES_CCM;
				WPA_len += WPA_SUITE_LEN;
			}
			if (WSEC_TKIP_ENABLED(wsec)) {
				bcopy(WPA_OUI, ucast->list[count].oui, DOT11_OUI_LEN);
				ucast->list[count++].type = WPA_CIPHER_TKIP;
				WPA_len += WPA_SUITE_LEN;
			}
			ASSERT(count);
			ucast->count.low = (uint8)count;
			ucast->count.high = (uint8)(count>>8);

			/* authenticated key management suite list */
			auth = (wpa_suite_auth_key_mgmt_t *)&ucast->list[count];
			count = 0;
			WPA_len += WPA_IE_SUITE_COUNT_LEN;
			ASSERT(!(WPA_auth & WPA_AUTH_NONE));
			if (WPA_auth & WPA_AUTH_UNSPECIFIED) {
				bcopy(WPA_OUI, auth->list[count].oui, DOT11_OUI_LEN);
				auth->list[count++].type = RSN_AKM_UNSPECIFIED;
				WPA_len += WPA_SUITE_LEN;
			}
			if (WPA_auth & WPA_AUTH_PSK) {
				bcopy(WPA_OUI, auth->list[count].oui, DOT11_OUI_LEN);
				auth->list[count++].type = RSN_AKM_PSK;
				WPA_len += WPA_SUITE_LEN;
			}
			ASSERT(count);
			auth->count.low = (uint8)count;
			auth->count.high = (uint8)(count>>8);

			/* WPA capabilities */
			cap = (uint8 *)&auth->list[count];
			wlc_wpa_cap(wlc, bsscfg, cap, WPA_CAP_LEN);
			WPA_len += WPA_CAP_LEN;

			/* update tag length */
			wpaie->length = (uint8)WPA_len;
		}

		if (WPA_len)
			cp += TLV_HDR_LEN + WPA_len;
	}

	bcn_len = (int)(cp - bcn);

	ASSERT(bcn_len <= *len);
	*len = bcn_len;

	return;
}

static uchar
wlc_assocscb_getcnt(wlc_info_t *wlc)
{
	struct scb *scb;
	uchar assoc = 0;
	struct scb_iter scbiter;
	ASSERT(MAXSCB <= 255);

	FOREACHSCB(&scbiter, wlc->scbstate->scb, scb) {
		if (SCB_ASSOCIATED(scb))
			assoc++;
	}
	return assoc;
}

/* return TRUE if wlc->brcm_ie[] changed, FALSE if not */
bool
wlc_update_brcm_ie(wlc_info_t *wlc)
{
	uchar assoc;
	brcm_ie_t *brcm_ie;
	volatile uchar flags = 0, flags1 = 0;


	brcm_ie = (brcm_ie_t *)(&wlc->brcm_ie[0]);

	/* count the total number of associated stas */
	assoc = wlc_assocscb_getcnt(wlc);

	/* no change? */
	if ((brcm_ie->assoc == assoc) && (brcm_ie->flags == flags) && (brcm_ie->flags1 == flags1))
		return (FALSE);

	brcm_ie->assoc = assoc;
	brcm_ie->flags = flags;
	brcm_ie->flags1 = flags1;

	ASSERT((wlc->brcm_ie[TLV_LEN_OFF]+TLV_HDR_LEN) < WLC_MAX_BRCM_ELT);

	return (TRUE);
}

/* process received BRCM info element */
static void
wlc_process_brcm_ie(wlc_info_t *wlc, struct scb *scb, brcm_ie_t *brcm_ie)
{

	if (!scb || !brcm_ie)
		return;

	/* remote station is BRCM AP/STA */
	scb->flags |= SCB_BRCM;

	/* legacy AES implementations */
	if (brcm_ie->ver <= BRCM_IE_LEGACY_AES_VER)
		scb->flags |= SCB_LEGACY_AES;
	else
		scb->flags &= ~SCB_LEGACY_AES;

	return;
}

void
wlc_read_tsf(wlc_info_t* wlc, uint32* tsf_l_ptr, uint32* tsf_h_ptr)
{
	d11regs_t *regs = wlc->regs;

	/* read the tsf timer low, then high to get an atomic read */
	*tsf_l_ptr = R_REG(&regs->tsf_timerlow);
	*tsf_h_ptr = R_REG(&regs->tsf_timerhigh);

	return;
}


static void*
wlc_frame_get_ps_ctl(wlc_info_t *wlc, const struct ether_addr *bssid)
{
	void *p;
	osl_t *osh;
	struct dot11_ps_poll_frame *hdr;

	osh = wlc->pub.osh;

	if ((p = wlc_frame_get_ctl(wlc, DOT11_PS_POLL_LEN)) == NULL) {
		return (NULL);
	}

	/* construct a PS-Poll frame */
	hdr = (struct dot11_ps_poll_frame *)PKTDATA(osh, p);
	hdr->fc = htol16(FC_PS_POLL);
	hdr->durid = htol16(wlc->AID);
	bcopy((char*)bssid, (char*)&hdr->bssid, ETHER_ADDR_LEN);
	bcopy((char*)&wlc->pub.cur_etheraddr, (char*)&hdr->ta, ETHER_ADDR_LEN);

	return (p);
}

void*
wlc_frame_get_ctl(wlc_info_t *wlc, uint len)
{
	void *p;
	osl_t *osh;

	ASSERT(len != 0);

	osh = wlc->pub.osh;
	if ((p = PKTGET(osh, (TXOFF + len), TRUE)) == NULL) {
		WL_ERROR(("wl%d: wlc_frame_get_ctl: pktget error for len %d\n",
			wlc->pub.unit, (TXOFF + len)));
		WLCNTINCR(wlc->pub._cnt.txnobuf);
		return (NULL);
	}
	ASSERT(ISALIGNED((uintptr)PKTDATA(osh, p), sizeof(uint32)));

	/* reserve TXOFF bytes of headroom */
	PKTPULL(osh, p, TXOFF);
	PKTSETLEN(osh, p, len);

	PKTSETPRIO(p, 0);

	return (p);
}

void*
wlc_frame_get_mgmt(wlc_info_t *wlc, uint16 fc, const struct ether_addr *da,
	const struct ether_addr *bssid, uint body_len, uint8 **pbody)
{
	uint len;
	void *p;
	osl_t *osh;
	struct dot11_management_header *hdr;

	osh = wlc->pub.osh;

	len = DOT11_MGMT_HDR_LEN + body_len;

	if ((p = PKTGET(osh, (TXOFF + len), TRUE)) == NULL) {
		WL_ERROR(("wl%d: wlc_frame_get_mgmt: pktget error for len %d fc %x\n",
			wlc->pub.unit, (TXOFF + len), fc));
		WLCNTINCR(wlc->pub._cnt.txnobuf);
		return (NULL);
	}
	ASSERT(ISALIGNED((uintptr)PKTDATA(osh, p), sizeof(uint32)));

	/* reserve TXOFF bytes of headroom */
	PKTPULL(osh, p, TXOFF);
	PKTSETLEN(osh, p, len);

	/* construct a management frame */
	hdr = (struct dot11_management_header *)PKTDATA(osh, p);
	hdr->fc = htol16(fc);
	hdr->durid = 0;
	bcopy((char*)da, (char*)&hdr->da, ETHER_ADDR_LEN);
	bcopy((char*)&wlc->pub.cur_etheraddr, (char*)&hdr->sa, ETHER_ADDR_LEN);
	bcopy((char*)bssid, (char*)&hdr->bssid, ETHER_ADDR_LEN);
	hdr->seq = 0;

	*pbody = (uint8*)&hdr[1];

	/* Set MAX Prio for MGMT packets */
	PKTSETPRIO(p, MAXPRIO);

	return (p);
}

bool
wlc_sendmgmt(wlc_info_t *wlc, void *p, struct scb *scb, int short_preamble, wsec_key_t *key,
	ratespec_t rate_override)
{
	int prio;
	uint fifo = TX_CTL_FIFO;

	ASSERT(wlc->pub.up);

	/* use hw rateset scb if the one passed in is NULL */
	if (scb == NULL)
		scb = wlc->band->hwrs_scb;

	prio = PKTPRIO(p);
	ASSERT(prio <= MAXPRIO);

	/* In WME mode, send mgmt frames based on priority setting */
	if (SCB_WME(scb))
		fifo = wme_prio2fifo[prio];

	/* add headers */
	wlc_d11hdrs(wlc, p, scb, short_preamble, 0, 1, fifo, 0, key, rate_override);
	wlc->counter++;

	WLPKTTAGSCBSET(p, scb);


	if (wlc_prec_enq(wlc, &wlc->txq, p, WLC_PRIO_TO_HI_PREC(prio))) {
		wlc_send_q(wlc, &wlc->txq);
		return TRUE;
	}

	wlc_pkt_flush(wlc, p, TX_STATUS_NO_ACK);
	WLCNTINCR(wlc->pub._cnt.txnobuf);
	return FALSE;
}

bool
wlc_sendctl(wlc_info_t *wlc, void *p, struct scb *scb, uint fifo,
	ratespec_t rate_override, bool enq_only)
{
	int prio;

	ASSERT(wlc->pub.up);

	/* SCB must not be NULL */
	ASSERT(scb);

	prio = PKTPRIO(p);
	ASSERT(prio <= MAXPRIO);

	/* WME: send frames based on priority setting */
	if (SCB_WME(scb))
		fifo = wme_prio2fifo[prio];

	/* add headers */
	wlc_d11hdrs(wlc, p, scb, 0, 0, 1, fifo, 0, NULL, rate_override);

	wlc->counter++;

	WLPKTTAGSCBSET(p, scb);


	if (wlc_prec_enq(wlc, &wlc->txq, p, WLC_PRIO_TO_HI_PREC(prio))) {
		if (!enq_only)
			wlc_send_q(wlc, &wlc->txq);
		return TRUE;
	}

	wlc_pkt_flush(wlc, p, TX_STATUS_NO_ACK);
	WLCNTINCR(wlc->pub._cnt.txnobuf);
	return FALSE;
}

/* prepares pdu for transmission. returns BCM error codes */
int
wlc_prep_pdu(wlc_info_t *wlc, void *pdu, uint *fifop)
{
	uint nbytes;
	osl_t *osh;
	uint fifo;
	d11txh_t *txh;
	struct dot11_header *h;
	struct scb *scb;
	uint16 fc;

	/* Make sure that it's a PDU */
	ASSERT(WLPKTTAG(pdu)->flags & WLF_MPDU);

	osh = wlc->pub.osh;

	txh = (d11txh_t *)PKTDATA(osh, pdu);
	h = (struct dot11_header *)((uint8 *)(txh + 1) + D11_PHY_HDR_LEN);
	fc = ltoh16(h->fc);

	/* get the pkt queue info. This was put at wlc_sendctl or wlc_send for PDU */
	fifo = ltoh16(txh->TxFrameID) & TXFID_QUEUE_MASK;

	scb = WLPKTTAGSCBGET(pdu);


	*fifop = fifo;

#if defined(WME)
	/* If sending APSD trigger frame, stay awake until EOSP */
	if (WME_ENAB(wlc) && wlc->PMenabled &&
	    !wlc->apsd_sta_usp && (fifo <= TX_AC_VO_FIFO) &&
	    AC_BITMAP_TST(scb->apsd.ac_trig, wme_fifo2ac[fifo]) &&
	    FC_TYPE(fc) == FC_TYPE_DATA && FC_SUBTYPE_ANY_QOS(FC_SUBTYPE(fc))) {
		WL_PS(("wl%d: APSD ctl wake\n", wlc->pub.unit));
		wlc->apsd_sta_usp = TRUE;
		wlc_set_ps_ctrl(wlc);
	}
#endif /* defined(STA) && defined(WME) */

	if (!PIO_ENAB(wlc)) {
		/* return if insufficient dma resources */
		if (*wlc->core->txavail[fifo] < 3) {
			/* Mark precedences related to this FIFO, unsendable */
			WLC_TX_FIFO_CLEAR(wlc, fifo);
			return BCME_BUSY;
		}
	} else {
		nbytes = pkttotlen(osh, pdu);

		/* return if insufficient pio resources */
		if (!wlc_pio_txavailable(wlc->core->pio[fifo], nbytes, 1)) {
			/* Mark precedences related to this FIFO, unsendable */
			WLC_TX_FIFO_CLEAR(wlc, fifo);
			return BCME_BUSY;
		}
	}

	if ((ltoh16(txh->MacFrameControl) & FC_KIND_MASK) != FC_DATA)
		WLCNTINCR(wlc->pub._cnt.txctl);

	return 0;
}


void
wlc_sendprobe(wlc_info_t *wlc, uchar ssid[], int ssid_len)
{
	void *p;
	uint8 *pbody;
	const struct rateset *rs;
	struct rateset sup_rates, ext_rates;
	int body_len;

	bzero(&sup_rates, sizeof(struct rateset));
	bzero(&ext_rates, sizeof(struct rateset));
	if ((ISGPHY(wlc->band->pi)) && (wlc->band->gmode == GMODE_LEGACY_B))
		rs = cck_rates;	/* only advertise 11b rates */
	else
		rs = &wlc->band->hw_rateset;

	wlc_rateset_elts(wlc, rs->count, rs->rates, &sup_rates, &ext_rates);

	/*
	 * get a packet - probe req has the following contents:
	 * InfoElt   SSID
	 * InfoElt   Supported rates
	 * InfoElt   Extended Supported rates (11g)
	 * InfoElt   BRCM proprietary
	 */

	body_len = TLV_HDR_LEN + ssid_len;
	body_len += TLV_HDR_LEN + sup_rates.count;
	if (ext_rates.count > 0)
		body_len += TLV_HDR_LEN + ext_rates.count;
	if (wlc->brcm_ie[TLV_LEN_OFF] > 0)
		body_len += TLV_HDR_LEN + wlc->brcm_ie[TLV_LEN_OFF];

	if ((p = wlc_frame_get_mgmt(wlc, FC_PROBE_REQ, &ether_bcast, &ether_bcast, body_len,
		&pbody)) == NULL)
		return;

	/* fill out the probe request body */

	/* SSID */
	pbody = wlc_write_info_elt(pbody, DOT11_MNG_SSID_ID, ssid_len, ssid);

	/* Supported Rates */
	pbody = wlc_write_info_elt(pbody, DOT11_MNG_RATES_ID, sup_rates.count, sup_rates.rates);

	/* Extended Supported Rates */
	if (ext_rates.count > 0)
		pbody = wlc_write_info_elt(pbody, DOT11_MNG_EXT_RATES_ID, ext_rates.count,
			ext_rates.rates);

	/* BRCM proprietary elt */
	if (wlc->brcm_ie[TLV_LEN_OFF] > 0)
		pbody = wlc_copy_info_elt(pbody, wlc->brcm_ie);


	wlc_sendmgmt(wlc, p, NULL, 0, NULL, 0);
}

static void *
wlc_sendauth(
	wlc_info_t *wlc,
	struct ether_addr *ea,
	struct ether_addr *bssid,
	struct scb *scb,
	int auth_alg,
	int auth_seq,
	int auth_status,
	wsec_key_t *scb_key,
	uint8 *challenge_text,
	int short_preamble
)
{
	void *p;
	uint8 *pbody;
	struct dot11_auth *auth;
	uint16 fc;
	int body_len;
	wsec_key_t *key = NULL;

	/* All Authentication frames have a common 6 byte contents:
	 * 2 bytes Authentication Alg Number
	 * 2 bytes Authentication Transaction Number
	 * 2 bytes Status Code
	 */
	body_len = 6;

	/* Authentications frames 2 and 3 for Shared Key auth have a 128 byte
	 * Challenge Text Info element
	 */
	if ((auth_alg == DOT11_SHARED_KEY) && (auth_status == DOT11_STATUS_SUCCESS) &&
		((auth_seq == 2) || (auth_seq == 3))) {
		ASSERT(challenge_text != NULL);
		body_len += (2 + challenge_text[1]);

		/* frame 3 is also WEP encrypted */
		if (auth_seq == 3) {
			if (scb_key)
				key = scb_key;
			else
				key = WSEC_BSS_DEFAULT_KEY(scb->bsscfg);

			if (key == NULL) {
				WL_ERROR(("wl%d: transmit key absent\n", wlc->pub.unit));
				wlc_auth_complete(wlc, TX_STATUS_NO_ACK, NULL);
				return NULL;
			}
		}
	} else {
		ASSERT(challenge_text == NULL);
	}

	/* get a packet */
	if (key) {
		fc = (FC_AUTH | FC_WEP);
		body_len += key->iv_len;
	} else
		fc = FC_AUTH;


	if (wlc->brcm_ie[TLV_LEN_OFF] > 0)
		body_len += TLV_HDR_LEN + wlc->brcm_ie[TLV_LEN_OFF];

	if ((p = wlc_frame_get_mgmt(wlc, fc, ea, bssid, body_len, &pbody)) == NULL) {
		wlc_auth_complete(wlc, TX_STATUS_NO_ACK, NULL);
		return NULL;
	}

	if (key) {
		wlc_key_iv_update(wlc, scb->bsscfg, key, pbody, 0);
		auth = (struct dot11_auth *)(pbody + key->iv_len);
	} else {
		auth = (struct dot11_auth *)pbody;
	}

	auth->alg = htol16((uint16)auth_alg);
	auth->seq = htol16((uint16)auth_seq);
	auth->status = htol16((uint16)auth_status);

	pbody = (void*)(&auth[1]);

	if (challenge_text) {
		bcopy((char*)challenge_text, pbody, 2 + challenge_text[1]);
		pbody = (void*)(pbody + 2 + challenge_text[1]);
	}


	/* BRCM proprietary elt */
	if (wlc->brcm_ie[TLV_LEN_OFF] > 0)
		pbody = wlc_copy_info_elt(pbody, wlc->brcm_ie);

	if (wlc_sendmgmt(wlc, p, scb, short_preamble, key, 0))
		return p;

	return NULL;
}

static void *
wlc_senddisassoc(wlc_info_t *wlc, const struct ether_addr *da, const struct ether_addr *bssid,
	struct scb *scb, uint16 reason_code)
{
	void *p = NULL;
	uint8 *pbody;
	uint16 *reason;

	if ((wlc->cfg.WPA_auth != WPA_AUTH_DISABLED) &&
	    WSEC_ENABLED(wlc->cfg.wsec)) {
		/* Signal WPA process. */
		wlc_mac_event(wlc, WLC_E_DISASSOC_IND, da, WLC_E_STATUS_SUCCESS, reason_code, 0, 0,
			0);
	}
	/*
	 * get a packet - disassoc pkt has the following contents:
	 * 2 bytes Reason Code
	 */

	if ((p = wlc_frame_get_mgmt(wlc, FC_DISASSOC, da, bssid, 2, &pbody)) == NULL)
		return p;

	/* fill out the disassociation reason code */
	reason = (uint16 *) pbody;
	reason[0] = htol16(reason_code);

	if (wlc_sendmgmt(wlc, p, scb, 0, NULL, 0))
		return p;

	return NULL;
}

void *
wlc_sendnulldata(wlc_info_t *wlc, struct ether_addr *da, ratespec_t rate_override,
                 uint32 pktflags, int prio)
{
	void *p;
	uint8 *pbody;
	uint16 fc;
	int body_len;
	struct ether_addr *bssid;
	struct scb *scb;

	if (!(scb = wlc_scbfind(wlc, da)))
		return NULL;

	if (SCB_WME(scb)) {
		fc = FC_QOS_NULL;
		body_len = 2;
	} else {
		fc = FC_NULL_DATA;
		body_len = 0;
	}


	if (wlc->pub.BSS)
		fc |= FC_TODS;
	bssid = &wlc->BSSID;

	if ((p = wlc_frame_get_mgmt(wlc, fc, da, bssid, body_len, &pbody)) == NULL)
		return p;

	WLPKTTAG(p)->flags |= pktflags;

	PKTSETPRIO(p, prio);

	if (SCB_WME(scb)) {
		uint16 *pqos;
		/* Initialize the QoS Control field that comes after all addresses. */
		pqos = (uint16 *)((uint8 *)PKTDATA(wlc->pub.osh, p) +
		                  DOT11_MGMT_HDR_LEN + body_len - DOT11_QOS_LEN);
		ASSERT(ISALIGNED((uintptr)pqos, sizeof(*pqos)));
		*pqos = htol16(((prio << QOS_PRIO_SHIFT) & QOS_PRIO_MASK) |
		               ((wlc->wme_noack << QOS_ACK_SHIFT) & QOS_ACK_MASK));
	}

	/* use sendctl to send packet */
	if (wlc_sendctl(wlc, p, scb, TX_DATA_FIFO, rate_override, FALSE))
		return p;

	return NULL;
}

void
wlc_rateprobe(wlc_info_t *wlc, struct ether_addr *ea, struct scb *scb, ratespec_t rate_override)
{
	void *pkt;

	pkt = wlc_sendnulldata(wlc, ea, rate_override, 0, PRIO_8021D_BE);

	if ((pkt == NULL) || wlc_pkt_callback_register(wlc, wlc_rateprobe_complete, (void*)ea,
	                                               pkt))
		WL_ERROR(("wl%d: wlc_rateprobe: could not register callback\n", wlc->pub.unit));
}

static void
wlc_rateprobe_complete(wlc_info_t *wlc, uint txstatus, void *ea_arg)
{
	struct scb *scb;
	struct ether_addr *ea = (struct ether_addr *)ea_arg;

	/* no ack */
	if (!(txstatus & TX_STATUS_ACK_RCV))
		return;

	/* no scb */
	if ((scb = wlc_scbfind(wlc, ea)) == NULL)
		return;

	/* upgrade the scb rateset to all supported network rates */
	wlc_rateset_filter((wlc->pub.associated ? &wlc->pub.current_bss.rateset :
		&wlc->target_bss.rateset),
		&scb->rateset, FALSE, FALSE, RATE_MASK);

	wlc_scb_rate_init(wlc, scb);
}

static void
wlc_rateprobe_scan(wlc_info_t *wlc)
{
	struct scb *scb;
	struct scb_iter scbiter;
	ASSERT(!wlc->pub.BSS && wlc->band->gmode);

	/*
	 * Send ofdm rate probe to any station
	 * which does not yet have an ofdm rate in its scb rateset
	 * and has been active within the past 60 seconds.
	 */

	FOREACHSCB(&scbiter, wlc->scbstate->scb, scb)
		if (!IS_OFDM(scb->rateset.rates[scb->rateset.count-1]) &&
			((wlc->pub.now - scb->used) < 60))
			wlc_rateprobe(wlc, &scb->ea, scb, WLC_RATEPROBE_RATE);
}

static bool
wlc_sendpspoll(wlc_info_t *wlc)
{
	void *p;
	struct scb *scb;

	if (!(scb = wlc_scbfind(wlc, &wlc->BSSID))) {
		WL_ERROR(("wl%d: wlc_sendpspoll: wlc_scbfind failed\n", wlc->pub.unit));
		return FALSE;
	}

	if ((p = wlc_frame_get_ps_ctl(wlc, &wlc->BSSID)) == NULL)
		return FALSE;

	return (wlc_sendctl(wlc, p, scb, TX_CTL_FIFO, 0, FALSE));
}

void *
wlc_senddeauth(wlc_info_t *wlc, struct ether_addr *da, struct ether_addr *bssid, struct scb *scb,
	uint16 reason_code)

{
	void *p;
	uint8 *pbody;
	uint16 *reason;

	/*
	 * get a packet - deauth pkt has the following contents:
	 * 2 bytes Reason Code
	 */
	if ((p = wlc_frame_get_mgmt(wlc, FC_DEAUTH, da, bssid, 2, &pbody)) == NULL)
		return NULL;

	/* fill out the deauthentication reason code */
	reason = (uint16 *)pbody;
	reason[0] = htol16(reason_code);

	if (wlc_sendmgmt(wlc, p, scb, 0, NULL, 0))
		return p;

	return NULL;
}

/* Look for vendor-specific IE with specified OUI and optional type */
bcm_tlv_t *
wlc_find_vendor_ie(uint8 *tlvs, int tlvs_len, uint8 *voui, uint8 *type, int type_len)
{
	bcm_tlv_t *ie = NULL;
	uint8 ie_len;

	if (tlvs == NULL)
		return NULL;

	while (tlvs_len >= TLV_HDR_LEN) {
		ie = (bcm_tlv_t*)tlvs;
		ie_len = ie->len;
		if ((ie->id == DOT11_MNG_PROPR_ID) &&
			(ie_len >= (DOT11_OUI_LEN + type_len)) &&
			!bcmp(ie->data, voui, DOT11_OUI_LEN))
		{
			if (type_len) {		/* compare optional type */
				if (!bcmp(&ie->data[DOT11_OUI_LEN], type, type_len))
					return (ie);		/* a match */
			}
			else
				return (ie);		/* a match */
		}

		/* No match with OUI, so move on to next IE */
		tlvs += ie_len + TLV_HDR_LEN;
		tlvs_len -= ie_len + TLV_HDR_LEN;
	}
	return NULL;
}

void
wlc_sendup_event(wlc_info_t *wlc, int bssid, const struct ether_addr *ea,
	const wlc_event_t *e, uint8 *data, uint32 len)
{
	struct wl_if *wlif = NULL;
	void *p, *databuf;
	bcm_event_t *msg;
	char *ptr;

	WL_ASSOC(("wlc_sendup_event START\n"));

	if ((p = PKTGET(wlc->pub.osh, RXBUFSZ, FALSE)) == NULL) {
		WL_ERROR(("wl%d: wlc_sendup_event: failed to get a pkt\n", wlc->pub.unit));
		return;
	}

	ASSERT(ISALIGNED((uintptr)PKTDATA(wlc->pub.osh, p), sizeof(uint32)));

	/* ensure we start on an odd 16 bit offset */
	PKTPULL(wlc->pub.osh, p, 2);
	msg = (bcm_event_t *) PKTDATA(wlc->pub.osh, p);

	bcopy(&wlc->pub.cur_etheraddr, (char *)&msg->eth.ether_dhost, ETHER_ADDR_LEN);
	bcopy(ea, (char *)&msg->eth.ether_shost, ETHER_ADDR_LEN);

	/* Set the locally administered bit on the source mac address if both
	 * SRC and DST mac addresses are the same. This prevents the downstream
	 * bridge from dropping the packet
	*/
	if (!bcmp(&msg->eth.ether_shost, &msg->eth.ether_dhost, ETHER_ADDR_LEN))
		ETHER_SET_LOCALADDR(&msg->eth.ether_shost);

	msg->eth.ether_type = hton16(ETHER_TYPE_BRCM);

	/* BCM Vendor specific header... */
	msg->bcm_hdr.subtype = hton16(BCMILCP_SUBTYPE_VENDOR_LONG);
	msg->bcm_hdr.version = BCMILCP_BCM_SUBTYPEHDR_VERSION;
	bcopy(BRCM_OUI, &msg->bcm_hdr.oui[0], DOT11_OUI_LEN);
	/* vendor spec header length + pvt data length (private indication hdr + actual message
	 * itself)
	 */
	msg->bcm_hdr.length = hton16(BCMILCP_BCM_SUBTYPEHDR_MINLENGTH + BCM_MSG_LEN + (uint16)len);
	msg->bcm_hdr.usr_subtype = hton16(BCMILCP_BCM_SUBTYPE_EVENT);

	PKTSETLEN(wlc->pub.osh, p, (sizeof(bcm_event_t) + len + 2));

	/* update the event struct */
	/* translate the wlc event into bcm event msg */
	msg->event.version = hton16(BCM_EVENT_MSG_VERSION);
	msg->event.event_type = hton32(e->event.event_type);
	msg->event.status = hton32(e->event.status);
	msg->event.reason = hton32(e->event.reason);
	msg->event.auth_type = hton32(e->event.auth_type);
	msg->event.datalen = hton32(e->event.datalen);
	msg->event.flags = hton16(e->event.flags);
	msg->event.ifname[0] = '\0';
	if (e->addr)
		bcopy(e->event.addr.octet, msg->event.addr.octet, ETHER_ADDR_LEN);

	/* fixup lengths */
	msg->bcm_hdr.length  = hton16(sizeof(wl_event_msg_t) + ntoh16(msg->bcm_hdr.length));
	PKTSETLEN(wlc->pub.osh, p, (sizeof(bcm_event_t) + len + 2));

	databuf = (char *)(msg + 1);
	if (len)
		bcopy(data, databuf, len);

	ptr = (char *)databuf;
	/* Last 2 bytes of the message are 0x00 0x00 to signal that there are no ethertypes which
	 * are following this
	 */
	ptr[len+0] = 0x00;
	ptr[len+1] = 0x00;


	if (isset(wlc->eventing, e->event.event_type) && wlc->callback.fn)
		wlc->callback.fn(wlc->callback.context, msg);

	if (isset(wlc->event_msgs, e->event.event_type))
		wl_sendup(wlc->wl, wlif, p);
	else
		PKTFREE(wlc->pub.osh, p, FALSE);
}

static void
wlc_rsn_build_ie(wlc_info_t *wlc, struct rsn_parms *rsn, void *pmcast)
{
	wlc_bsscfg_t* cfg = &wlc->cfg;
	wpa_suite_mcast_t *mcast;
	wpa_suite_ucast_t *ucast;
	wpa_suite_auth_key_mgmt_t *mgmt;
	uint8 *cap;

	/* copy multicast cipher */
	mcast = (wpa_suite_mcast_t *)pmcast;
	mcast->type = (uint8)rsn->multicast;

	/* select unicast cipher */
	ucast = (wpa_suite_ucast_t *)&mcast[1];
	ucast->list[0].type = WPA_CIPHER_NONE;
	if (WSEC_AES_ENABLED(cfg->wsec) && !UCAST_NONE(rsn))
		ucast->list[0].type = WPA_CIPHER_AES_CCM;
	else if (!WSEC_AES_ENABLED(cfg->wsec) &&
		WSEC_TKIP_ENABLED(cfg->wsec) && !UCAST_NONE(rsn))
		ucast->list[0].type = WPA_CIPHER_TKIP;

	/* copy authenticated key management */
	mgmt = (wpa_suite_auth_key_mgmt_t *)&ucast->list[1];
	if (cfg->WPA_auth == WPA_AUTH_UNSPECIFIED)
		mgmt->list[0].type = RSN_AKM_UNSPECIFIED;
	else if (cfg->WPA_auth == WPA_AUTH_PSK)
		mgmt->list[0].type = RSN_AKM_PSK;
#ifdef BCMWPA2
	else if (cfg->WPA_auth == WPA2_AUTH_UNSPECIFIED)
		mgmt->list[0].type = RSN_AKM_UNSPECIFIED;
	else if (cfg->WPA_auth == WPA2_AUTH_PSK)
		mgmt->list[0].type = RSN_AKM_PSK;
#endif /* BCMWPA2 */

	/* WPA capabilities */
	cap = (uint8 *)&mgmt->list[1];
	wlc_wpa_cap(wlc, cfg, cap, WPA_CAP_LEN);

}

static void *
wlc_sendassocreq(wlc_info_t *wlc, wlc_bss_info_t *bi, struct scb *scb, bool reassoc)
{
	uint len, assoc_req_len, ssid_len, WPA_len, parse_len;
	struct rateset sup_rates, ext_rates;
	void *pkt;
	uint8 *pbody, *elt, *parse, *pkt_hdr;
	struct dot11_assoc_req *assoc;
	uint16 type;
	uchar *ssid;
	wpa_ie_fixed_t *wpa_ie;
#ifdef BCMWPA2
	bcm_tlv_t *wpa2_ie = NULL;
	uint8 *pmkid[MAXPMKID];
	uint npmkid = 0;
#endif /* BCMWPA2 */
	bcm_tlv_t *ie;
	bcm_tlv_t *sup_rates_ovrd, *ext_rates_ovrd;
	dot11_power_cap_t pwr_cap;
	uint8 WPA_auth = wlc->cfg.WPA_auth;

	/* get a packet
	 * (Re)Assoc Request packet has the following contents:
	 * 2 bytes Capability
	 * 2 bytes Listen Interval
	 * 6 bytes current BSSID (Reassoc only)
	 * InfoElt SSID
	 * InfoElt Supported rates
	 * InfoElt Power Capability (11h)
	 * InfoElt Supported channels (11h)
	 * InfoElt Extended Supported rates (11g)
	 * InfoElt BRCM proprietary
	 * InfoElt WME
	 */
	/* SSID */
	ie = bcm_parse_tlvs(wlc->assoc_elts, wlc->assoc_elts_len, DOT11_MNG_SSID_ID);
	if (ie) {
		ssid = ie->data;
		ssid_len = ie->len;
	} else {
		ssid = bi->SSID;
		ssid_len = bi->SSID_len;
	}

	/* Supported Rates and Extended Supported Rates */

	/* We may have 1 or 2 ratesets.
	 * One or both may be overridden by a configured set of info elements.
	 */
	bzero(&sup_rates, sizeof(struct rateset));
	bzero(&ext_rates, sizeof(struct rateset));
	sup_rates_ovrd = bcm_parse_tlvs(wlc->assoc_elts, wlc->assoc_elts_len, DOT11_MNG_RATES_ID);
	ext_rates_ovrd = bcm_parse_tlvs(wlc->assoc_elts, wlc->assoc_elts_len,
		DOT11_MNG_EXT_RATES_ID);

	/* Supported Rates override */
	if (sup_rates_ovrd) {
		/* subtract the length for the elt from the total and
		 * we will add it back in below if needed
		 */
		if (sup_rates_ovrd->len <= WLC_NUMRATES) {
			sup_rates.count = sup_rates_ovrd->len;
			bcopy(sup_rates_ovrd->data, sup_rates.rates, sup_rates.count);
		} else {
			/* Supported Rates override is invalid */
			sup_rates_ovrd = NULL;
		}
	}

	/* Extended Supported Rates override */
	if (ext_rates_ovrd) {
		/* subtract the length for the elt from the total and
		 * we will add it back in below if needed
		 */
		if (ext_rates_ovrd->len <= WLC_NUMRATES) {
			ext_rates.count = ext_rates_ovrd->len;
			bcopy(ext_rates_ovrd->data, ext_rates.rates, ext_rates.count);
		} else {
			/* Extended Supported Rates override is invalid */
			ext_rates_ovrd = NULL;
		}
	}

	if (bi->flags & WLC_BSS_54G &&
	    !(wlc->band->gmode == GMODE_LEGACY_B)) {
		/* Legacy 54g AP, so ignore rates overrides and put
		 * all rates in the supported rates elt
		 */
		sup_rates.count = wlc->band->hw_rateset.count;
		bcopy(wlc->band->hw_rateset.rates, sup_rates.rates, sup_rates.count);
		ext_rates.count = 0;
	} else {
		/* create the ratesets with overrides taken into account
		 * target rateset has been filtered against our hw_rateset
		 */
		wlc_rateset_elts(wlc, bi->rateset.count, bi->rateset.rates, &sup_rates, &ext_rates);
	}

	/* Default WPA */
	WPA_len = 0;

	parse = wlc->assoc_elts;
	parse_len = wlc->assoc_elts_len;
	if ((wpa_ie = bcm_find_wpaie(parse, parse_len)) != NULL) {
		WL_INFORM(("got the wpaie from assoc elts\n"));
		WPA_len = wpa_ie->length;
#ifdef BCMWPA2
	} else if ((wpa2_ie = bcm_parse_tlvs(wlc->assoc_elts, wlc->assoc_elts_len,
		DOT11_MNG_RSN_ID)) != NULL) {
		WPA_len = wpa2_ie->len;
	} else if (IS_WPA2_AUTH(WPA_auth) && WSEC_ENABLED(wlc->cfg.wsec)) {
		WPA_len = WPA2_info_element[TLV_LEN_OFF];

		/* PMKID in cache? */
		if (WPA_auth == WPA2_AUTH_UNSPECIFIED) {
			uint i;

			for (i = 0; i < wlc->pub.npmkid; i++) {
				if (bcmp((void *)&bi->BSSID,
				         (void *)&wlc->pub.pmkid[i].BSSID, ETHER_ADDR_LEN) == 0) {
					pmkid[npmkid++] = wlc->pub.pmkid[i].PMKID;
				}
			}
		}
#endif /* BCMWPA2 */
	} else if (WPA_auth != WPA_AUTH_DISABLED && WSEC_ENABLED(wlc->cfg.wsec)) {
		WL_INFORM(("using the wpaie from wpainfo elt\n"));
		WPA_len = WPA_info_element[TLV_LEN_OFF];
	}

	/* get a control packet */
	type = (uint16)(reassoc ? FC_REASSOC_REQ : FC_ASSOC_REQ);
	if ((pkt = wlc_frame_get_mgmt(wlc, type, &bi->BSSID, &bi->BSSID,
		ETHER_MAX_LEN, &pbody)) == NULL) {
		wlc_assocreq_complete(wlc, TX_STATUS_NO_ACK, NULL);
		return NULL;
	}
	pkt_hdr = pbody - DOT11_MGMT_HDR_LEN;	/* point to beginning of packet */

	/* fill out the association request body */
	assoc = (struct dot11_assoc_req *) pbody;

	/* Capability information */
	if (wlc->assoc_capability)
		assoc->capability = htol16(wlc->assoc_capability);
	else {
		assoc->capability = DOT11_CAP_ESS;
		if (WSEC_ENABLED(wlc->cfg.wsec))
			assoc->capability |= DOT11_CAP_PRIVACY;
		if (ISGPHY(wlc->band->pi) &&
		    wlc->PLCPHdr_override == WLC_PLCP_SHORT)
			assoc->capability |= DOT11_CAP_SHORT;
		if (wlc->band->gmode && wlc->shortslot_override != WLC_SHORTSLOT_OFF)
			assoc->capability |= DOT11_CAP_SHORTSLOT;
		if (WL11H_ENAB(wlc))
			assoc->capability |= DOT11_CAP_SPECTRUM;
		assoc->capability = htol16(assoc->capability);
	}

	/* Listen interval */
	if (wlc->assoc_listen)
		assoc->listen = htol16(wlc->assoc_listen);
	else
		assoc->listen = htol16(10);

	pbody = (uint8*)&assoc[1];

	/* Current AP address */
	if (reassoc) {
		if (!ETHER_ISNULLADDR(&wlc->assoc_bssid))
			bcopy(wlc->assoc_bssid.octet, pbody, ETHER_ADDR_LEN);
		else
			bcopy(wlc->prev_BSSID.octet, pbody, ETHER_ADDR_LEN);
		pbody += ETHER_ADDR_LEN;
	}

	/* SSID */
	pbody = wlc_write_info_elt(pbody, DOT11_MNG_SSID_ID, ssid_len, ssid);

	/* Supported Rates */
	pbody = wlc_write_info_elt(pbody, DOT11_MNG_RATES_ID, sup_rates.count, sup_rates.rates);

	if (WL11H_ENAB(wlc)) {
		int min, max;

		/* Power Capability */
		wlc_txpwr_limits(wlc->band->pi, bi->channel, &min, &max, -1);
		pwr_cap.min = (int8)min / WLC_TXPWR_DB_FACTOR; /* convert .25 dBm to dBm */
		pwr_cap.max = (int8)max / WLC_TXPWR_DB_FACTOR; /* convert .25 dBm to dBm */
		pbody = wlc_write_info_elt(pbody, DOT11_MNG_PWR_CAP_ID, sizeof(pwr_cap), &pwr_cap);

		/* Supported Channels */
		pbody = wlc_copy_info_elt(pbody, wlc->band->sup_chan_ie);
	}

#ifdef BCMWPA2
	/* WPA2 IE */
	if (WPA_len) {
		if (wpa2_ie) {
			/* copy WPA2 info element */
			pbody = wlc_copy_info_elt(pbody, wpa2_ie);
		} else if (IS_WPA2_AUTH(WPA_auth)) {
			/* copy WPA2 info element template */
			wpa2_ie = (bcm_tlv_t *)pbody;
			pbody = wlc_copy_info_elt(pbody, WPA2_info_element);

			/* fill in cipher suites, AKM, etc. */
			wlc_rsn_build_ie(wlc, &(bi->wpa2), &wpa2_ie->data[WPA2_VERSION_LEN]);

			/* use cached PMKIDs */
			if (npmkid) {
				wpa_pmkid_list_t *pmkid_list;
				uint i;

				pmkid_list =
					(wpa_pmkid_list_t *)&wpa2_ie->data[wpa2_ie->len];

				/* fill in PMKID count */
				pmkid_list->count.low = (uint8)npmkid;
				pmkid_list->count.high = (uint8)(npmkid >> 8);
				wpa2_ie->len += 2;
				pbody += 2;

				/* fill in PMKID list */
				for (i = 0; i < npmkid; i++) {
					bcopy(pmkid[i], &pmkid_list->list[i], WPA2_PMKID_LEN);
					wpa2_ie->len += WPA2_PMKID_LEN;
					pbody += WPA2_PMKID_LEN;
				}
			}
		}
	}
#endif /* BCMWPA2 */

	/* Extended Supported Rates */
	if (ext_rates.count > 0)
		pbody = wlc_write_info_elt(pbody, DOT11_MNG_EXT_RATES_ID, ext_rates.count,
			ext_rates.rates);

	/* Merge in additional elements */
	for (elt = wlc->assoc_elts;
		elt && (elt < &wlc->assoc_elts[wlc->assoc_elts_len]);
		elt += 2 + elt[1]) {
		/* Sanity check */
		if ((uint)(elt - wlc->assoc_elts + 2 + elt[1]) > wlc->assoc_elts_len)
			break;

		switch (elt[0]) {
		case DOT11_MNG_SSID_ID:
		case DOT11_MNG_RATES_ID:
#ifdef BCMWPA2
		case DOT11_MNG_RSN_ID:
#endif /* BCMWPA2 */
			/* Already added above */
			break;
		default:
			bcopy(elt, pbody, 2 + elt[1]);
			pbody += TLV_HDR_LEN + elt[1];
			break;
		}
	}

	/* BRCM proprietary elt */
	if (wlc->brcm_ie[TLV_LEN_OFF] > 0)
		pbody = wlc_copy_info_elt(pbody, wlc->brcm_ie);

	/* Include a WME info element if the AP supports WME */
	if (WME_ENAB(wlc) && (bi->flags & WLC_BSS_WME)) {
		wme_ie_t wme_ie;
		ASSERT(sizeof(wme_ie) == WME_IE_LEN);
		bcopy(WME_OUI, (char*)&wme_ie.oui[0], DOT11_OUI_LEN);
		wme_ie.type = WME_TYPE;
		wme_ie.subtype = WME_SUBTYPE_IE;
		wme_ie.version = WME_VER;
		/* Send QoS Info in request only if both STA and AP support APSD */
		wme_ie.qosinfo = (wlc->wme_apsd && (bi->wme_qosinfo & WME_QI_AP_APSD_MASK) ?
		                  wlc->apsd_sta_qosinfo : 0);
		pbody = wlc_write_info_elt(pbody, DOT11_MNG_PROPR_ID,
		                           sizeof(wme_ie), (char*)&wme_ie);
	}

	if (WPA_len) {
		/* copy WPA info element */
		if (wpa_ie)
			pbody = wlc_copy_info_elt(pbody, wpa_ie);
#ifdef BCMWPA2
		else if (!IS_WPA2_AUTH(WPA_auth)) {
#else /* BCMWPA2 */
		else {
#endif /* BCMWPA2 */
			/* copy WPA info element template */
			wpa_ie = (wpa_ie_fixed_t *)pbody;
			pbody = wlc_copy_info_elt(pbody, WPA_info_element);

			/* fill in cipher suites, AKM, etc. */
			wlc_rsn_build_ie(wlc, &(bi->wpa), &wpa_ie[1]);
		}
	}

	len = (uint)(pbody - pkt_hdr);	/* 802.11 packet length */
	ASSERT(len <= ETHER_MAX_LEN);
	PKTSETLEN(wlc->pub.osh, pkt, len);	/* adjust the packet length */

	/* Save a copy of the last association request */
	if (wlc->assoc_req) {
		MFREE(wlc->pub.osh, wlc->assoc_req, wlc->assoc_req_len);
	}

	assoc_req_len = len - (uint)((uint8 *)assoc - pkt_hdr);
	if (!(wlc->assoc_req = MALLOC(wlc->pub.osh, assoc_req_len)))
		WL_ERROR(("wl%d: wlc_sendassocreq: out of memory, malloced %d bytes\n",
			wlc->pub.unit, MALLOCED(wlc->pub.osh)));
	else {
		wlc->assoc_req_len = assoc_req_len;
		wlc->assoc_req_is_reassoc = reassoc;
		bcopy((char*)assoc, (char*)wlc->assoc_req, assoc_req_len);
	}

	WL_ASSOC(("JOIN: sending %s REQ ...\n", (reassoc ? "REASSOC" : "ASSOC")));

	if (wlc_sendmgmt(wlc, pkt, scb, bi->capability & DOT11_CAP_SHORT, NULL, 0))
		return pkt;

	return NULL;
}

static void
wlc_authresp_client(wlc_info_t *wlc, d11rxhdr_t *rxh, struct dot11_management_header *hdr,
	void *body, uint len)
{
	struct dot11_auth *auth = (struct dot11_auth *) body;
	uint16 auth_alg, auth_seq, auth_status;
	uint8 *challenge = (uint8 *) &auth[1];
	struct scb *scb;
	void *pkt;

	WL_TRACE(("wl%d: wlc_authresp_client\n", wlc->pub.unit));

	if (len < sizeof(struct dot11_auth))
		return;

	auth_alg = ltoh16(auth->alg);
	auth_seq = ltoh16(auth->seq);
	auth_status = ltoh16(auth->status);

	/* ignore authentication frames from other stations */
	if (bcmp((char*)&hdr->sa, (char*)&wlc->target_bss.BSSID, ETHER_ADDR_LEN) ||
		!(scb = wlc_scblookup(wlc, (struct ether_addr *) &hdr->sa)) ||
		(wlc->assoc_state != AS_SENT_AUTH_1 && wlc->assoc_state != AS_SENT_AUTH_3)) {
		WL_ERROR(("wl%d: unsolicited authentication response from %s\n", wlc->pub.unit,
			sa));
		wlc_mac_event(wlc, WLC_E_AUTH, &hdr->sa, WLC_E_STATUS_UNSOLICITED, 0, 0, 0, 0);
		return;
	}

	scb->flags |= SCB_MYAP;

	if ((wlc->assoc_state == AS_SENT_AUTH_1 && auth_seq != 2) ||
		(wlc->assoc_state == AS_SENT_AUTH_3 && auth_seq != 4)) {
		WL_ERROR(("wl%d: out-of-sequence authentication response from %s\n", wlc->pub.unit,
			sa));
		wlc_mac_event(wlc, WLC_E_AUTH, &hdr->sa, WLC_E_STATUS_UNSOLICITED, 0, 0, 0, 0);
		return;
	}

	/* authentication error */
	if (auth_status != DOT11_STATUS_SUCCESS) {
		wlc_mac_event(wlc, WLC_E_AUTH, &hdr->sa, WLC_E_STATUS_FAIL, auth_status, auth_alg,
			0, 0);
		return;
	}

	if (auth_alg == DOT11_SHARED_KEY && auth_seq == 2) {
		/* respond to the challenge */
		if (len < (sizeof(struct dot11_auth) + 2 + challenge[1])) {
			WL_ERROR(("wl%d: shared key auth pkt too short, got %d, expected %u\n",
				wlc->pub.unit, len,
				(uint)(sizeof(struct dot11_auth) + 2 + challenge[1])));
			return;
		}
		WL_ASSOC(("JOIN: got authentication response seq 2 ...\n"));
		wlc_assoc_change_state(wlc, AS_SENT_AUTH_3);
		pkt = wlc_sendauth(wlc, &scb->ea, &wlc->target_bss.BSSID, scb,
		                   DOT11_SHARED_KEY, 3, DOT11_STATUS_SUCCESS, scb->key, challenge,
		                   rxh->PhyRxStatus_0 & PRXS0_SHORTH);
		wl_del_timer(wlc->wl, wlc->assoc_timer);
		if (pkt != NULL)
			wlc_pkt_callback_register(wlc, wlc_auth_complete, NULL, pkt);
		else
			wl_add_timer(wlc->wl, wlc->assoc_timer, WECA_ASSOC_TIMEOUT + 10, 0);
	} else {
		/* authentication success */
		ASSERT(auth_seq == 2 || (auth_alg == DOT11_SHARED_KEY && auth_seq == 4));
		wlc_mac_event(wlc, WLC_E_AUTH, &hdr->sa, WLC_E_STATUS_SUCCESS, auth_status,
			auth_alg, 0, 0);
	}
}

static void
wlc_clear_hw_association(wlc_info_t *wlc)
{
	d11regs_t *regs = wlc->regs;

	wlc->PM_override = FALSE;

	/* zero the BSSID so the core will not process TSF updates */
	bzero(wlc->BSSID.octet, ETHER_ADDR_LEN);
	bzero(wlc->pub.current_bss.BSSID.octet, ETHER_ADDR_LEN);

	bzero(wlc->announce_BSSID.octet, ETHER_ADDR_LEN);

	/* Clear any possible Channel Switch Announcement */
	wlc->channel_sw.secs = 0;

	/* reset any BSS SpectrumManagement derived channel or txpwr information */
	if (WL11H_ENAB(wlc))
		wlc_channels_init(wlc);

	if (!DEVICEREMOVED(wlc)) {
		wlc_suspend_mac_and_wait(wlc);

		if (QUIET_CHANNEL(wlc, wlc->band->pi->radio_channel))
			wlc_mute(wlc, ON);

		/* clear BSSID in PSM */
		wlc_set_bssid(wlc);

		/* write the beacon interval to the TSF block */
		W_REG(&regs->tsf_cfprep, 0x80000000);

		/* gphy, aphy use the same CWMIN */
		wlc_set_cwmin(wlc, APHY_CWMIN);
		wlc_set_cwmax(wlc, PHY_CWMAX);

		if (!ISAPHY(wlc->band->pi))
			wlc_update_slot_timing(wlc, wlc->shortslot);

		/* Restore the frequency tracking bandwidth of the PHY, if we modified it */
		wlc_freqtrack_reset(wlc);

		wlc_enable_mac(wlc);
	}
}

static void
wlc_join_start_ibss(wlc_info_t *wlc, const uchar ssid[], int ssid_len)
{
	wlc_bss_info_t bi;
	uint16 r;

	ASSERT(ssid_len > 0);
	ASSERT(ssid_len <= DOT11_MAX_SSID_LEN);

	/* set up default BSS params */
	bcopy(&wlc->default_bss, &bi, sizeof(bi));

	/* create a IBSS BSSID using a random number */
	r = R_REG(&wlc->regs->tsf_random);
	bi.BSSID.octet[0] = (uint8)(r);
	bi.BSSID.octet[1] = (uint8)(r>>8);
	r = R_REG(&wlc->regs->tsf_random);
	bi.BSSID.octet[2] = (uint8)(r);
	bi.BSSID.octet[3] = (uint8)(r>>8);
	r = R_REG(&wlc->regs->tsf_random);
	bi.BSSID.octet[4] = (uint8)(r);
	bi.BSSID.octet[5] = (uint8)(r>>8);
	/* Set the first 2 MAC addr bits to "locally administered" */
	bi.BSSID.octet[0] &= ~0x03;
	bi.BSSID.octet[0] |= 0x02;

	/* adopt previously specified params */
	bi.infra = 0;
	bzero(bi.SSID, sizeof(bi.SSID));
	bi.SSID_len = (uint8)ssid_len;
	bcopy(ssid, bi.SSID, ssid_len);

	/*
	   Validate or fixup default channel value.
	   Don't want to create ibss on quiet channel since it hasn't be
	   verified as radar-free.
	   For future reference, if we do support ibss creation
	   on radar channels, need to unmute and clear quiet bit.
	*/
	if (!VALID_CHANNEL_DB(wlc, bi.channel) ||
	    QUIET_CHANNEL(wlc, bi.channel)) {
		uint8 chan;
		if ((chan = wlc_next_channel(wlc, bi.channel, CHAN_TYPE_CHATTY, 0)) == INVCHANNEL) {
			wlc_mac_event(wlc, WLC_E_SET_SSID, &wlc->BSSID, WLC_E_STATUS_NO_NETWORKS, 0,
				0, 0, 0);
			return;
		}
		bi.channel = chan;
	}

	wlc_suspend_mac_and_wait(wlc);
	wlc_BSSinit(wlc, &bi, &wlc->cfg, WLC_BSS_START);
	wlc_enable_mac(wlc);
	/* initialize link state tracking to the lost beacons condition */
	wlc->time_since_bcn = 1;
	wlc->bcns_lost = TRUE;

	wlc->pub.associated = TRUE;

	WL_ERROR(("wl%d: IBSS started\n", wlc->pub.unit));
	wlc_mac_event(wlc, WLC_E_START, &wlc->BSSID, WLC_E_STATUS_SUCCESS, 0, 0, 0, 0);
}

/* Temporarily suspend IBSS operation, disabling beacons and ignoring probe requests */
static void
wlc_ibss_suspend(wlc_info_t *wlc)
{
	WL_TRACE(("wl%d: wlc_ibss_suspend\n", wlc->pub.unit));

	/* Must be in IBSS mode */
	ASSERT(!wlc->pub.BSS);
	ASSERT(wlc->pub.associated);

	/* Switch to BSS STA mode so that MAC will not send IBSS Beacons
	 * or respond to probe requests
	 */
	wlc->bcnmisc_ibss = FALSE;
	wlc_mac_bcn_promisc(wlc);
	wlc_mctrl(wlc, MCTL_INFRA, MCTL_INFRA);
}

/* Resume IBSS operation, enabling beacons and probe responses */
static void
wlc_ibss_resume(wlc_info_t *wlc)
{
	WL_TRACE(("wl%d: wlc_ibss_resume\n", wlc->pub.unit));

	/* Must be in IBSS mode */
	ASSERT(!wlc->pub.BSS);
	ASSERT(wlc->pub.associated);

	/* Switch back to IBSS STA mode */
	wlc->bcnmisc_ibss = TRUE;
	wlc_mac_bcn_promisc(wlc);
	wlc_mctrl(wlc, MCTL_INFRA, 0);
}

static void
wlc_tkip_countermeasures(wlc_info_t *wlc, uint txstatus, void *dummy)
{
	struct scb *scb;

	WL_WSEC(("wl%d: wlc_tkip_countermeasures: TKIP countermeasures: txstatus received for 2nd"
		" MIC failure indication...\n", wlc->pub.unit));

	if (ETHER_ISNULLADDR(wlc->BSSID.octet)) {
		/* no need to disassoc...already gone */
		return;
	}


	WL_WSEC(("wl%d: wlc_tkip_countermeasures: disassociating %s, BSS under attack\n",
		wlc->pub.unit, eabuf));

	scb = wlc_scbfind(wlc, &wlc->BSSID);
	ASSERT(scb);
	ASSERT(SCB_ASSOCIATED(scb));

	/* send disassociate packet */
	(void) wlc_senddisassoc(wlc, &wlc->BSSID, &wlc->BSSID, scb, DOT11_RC_MIC_FAILURE);

	/* delete pairwise key */
	if (scb->key) {
		WL_WSEC(("wl%d: %s: deleting pairwise key for %s\n",
			wlc->pub.unit, __FUNCTION__, eabuf));
		wlc_scb_key_delete(wlc, scb);
	}

	/* change scb state */
	wlc_scb_clearstatebit(scb, ASSOCIATED | AUTHORIZED);
	wlc_mac_event(wlc, WLC_E_DISASSOC, &wlc->BSSID, WLC_E_STATUS_SUCCESS, DOT11_RC_MIC_FAILURE,
		0, 0, 0);

	/* establish blocking timer */
	wlc->cfg.tk_cm_bt = WPA_TKIP_CM_BLOCK;

	/* force STA to roam (allowing it to roam back) */
	bzero(wlc->BSSID.octet, ETHER_ADDR_LEN);
	bzero(wlc->pub.current_bss.BSSID.octet, ETHER_ADDR_LEN);
}

int
wlc_disassociate_client(wlc_info_t *wlc, bool send_disassociate, pkcb_fn_t fn, void *arg)
{
	struct scb *scb;
	void *pkt;
	struct ether_addr BSSID;

	if (wlc->pub.associated == FALSE)
		return (-1);

	WL_TRACE(("wl%d: wlc_disassociate_client\n", wlc->pub.unit));

	if (wlc->pub.BSS) {
		bcopy(&wlc->prev_BSSID, &BSSID, ETHER_ADDR_LEN);
	}

	wlc->check_for_unaligned_tbtt = FALSE;


	if (DEVICEREMOVED(wlc)) {
		wlc->pub.associated = FALSE;
		wlc_clear_hw_association(wlc);
		wlc_mac_event(wlc, WLC_E_DISASSOC, &BSSID, WLC_E_STATUS_SUCCESS,
			DOT11_RC_DISASSOC_LEAVING, 0, 0, 0);
		return (0);
	}

	/* BSS STA */
	if (wlc->pub.BSS) {
		/* abort any association state machine in process */
		if ((wlc->assoc_state != AS_IDLE) && (wlc->assoc_state != AS_WAIT_DISASSOC))
		    wlc_assoc_abort(wlc);

		/* reset scb state */
		if ((scb = wlc_scbfind(wlc, &BSSID))) {

			wlc_scb_clearstatebit(scb, ASSOCIATED | PENDING_AUTH | PENDING_ASSOC);



			/* delete pairwise key on disassociate */
			if (scb && scb->key) {
				WL_WSEC(("wl%d: %s: disassociating, deleting pairwise key for %s\n",
					wlc->pub.unit, __FUNCTION__, eabuf));
				wlc_scb_key_delete(wlc, scb);
			}
		}

		/* clear PM state, (value used for wake shouldn't matter) */
		wlc->dtim_programmed = FALSE;
		wlc_set_ps_ctrl(wlc);

#ifdef QUIET_DISASSOC
		if (send_disassociate) {
			send_disassociate = FALSE;
		}
#endif /* QUIET_DISASSOC */
		/* Send disassociate packet and (attempt to) schedule callback */
		if (send_disassociate) {
			pkt = NULL;
			if (ETHER_ISNULLADDR(wlc->BSSID.octet)) {
				WL_ASSOC(("JOIN: skipping DISASSOC to %s since we lost contact.\n",
					eabuf));
			} else if (QUIET_CHANNEL(wlc, wlc->pub.current_bss.channel) ||
				(wlc_radar_channel(wlc, (uint)wlc->pub.current_bss.channel) ==
				TRUE)) {
				/* note that if the channel is a radar channel, after this function
				 * is done it will be a quiet channel since we will no longer be
				 * associated
				 */
				WL_ASSOC(("JOIN: skipping DISASSOC to %s since channel %d is"
					" quiet.\n",
					  eabuf, wlc->pub.current_bss.channel));
			} else {
				WL_ASSOC(("JOIN: sending DISASSOC to %s\n", eabuf));
				pkt = wlc_senddisassoc(wlc, &BSSID, &BSSID, scb,
				                       DOT11_RC_DISASSOC_LEAVING);
				if (pkt == NULL)
					WL_ASSOC(("JOIN: error sending DISASSOC\n"));
			}

			if (fn && pkt != NULL) {
				if (wlc_pkt_callback_register(wlc, fn, arg, pkt)) {
					WL_ERROR(("wl%d: wlc_disassociate_client out of pkt"
						" callbacks\n", wlc->pub.unit));
				} else {
					/* the callback was registered, so clear fn local so it will
					 * not be called at the end of this function
					 */
					fn = NULL;
				}

			}
		}
	}
	/* IBSS STA */
	else {
		/* Switch to BSS STA mode */
		wlc_mctrl(wlc, MCTL_INFRA, MCTL_INFRA);
		wlc->pub.BSS = TRUE;

		/* clear bcn promisc override for IBSS */
		wlc->bcnmisc_ibss = FALSE;
		wlc_mac_bcn_promisc(wlc);
	}
	wlc->pub.associated = FALSE;
	if (wlc->assoc_elts) {
		MFREE(wlc->pub.osh, wlc->assoc_elts, wlc->assoc_elts_len);
		wlc->assoc_elts = NULL;
		wlc->assoc_elts_len = 0;
	}


	if (!wlc->pub.associated) {
		/* unassociated CCK preamble mode */
		wlc->shortpreamble = FALSE;

		/* if auto shortslot, switch back to 11b compatible long slot */
		if (wlc->shortslot_override == WLC_SHORTSLOT_AUTO)
			wlc->shortslot = FALSE;

		/* unassociated 11g protection mode */
		if (wlc->g_protection_override == WLC_PROTECTION_AUTO)
			wlc->g_protection = FALSE;
	}

	wlc_clear_hw_association(wlc);

	if (wlc->pub.current_bss.bcn_prb) {
		MFREE(wlc->pub.osh, wlc->pub.current_bss.bcn_prb, wlc->pub.current_bss.bcn_prb_len);
		wlc->pub.current_bss.bcn_prb = NULL;
		wlc->pub.current_bss.bcn_prb_len = 0;
	}

	wlc_mac_event(wlc, WLC_E_DISASSOC, &BSSID, WLC_E_STATUS_SUCCESS, DOT11_RC_DISASSOC_LEAVING,
		0, 0, 0);

	wlc_link(wlc, FALSE, &BSSID, wlc->bsscfg[0]);

	wlc_led_event(wlc->ledh);

	/* call the given callback fn if it has not been taken care of with
	 * a disassoc packet callback.
	 */
	if (fn)
		(*fn)(wlc, TX_STATUS_NO_ACK, arg);

	return (0);
}




void
wlc_getrand(wlc_info_t *wlc, uint8 *buf, int buflen)
{
	int rand;

	if (buf != NULL) {
		while (buflen > 1) {
			rand = R_REG(&wlc->regs->tsf_random);
			*buf++ = (rand >> 8) & 0xff;
			*buf++ = rand & 0xff;
			buflen -= 2;
		}
		/* Was buflen odd? */
		if (buflen == 1) {
			rand = R_REG(&wlc->regs->tsf_random);
			*buf = rand & 0xff;
		}
	}
}

/* Append pkt's callback list to new_pkt. */
void
wlc_pkt_callback_append(wlc_info_t *wlc, void *new_pkt, void *pkt)
{
	uint8 idx = WLPKTTAG(pkt)->callbackidx;

	if (!idx)
		return;

	/* Go to the end of the chain */
	while (wlc->pkt_callback[idx].nextidx)
		idx = wlc->pkt_callback[idx].nextidx;

	wlc->pkt_callback[idx].nextidx = WLPKTTAG(new_pkt)->callbackidx;
	WLPKTTAG(new_pkt)->callbackidx = idx;
	WLPKTTAG(pkt)->callbackidx = 0;
}

int
wlc_pkt_callback_register(wlc_info_t *wlc, pkcb_fn_t fn, void *arg, void *pkt)
{
	pkt_cb_t	*pcb;
	uint8 i;

	/* find a free entry */
	for (i = 1; i <= MAX_PKT_CB; i++) {
		if (wlc->pkt_callback[i].fn == NULL)
			break;
	}

	if (i > MAX_PKT_CB) {
		WLCNTINCR(wlc->pub._cnt.pkt_callback_reg_fail);
		WL_ERROR(("wl%d: failed to register callback %p\n", wlc->pub.unit, fn));
		ASSERT(0);
		return (-1);
	}

	/* alloc and init next free callback struct */
	pcb = &wlc->pkt_callback[i];
	pcb->fn = fn;
	pcb->arg = arg;
	pcb->entered = FALSE;
	/* Chain this callback */
	pcb->nextidx = WLPKTTAG(pkt)->callbackidx;
	WLPKTTAG(pkt)->callbackidx = i;

	return (0);
}

void
wlc_pkt_callback(wlc_info_t *wlc, void *pkt, uint16 tx_status)
{
	pkt_cb_t	*pcb;
	uint8 idx = WLPKTTAG(pkt)->callbackidx;

	ASSERT(idx <= MAX_PKT_CB && wlc->pkt_callback[idx].fn != NULL);
	while (idx != 0 && idx <= MAX_PKT_CB) {
		pcb = &wlc->pkt_callback[idx];

		/* ensure the callback is not called recursively */
		ASSERT(pcb->entered == FALSE);
		ASSERT(pcb->fn != NULL);
		pcb->entered = TRUE;
		/* call the function */
		(pcb->fn)(wlc, tx_status, pcb->arg);
		pcb->fn = NULL;
		pcb->entered = FALSE;
		idx = pcb->nextidx;
		pcb->nextidx = 0;
	}
	WLPKTTAG(pkt)->callbackidx = 0;
}

bool
BCMINITFN(wlc_validate_chip_access)(wlc_info_t *wlc)
{
	d11regs_t *regs;
	uint32 w, val;
	uint16 *reg16;

	WL_TRACE(("wl%d: validate_chip_access\n", wlc->pub.unit));

	regs = wlc->regs;

	/* Validate dchip register access */

	W_REG(&regs->objaddr, OBJADDR_SHM_SEL | 0);
	w = R_REG(&regs->objdata);

	/* Can we write and read back a 32bit register? */
	W_REG(&regs->objaddr, OBJADDR_SHM_SEL | 0);
	W_REG(&regs->objdata, (uint32)0xaa5555aa);

	W_REG(&regs->objaddr, OBJADDR_SHM_SEL | 0);
	val = R_REG(&regs->objdata);
	if (val != (uint32)0xaa5555aa) {
		WL_ERROR(("wl%d: validate_chip_access: SHM = 0x%x, expected 0xaa5555aa\n",
			wlc->pub.unit, val));
		return (FALSE);
	}

	W_REG(&regs->objaddr, OBJADDR_SHM_SEL | 0);
	W_REG(&regs->objdata, (uint32)0x55aaaa55);

	W_REG(&regs->objaddr, OBJADDR_SHM_SEL | 0);
	val = R_REG(&regs->objdata);
	if (val != (uint32)0x55aaaa55) {
		WL_ERROR(("wl%d: validate_chip_access: SHM = 0x%x, expected 0x55aaaa55\n",
			wlc->pub.unit, val));
		return (FALSE);
	}

	W_REG(&regs->objaddr, OBJADDR_SHM_SEL | 0);
	W_REG(&regs->objdata, w);

	if ((D11REV_GE(wlc->pub.corerev, 3)) && (D11REV_LT(wlc->pub.corerev, 11))) {
		/* if 32 bit writes are split into 16 bit writes, are they in the correct order
		 * for our interface, low to high
		 */
		reg16 = (uint16*)&regs->tsf_cfpstart;

		/* write the CFPStart register low half explicitly, starting a buffered write */
		W_REG(reg16, 0xAAAA);

		/* Write a 32 bit value to CFPStart to test the 16 bit split order.
		 * If the low 16 bits are written first, followed by the high 16 bits then the
		 * 32 bit value 0xCCCCBBBB should end up in the register.
		 * If the order is reversed, then the write to the high half will trigger a buffered
		 * write of 0xCCCCAAAA.
		 * If the bus is 32 bits, then this is not much of a test, and the reg should
		 * have the correct value 0xCCCCBBBB.
		 */
		W_REG(&regs->tsf_cfpstart, 0xCCCCBBBB);

		/* verify with the 16 bit registers that have no side effects */
		val = R_REG(&regs->tsf_cfpstrt_l);
		if (val != (uint)0xBBBB) {
			WL_ERROR(("wl%d: validate_chip_access: tsf_cfpstrt_l = 0x%x, expected"
				" 0x%x\n",
				wlc->pub.unit, val, 0xBBBB));
			return (FALSE);
		}
		val = R_REG(&regs->tsf_cfpstrt_h);
		if (val != (uint)0xCCCC) {
			WL_ERROR(("wl%d: validate_chip_access: tsf_cfpstrt_h = 0x%x, expected"
				" 0x%x\n",
				wlc->pub.unit, val, 0xCCCC));
			return (FALSE);
		}

	}

	/* clear CFPStart */
	W_REG(&regs->tsf_cfpstart, 0);

	w = R_REG(&regs->maccontrol);
	if ((w != (uint)MCTL_IHR_EN) && (w != (uint)(MCTL_IHR_EN | MCTL_GMODE))) {
		WL_ERROR(("wl%d: validate_chip_access: maccontrol = 0x%x, expected 0x%x or 0x%x\n",
			wlc->pub.unit, w, MCTL_IHR_EN, (MCTL_IHR_EN | MCTL_GMODE)));
		return (FALSE);
	}

	return (TRUE);
}

/* init tx reported rate mechanism */
static void
wlc_reprate_init(wlc_info_t *wlc)
{
	wlc->txrateidx = 0;
	bzero((char*)wlc->txrate, sizeof(wlc->txrate));
}

/* return the "current tx rate" in units of 500kbps */
static uint
wlc_get_reprate(wlc_info_t *wlc)
{
	ratespec_t hw_rates_used[WLC_MAXRATE + 1];
	ratespec_t reprate, max, rate;
	int i;
	struct rateset	pub_bss_rateset;

	reprate = wlc->band->rspec_override ? wlc->band->rspec_override : 0;

	if (reprate)
		return (RSPEC2RATE(reprate));

	/*
	 * Loop over txrate history, looking up rate bins, and summing
	 * nfrags into appropriate supported rate bin.
	 */
	max = 0;
	bzero((char*)hw_rates_used, sizeof(hw_rates_used));
	for (i = 0; i < NTXRATE; i++) {
		rate = wlc->txrate[i][0];
		hw_rates_used[rate] += wlc->txrate[i][1];
		if (hw_rates_used[rate] > max) {
			max = hw_rates_used[rate];
			reprate = rate;
		}
	}

	/* check for an empty history */
	if (reprate == 0) {
		/* return a default value for rate */
		if (wlc->pub.associated) {
			pub_bss_rateset = wlc->pub.current_bss.rateset;
			/* associated, so return the max supported rate */
			reprate = pub_bss_rateset.rates[pub_bss_rateset.count - 1] & RATE_MASK;
		} else {
			/* not associated, so return the max rate we can send */
			reprate = wlc->band->hw_rateset.rates[wlc->band->hw_rateset.count - 1];

			/* Report the highest CCK rate if gmode == GMODE_LEGACY_B */
			if (ISGPHY(wlc->band->pi) && wlc->band->gmode == GMODE_LEGACY_B) {
				rateset_t rateset;
				wlc_rateset_filter(&wlc->band->hw_rateset, &rateset,
					FALSE, TRUE, RATE_MASK_FULL);
				reprate = rateset.rates[rateset.count - 1];
			}
		}
	}

	return (reprate);
}

uint16
wlc_rate_shm_offset(wlc_info_t *wlc, uint8 rate)
{
	uint16 table_ptr;
	uint8 phy_rate, index;

	/* get the phy specific rate encoding for the PLCP SIGNAL field */
	if (IS_OFDM(rate))
		table_ptr = M_RT_DIRMAP_A;
	else
		table_ptr = M_RT_DIRMAP_B;

	/* for a given rate, the LS-nibble of the PLCP SIGNAL field is
	 * the index into the rate table.
	 */
	phy_rate = rate_info[rate] & RATE_MASK;
	index = phy_rate & 0xf;

	/* Find the SHM pointer to the rate table entry by looking in the
	 * Direct-map Table
	 */
	return (2*wlc_read_shm(wlc, table_ptr + (index * 2)));
}

static bool
wlc_radar_channel(wlc_info_t *wlc, uint channel)
{
	bool is_radar = FALSE;
#if ACONF     /* RADAR */
	int bandunit;

	if (VALID_CHANNEL_DB(wlc, channel)) {
		bandunit = CHANNEL_BANDUNIT(wlc, channel);
		if (wlc_check_locale_radar(wlc->bandstate[bandunit].bandtype, channel))
			is_radar = TRUE;
	}
#endif /* ACONF */
	return is_radar;
}


void
wlc_default_rateset(wlc_info_t *wlc, rateset_t *rs)
{
	const rateset_t *phy_rs = NULL;


	if (ISAPHY(wlc->band->pi))
		phy_rs = ofdm_rates;
	else if (ISGPHY(wlc->band->pi))
		phy_rs = cck_ofdm_rates;

	ASSERT(phy_rs);
	wlc_rateset_copy(phy_rs, rs);
}

void
wlc_default_bss(wlc_info_t *wlc)
{
	uint8 chan;

	/* init default and target BSS with some sane initial values */
	bzero((char*)(&wlc->default_bss), sizeof(wlc_bss_info_t));
	wlc->default_bss.beacon_period = BEACON_INTERVAL_DEFAULT;
	wlc->default_bss.dtim_period = DTIM_INTERVAL_DEFAULT;

	/* default to infrastructure mode */
	wlc->default_bss.infra = 1;

	/* fill the default channel */
	chan = wlc_next_channel(wlc, 0, CHAN_TYPE_ANY, 0);
	if (chan == INVCHANNEL) {
		/* init radio channel to be first hw supported channel */
		chan = wlc_band_first_channel(wlc->band->pi, wlc->band->bandtype);
	}
	wlc->default_bss.channel = chan;

	bcopy((char*)&wlc->default_bss, (char*)&wlc->target_bss, sizeof(wlc_bss_info_t));
	bcopy((char*)&wlc->default_bss, (char*)&wlc->pub.current_bss, sizeof(wlc_bss_info_t));
}

/*
 * Parse a legacy rateset from a TLV section, optionally copy the raw version
 * into a *bss, sanitize and sort it, then optionally copy it into an scb.
 * Return 0 on success, nonzero on error.
 */
static int
wlc_parse_rates(wlc_info_t *wlc, uchar *tlvs, uint tlvs_len, wlc_bss_info_t *bi,
	struct rateset *rates)
{
	struct rateset rs;
	int ext_count;
	bcm_tlv_t *ie;

	/* pick out rateset from tlv section */
	if ((ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_RATES_ID)) == NULL)
		return (-1);

	rs.count = MIN(ie->len, WLC_NUMRATES);
	bcopy(ie->data, (char*)rs.rates, rs.count);

	/* pick out extended rateset from tlv section */
	if ((ie = bcm_parse_tlvs(tlvs, tlvs_len, DOT11_MNG_EXT_RATES_ID)) != NULL) {
		ext_count = MIN(ie->len, WLC_NUMRATES - (int)rs.count);
		bcopy(ie->data, (char*)(rs.rates + rs.count), ext_count);
		rs.count += ext_count;
	}

	/* copy raw set to bss_info */
	if (bi)
		bcopy((char*)&rs, (char*) &bi->rateset, sizeof(struct rateset));

	/* sort and sanitize them */
	wlc_sort_rates(&rs, &wlc->band->hw_rateset);

	ASSERT(rs.count <= WLC_NUMRATES);

	if (rs.count == 0)
		return (-1);

	if (rates)
		bcopy((char*)&rs, (char*)rates, sizeof(struct rateset));

	return (0);
}


void
wlc_rateset_elts(wlc_info_t *wlc, uint count, const uint8 *rates, struct rateset *sup_rates,
	struct rateset *ext_rates)
{
	uint8 sup_bin[WLC_MAXRATE+1];
	uint8 ext_bin[WLC_MAXRATE+1];
	uint8 sup_pref_bin[WLC_MAXRATE+1];
	const struct rateset *sup_pref_rates;
	const struct rateset *hw_rs = &wlc->band->hw_rateset;
	uint8 r;
	uint sup_count, ext_count;
	uint i;
	int j;

	ASSERT(count <= WLC_NUMRATES);

	bzero(sup_bin, sizeof(sup_bin));
	bzero(ext_bin, sizeof(ext_bin));
	bzero(sup_pref_bin, sizeof(sup_pref_bin));

	if (!wlc->band->gmode || ext_rates == NULL) {
		/* All rates end up in the Supported Rates elt */
		sup_rates->count = count;
		bcopy(rates, sup_rates->rates, count);
		return;
	}

	/* check for a supported rates override */
	if (sup_rates->count > 0) {
		/* split the rateset into supported and extended using the provided sup_rates
		 * as the guide
		 */
		sup_pref_rates = sup_rates;
	} else {
		/* use the default rate split of LRS rates (wlc_lrs_rates -> 1 2 5.5 11 18 24 36 54)
		 * in the supported rates, and others in extended rates
		 */
		sup_pref_rates = wlc_lrs_rates;
	}

	/* init the supported rates preferred lookup */
	for (i = 0; i < sup_pref_rates->count; i++) {
		r = sup_pref_rates->rates[i] & RATE_MASK;
		sup_pref_bin[r] = r;
	}

	sup_count = 0;
	ext_count = 0;

	/* split the rates into a supported and ext supported using the preferred split. */
	for (i = 0; i < count; i++) {
		r = rates[i] & RATE_MASK;
		if (sup_pref_bin[r]) {
			sup_bin[r] = rates[i];
			sup_count++;
		} else {
			ext_bin[r] = rates[i];
			ext_count++;
		}
	}

	if (sup_rates->count == 0 && ext_rates->count == 0) {
		/* fill up the supported rates to the full 8 rates unless there was an override */
		j = (int)hw_rs->count;
		for (i = sup_count; i < 8 && ext_count > 0; i++) {
			for (; j >= 0; j--) {
				r = hw_rs->rates[j] & RATE_MASK;
				if (ext_bin[r] != 0) {
					sup_bin[r] = ext_bin[r];
					sup_count++;
					ext_bin[r] = 0;
					ext_count--;
					break;
				}
			}
		}
	}

	/* fill out the sup_rates and ext_rates in sorted order */
	bzero(sup_rates, sizeof(struct rateset));
	bzero(ext_rates, sizeof(struct rateset));
	for (i = 0; i < hw_rs->count; i++) {
		r = hw_rs->rates[i] & RATE_MASK;
		if (sup_bin[r] != 0)
			sup_rates->rates[sup_rates->count++] = sup_bin[r];
		if (ext_bin[r] != 0)
			ext_rates->rates[ext_rates->count++] = ext_bin[r];
	}
}

void
wlc_event(wlc_info_t *wlc, wlc_event_t *e)
{
	struct ether_addr* addr = &(e->event.addr);
	uint msg, result, status, auth_type;
	wlc_event_t **e2;
	void *pkt;
	struct scb *scb;



	msg = e->event.event_type;
	result = e->event.status;
	status = e->event.reason;
	auth_type = e->event.auth_type;

	e->next = NULL;
	/* add it to the ordered list of pending events to report */
	for (e2 = &wlc->events; *e2; e2 = &((*e2)->next))
		;
	*e2 = e;

	switch (msg) {

	case WLC_E_AUTH:
		if (result == WLC_E_STATUS_UNSOLICITED)
			break;

		/* Clear pending bits */
		scb = wlc_scbfind(wlc, addr);
		if (scb)
			wlc_scb_clearstatebit(scb, PENDING_AUTH | PENDING_ASSOC);
		if (result != WLC_E_STATUS_TIMEOUT)
			wl_del_timer(wlc->wl, wlc->assoc_timer);

		if (result == WLC_E_STATUS_SUCCESS) {
			WL_ASSOC(("JOIN: authentication success\n"));

			/* call wlc_scblookup here with the TRUE allocation param in case
			 * we do not already have one allocated
			 */
			if (scb || (scb = wlc_scblookup(wlc, addr))) {
				wlc_scb_setstatebit(scb, AUTHENTICATED | PENDING_ASSOC);
				scb->flags &= ~SCB_SHORTPREAMBLE;
				if ((wlc->target_bss.capability & DOT11_CAP_SHORT) != 0)
					scb->flags |= SCB_SHORTPREAMBLE;
			}
			if ((scb->flags & SCB_MYAP) == 0) {
				scb->flags |= SCB_MYAP;
			}
			wlc_assoc_change_state(wlc, AS_SENT_ASSOC);
			pkt = wlc_sendassocreq(wlc, &wlc->target_bss, scb, wlc->pub.associated);
			wl_del_timer(wlc->wl, wlc->assoc_timer);
			if (pkt != NULL)
				wlc_pkt_callback_register(wlc, wlc_assocreq_complete, NULL, pkt);
			else
				wl_add_timer(wlc->wl, wlc->assoc_timer, WECA_ASSOC_TIMEOUT + 10, 0);
		} else if (result == WLC_E_STATUS_TIMEOUT) {
			WL_ASSOC(("JOIN: timeout waiting for authentication response, assoc_state"
				" %d\n", wlc->assoc_state));
		} else if (result == WLC_E_STATUS_NO_ACK) {
			WL_ASSOC(("JOIN: authentication failure, no ack from %s\n", eabuf));
		} else if (result == WLC_E_STATUS_FAIL) {
			WL_ASSOC(("JOIN: authentication failure status %d from %s\n", (int)status,
				eabuf));
			if (scb)
				wlc_scb_clearstatebit(scb, AUTHENTICATED);
		} else {
			WL_ERROR(("MACEVENT: AUTH, unexpected result param %d\n", (int)result));
			break;
		}

		if (result != WLC_E_STATUS_SUCCESS) {
			/* Try next BSS */
			wlc_join_attempt(wlc);
		}
		break;

	case WLC_E_ASSOC:
	case WLC_E_REASSOC: {
		if (result == WLC_E_STATUS_UNSOLICITED)
			break;

		/* Clear pending bits */
		scb = wlc_scbfind(wlc, addr);
		if (scb)
			wlc_scb_clearstatebit(scb, PENDING_AUTH | PENDING_ASSOC);

		if (result != WLC_E_STATUS_TIMEOUT)
			wl_del_timer(wlc->wl, wlc->assoc_timer);

		if (result == WLC_E_STATUS_SUCCESS) {
			struct scb *prev_scb;

			WL_ASSOC(("JOIN: %s success ...\n", action));
			/* clean up before leaving the BSS */
			if (wlc->pub.BSS && wlc->pub.associated) {
				prev_scb = wlc_scbfindband(wlc, &wlc->prev_BSSID,
					CHANNEL_BANDUNIT(wlc, wlc->pub.current_bss.channel));
				if (prev_scb) {
					wlc_scb_clearstatebit(prev_scb, ASSOCIATED | AUTHORIZED);


					/* delete old AP's pairwise key */
					if (prev_scb->key) {
						WL_WSEC(("wl%d: %s: roamed, deleting old pairwise"
							" key for %s\n",
							wlc->pub.unit, __FUNCTION__,
							bcm_ether_ntoa(&wlc->BSSID,
							prev_bssid)));
						wlc_scb_key_delete(wlc, prev_scb);
					}
				}
			}

			/* clear PM state */
			wlc->PM_override = FALSE;
			wlc->dtim_programmed = FALSE;
			wlc_set_ps_ctrl(wlc);

			/* update scb state */
			wlc_scb_setstatebit(scb, ASSOCIATED);

			/* init per scb WPA_auth */
			scb->WPA_auth = wlc->cfg.WPA_auth;
			WL_WSEC(("wl%d: %s: WPA_auth 0x%x\n", wlc->pub.unit, action,
				scb->WPA_auth));

			/* adopt the BSS parameters */
			wlc_join_adopt_bss(wlc);
		} else if (result == WLC_E_STATUS_TIMEOUT) {
			WL_ASSOC(("JOIN: timeout waiting for %s response\n", action));
		} else if (result == WLC_E_STATUS_NO_ACK) {
			WL_ASSOC(("JOIN: association failure, no ack from %s\n", eabuf));
		} else if (result == WLC_E_STATUS_FAIL) {
			WL_ASSOC(("JOIN: %s failure %d\n", action, (int)status));
		} else {
			WL_ERROR(("MACEVENT: %s, unexpected result param %d\n", action,
				(int)result));
			break;
		}

		if (result != WLC_E_STATUS_SUCCESS) {
			/* Try next BSS */
			wlc_join_attempt(wlc);
		}
		break;
	}

	case WLC_E_JOIN:
		if (result != WLC_E_STATUS_SUCCESS && result != WLC_E_STATUS_FAIL) {
			WL_ERROR(("MACEVENT: JOIN, unexpected result param %d\n", (int)result));
		}

		/* If assoc_state is not AS_IDLE, then this join event is completing a roam or a
		 * set_ssid operation.
		 * If assoc_state is AS_IDLE, then we are here due to an IBSS coalesce and there is
		 * no more follow on work.
		 */
		if (wlc->assoc_state != AS_IDLE) {
			uint next_msg;
			struct ether_addr* next_addr;

			next_msg = (wlc->assoc_type == AS_ASSOCIATION) ? WLC_E_SET_SSID :
				WLC_E_ROAM;
			next_addr = (result == WLC_E_STATUS_SUCCESS) ? addr : NULL;

			wlc_mac_event(wlc, next_msg, next_addr, result, 0, 0, 0, 0);
		}
		break;

	case WLC_E_SET_SSID:
		/* Association state machine is halting, clear state and allow core to sleep */
		wlc_bss_list_free(wlc, &wlc->join_targets);

		wlc->assoc_type = AS_IDLE;
		wlc_assoc_change_state(wlc, AS_IDLE);

		/* allow the core to sleep if we were waiting on assoc_state */
		wlc_set_ps_ctrl(wlc);

		if ((result != WLC_E_STATUS_SUCCESS) && (result != WLC_E_STATUS_FAIL) &&
			(result != WLC_E_STATUS_NO_NETWORKS)) {
			WL_ERROR(("MACEVENT: SET_SSID, unexpected result param %d\n", (int)result));
		}

		if (result != WLC_E_STATUS_SUCCESS) {
			WL_ASSOC(("MACEVT: JOIN ends, E_SET_SSID failed, assoc_state==AS_IDLE\n"));
			/* STA only, no aps_associated issues */
			wlc->cfg.up = FALSE;
			/* leave enabled if retry time configured, else disable */
			if (wlc->sta_retry_time) {
				wl_del_timer(wlc->wl, wlc->assoc_timer);
				wl_add_timer(wlc->wl, wlc->assoc_timer,
				             wlc->sta_retry_time*1000, 0);
			} else {
				wlc->cfg.enable = FALSE;
			}
		}
		break;

	case WLC_E_ROAM:
		if (result == WLC_E_STATUS_SUCCESS) {
			WL_ASSOC(("JOIN: roam success\n"));
		} else if (result == WLC_E_STATUS_FAIL) {
			WL_ASSOC(("JOIN: roam failure\n"));
		} else if (result == WLC_E_STATUS_NO_NETWORKS) {
			WL_ASSOC(("JOIN: roam found no networks\n"));
		} else {
			WL_ERROR(("MACEVENT: ROAM, unexpected result param %d\n", (int)result));
		}

		/* Association state machine is halting, clear state */
		wlc_bss_list_free(wlc, &wlc->join_targets);

		wlc->assoc_type = AS_IDLE;
		wlc_assoc_change_state(wlc, AS_IDLE);


		/* allow the core to sleep if we were waiting on assoc_state */
		wlc_set_ps_ctrl(wlc);

		/* If a roam fails, restore state to that of our current association */
		if (result == WLC_E_STATUS_FAIL) {
			/* restore old channel */
			if (wlc->band->pi->radio_channel != wlc->pub.current_bss.channel) {
				wlc_suspend_mac_and_wait(wlc);
				wlc_set_channel(wlc, wlc->pub.current_bss.channel);
				wlc_enable_mac(wlc);
			}

			/* restore old basic rates */
			wlc_rate_lookup_init(wlc, &wlc->pub.current_bss.rateset);
		}
		e->event.reason = wlc->roam_reason;
		break;

	case WLC_E_DEAUTH_IND:
	case WLC_E_DISASSOC_IND:
		break;

	case WLC_E_LINK:
#ifdef BCMWPA2
		if (IS_WPA2_AUTH(wlc->cfg.WPA_auth) && WSEC_ENABLED(wlc->cfg.wsec)) {
			if (wlc->pub.associated &&
			    (wlc->pub.current_bss.bcn_prb != NULL) &&
			    (wlc->pub.current_bss.bcn_prb_len > DOT11_BCN_PRB_LEN)) {
				uint8 *parse;
				wpa_rsn_ie_fixed_t *ie;
				uint len;

				len = wlc->pub.current_bss.bcn_prb_len -
					sizeof(struct dot11_bcn_prb);
				parse = (uint8 *)wlc->pub.current_bss.bcn_prb +
					sizeof(struct dot11_bcn_prb);

				if ((ie = (wpa_rsn_ie_fixed_t *)bcm_parse_tlvs(parse, len,
					DOT11_MNG_RSN_ID)) != NULL) {
					e->data = (uint8*)ie;
					e->event.datalen = ie->length + TLV_HDR_LEN;
				}
			}
		} else
#endif /* BCMWPA2 */
		if (wlc->pub.associated &&
			(wlc->pub.current_bss.bcn_prb != NULL) &&
			(wlc->pub.current_bss.bcn_prb_len > DOT11_BCN_PRB_LEN)) {
			uint8 *parse;
			wpa_ie_fixed_t *ie;
			uint len;
			extern wpa_ie_fixed_t *bcm_find_wpaie(uint8 *, uint);

			len = wlc->pub.current_bss.bcn_prb_len -
				sizeof(struct dot11_bcn_prb);
			parse = (uint8 *)wlc->pub.current_bss.bcn_prb +
				sizeof(struct dot11_bcn_prb);

			if ((ie = bcm_find_wpaie(parse, len)) != NULL) {

				e->data = (uint8*)ie;
				e->event.datalen = ie->length + TLV_HDR_LEN;
			}
		}
		break;

	case WLC_E_ADDTS_IND:
		if (result == WLC_E_STATUS_SUCCESS)
			WL_CAC(("ADDTS: success ...\n"));
		else if (result == WLC_E_STATUS_TIMEOUT)
			WL_CAC(("ADDTS: timeout waiting for ADDTS response\n"));
		else if (result == WLC_E_STATUS_FAIL)
			WL_CAC(("ADDTS: failure, TSPEC rejected\n"));
		else
			WL_ERROR(("MACEVENT: ADDTS, unexpected result param %d\n", (int)result));
		break;

	case WLC_E_DELTS_IND:
		if (result == WLC_E_STATUS_SUCCESS)
			WL_CAC(("DELTS: success ...\n"));
		else if (result == WLC_E_STATUS_UNSOLICITED)
			WL_CAC(("DELTS: unsolicited\n"));
		else
			WL_ERROR(("MACEVENT: DELTS, unexpected result param %d\n", (int)result));
		break;

	case WLC_E_MIC_ERROR:
	case WLC_E_ASSOC_IND:
	case WLC_E_REASSOC_IND:
	case WLC_E_PMKID_CACHE:
	case WLC_E_START:
	case WLC_E_AUTH_IND:
	case WLC_E_DEAUTH:
	case WLC_E_DISASSOC:
	case WLC_E_QUIET_START:
	case WLC_E_QUIET_END:
	case WLC_E_GOT_BEACONS:
	case WLC_E_TXFAIL:
	case WLC_E_EAPOL_MSG:
	case WLC_E_PRUNE:
	case WLC_E_AUTOAUTH:
	case WLC_E_SCAN_COMPLETE:
		/* no special action required */
		break;

	default:
		WL_ERROR(("MACEVENT: UNKNOWN %d, MAC %s, result %d, status %d, auth %d\n",
			msg, eabuf, (int)result, (int)status, (int)auth_type));
		break;
	}

/* When an event is added, add a case to the switch above to handle the new event
 * and adjust the preprocessor error check below
 */
/* check if WLC_E_LAST changed since the last time this function was updated */
#if WLC_E_LAST != 29
#error "You need to add a wlc_event case for the new event"
#endif /* WLC_E_LAST != 29 */

	addr = 0;

	/* report pending events to interested parties in correct order */
	for (e = wlc->events; e; e = e->next) {
		int bssid = 0;
		wl_event(wlc->wl, e);
		/* send encapsulated version of the message if requested */
		if (isset(wlc->event_msgs, e->event.event_type) || (
			isset(wlc->eventing, e->event.event_type) && wlc->callback.fn))
			wlc_sendup_event(wlc, bssid, &wlc->pub.cur_etheraddr, e, e->data,
				e->event.datalen);
	}
	wlc->events = NULL;
}

void
wlc_mac_event(wlc_info_t* wlc, uint msg, const struct ether_addr* addr, uint result, uint status,
	uint auth_type, void *data, int datalen)
{
	wlc_event_t e;

	bzero(&e, sizeof(e));
	e.event.event_type = msg;
	if (addr != NULL) {
		bcopy(addr->octet, e.event.addr.octet, ETHER_ADDR_LEN);
		e.addr = &(e.event.addr);
	}
	e.event.status = result;
	e.event.reason = status;
	e.event.auth_type = auth_type;
	e.data = data;
	e.event.datalen = datalen;

	wlc_event(wlc, &e);
}

/*
 * The struct ether_addr arg is (when non-NULL) the mac addr
 * of the AP to which we're associating. It's used by nas supplicant
 * in the ap-sta build variant.
 *
 */
void
wlc_link(wlc_info_t *wlc, bool up, struct ether_addr *addr, wlc_bsscfg_t *bsscfg)
{
	wlc_event_t e;

	bzero(&e, sizeof(e));
	e.event.event_type = WLC_E_LINK;
	e.event.flags = up?WLC_EVENT_MSG_LINK:0;
	if (addr != NULL) {
		bcopy(addr->octet, e.event.addr.octet, ETHER_ADDR_LEN);
		e.addr = &(e.event.addr);
	}

	wlc_event(wlc, &e);
}


/* Returns TRUE if the request can proceed, or FALSE if the request is blocked.
 * This routine will make sure any lower priority MAC action is aborted.
 */
static bool
wlc_mac_request_entry(wlc_info_t* wlc, int req)
{
	bool entry_ok = TRUE;

	switch (req) {
	case WLC_ACTION_ASSOC:
		/* The Association code, wlc_set_ssid(), handles canceling scans, associations,
		 * and quiet periods, and takes precedence over all other actions
		 */
		break;
	case WLC_ACTION_ROAM:
		if (wlc->assoc_state != AS_IDLE) {
			WL_INFORM(("wl%d: roam scan blocked for association in progress\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		else if (SCAN_IN_PROGRESS(wlc)) {
			WL_INFORM(("wl%d: roam scan blocked for scan in progress\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		else if (wlc->PSpoll) {
			WL_INFORM(("wl%d: roam scan blocked for outstanding PS-Poll\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		else if (QUIET_STATE(wlc) & SILENCE) {
			WL_INFORM(("wl%d: roam scan blocked for 802.11h Quiet Period\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		break;
	case WLC_ACTION_SCAN:
		if (QUIET_STATE(wlc) & SILENCE) {
			WL_INFORM(("wl%d: scan request blocked for 802.11h Quiet Period\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		else if ((wlc->assoc_state != AS_IDLE) && (wlc->assoc_type == AS_ASSOCIATION)) {
			WL_ERROR(("wl%d: scan request blocked for association in progress\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		else if ((wlc->assoc_state != AS_IDLE) && (wlc->assoc_type == AS_ROAM)) {
			{
				WL_INFORM(("wl%d: scan request while roam is in progress, aborting"
					" roam\n", wlc->pub.unit));
				wlc_assoc_abort(wlc);
			}
		}
		else
		if (SCAN_IN_PROGRESS(wlc)) {
			WL_ERROR(("wl%d: scan request blocked for scan in progress assoc_state %d assoc_type %d , scan channel is %d\n",
				wlc->pub.unit, wlc->assoc_state, wlc->assoc_type, wlc->scan.channel_idx));
			entry_ok = FALSE;
		}
		break;


	case WLC_ACTION_RM:
		if ((wlc->assoc_state != AS_IDLE) && (wlc->assoc_type == AS_ASSOCIATION)) {
			WL_INFORM(("wl%d: radio measure blocked for association in progress\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		else if ((wlc->assoc_state != AS_IDLE) && (wlc->assoc_type == AS_ROAM)) {
			WL_INFORM(("wl%d: radio measure blocked for roam in progress\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		else if (SCAN_IN_PROGRESS(wlc)) {
			WL_INFORM(("wl%d: radio measure blocked for scan in progress\n",
				wlc->pub.unit));
			entry_ok = FALSE;
		}
		break;

	default:
		ASSERT(0);
		entry_ok = FALSE;
	}

	return entry_ok;
}

#ifdef BCMWPA2
void
wlc_pmkid_event(wlc_info_t *wlc)
{
	wlc_event_t e;
	uint32 i, pmkid_list_len = wlc->pub.npmkid_cand*sizeof(pmkid_cand_t) + sizeof(uint32);
	struct {
		pmkid_cand_list_t pmkids;
		pmkid_cand_t foo[MAXPMKID-1];
	} pmkid_list;
	pmkid_cand_t *cand = (pmkid_cand_t *)&pmkid_list.pmkids.pmkid_cand[0];

	bzero((char*)&e, sizeof(wlc_event_t));
	bzero((char*)&pmkid_list, pmkid_list_len);
	e.event.event_type = WLC_E_PMKID_CACHE;

	/* creat a copy of the pmkid cand list, store the list elements in network order */
	pmkid_list.pmkids.npmkid_cand = hton32(wlc->pub.npmkid_cand);

	for (i = 0; i < wlc->pub.npmkid_cand; i++) {
		bcopy(&wlc->pub.pmkid_cand[i].BSSID.octet[0], &cand->BSSID.octet[0],
			ETHER_ADDR_LEN);
		cand->preauth = wlc->pub.pmkid_cand[i].preauth;
		cand++;
	}
	/* point the event data at it */
	e.data = (uchar*)&pmkid_list;
	e.event.datalen = pmkid_list_len;

	wlc_event(wlc, &e);
}
#endif /* BCMWPA2 */

void
wlc_coredisable(wlc_info_t* wlc)
{
	bool dev_gone;

	WL_TRACE(("wl%d: %s\n", wlc->pub.unit, __FUNCTION__));

	ASSERT(!wlc->pub.up);

	dev_gone = DEVICEREMOVED(wlc);

	if (dev_gone)
		return;


	/* radio off */
	wlc_switch_radio(wlc->band->pi, OFF);

	wlc_phy_analogoff((void *)wlc->regs);

	/* remove gpio controls */
	if (wlc->dbgsel)
		sb_gpiocontrol(wlc->pub.sbh, ~0, 0, GPIO_DRV_PRIORITY);

	wlc->core->clk = FALSE;
	sb_core_disable(wlc->pub.sbh, 0);
}

/* power both the pll and external oscillator on/off */
void
wlc_xtal(wlc_info_t* wlc, bool want)
{
	WL_TRACE(("wl%d: wlc_xtal: want %d\n", wlc->pub.unit, want));

	/* dont power down if plldown is false or we must poll hw radio disable */
	if (!want && (!wlc->plldown || wlc->radio_monitor))
		return;

	if (wlc->pub.sbh)
		sb_clkctl_xtal(wlc->pub.sbh, XTAL|PLL, want);

	wlc->sbclk = want;
	if (!wlc->sbclk)
		wlc->corestate[0].clk = wlc->corestate[1].clk = FALSE;
}






bool
wlc_bssid_is_current(wlc_info_t *wlc, struct ether_addr *bssid)
{
	return WLC_IS_CURRENT_BSSID(wlc, bssid);
}

static void
wlc_flushqueues(wlc_info_t *wlc)
{
	uint i;

	if (!PIO_ENAB(wlc)) {
		wlc->txpend16165war = 0;

		/* free any posted tx packets */
		for (i = 0; i < NFIFO; i++)
			if (wlc->core->di[i])
				dma_txreclaim(wlc->core->di[i], TRUE);

		/* free any posted rx packets */
		dma_rxreclaim(wlc->core->di[RX_FIFO]);
		if (D11REV_LT(wlc->pub.corerev, 5))
			dma_rxreclaim(wlc->core->di[RX_TXSTATUS_FIFO]);
	} else {
		for (i = 0; i < NFIFO; i++) {
			if (wlc->core->pio[i]) {
				/* include reset the counter */
				wlc_pio_txreclaim(wlc->core->pio[i]);
			}
		}
		/* For PIO, no rx sw queue to reclaim */
	}
}

/* Convert RX hardware status to standard format and send to wl_monitor */
static void
wlc_monitor(wlc_info_t *wlc, d11rxhdr_t *rxh, void *p)
{
	struct wl_rxsts sts;
	uint32 dummy_tsf_h, tsf_l;
	uint32 rx_tsf_l;

	/* read the tsf from our chip */
	wlc_read_tsf(wlc, &tsf_l, &dummy_tsf_h);
	rx_tsf_l = (tsf_l & 0xffff0000) | rxh->RxTSFTime;
	if (rxh->RxTSFTime > (tsf_l & 0xffff)) {
		/* a greater rx frame time indicates the low 16 bits of
		 * tsf_l wrapped, so decrement the high 16 bits.
		 */
		rx_tsf_l -= 0x10000;
	}

	bzero((void *)&sts, sizeof(wl_rxsts_t));

	sts.mactime = rx_tsf_l;
	sts.antenna = (rxh->PhyRxStatus_0 & PRXS0_RXANT) ? 1 : 0;
	sts.signal = wlc_compute_rssi(wlc->band->pi, rxh);
	sts.datarate = wlc_compute_rate(rxh, PKTDATA(wlc->pub.osh, p));
	sts.pktlength = rxh->RxFrameSize - D11_PHY_HDR_LEN;
	sts.sq = ((rxh->PhyRxStatus_1 & PRXS1_SQ_MASK) >> PRXS1_SQ_SHIFT);

#if PHY_TYPE_A != WL_RXS_PHY_A
#error "RxChan phy type does not match"
#endif /* PHY_TYPE_A != WL_RXS_PHY_A */
	sts.phytype = WLC_CHAN_PHYTYPE(rxh->RxChan);
	sts.channel = wlc_radiocode2channel(WLC_CHAN_ID(rxh->RxChan),
		WLC_CHAN_PHYTYPE(rxh->RxChan));

	if (IS_CCK(sts.datarate))
		sts.preamble = (rxh->PhyRxStatus_0 &
			PRXS0_SHORTH)?WL_RXS_PREAMBLE_SHORT:WL_RXS_PREAMBLE_LONG;
	else
		sts.preamble = WL_RXS_PREAMBLE_SHORT;

	/* translate error code */
	if (rxh->RxStatus1 & RXS_DECERR)
		sts.pkterror |= WL_RXS_DECRYPT_ERR;
	if (rxh->RxStatus1 & RXS_FCSERR)
		sts.pkterror |= WL_RXS_CRC_ERROR;

	wl_monitor(wlc->wl, &sts, p);
}



void
wlc_uint64_add(uint32* high, uint32* low, uint32 inc_high, uint32 inc_low)
{
	uint32 old_l;
	uint32 new_l;

	old_l = *low;
	new_l = old_l + inc_low;
	*low = new_l;
	if (new_l < old_l) {
		/* carry */
		inc_high += 1;
	}
	*high += inc_high;
}

void
wlc_uint64_sub(uint32* a_high, uint32* a_low, uint32 b_high, uint32 b_low)
{
	if (b_low > *a_low) {
		/* low half needs a carry */
		b_high += 1;
	}
	*a_low -= b_low;
	*a_high -= b_high;
}

bool
wlc_uint64_lt(uint32 a_high, uint32 a_low, uint32 b_high, uint32 b_low)
{
	return (a_high < b_high ||
		(a_high == b_high && a_low < b_low));
}

static void
wlc_freqtrack(wlc_info_t *wlc)
{
	if (wlc->freqtrack_override != FREQTRACK_AUTO)
		return;

	ASSERT(wlc->pub.associated);

	if (!wlc->freqtrack && (wlc->pub.current_bss.channel <= WLC_MAX_2G_CHANNEL)) {
		WL_INFORM(("wl%d: FREQTRACK: Increasing freq tracking bw\n", wlc->pub.unit));
		wlc->freqtrack = TRUE;
		wlc->freqtrack_starttime = wlc->pub.now;
		wlc->freqtrack_attempts++;

		if (ISGPHY(wlc->band->pi))
			wlc_phy_freqtrack_start(wlc->band->pi);
	}
}

static void
wlc_freqtrack_reset(wlc_info_t *wlc)
{
	if (wlc->freqtrack_override != FREQTRACK_AUTO)
		return;

	wlc->freqtrack_attempts = 0;
	wlc->freqtrack_starttime = 0;

	if (wlc->freqtrack) {
		wlc->freqtrack = FALSE;

		if (ISGPHY(wlc->band->pi))
			wlc_phy_freqtrack_end(wlc->band->pi);
	}
}

static void
wlc_freqtrack_verify(wlc_info_t *wlc)
{
	if (wlc->freqtrack_override != FREQTRACK_AUTO)
		return;

	ASSERT((wlc->freqtrack_attempts > 0) &&
		(wlc->freqtrack_attempts <= WLC_FREQTRACK_MIN_ATTEMPTS));

	if (wlc->freqtrack) {
		if (wlc->time_since_bcn == 0) {
			/* Restore the frequency tracking bandwidth and check if we get
			 * beacons with the default settings. We will try this a few times
			 * before committing to the wide bandwidth
			 */
			WL_INFORM(("wl%d: FREQTRACK: Restoring freq tracking bw (attempts = %d)\n",
			           wlc->pub.unit, wlc->freqtrack_attempts));

			wlc->freqtrack = FALSE;

			if (ISGPHY(wlc->band->pi))
				wlc_phy_freqtrack_end(wlc->band->pi);
		} else if ((wlc->pub.now - wlc->freqtrack_starttime) > WLC_FREQTRACK_DETECT_TIME) {
			/* We increased the bandwidth but did not get any beacons */
			wlc_freqtrack_reset(wlc);
		}
	}
	else if ((wlc->pub.now - wlc->freqtrack_starttime) > WLC_FREQTRACK_TIMEOUT) {
		/* Either the link is down or we have been receiving beacons with
		 * the default freq tracking b/w, for quite some time now. Clear
		 * the number of attempts.
		 */
		wlc_freqtrack_reset(wlc);
	}
}



static int
wlc_set_rate_override(wlc_info_t *wlc, int band, int32 rate, bool mcast)
{
	wlcband_t *bandp;

	if (rate == -1)
		rate = 0;

	switch (band) {
	case WLC_BAND_AUTO:
		if (IS_MBAND_UNLOCKED(wlc)) {
			WL_ERROR(("wl%d: wlc_set_rate_override: band must be set\n",
			          wlc->pub.unit));
			return -1;
		}
		bandp = wlc->band;
		break;
	case WLC_BAND_5G:
		if (!BAND_5G(wlc->band->bandtype) && (NBANDS(wlc) <= 1)) {
			WL_ERROR(("wl%d: wlc_set_rate_override: 5G band not available\n",
				wlc->pub.unit));
			return -1;
		}
		bandp = &wlc->bandstate[BAND_5G_INDEX];
		break;
	case WLC_BAND_2G:
		if (!BAND_2G(wlc->band->bandtype) && (NBANDS(wlc) <= 1)) {
			WL_ERROR(("wl%d: wlc_set_rate_override: 2.4G band not available\n",
				wlc->pub.unit));
			return -1;
		}
		bandp = &wlc->bandstate[BAND_2G_INDEX];
		break;
	default:
		WL_ERROR(("wl%d: wlc_set_rate_override: invalid band %d\n", wlc->pub.unit, band));
		return -1;
	}

	/* check for a valid rate */
	if ((rate != 0) && (!wlc_valid_rate(wlc, rate, band, TRUE)))
		return -1;

	/* allow only CCK if gmode == GMODE_LEGACY_B */
	if (ISGPHY(bandp->pi) && bandp->gmode == GMODE_LEGACY_B &&
	    (rate != 0) && !IS_CCK(rate))
		return -1;

	if (mcast) {
		bandp->mrspec_override &= ~(RSPC_RATE_MASK);
		bandp->mrspec_override |= rate;
	} else {
		bandp->rspec_override &= ~(RSPC_RATE_MASK);
		bandp->rspec_override |= rate;
	}

	if (!mcast)
		wlc_reprate_init(wlc);

	return 0;
}

/* flush a tx packet by calling any registered callbacks and freeing the packet */
void
wlc_pkt_flush(wlc_info_t *wlc, void *pkt, uint16 txstatus)
{
	/* get the pkt frameid and fire off any packet callbacks */
	if (WLPKTTAG(pkt)->callbackidx)
		wlc_pkt_callback(wlc, pkt, txstatus);
	PKTFREE(wlc->pub.osh, pkt, TRUE);
}

void
wlc_txq_scb_init(void *ctx, struct scb *scb)
{
	/* Init the basic feature tx path to regular tx function */
	wlc_txmod_config((wlc_info_t *)ctx, scb, SCB_TRANSMIT);
}

void
wlc_txq_scb_free_notify(void *context, struct scb *remove)
{
	int prec;
	void *head_pkt, *pkt;
	wlc_info_t *wlc = (wlc_info_t *) context;
	struct pktq *q = &wlc->txq;

	PKTQ_PREC_ITER(q, prec) {
		head_pkt = NULL;
		while (pktq_ppeek(q, prec) != head_pkt) {
			pkt = pktq_pdeq(q, prec);
			if (WLPKTTAGSCBGET(pkt) != remove) {
				if (!head_pkt) head_pkt = pkt;
				pktq_penq(q, prec, pkt);
			} else
				wlc_pkt_flush(wlc, pkt, TX_STATUS_NO_ACK);
		}
	}
}

/* Lower BTC GPIO through ChipCommon when BTC is OFF
 * or D11 MAC is in reset or on powerup
 */
void
wlc_btc_gpio_disable(wlc_info_t *wlc)
{
	uint32 gm;
	sb_t *sbh;

	/* Proceed only if BTC is possible or set manually */
	if (!(wlc->pub.boardflags & BFL_BTCOEXIST) &&
	    ((wlc->btc_mhf & (MHF_BTCMOD | MHF_BTCOEXIST)) == 0))
		return;

	sbh = wlc->pub.sbh;

	if ((wlc->pub.boardflags & BFL_BTCMOD) ||
	    (wlc->btc_mhf & MHF_BTCMOD))
		gm = BOARD_GPIO_BTCMOD_OUT;
	else
	        gm = BOARD_GPIO_BTC_OUT;

	/* Set the control of GPIO back and lower the pin */
	sb_gpiocontrol(sbh, gm, 0, GPIO_DRV_PRIORITY);
	sb_gpioouten(sbh, gm, gm, GPIO_DRV_PRIORITY);
	sb_gpioout(sbh, gm, 0, GPIO_DRV_PRIORITY);

	/* On someboards, which give GPIOs to UART via strapping,
	 * GPIO_BTC_OUT is not directly controlled by gpioout on CC
	 */
	if ((BUSTYPE(wlc->pub.sbh->bustype) == PCI_BUS) &&
	    (gm & BOARD_GPIO_BTC_OUT))
		sb_btcgpiowar(sbh);
}

/* Register the function to handle this feature */
void
wlc_txmod_fn_register(wlc_info_t *wlc, scb_txmod_t feature_id, txmod_tx_fn_t tx_fn,
                      void *ctx, txmod_deactivate_fn_t deactivate_fn,
                      txmod_activate_fn_t activate_fn)
{
	ASSERT(feature_id < SCB_LAST);
	/* tx_fn can't be NULL */
	ASSERT(tx_fn != NULL);
	wlc->txmod_fns[feature_id].tx_fn = tx_fn;
	wlc->txmod_fns[feature_id].deactivate_notify_fn = deactivate_fn;
	wlc->txmod_fns[feature_id].activate_notify_fn = activate_fn;
	wlc->txmod_fns[feature_id].ctx = ctx;
}

